Imports System
Imports System.Drawing
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class AddEditCustomers

        Dim _isCustomerIDValid As Boolean
        Dim _isCompanyNameValid As Boolean
        Dim _customerID As String
        Private _operation As CrudOperation

        Public Property CustomerID() As String
            Get
                Return _customerID
            End Get
            Set(ByVal value As String)
                _customerID = value
            End Set
        End Property


        Public Property Operation() As String
            Get
                Return _operation
            End Get
            Set(ByVal value As String)
                _operation = value
            End Set
        End Property

        Private Sub TxtCustomerID_TextChanged(sender As Object, e As EventArgs) Handles TxtCustomerID.TextChanged
            If Not String.IsNullOrEmpty(TxtCustomerID.Text) Then
                TxtCustomerID.BackColor = Color.White
                _isCustomerIDValid = True
            Else
                _isCustomerIDValid = False
            End If
        End Sub

        Private Sub TxtCompanyName_TextChanged(sender As Object, e As EventArgs) Handles TxtCompanyName.TextChanged
            If Not String.IsNullOrEmpty(TxtCompanyName.Text) Then
                TxtCompanyName.BackColor = Color.White
                _isCompanyNameValid = True
            Else
                _isCompanyNameValid = False
            End If
        End Sub

        Private Sub BtnSave_Click(sender As Object, e As EventArgs) Handles BtnSave.Click
            Dim errors As String = ValidateEntries()

            If String.IsNullOrEmpty(errors) Then
                AddOrUpdateCustomers()
                WindowHelper.CloseParent(Me)
            Else
                MessageBox.Show(errors, "Validation errors")
            End If
        End Sub

        Private Sub BtnCancel_Click(sender As Object, e As EventArgs) Handles BtnCancel.Click
            WindowHelper.CloseParent(Me)
        End Sub

        Private Function ValidateEntries() As String
            Dim errors As String = String.Empty

            ' validate
            If Not _isCustomerIDValid Then
                errors += "Customer ID is required!" & vbCrLf
                TxtCustomerID.BackColor = Color.LightPink
            End If

            If Not _isCompanyNameValid Then
                errors += "Company Name is required!" & vbCrLf
                TxtCompanyName.BackColor = Color.LightPink
            End If

            Return errors
        End Function

        Friend Sub GetCustomersToEdit()
            Dim objCustomers As Customers = Customers.SelectByPrimaryKey(CustomerID)

            If objCustomers IsNot Nothing Then
                TxtCustomerID.Text = objCustomers.CustomerID
                TxtCompanyName.Text = objCustomers.CompanyName
                TxtContactName.Text = objCustomers.ContactName
                TxtContactTitle.Text = objCustomers.ContactTitle
                TxtAddress.Text = objCustomers.Address
                TxtCity.Text = objCustomers.City
                TxtRegion1.Text = objCustomers.Region1
                TxtPostalCode.Text = objCustomers.PostalCode
                TxtCountry.Text = objCustomers.Country
                TxtPhone.Text = objCustomers.Phone
                TxtFax.Text = objCustomers.Fax
            End If
        End Sub

        Private Sub AddOrUpdateCustomers()
            Dim objCustomers As Customers

            If Operation = CrudOperation.Add Then
                objCustomers = New Customers()
            Else
                objCustomers = Customers.SelectByPrimaryKey(CustomerID)
                objCustomers.CustomerID = CustomerID
            End If

            objCustomers.CompanyName = TxtCompanyName.Text.ToString()

            If String.IsNullOrEmpty(TxtContactName.Text.Trim()) Then
                objCustomers.ContactName = Nothing
            Else
                objCustomers.ContactName = TxtContactName.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtContactTitle.Text.Trim()) Then
                objCustomers.ContactTitle = Nothing
            Else
                objCustomers.ContactTitle = TxtContactTitle.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtAddress.Text.Trim()) Then
                objCustomers.Address = Nothing
            Else
                objCustomers.Address = TxtAddress.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtCity.Text.Trim()) Then
                objCustomers.City = Nothing
            Else
                objCustomers.City = TxtCity.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtRegion1.Text.Trim()) Then
                objCustomers.Region1 = Nothing
            Else
                objCustomers.Region1 = TxtRegion1.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtPostalCode.Text.Trim()) Then
                objCustomers.PostalCode = Nothing
            Else
                objCustomers.PostalCode = TxtPostalCode.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtCountry.Text.Trim()) Then
                objCustomers.Country = Nothing
            Else
                objCustomers.Country = TxtCountry.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtPhone.Text.Trim()) Then
                objCustomers.Phone = Nothing
            Else
                objCustomers.Phone = TxtPhone.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtFax.Text.Trim()) Then
                objCustomers.Fax = Nothing
            Else
                objCustomers.Fax = TxtFax.Text.ToString()
            End If


            Try
                If Operation = CrudOperation.Add Then
                    objCustomers.Insert()
                Else
                    objCustomers.Update()
                End If
            Catch ex As Exception
                If Operation = CrudOperation.Add Then
                    MessageBox.Show(ex.Message, "An error occured on item insertion!")
                Else
                    MessageBox.Show(ex.Message, "An error occured on item update!")
                End If
            End Try
        End Sub
    End Class
