Imports System
Imports System.Drawing
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class AddEditEmployees

        Dim _isLastNameValid As Boolean
        Dim _isFirstNameValid As Boolean
        Dim _employeeID As Integer
        Private _operation As CrudOperation

        Public Property EmployeeID() As Integer
            Get
                Return _employeeID
            End Get
            Set(ByVal value As Integer)
                _employeeID = value
            End Set
        End Property


        Public Property Operation() As String
            Get
                Return _operation
            End Get
            Set(ByVal value As String)
                _operation = value
            End Set
        End Property

        Private Sub AddEditEmployees_Load(sender As Object, e As EventArgs) Handles MyBase.Load
            SetDateFieldsToDefault()
        End Sub

        Private Sub TxtLastName_TextChanged(sender As Object, e As EventArgs) Handles TxtLastName.TextChanged
            If Not String.IsNullOrEmpty(TxtLastName.Text) Then
                TxtLastName.BackColor = Color.White
                _isLastNameValid = True
            Else
                _isLastNameValid = False
            End If
        End Sub

        Private Sub TxtFirstName_TextChanged(sender As Object, e As EventArgs) Handles TxtFirstName.TextChanged
            If Not String.IsNullOrEmpty(TxtFirstName.Text) Then
                TxtFirstName.BackColor = Color.White
                _isFirstNameValid = True
            Else
                _isFirstNameValid = False
            End If
        End Sub

        Private Sub BtnSave_Click(sender As Object, e As EventArgs) Handles BtnSave.Click
            Dim errors As String = ValidateEntries()

            If String.IsNullOrEmpty(errors) Then
                AddOrUpdateEmployees()
                WindowHelper.CloseParent(Me)
            Else
                MessageBox.Show(errors, "Validation errors")
            End If
        End Sub

        Private Sub BtnCancel_Click(sender As Object, e As EventArgs) Handles BtnCancel.Click
            WindowHelper.CloseParent(Me)
        End Sub

        Private Sub DtpBirthDate_ValueChanged(sender As Object, e As EventArgs) Handles DtpBirthDate.ValueChanged
            Me.DtpBirthDate.CustomFormat = "MM/dd/yyyy"
        End Sub

        Private Sub DtpHireDate_ValueChanged(sender As Object, e As EventArgs) Handles DtpHireDate.ValueChanged
            Me.DtpHireDate.CustomFormat = "MM/dd/yyyy"
        End Sub


        Private Sub SetDateFieldsToDefault()
            Me.DtpBirthDate.CustomFormat = " "
            Me.DtpBirthDate.Format = DateTimePickerFormat.Custom

            Me.DtpHireDate.CustomFormat = " "
            Me.DtpHireDate.Format = DateTimePickerFormat.Custom

        End Sub

        Private Function ValidateEntries() As String
            Dim errors As String = String.Empty

            ' validate
            If Not _isLastNameValid Then
                errors += "Last Name is required!" & vbCrLf
                TxtLastName.BackColor = Color.LightPink
            End If

            If Not _isFirstNameValid Then
                errors += "First Name is required!" & vbCrLf
                TxtFirstName.BackColor = Color.LightPink
            End If

            Return errors
        End Function

        Friend Sub GetEmployeesToEdit()
            Dim objEmployees As Employees = Employees.SelectByPrimaryKey(EmployeeID)

            If objEmployees IsNot Nothing Then
                TxtEmployeeID.Text = objEmployees.EmployeeID.ToString()
                TxtLastName.Text = objEmployees.LastName
                TxtFirstName.Text = objEmployees.FirstName
                TxtTitle.Text = objEmployees.Title
                TxtTitleOfCourtesy.Text = objEmployees.TitleOfCourtesy
                TxtAddress.Text = objEmployees.Address
                TxtCity.Text = objEmployees.City
                TxtRegion1.Text = objEmployees.Region1
                TxtPostalCode.Text = objEmployees.PostalCode
                TxtCountry.Text = objEmployees.Country
                TxtHomePhone.Text = objEmployees.HomePhone
                TxtExtension.Text = objEmployees.Extension
                TxtNotes.Text = objEmployees.Notes
                TxtPhotoPath.Text = objEmployees.PhotoPath

                If objEmployees.BirthDate.HasValue Then
                    DtpBirthDate.Text = objEmployees.BirthDate.ToString()
                End If

                If objEmployees.HireDate.HasValue Then
                    DtpHireDate.Text = objEmployees.HireDate.ToString()
                End If

                If objEmployees.ReportsTo.HasValue Then
                    CbxReportsTo.SelectedValue = objEmployees.ReportsTo.Value.ToString()
                End If
            End If
        End Sub

        Private Sub AddOrUpdateEmployees()
            Dim objEmployees As Employees

            If Operation = CrudOperation.Add Then
                objEmployees = New Employees()
            Else
                objEmployees = Employees.SelectByPrimaryKey(EmployeeID)
                objEmployees.EmployeeID = EmployeeID
            End If

            objEmployees.LastName = TxtLastName.Text.ToString()

            objEmployees.FirstName = TxtFirstName.Text.ToString()

            If String.IsNullOrEmpty(TxtTitle.Text.Trim()) Then
                objEmployees.Title = Nothing
            Else
                objEmployees.Title = TxtTitle.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtTitleOfCourtesy.Text.Trim()) Then
                objEmployees.TitleOfCourtesy = Nothing
            Else
                objEmployees.TitleOfCourtesy = TxtTitleOfCourtesy.Text.ToString()
            End If

            If String.IsNullOrEmpty(DtpBirthDate.Text.Trim()) Then
                objEmployees.BirthDate = Nothing
            Else
                objEmployees.BirthDate = Convert.ToDateTime(DtpBirthDate.Text.ToString())
            End If

            If String.IsNullOrEmpty(DtpHireDate.Text.Trim()) Then
                objEmployees.HireDate = Nothing
            Else
                objEmployees.HireDate = Convert.ToDateTime(DtpHireDate.Text.ToString())
            End If

            If String.IsNullOrEmpty(TxtAddress.Text.Trim()) Then
                objEmployees.Address = Nothing
            Else
                objEmployees.Address = TxtAddress.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtCity.Text.Trim()) Then
                objEmployees.City = Nothing
            Else
                objEmployees.City = TxtCity.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtRegion1.Text.Trim()) Then
                objEmployees.Region1 = Nothing
            Else
                objEmployees.Region1 = TxtRegion1.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtPostalCode.Text.Trim()) Then
                objEmployees.PostalCode = Nothing
            Else
                objEmployees.PostalCode = TxtPostalCode.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtCountry.Text.Trim()) Then
                objEmployees.Country = Nothing
            Else
                objEmployees.Country = TxtCountry.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtHomePhone.Text.Trim()) Then
                objEmployees.HomePhone = Nothing
            Else
                objEmployees.HomePhone = TxtHomePhone.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtExtension.Text.Trim()) Then
                objEmployees.Extension = Nothing
            Else
                objEmployees.Extension = TxtExtension.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtNotes.Text.Trim()) Then
                objEmployees.Notes = Nothing
            Else
                objEmployees.Notes = TxtNotes.Text.ToString()
            End If

            If CbxReportsTo.SelectedIndex < 1 Then
                objEmployees.ReportsTo = Nothing
            Else
                objEmployees.ReportsTo = Convert.ToInt32(CbxReportsTo.SelectedValue.ToString())
            End If

            If String.IsNullOrEmpty(TxtPhotoPath.Text.Trim()) Then
                objEmployees.PhotoPath = Nothing
            Else
                objEmployees.PhotoPath = TxtPhotoPath.Text.ToString()
            End If


            Try
                If Operation = CrudOperation.Add Then
                    objEmployees.Insert()
                Else
                    objEmployees.Update()
                End If
            Catch ex As Exception
                If Operation = CrudOperation.Add Then
                    MessageBox.Show(ex.Message, "An error occured on item insertion!")
                Else
                    MessageBox.Show(ex.Message, "An error occured on item update!")
                End If
            End Try
        End Sub
    End Class
