Imports System
Imports System.Drawing
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class AddEditOrderDetails

        Dim _isOrderIDValid As Boolean
        Dim _isProductIDValid As Boolean
        Dim _isUnitPriceValid As Boolean
        Dim _isQuantityValid As Boolean
        Dim _isDiscountValid As Boolean
        Dim _orderID As Integer
        Dim _productID As Integer
        Private _operation As CrudOperation

        Public Property OrderID() As Integer
            Get
                Return _orderID
            End Get
            Set(ByVal value As Integer)
                _orderID = value
            End Set
        End Property

        Public Property ProductID() As Integer
            Get
                Return _productID
            End Get
            Set(ByVal value As Integer)
                _productID = value
            End Set
        End Property


        Public Property Operation() As String
            Get
                Return _operation
            End Get
            Set(ByVal value As String)
                _operation = value
            End Set
        End Property

        Private Sub CbxOrderID_SelectedIndexChanged(sender As Object, e As EventArgs) Handles CbxOrderID.SelectedIndexChanged
            If CbxOrderID.SelectedIndex <> 0 Then
                CbxOrderID.BackColor = Color.White
                _isOrderIDValid = True
            Else
                _isOrderIDValid = False
            End If
        End Sub

        Private Sub CbxProductID_SelectedIndexChanged(sender As Object, e As EventArgs) Handles CbxProductID.SelectedIndexChanged
            If CbxProductID.SelectedIndex <> 0 Then
                CbxProductID.BackColor = Color.White
                _isProductIDValid = True
            Else
                _isProductIDValid = False
            End If
        End Sub

        Private Sub MtbUnitPrice_TextChanged(sender As Object, e As EventArgs) Handles MtbUnitPrice.TextChanged
            If Not String.IsNullOrEmpty(MtbUnitPrice.Text) Then
                MtbUnitPrice.BackColor = Color.White
                _isUnitPriceValid = True
            Else
                _isUnitPriceValid = False
            End If
        End Sub

        Private Sub MtbQuantity_TextChanged(sender As Object, e As EventArgs) Handles MtbQuantity.TextChanged
            If Not String.IsNullOrEmpty(MtbQuantity.Text) Then
                MtbQuantity.BackColor = Color.White
                _isQuantityValid = True
            Else
                _isQuantityValid = False
            End If
        End Sub

        Private Sub MtbDiscount_TextChanged(sender As Object, e As EventArgs) Handles MtbDiscount.TextChanged
            If Not String.IsNullOrEmpty(MtbDiscount.Text) Then
                MtbDiscount.BackColor = Color.White
                _isDiscountValid = True
            Else
                _isDiscountValid = False
            End If
        End Sub

        Private Sub BtnSave_Click(sender As Object, e As EventArgs) Handles BtnSave.Click
            Dim errors As String = ValidateEntries()

            If String.IsNullOrEmpty(errors) Then
                AddOrUpdateOrderDetails()
                WindowHelper.CloseParent(Me)
            Else
                MessageBox.Show(errors, "Validation errors")
            End If
        End Sub

        Private Sub BtnCancel_Click(sender As Object, e As EventArgs) Handles BtnCancel.Click
            WindowHelper.CloseParent(Me)
        End Sub

        Private Function ValidateEntries() As String
            Dim errors As String = String.Empty

            ' validate
            If Not _isOrderIDValid Then
                errors += "Order ID is required!" & vbCrLf
                CbxOrderID.BackColor = Color.LightPink
            End If

            If Not _isProductIDValid Then
                errors += "Product ID is required!" & vbCrLf
                CbxProductID.BackColor = Color.LightPink
            End If

            If Not _isUnitPriceValid Then
                errors += "Unit Price is required!" & vbCrLf
                MtbUnitPrice.BackColor = Color.LightPink
            End If

            If Not _isQuantityValid Then
                errors += "Quantity is required!" & vbCrLf
                MtbQuantity.BackColor = Color.LightPink
            End If

            If Not _isDiscountValid Then
                errors += "Discount is required!" & vbCrLf
                MtbDiscount.BackColor = Color.LightPink
            End If

            Return errors
        End Function

        Friend Sub GetOrderDetailsToEdit()
            Dim objOrderDetails As OrderDetails = OrderDetails.SelectByPrimaryKey(OrderID, ProductID)

            If objOrderDetails IsNot Nothing Then
                CbxOrderID.SelectedValue = objOrderDetails.OrderID.ToString()
                CbxProductID.SelectedValue = objOrderDetails.ProductID.ToString()
                MtbUnitPrice.Text = objOrderDetails.UnitPrice.ToString()
                MtbQuantity.Text = objOrderDetails.Quantity.ToString()
                MtbDiscount.Text = objOrderDetails.Discount.ToString()
            End If
        End Sub

        Private Sub AddOrUpdateOrderDetails()
            Dim objOrderDetails As OrderDetails
            Dim mtbUnitPriceString As String = MtbUnitPrice.Text.Trim().Replace(",", "").Replace(" ", "")
            Dim mtbQuantityString As String = MtbQuantity.Text.Trim().Replace(",", "").Replace(" ", "")
            Dim mtbDiscountString As String = MtbDiscount.Text.Trim().Replace(",", "").Replace(" ", "")

            If mtbUnitPriceString = "." Then
                mtbUnitPriceString = Nothing
            End If


            If Operation = CrudOperation.Add Then
                objOrderDetails = New OrderDetails()
            Else
                objOrderDetails = OrderDetails.SelectByPrimaryKey(OrderID, ProductID)
                objOrderDetails.OrderID = OrderID
                objOrderDetails.ProductID = ProductID
            End If

            objOrderDetails.UnitPrice = Convert.ToDecimal(MtbUnitPrice.Text.ToString())

            objOrderDetails.Quantity = Convert.ToInt16(MtbQuantity.Text.ToString())

            objOrderDetails.Discount = Convert.ToSingle(MtbDiscount.Text.ToString())


            Try
                If Operation = CrudOperation.Add Then
                    objOrderDetails.Insert()
                Else
                    objOrderDetails.Update()
                End If
            Catch ex As Exception
                If Operation = CrudOperation.Add Then
                    MessageBox.Show(ex.Message, "An error occured on item insertion!")
                Else
                    MessageBox.Show(ex.Message, "An error occured on item update!")
                End If
            End Try
        End Sub
    End Class
