Imports System
Imports System.Drawing
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class AddEditOrders

        Dim _orderID As Integer
        Private _operation As CrudOperation

        Public Property OrderID() As Integer
            Get
                Return _orderID
            End Get
            Set(ByVal value As Integer)
                _orderID = value
            End Set
        End Property


        Public Property Operation() As String
            Get
                Return _operation
            End Get
            Set(ByVal value As String)
                _operation = value
            End Set
        End Property

        Private Sub AddEditOrders_Load(sender As Object, e As EventArgs) Handles MyBase.Load
            SetDateFieldsToDefault()
        End Sub

        Private Sub BtnSave_Click(sender As Object, e As EventArgs) Handles BtnSave.Click
            Dim errors As String = ValidateEntries()

            If String.IsNullOrEmpty(errors) Then
                AddOrUpdateOrders()
                WindowHelper.CloseParent(Me)
            Else
                MessageBox.Show(errors, "Validation errors")
            End If
        End Sub

        Private Sub BtnCancel_Click(sender As Object, e As EventArgs) Handles BtnCancel.Click
            WindowHelper.CloseParent(Me)
        End Sub

        Private Sub DtpOrderDate_ValueChanged(sender As Object, e As EventArgs) Handles DtpOrderDate.ValueChanged
            Me.DtpOrderDate.CustomFormat = "MM/dd/yyyy"
        End Sub

        Private Sub DtpRequiredDate_ValueChanged(sender As Object, e As EventArgs) Handles DtpRequiredDate.ValueChanged
            Me.DtpRequiredDate.CustomFormat = "MM/dd/yyyy"
        End Sub

        Private Sub DtpShippedDate_ValueChanged(sender As Object, e As EventArgs) Handles DtpShippedDate.ValueChanged
            Me.DtpShippedDate.CustomFormat = "MM/dd/yyyy"
        End Sub


        Private Sub SetDateFieldsToDefault()
            Me.DtpOrderDate.CustomFormat = " "
            Me.DtpOrderDate.Format = DateTimePickerFormat.Custom

            Me.DtpRequiredDate.CustomFormat = " "
            Me.DtpRequiredDate.Format = DateTimePickerFormat.Custom

            Me.DtpShippedDate.CustomFormat = " "
            Me.DtpShippedDate.Format = DateTimePickerFormat.Custom

        End Sub

        Private Function ValidateEntries() As String
            Dim errors As String = String.Empty

            ' validate
            Return errors
        End Function

        Friend Sub GetOrdersToEdit()
            Dim objOrders As Orders = Orders.SelectByPrimaryKey(OrderID)

            If objOrders IsNot Nothing Then
                TxtOrderID.Text = objOrders.OrderID.ToString()
                CbxCustomerID.SelectedValue = objOrders.CustomerID
                TxtShipName.Text = objOrders.ShipName
                TxtShipAddress.Text = objOrders.ShipAddress
                TxtShipCity.Text = objOrders.ShipCity
                TxtShipRegion.Text = objOrders.ShipRegion
                TxtShipPostalCode.Text = objOrders.ShipPostalCode
                TxtShipCountry.Text = objOrders.ShipCountry

                If objOrders.EmployeeID.HasValue Then
                    CbxEmployeeID.SelectedValue = objOrders.EmployeeID.Value.ToString()
                End If

                If objOrders.OrderDate.HasValue Then
                    DtpOrderDate.Text = objOrders.OrderDate.ToString()
                End If

                If objOrders.RequiredDate.HasValue Then
                    DtpRequiredDate.Text = objOrders.RequiredDate.ToString()
                End If

                If objOrders.ShippedDate.HasValue Then
                    DtpShippedDate.Text = objOrders.ShippedDate.ToString()
                End If

                If objOrders.ShipVia.HasValue Then
                    CbxShipVia.SelectedValue = objOrders.ShipVia.Value.ToString()
                End If

                If objOrders.Freight.HasValue Then
                    MtbFreight.Text = objOrders.Freight.Value.ToString("N2")
                End If
            End If
        End Sub

        Private Sub AddOrUpdateOrders()
            Dim objOrders As Orders
            Dim mtbFreightString As String = MtbFreight.Text.Trim().Replace(",", "").Replace(" ", "")

            If mtbFreightString = "." Then
                mtbFreightString = Nothing
            End If


            If Operation = CrudOperation.Add Then
                objOrders = New Orders()
            Else
                objOrders = Orders.SelectByPrimaryKey(OrderID)
                objOrders.OrderID = OrderID
            End If

            If CbxCustomerID.SelectedIndex < 1 Then
                objOrders.CustomerID = Nothing
            Else
                objOrders.CustomerID = CbxCustomerID.SelectedValue.ToString()
            End If

            If CbxEmployeeID.SelectedIndex < 1 Then
                objOrders.EmployeeID = Nothing
            Else
                objOrders.EmployeeID = Convert.ToInt32(CbxEmployeeID.SelectedValue.ToString())
            End If

            If String.IsNullOrEmpty(DtpOrderDate.Text.Trim()) Then
                objOrders.OrderDate = Nothing
            Else
                objOrders.OrderDate = Convert.ToDateTime(DtpOrderDate.Text.ToString())
            End If

            If String.IsNullOrEmpty(DtpRequiredDate.Text.Trim()) Then
                objOrders.RequiredDate = Nothing
            Else
                objOrders.RequiredDate = Convert.ToDateTime(DtpRequiredDate.Text.ToString())
            End If

            If String.IsNullOrEmpty(DtpShippedDate.Text.Trim()) Then
                objOrders.ShippedDate = Nothing
            Else
                objOrders.ShippedDate = Convert.ToDateTime(DtpShippedDate.Text.ToString())
            End If

            If CbxShipVia.SelectedIndex < 1 Then
                objOrders.ShipVia = Nothing
            Else
                objOrders.ShipVia = Convert.ToInt32(CbxShipVia.SelectedValue.ToString())
            End If

            if (String.IsNullOrEmpty(mtbFreightString))
                objOrders.Freight = Nothing
            Else
                objOrders.Freight = Convert.ToDecimal(mtbFreightString)
            End If

            If String.IsNullOrEmpty(TxtShipName.Text.Trim()) Then
                objOrders.ShipName = Nothing
            Else
                objOrders.ShipName = TxtShipName.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtShipAddress.Text.Trim()) Then
                objOrders.ShipAddress = Nothing
            Else
                objOrders.ShipAddress = TxtShipAddress.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtShipCity.Text.Trim()) Then
                objOrders.ShipCity = Nothing
            Else
                objOrders.ShipCity = TxtShipCity.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtShipRegion.Text.Trim()) Then
                objOrders.ShipRegion = Nothing
            Else
                objOrders.ShipRegion = TxtShipRegion.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtShipPostalCode.Text.Trim()) Then
                objOrders.ShipPostalCode = Nothing
            Else
                objOrders.ShipPostalCode = TxtShipPostalCode.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtShipCountry.Text.Trim()) Then
                objOrders.ShipCountry = Nothing
            Else
                objOrders.ShipCountry = TxtShipCountry.Text.ToString()
            End If


            Try
                If Operation = CrudOperation.Add Then
                    objOrders.Insert()
                Else
                    objOrders.Update()
                End If
            Catch ex As Exception
                If Operation = CrudOperation.Add Then
                    MessageBox.Show(ex.Message, "An error occured on item insertion!")
                Else
                    MessageBox.Show(ex.Message, "An error occured on item update!")
                End If
            End Try
        End Sub
    End Class
