Imports System
Imports System.Drawing
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class AddEditProducts

        Dim _isProductNameValid As Boolean
        Dim _productID As Integer
        Private _operation As CrudOperation

        Public Property ProductID() As Integer
            Get
                Return _productID
            End Get
            Set(ByVal value As Integer)
                _productID = value
            End Set
        End Property


        Public Property Operation() As String
            Get
                Return _operation
            End Get
            Set(ByVal value As String)
                _operation = value
            End Set
        End Property

        Private Sub TxtProductName_TextChanged(sender As Object, e As EventArgs) Handles TxtProductName.TextChanged
            If Not String.IsNullOrEmpty(TxtProductName.Text) Then
                TxtProductName.BackColor = Color.White
                _isProductNameValid = True
            Else
                _isProductNameValid = False
            End If
        End Sub

        Private Sub BtnSave_Click(sender As Object, e As EventArgs) Handles BtnSave.Click
            Dim errors As String = ValidateEntries()

            If String.IsNullOrEmpty(errors) Then
                AddOrUpdateProducts()
                WindowHelper.CloseParent(Me)
            Else
                MessageBox.Show(errors, "Validation errors")
            End If
        End Sub

        Private Sub BtnCancel_Click(sender As Object, e As EventArgs) Handles BtnCancel.Click
            WindowHelper.CloseParent(Me)
        End Sub

        Private Function ValidateEntries() As String
            Dim errors As String = String.Empty

            ' validate
            If Not _isProductNameValid Then
                errors += "Product Name is required!" & vbCrLf
                TxtProductName.BackColor = Color.LightPink
            End If

            Return errors
        End Function

        Friend Sub GetProductsToEdit()
            Dim objProducts As Products = Products.SelectByPrimaryKey(ProductID)

            If objProducts IsNot Nothing Then
                TxtProductID.Text = objProducts.ProductID.ToString()
                TxtProductName.Text = objProducts.ProductName
                TxtQuantityPerUnit.Text = objProducts.QuantityPerUnit
                CbxDiscontinued.Checked = objProducts.Discontinued

                If objProducts.SupplierID.HasValue Then
                    CbxSupplierID.SelectedValue = objProducts.SupplierID.Value.ToString()
                End If

                If objProducts.CategoryID.HasValue Then
                    CbxCategoryID.SelectedValue = objProducts.CategoryID.Value.ToString()
                End If

                If objProducts.UnitPrice.HasValue Then
                    MtbUnitPrice.Text = objProducts.UnitPrice.Value.ToString("N2")
                End If

                If objProducts.UnitsInStock.HasValue Then
                    MtbUnitsInStock.Text = objProducts.UnitsInStock.Value.ToString("N0")
                End If

                If objProducts.UnitsOnOrder.HasValue Then
                    MtbUnitsOnOrder.Text = objProducts.UnitsOnOrder.Value.ToString("N0")
                End If

                If objProducts.ReorderLevel.HasValue Then
                    MtbReorderLevel.Text = objProducts.ReorderLevel.Value.ToString("N0")
                End If
            End If
        End Sub

        Private Sub AddOrUpdateProducts()
            Dim objProducts As Products
            Dim mtbUnitPriceString As String = MtbUnitPrice.Text.Trim().Replace(",", "").Replace(" ", "")
            Dim mtbUnitsInStockString As String = MtbUnitsInStock.Text.Trim().Replace(",", "").Replace(" ", "")
            Dim mtbUnitsOnOrderString As String = MtbUnitsOnOrder.Text.Trim().Replace(",", "").Replace(" ", "")
            Dim mtbReorderLevelString As String = MtbReorderLevel.Text.Trim().Replace(",", "").Replace(" ", "")

            If mtbUnitPriceString = "." Then
                mtbUnitPriceString = Nothing
            End If


            If Operation = CrudOperation.Add Then
                objProducts = New Products()
            Else
                objProducts = Products.SelectByPrimaryKey(ProductID)
                objProducts.ProductID = ProductID
            End If

            objProducts.ProductName = TxtProductName.Text.ToString()

            If CbxSupplierID.SelectedIndex < 1 Then
                objProducts.SupplierID = Nothing
            Else
                objProducts.SupplierID = Convert.ToInt32(CbxSupplierID.SelectedValue.ToString())
            End If

            If CbxCategoryID.SelectedIndex < 1 Then
                objProducts.CategoryID = Nothing
            Else
                objProducts.CategoryID = Convert.ToInt32(CbxCategoryID.SelectedValue.ToString())
            End If

            If String.IsNullOrEmpty(TxtQuantityPerUnit.Text.Trim()) Then
                objProducts.QuantityPerUnit = Nothing
            Else
                objProducts.QuantityPerUnit = TxtQuantityPerUnit.Text.ToString()
            End If

            if (String.IsNullOrEmpty(mtbUnitPriceString))
                objProducts.UnitPrice = Nothing
            Else
                objProducts.UnitPrice = Convert.ToDecimal(mtbUnitPriceString)
            End If

            if (String.IsNullOrEmpty(mtbUnitsInStockString))
                objProducts.UnitsInStock = Nothing
            Else
                objProducts.UnitsInStock = Convert.ToInt16(mtbUnitsInStockString)
            End If

            if (String.IsNullOrEmpty(mtbUnitsOnOrderString))
                objProducts.UnitsOnOrder = Nothing
            Else
                objProducts.UnitsOnOrder = Convert.ToInt16(mtbUnitsOnOrderString)
            End If

            if (String.IsNullOrEmpty(mtbReorderLevelString))
                objProducts.ReorderLevel = Nothing
            Else
                objProducts.ReorderLevel = Convert.ToInt16(mtbReorderLevelString)
            End If

            objProducts.Discontinued = CbxDiscontinued.Checked


            Try
                If Operation = CrudOperation.Add Then
                    objProducts.Insert()
                Else
                    objProducts.Update()
                End If
            Catch ex As Exception
                If Operation = CrudOperation.Add Then
                    MessageBox.Show(ex.Message, "An error occured on item insertion!")
                Else
                    MessageBox.Show(ex.Message, "An error occured on item update!")
                End If
            End Try
        End Sub
    End Class
