Imports System
Imports System.Drawing
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class AddEditRegion

        Dim _isRegionIDValid As Boolean
        Dim _isRegionDescriptionValid As Boolean
        Dim _regionID As Integer
        Private _operation As CrudOperation

        Public Property RegionID() As Integer
            Get
                Return _regionID
            End Get
            Set(ByVal value As Integer)
                _regionID = value
            End Set
        End Property


        Public Property Operation() As String
            Get
                Return _operation
            End Get
            Set(ByVal value As String)
                _operation = value
            End Set
        End Property

        Private Sub MtbRegionID_TextChanged(sender As Object, e As EventArgs) Handles MtbRegionID.TextChanged
            If Not String.IsNullOrEmpty(MtbRegionID.Text) Then
                MtbRegionID.BackColor = Color.White
                _isRegionIDValid = True
            Else
                _isRegionIDValid = False
            End If
        End Sub

        Private Sub TxtRegionDescription_TextChanged(sender As Object, e As EventArgs) Handles TxtRegionDescription.TextChanged
            If Not String.IsNullOrEmpty(TxtRegionDescription.Text) Then
                TxtRegionDescription.BackColor = Color.White
                _isRegionDescriptionValid = True
            Else
                _isRegionDescriptionValid = False
            End If
        End Sub

        Private Sub BtnSave_Click(sender As Object, e As EventArgs) Handles BtnSave.Click
            Dim errors As String = ValidateEntries()

            If String.IsNullOrEmpty(errors) Then
                AddOrUpdateRegion()
                WindowHelper.CloseParent(Me)
            Else
                MessageBox.Show(errors, "Validation errors")
            End If
        End Sub

        Private Sub BtnCancel_Click(sender As Object, e As EventArgs) Handles BtnCancel.Click
            WindowHelper.CloseParent(Me)
        End Sub

        Private Function ValidateEntries() As String
            Dim errors As String = String.Empty

            ' validate
            If Not _isRegionIDValid Then
                errors += "Region ID is required!" & vbCrLf
                MtbRegionID.BackColor = Color.LightPink
            End If

            If Not _isRegionDescriptionValid Then
                errors += "Region Description is required!" & vbCrLf
                TxtRegionDescription.BackColor = Color.LightPink
            End If

            Return errors
        End Function

        Friend Sub GetRegionToEdit()
            Dim objRegion As BusinessObject.Region = BusinessObject.Region.SelectByPrimaryKey(RegionID)

            If objRegion IsNot Nothing Then
                MtbRegionID.Text = objRegion.RegionID.ToString()
                TxtRegionDescription.Text = objRegion.RegionDescription
            End If
        End Sub

        Private Sub AddOrUpdateRegion()
            Dim objRegion As BusinessObject.Region
            Dim mtbRegionIDString As String = MtbRegionID.Text.Trim().Replace(",", "").Replace(" ", "")

            If Operation = CrudOperation.Add Then
                objRegion = New BusinessObject.Region()
            Else
                objRegion = BusinessObject.Region.SelectByPrimaryKey(RegionID)
                objRegion.RegionID = RegionID
            End If

            objRegion.RegionDescription = TxtRegionDescription.Text.ToString()


            Try
                If Operation = CrudOperation.Add Then
                    objRegion.Insert()
                Else
                    objRegion.Update()
                End If
            Catch ex As Exception
                If Operation = CrudOperation.Add Then
                    MessageBox.Show(ex.Message, "An error occured on item insertion!")
                Else
                    MessageBox.Show(ex.Message, "An error occured on item update!")
                End If
            End Try
        End Sub
    End Class
