Imports System
Imports System.Drawing
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class AddEditSuppliers

        Dim _isCompanyNameValid As Boolean
        Dim _supplierID As Integer
        Private _operation As CrudOperation

        Public Property SupplierID() As Integer
            Get
                Return _supplierID
            End Get
            Set(ByVal value As Integer)
                _supplierID = value
            End Set
        End Property


        Public Property Operation() As String
            Get
                Return _operation
            End Get
            Set(ByVal value As String)
                _operation = value
            End Set
        End Property

        Private Sub TxtCompanyName_TextChanged(sender As Object, e As EventArgs) Handles TxtCompanyName.TextChanged
            If Not String.IsNullOrEmpty(TxtCompanyName.Text) Then
                TxtCompanyName.BackColor = Color.White
                _isCompanyNameValid = True
            Else
                _isCompanyNameValid = False
            End If
        End Sub

        Private Sub BtnSave_Click(sender As Object, e As EventArgs) Handles BtnSave.Click
            Dim errors As String = ValidateEntries()

            If String.IsNullOrEmpty(errors) Then
                AddOrUpdateSuppliers()
                WindowHelper.CloseParent(Me)
            Else
                MessageBox.Show(errors, "Validation errors")
            End If
        End Sub

        Private Sub BtnCancel_Click(sender As Object, e As EventArgs) Handles BtnCancel.Click
            WindowHelper.CloseParent(Me)
        End Sub

        Private Function ValidateEntries() As String
            Dim errors As String = String.Empty

            ' validate
            If Not _isCompanyNameValid Then
                errors += "Company Name is required!" & vbCrLf
                TxtCompanyName.BackColor = Color.LightPink
            End If

            Return errors
        End Function

        Friend Sub GetSuppliersToEdit()
            Dim objSuppliers As Suppliers = Suppliers.SelectByPrimaryKey(SupplierID)

            If objSuppliers IsNot Nothing Then
                TxtSupplierID.Text = objSuppliers.SupplierID.ToString()
                TxtCompanyName.Text = objSuppliers.CompanyName
                TxtContactName.Text = objSuppliers.ContactName
                TxtContactTitle.Text = objSuppliers.ContactTitle
                TxtAddress.Text = objSuppliers.Address
                TxtCity.Text = objSuppliers.City
                TxtRegion1.Text = objSuppliers.Region1
                TxtPostalCode.Text = objSuppliers.PostalCode
                TxtCountry.Text = objSuppliers.Country
                TxtPhone.Text = objSuppliers.Phone
                TxtFax.Text = objSuppliers.Fax
                TxtHomePage.Text = objSuppliers.HomePage
            End If
        End Sub

        Private Sub AddOrUpdateSuppliers()
            Dim objSuppliers As Suppliers

            If Operation = CrudOperation.Add Then
                objSuppliers = New Suppliers()
            Else
                objSuppliers = Suppliers.SelectByPrimaryKey(SupplierID)
                objSuppliers.SupplierID = SupplierID
            End If

            objSuppliers.CompanyName = TxtCompanyName.Text.ToString()

            If String.IsNullOrEmpty(TxtContactName.Text.Trim()) Then
                objSuppliers.ContactName = Nothing
            Else
                objSuppliers.ContactName = TxtContactName.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtContactTitle.Text.Trim()) Then
                objSuppliers.ContactTitle = Nothing
            Else
                objSuppliers.ContactTitle = TxtContactTitle.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtAddress.Text.Trim()) Then
                objSuppliers.Address = Nothing
            Else
                objSuppliers.Address = TxtAddress.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtCity.Text.Trim()) Then
                objSuppliers.City = Nothing
            Else
                objSuppliers.City = TxtCity.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtRegion1.Text.Trim()) Then
                objSuppliers.Region1 = Nothing
            Else
                objSuppliers.Region1 = TxtRegion1.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtPostalCode.Text.Trim()) Then
                objSuppliers.PostalCode = Nothing
            Else
                objSuppliers.PostalCode = TxtPostalCode.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtCountry.Text.Trim()) Then
                objSuppliers.Country = Nothing
            Else
                objSuppliers.Country = TxtCountry.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtPhone.Text.Trim()) Then
                objSuppliers.Phone = Nothing
            Else
                objSuppliers.Phone = TxtPhone.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtFax.Text.Trim()) Then
                objSuppliers.Fax = Nothing
            Else
                objSuppliers.Fax = TxtFax.Text.ToString()
            End If

            If String.IsNullOrEmpty(TxtHomePage.Text.Trim()) Then
                objSuppliers.HomePage = Nothing
            Else
                objSuppliers.HomePage = TxtHomePage.Text.ToString()
            End If


            Try
                If Operation = CrudOperation.Add Then
                    objSuppliers.Insert()
                Else
                    objSuppliers.Update()
                End If
            Catch ex As Exception
                If Operation = CrudOperation.Add Then
                    MessageBox.Show(ex.Message, "An error occured on item insertion!")
                Else
                    MessageBox.Show(ex.Message, "An error occured on item update!")
                End If
            End Try
        End Sub
    End Class
