Imports System
Imports System.Drawing
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class AddEditTerritories

        Dim _isTerritoryIDValid As Boolean
        Dim _isTerritoryDescriptionValid As Boolean
        Dim _isRegionIDValid As Boolean
        Dim _territoryID As String
        Private _operation As CrudOperation

        Public Property TerritoryID() As String
            Get
                Return _territoryID
            End Get
            Set(ByVal value As String)
                _territoryID = value
            End Set
        End Property


        Public Property Operation() As String
            Get
                Return _operation
            End Get
            Set(ByVal value As String)
                _operation = value
            End Set
        End Property

        Private Sub TxtTerritoryID_TextChanged(sender As Object, e As EventArgs) Handles TxtTerritoryID.TextChanged
            If Not String.IsNullOrEmpty(TxtTerritoryID.Text) Then
                TxtTerritoryID.BackColor = Color.White
                _isTerritoryIDValid = True
            Else
                _isTerritoryIDValid = False
            End If
        End Sub

        Private Sub TxtTerritoryDescription_TextChanged(sender As Object, e As EventArgs) Handles TxtTerritoryDescription.TextChanged
            If Not String.IsNullOrEmpty(TxtTerritoryDescription.Text) Then
                TxtTerritoryDescription.BackColor = Color.White
                _isTerritoryDescriptionValid = True
            Else
                _isTerritoryDescriptionValid = False
            End If
        End Sub

        Private Sub CbxRegionID_SelectedIndexChanged(sender As Object, e As EventArgs) Handles CbxRegionID.SelectedIndexChanged
            If CbxRegionID.SelectedIndex <> 0 Then
                CbxRegionID.BackColor = Color.White
                _isRegionIDValid = True
            Else
                _isRegionIDValid = False
            End If
        End Sub

        Private Sub BtnSave_Click(sender As Object, e As EventArgs) Handles BtnSave.Click
            Dim errors As String = ValidateEntries()

            If String.IsNullOrEmpty(errors) Then
                AddOrUpdateTerritories()
                WindowHelper.CloseParent(Me)
            Else
                MessageBox.Show(errors, "Validation errors")
            End If
        End Sub

        Private Sub BtnCancel_Click(sender As Object, e As EventArgs) Handles BtnCancel.Click
            WindowHelper.CloseParent(Me)
        End Sub

        Private Function ValidateEntries() As String
            Dim errors As String = String.Empty

            ' validate
            If Not _isTerritoryIDValid Then
                errors += "Territory ID is required!" & vbCrLf
                TxtTerritoryID.BackColor = Color.LightPink
            End If

            If Not _isTerritoryDescriptionValid Then
                errors += "Territory Description is required!" & vbCrLf
                TxtTerritoryDescription.BackColor = Color.LightPink
            End If

            If Not _isRegionIDValid Then
                errors += "Region ID is required!" & vbCrLf
                CbxRegionID.BackColor = Color.LightPink
            End If

            Return errors
        End Function

        Friend Sub GetTerritoriesToEdit()
            Dim objTerritories As Territories = Territories.SelectByPrimaryKey(TerritoryID)

            If objTerritories IsNot Nothing Then
                TxtTerritoryID.Text = objTerritories.TerritoryID
                TxtTerritoryDescription.Text = objTerritories.TerritoryDescription
                CbxRegionID.SelectedValue = objTerritories.RegionID.ToString()
            End If
        End Sub

        Private Sub AddOrUpdateTerritories()
            Dim objTerritories As Territories

            If Operation = CrudOperation.Add Then
                objTerritories = New Territories()
            Else
                objTerritories = Territories.SelectByPrimaryKey(TerritoryID)
                objTerritories.TerritoryID = TerritoryID
            End If

            objTerritories.TerritoryDescription = TxtTerritoryDescription.Text.ToString()

            objTerritories.RegionID = Convert.ToInt32(CbxRegionID.SelectedValue.ToString())


            Try
                If Operation = CrudOperation.Add Then
                    objTerritories.Insert()
                Else
                    objTerritories.Update()
                End If
            Catch ex As Exception
                If Operation = CrudOperation.Add Then
                    MessageBox.Show(ex.Message, "An error occured on item insertion!")
                Else
                    MessageBox.Show(ex.Message, "An error occured on item update!")
                End If
            End Try
        End Sub
    End Class
