Imports System
Imports System.Data
Imports NorthwindWinFormDynamic_VB.DataLayer
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Models
Imports System.Runtime.InteropServices

Namespace BusinessObject.Base

     ''' <summary>
     ''' Base class for Customers.  Do not make changes to this class,
     ''' instead, put additional code in the Customers class
     ''' </summary>
     Public Class CustomersBase
         ' private members
         Private _customerID As String
         Private _companyName As String
         Private _contactName As String
         Private _contactTitle As String
         Private _address As String
         Private _city As String
         Private _region1 As String
         Private _postalCode As String
         Private _country As String
         Private _phone As String
         Private _fax As String

         ''' <summary>
         ''' Gets or Sets CustomerID
         ''' </summary>
         Public Property CustomerID() As String
             Get
                 Return _customerID
             End Get
             Set(ByVal value As String)
                 _customerID = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets CompanyName
         ''' </summary>
         Public Property CompanyName() As String
             Get
                 Return _companyName
             End Get
             Set(ByVal value As String)
                 _companyName = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets ContactName
         ''' </summary>
         Public Property ContactName() As String
             Get
                 Return _contactName
             End Get
             Set(ByVal value As String)
                 _contactName = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets ContactTitle
         ''' </summary>
         Public Property ContactTitle() As String
             Get
                 Return _contactTitle
             End Get
             Set(ByVal value As String)
                 _contactTitle = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets Address
         ''' </summary>
         Public Property Address() As String
             Get
                 Return _address
             End Get
             Set(ByVal value As String)
                 _address = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets City
         ''' </summary>
         Public Property City() As String
             Get
                 Return _city
             End Get
             Set(ByVal value As String)
                 _city = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets Region1
         ''' </summary>
         Public Property Region1() As String
             Get
                 Return _region1
             End Get
             Set(ByVal value As String)
                 _region1 = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets PostalCode
         ''' </summary>
         Public Property PostalCode() As String
             Get
                 Return _postalCode
             End Get
             Set(ByVal value As String)
                 _postalCode = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets Country
         ''' </summary>
         Public Property Country() As String
             Get
                 Return _country
             End Get
             Set(ByVal value As String)
                 _country = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets Phone
         ''' </summary>
         Public Property Phone() As String
             Get
                 Return _phone
             End Get
             Set(ByVal value As String)
                 _phone = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets Fax
         ''' </summary>
         Public Property Fax() As String
             Get
                 Return _fax
             End Get
             Set(ByVal value As String)
                 _fax = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or sets the related Orders(s) by CustomerID
         ''' </summary>
         Public ReadOnly Property OrdersCollection() As Lazy(Of OrdersCollection)
             Get
                 If Not String.IsNullOrEmpty(CustomerID) Then
                     Return New Lazy(Of OrdersCollection)(Function() Orders.SelectOrdersCollectionByCustomerID(CustomerID))
                 Else
                     Return Nothing
                 End If
             End Get
         End Property


         ''' <summary>
         ''' Constructor
         ''' </summary>
         Public Sub New()
         End Sub

         ''' <summary>
         ''' Selects a record by primary key(s)
         ''' </summary>
         Public Shared Function SelectByPrimaryKey(ByVal customerID As String) As Customers
             Return CustomersDataLayer.SelectByPrimaryKey(customerID)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Customers table
         ''' </summary>
         Public Shared Function GetRecordCount() As Integer
             Return CustomersDataLayer.GetRecordCount()
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Customers table based on search parameters
         ''' </summary>
         Public Shared Function GetRecordCountDynamicWhere(customerID As String, companyName As String, contactName As String, contactTitle As String, address As String, city As String, region1 As String, postalCode As String, country As String, phone As String, fax As String) As Integer
             Return CustomersDataLayer.GetRecordCountDynamicWhere(customerID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Customers sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared Function SelectSkipAndTake(rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer, sortByExpression As String) As CustomersCollection
             totalRowCount = GetRecordCount()
             sortByExpression = GetSortExpression(sortByExpression)

             Return CustomersDataLayer.SelectSkipAndTake(sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Customers sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex.  Use for non model-binded gridview control.  Does not return total row count.
         ''' </summary>
         Public Shared Function SelectSkipAndTake(rows As Integer, startRowIndex As Integer, sortByExpression As String) As CustomersCollection
             sortByExpression = GetSortExpression(sortByExpression)
             Return CustomersDataLayer.SelectSkipAndTake(sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Customers sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex, based on the search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(customerID As String, companyName As String, contactName As String, contactTitle As String, address As String, city As String, region1 As String, postalCode As String, country As String, phone As String, fax As String, rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer, sortByExpression As String) As CustomersCollection
             totalRowCount = GetRecordCountDynamicWhere(customerID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax)
             sortByExpression = GetSortExpression(sortByExpression)
             Return CustomersDataLayer.SelectSkipAndTakeDynamicWhere(customerID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Customers sorted by the sortByExpression starting from the startRowIndex, based on the search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(customerID As String, companyName As String, contactName As String, contactTitle As String, address As String, city As String, region1 As String, postalCode As String, country As String, phone As String, fax As String, rows As Integer, startRowIndex As Integer, sortByExpression As String) As CustomersCollection
             sortByExpression = GetSortExpression(sortByExpression)
             Return CustomersDataLayer.SelectSkipAndTakeDynamicWhere(customerID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects all records as a collection (List) of Customers
         ''' </summary>
         Public Shared Function SelectAll() As CustomersCollection
             Return CustomersDataLayer.SelectAll()
         End Function

         ''' <summary>
         ''' Selects all records as a collection (List) of Customers sorted by the sort expression
         ''' </summary>
         Public Shared Function SelectAll(sortExpression As String) As CustomersCollection
             Dim objCustomersCol As CustomersCollection = CustomersDataLayer.SelectAll()
             Return SortByExpression(objCustomersCol, sortExpression)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Customers.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(customerID As String, companyName As String, contactName As String, contactTitle As String, address As String, city As String, region1 As String, postalCode As String, country As String, phone As String, fax As String) As CustomersCollection
             Return CustomersDataLayer.SelectAllDynamicWhere(customerID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Customers sorted by the sort expression.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(customerID As String, companyName As String, contactName As String, contactTitle As String, address As String, city As String, region1 As String, postalCode As String, country As String, phone As String, fax As String, sortExpression As String) As CustomersCollection
             Dim objCustomersCol As CustomersCollection = CustomersDataLayer.SelectAllDynamicWhere(customerID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax)
             Return SortByExpression(objCustomersCol, sortExpression)
         End Function

         ''' <summary>
         ''' Selects CustomerID and CompanyName columns for use with a DropDownList web control, ComboBox, CheckedBoxList, ListView, ListBox, etc
         ''' </summary>
         Public Shared Function SelectCustomersDropDownListData() As CustomersCollection
             Return CustomersDataLayer.SelectCustomersDropDownListData()
         End Function

         ''' <summary>
         ''' Sorts the CustomersCollection by sort expression
         ''' </summary>
         Public Shared Function SortByExpression(objCustomersCol As CustomersCollection, sortExpression As String) As CustomersCollection
             Dim isSortDescending As Boolean = sortExpression.ToLower().Contains(" desc")

             If isSortDescending Then
                 sortExpression = sortExpression.Replace(" DESC", "")
                 sortExpression = sortExpression.Replace(" desc", "")
             Else
                 sortExpression = sortExpression.Replace(" ASC", "")
                 sortExpression = sortExpression.Replace(" asc", "")
             End If

             Select Case sortExpression
                 Case "CustomerID"
                     objCustomersCol.Sort(Customers.ByCustomerID)
                     Exit Select
                 Case "CompanyName"
                     objCustomersCol.Sort(Customers.ByCompanyName)
                     Exit Select
                 Case "ContactName"
                     objCustomersCol.Sort(Customers.ByContactName)
                     Exit Select
                 Case "ContactTitle"
                     objCustomersCol.Sort(Customers.ByContactTitle)
                     Exit Select
                 Case "Address"
                     objCustomersCol.Sort(Customers.ByAddress)
                     Exit Select
                 Case "City"
                     objCustomersCol.Sort(Customers.ByCity)
                     Exit Select
                 Case "Region1"
                     objCustomersCol.Sort(Customers.ByRegion1)
                     Exit Select
                 Case "PostalCode"
                     objCustomersCol.Sort(Customers.ByPostalCode)
                     Exit Select
                 Case "Country"
                     objCustomersCol.Sort(Customers.ByCountry)
                     Exit Select
                 Case "Phone"
                     objCustomersCol.Sort(Customers.ByPhone)
                     Exit Select
                 Case "Fax"
                     objCustomersCol.Sort(Customers.ByFax)
                     Exit Select
                 Case Else
                     Exit Select
             End Select

             If isSortDescending Then
                 objCustomersCol.Reverse()
             End If

             Return objCustomersCol
         End Function

         ''' <summary>
         ''' Inserts a record
         ''' </summary>
         Public Function Insert() As String
             Dim objCustomers As Customers = DirectCast(Me, Customers)
             Return CustomersDataLayer.Insert(objCustomers)
         End Function

         ''' <summary>
         ''' Updates a record
         ''' </summary>
         Public Sub Update()
             Dim objCustomers As Customers = DirectCast(Me, Customers)
             CustomersDataLayer.Update(objCustomers)
         End Sub

         ''' <summary>
         ''' Deletes a record based on primary key(s)
         ''' </summary>
         Public Shared Sub Delete(ByVal customerID As String)
             CustomersDataLayer.Delete(customerID)
         End Sub

         Private Shared Function GetSortExpression(sortByExpression As String) As String
             If [String].IsNullOrEmpty(sortByExpression) Or sortByExpression = " asc" Then
                 sortByExpression = "CustomerID"
             Else If sortByExpression.Contains(" asc") Then
                 sortByExpression = sortByExpression.Replace(" asc", "")
             End If

             Return sortByExpression
         End Function

         ''' <summary>
         ''' Compares CustomerID used for sorting
         ''' </summary>
         Public Shared ByCustomerID As Comparison(Of Customers) =
         Function(ByVal x As Customers, ByVal y As Customers)
             Dim value1 As String = If(x.CustomerID, String.Empty)
             Dim value2 As String = If(y.CustomerID, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares CompanyName used for sorting
         ''' </summary>
         Public Shared ByCompanyName As Comparison(Of Customers) =
         Function(ByVal x As Customers, ByVal y As Customers)
             Dim value1 As String = If(x.CompanyName, String.Empty)
             Dim value2 As String = If(y.CompanyName, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares ContactName used for sorting
         ''' </summary>
         Public Shared ByContactName As Comparison(Of Customers) =
         Function(ByVal x As Customers, ByVal y As Customers)
             Dim value1 As String = If(x.ContactName, String.Empty)
             Dim value2 As String = If(y.ContactName, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares ContactTitle used for sorting
         ''' </summary>
         Public Shared ByContactTitle As Comparison(Of Customers) =
         Function(ByVal x As Customers, ByVal y As Customers)
             Dim value1 As String = If(x.ContactTitle, String.Empty)
             Dim value2 As String = If(y.ContactTitle, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares Address used for sorting
         ''' </summary>
         Public Shared ByAddress As Comparison(Of Customers) =
         Function(ByVal x As Customers, ByVal y As Customers)
             Dim value1 As String = If(x.Address, String.Empty)
             Dim value2 As String = If(y.Address, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares City used for sorting
         ''' </summary>
         Public Shared ByCity As Comparison(Of Customers) =
         Function(ByVal x As Customers, ByVal y As Customers)
             Dim value1 As String = If(x.City, String.Empty)
             Dim value2 As String = If(y.City, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares Region1 used for sorting
         ''' </summary>
         Public Shared ByRegion1 As Comparison(Of Customers) =
         Function(ByVal x As Customers, ByVal y As Customers)
             Dim value1 As String = If(x.Region1, String.Empty)
             Dim value2 As String = If(y.Region1, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares PostalCode used for sorting
         ''' </summary>
         Public Shared ByPostalCode As Comparison(Of Customers) =
         Function(ByVal x As Customers, ByVal y As Customers)
             Dim value1 As String = If(x.PostalCode, String.Empty)
             Dim value2 As String = If(y.PostalCode, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares Country used for sorting
         ''' </summary>
         Public Shared ByCountry As Comparison(Of Customers) =
         Function(ByVal x As Customers, ByVal y As Customers)
             Dim value1 As String = If(x.Country, String.Empty)
             Dim value2 As String = If(y.Country, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares Phone used for sorting
         ''' </summary>
         Public Shared ByPhone As Comparison(Of Customers) =
         Function(ByVal x As Customers, ByVal y As Customers)
             Dim value1 As String = If(x.Phone, String.Empty)
             Dim value2 As String = If(y.Phone, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares Fax used for sorting
         ''' </summary>
         Public Shared ByFax As Comparison(Of Customers) =
         Function(ByVal x As Customers, ByVal y As Customers)
             Dim value1 As String = If(x.Fax, String.Empty)
             Dim value2 As String = If(y.Fax, String.Empty)
             Return value1.CompareTo(value2)
         End Function

     End Class
End Namespace
