Imports System
Imports System.Data
Imports NorthwindWinFormDynamic_VB.DataLayer
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Models
Imports System.Runtime.InteropServices

Namespace BusinessObject.Base

     ''' <summary>
     ''' Base class for Employees.  Do not make changes to this class,
     ''' instead, put additional code in the Employees class
     ''' </summary>
     Public Class EmployeesBase
         ' private members
         Private _employeeID As Integer
         Private _lastName As String
         Private _firstName As String
         Private _title As String
         Private _titleOfCourtesy As String
         Private _birthDate As System.Nullable(Of Date)
         Private _hireDate As System.Nullable(Of Date)
         Private _address As String
         Private _city As String
         Private _region1 As String
         Private _postalCode As String
         Private _country As String
         Private _homePhone As String
         Private _extension As String
         Private _notes As String
         Private _reportsTo As System.Nullable(Of Integer)
         Private _photoPath As String

         ''' <summary>
         ''' Gets or Sets EmployeeID
         ''' </summary>
         Public Property EmployeeID() As Integer
             Get
                 Return _employeeID
             End Get
             Set(ByVal value As Integer)
                 _employeeID = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets LastName
         ''' </summary>
         Public Property LastName() As String
             Get
                 Return _lastName
             End Get
             Set(ByVal value As String)
                 _lastName = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets FirstName
         ''' </summary>
         Public Property FirstName() As String
             Get
                 Return _firstName
             End Get
             Set(ByVal value As String)
                 _firstName = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets Title
         ''' </summary>
         Public Property Title() As String
             Get
                 Return _title
             End Get
             Set(ByVal value As String)
                 _title = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets TitleOfCourtesy
         ''' </summary>
         Public Property TitleOfCourtesy() As String
             Get
                 Return _titleOfCourtesy
             End Get
             Set(ByVal value As String)
                 _titleOfCourtesy = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets BirthDate
         ''' </summary>
         Public Property BirthDate() As System.Nullable(Of Date)
             Get
                 Return _birthDate
             End Get
             Set(ByVal value As System.Nullable(Of Date))
                 _birthDate = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets HireDate
         ''' </summary>
         Public Property HireDate() As System.Nullable(Of Date)
             Get
                 Return _hireDate
             End Get
             Set(ByVal value As System.Nullable(Of Date))
                 _hireDate = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets Address
         ''' </summary>
         Public Property Address() As String
             Get
                 Return _address
             End Get
             Set(ByVal value As String)
                 _address = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets City
         ''' </summary>
         Public Property City() As String
             Get
                 Return _city
             End Get
             Set(ByVal value As String)
                 _city = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets Region1
         ''' </summary>
         Public Property Region1() As String
             Get
                 Return _region1
             End Get
             Set(ByVal value As String)
                 _region1 = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets PostalCode
         ''' </summary>
         Public Property PostalCode() As String
             Get
                 Return _postalCode
             End Get
             Set(ByVal value As String)
                 _postalCode = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets Country
         ''' </summary>
         Public Property Country() As String
             Get
                 Return _country
             End Get
             Set(ByVal value As String)
                 _country = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets HomePhone
         ''' </summary>
         Public Property HomePhone() As String
             Get
                 Return _homePhone
             End Get
             Set(ByVal value As String)
                 _homePhone = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets Extension
         ''' </summary>
         Public Property Extension() As String
             Get
                 Return _extension
             End Get
             Set(ByVal value As String)
                 _extension = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets Notes
         ''' </summary>
         Public Property Notes() As String
             Get
                 Return _notes
             End Get
             Set(ByVal value As String)
                 _notes = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets ReportsTo
         ''' </summary>
         Public Property ReportsTo() As System.Nullable(Of Integer)
             Get
                 Return _reportsTo
             End Get
             Set(ByVal value As System.Nullable(Of Integer))
                 _reportsTo = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets PhotoPath
         ''' </summary>
         Public Property PhotoPath() As String
             Get
                 Return _photoPath
             End Get
             Set(ByVal value As String)
                 _photoPath = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or sets the Related Employees.  Related to column ReportsTo
         ''' </summary>
         Public ReadOnly Property EmployeesReportsTo() As Lazy(Of Employees)
             Get
                 Dim value As Integer
                 Dim hasValue As Boolean = Int32.TryParse(ReportsTo.ToString(), value)

                 If hasValue Then
                     Return New Lazy(Of Employees)(Function() BusinessObject.Employees.SelectByPrimaryKey(value))
                 Else
                     Return Nothing
                 End If
             End Get
         End Property

         ''' <summary>
         ''' Gets or sets the related Employees(s) by EmployeeID
         ''' </summary>
         Public ReadOnly Property EmployeesCollection() As Lazy(Of EmployeesCollection)
             Get
                 Dim value As Integer
                 Dim hasValue As Boolean = Int32.TryParse(EmployeeID.ToString(), value)

                 If hasValue Then
                     Return New Lazy(Of EmployeesCollection)(Function() Employees.SelectEmployeesCollectionByReportsTo(value))
                 Else
                     Return Nothing
                 End If
             End Get
         End Property

         ''' <summary>
         ''' Gets or sets the related Orders(s) by EmployeeID
         ''' </summary>
         Public ReadOnly Property OrdersCollection() As Lazy(Of OrdersCollection)
             Get
                 Dim value As Integer
                 Dim hasValue As Boolean = Int32.TryParse(EmployeeID.ToString(), value)

                 If hasValue Then
                     Return New Lazy(Of OrdersCollection)(Function() Orders.SelectOrdersCollectionByEmployeeID(value))
                 Else
                     Return Nothing
                 End If
             End Get
         End Property


         ''' <summary>
         ''' Constructor
         ''' </summary>
         Public Sub New()
         End Sub

         ''' <summary>
         ''' Selects a record by primary key(s)
         ''' </summary>
         Public Shared Function SelectByPrimaryKey(ByVal employeeID As Integer) As Employees
             Return EmployeesDataLayer.SelectByPrimaryKey(employeeID)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Employees table
         ''' </summary>
         Public Shared Function GetRecordCount() As Integer
             Return EmployeesDataLayer.GetRecordCount()
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Employees table by ReportsTo
         ''' </summary>
         Public Shared Function GetRecordCountByReportsTo(reportsTo As Integer) As Integer
             Return EmployeesDataLayer.GetRecordCountByReportsTo(reportsTo)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Employees table based on search parameters
         ''' </summary>
         Public Shared Function GetRecordCountDynamicWhere(employeeID As Integer?, lastName As String, firstName As String, title As String, titleOfCourtesy As String, birthDate As Date?, hireDate As Date?, address As String, city As String, region1 As String, postalCode As String, country As String, homePhone As String, extension As String, reportsTo As Integer?, photoPath As String) As Integer
             Return EmployeesDataLayer.GetRecordCountDynamicWhere(employeeID, lastName, firstName, title, titleOfCourtesy, birthDate, hireDate, address, city, region1, postalCode, country, homePhone, extension, reportsTo, photoPath)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Employees sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared Function SelectSkipAndTake(rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer, sortByExpression As String) As EmployeesCollection
             totalRowCount = GetRecordCount()
             sortByExpression = GetSortExpression(sortByExpression)

             Return EmployeesDataLayer.SelectSkipAndTake(sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Employees sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex.  Use for non model-binded gridview control.  Does not return total row count.
         ''' </summary>
         Public Shared Function SelectSkipAndTake(rows As Integer, startRowIndex As Integer, sortByExpression As String) As EmployeesCollection
             sortByExpression = GetSortExpression(sortByExpression)
             Return EmployeesDataLayer.SelectSkipAndTake(sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records by ReportsTo as a collection (List) of Employees sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared  Function SelectSkipAndTakeByReportsTo(rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer,  sortByExpression As String, reportsTo As Integer) As EmployeesCollection
             totalRowCount = EmployeesDataLayer.GetRecordCountByReportsTo(reportsTo)
             Dim ending As Integer = startRowIndex + rows
             sortByExpression = GetSortExpression(sortByExpression)
             Return EmployeesDataLayer.SelectSkipAndTakeByReportsTo(sortByExpression, startRowIndex, ending, reportsTo)
         End Function

         ''' <summary>
         ''' Selects records by ReportsTo as a collection (List) of Employees sorted by the sortByExpression starting from the startRowIndex
         ''' </summary>
         Public Shared  Function SelectSkipAndTakeByReportsTo(rows As Integer, startRowIndex As Integer,  sortByExpression As String, reportsTo As Integer) As EmployeesCollection
             Dim ending As Integer = startRowIndex + rows
             sortByExpression = GetSortExpression(sortByExpression)
             Return EmployeesDataLayer.SelectSkipAndTakeByReportsTo(sortByExpression, startRowIndex, ending, reportsTo)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Employees sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex, based on the search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(employeeID As Integer?, lastName As String, firstName As String, title As String, titleOfCourtesy As String, birthDate As Date?, hireDate As Date?, address As String, city As String, region1 As String, postalCode As String, country As String, homePhone As String, extension As String, reportsTo As Integer?, photoPath As String, rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer, sortByExpression As String) As EmployeesCollection
             totalRowCount = GetRecordCountDynamicWhere(employeeID, lastName, firstName, title, titleOfCourtesy, birthDate, hireDate, address, city, region1, postalCode, country, homePhone, extension, reportsTo, photoPath)
             sortByExpression = GetSortExpression(sortByExpression)
             Return EmployeesDataLayer.SelectSkipAndTakeDynamicWhere(employeeID, lastName, firstName, title, titleOfCourtesy, birthDate, hireDate, address, city, region1, postalCode, country, homePhone, extension, reportsTo, photoPath, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Employees sorted by the sortByExpression starting from the startRowIndex, based on the search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(employeeID As Integer?, lastName As String, firstName As String, title As String, titleOfCourtesy As String, birthDate As Date?, hireDate As Date?, address As String, city As String, region1 As String, postalCode As String, country As String, homePhone As String, extension As String, reportsTo As Integer?, photoPath As String, rows As Integer, startRowIndex As Integer, sortByExpression As String) As EmployeesCollection
             sortByExpression = GetSortExpression(sortByExpression)
             Return EmployeesDataLayer.SelectSkipAndTakeDynamicWhere(employeeID, lastName, firstName, title, titleOfCourtesy, birthDate, hireDate, address, city, region1, postalCode, country, homePhone, extension, reportsTo, photoPath, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects all records as a collection (List) of Employees
         ''' </summary>
         Public Shared Function SelectAll() As EmployeesCollection
             Return EmployeesDataLayer.SelectAll()
         End Function

         ''' <summary>
         ''' Selects all records as a collection (List) of Employees sorted by the sort expression
         ''' </summary>
         Public Shared Function SelectAll(sortExpression As String) As EmployeesCollection
             Dim objEmployeesCol As EmployeesCollection = EmployeesDataLayer.SelectAll()
             Return SortByExpression(objEmployeesCol, sortExpression)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Employees.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(employeeID As Integer?, lastName As String, firstName As String, title As String, titleOfCourtesy As String, birthDate As Date?, hireDate As Date?, address As String, city As String, region1 As String, postalCode As String, country As String, homePhone As String, extension As String, reportsTo As Integer?, photoPath As String) As EmployeesCollection
             Return EmployeesDataLayer.SelectAllDynamicWhere(employeeID, lastName, firstName, title, titleOfCourtesy, birthDate, hireDate, address, city, region1, postalCode, country, homePhone, extension, reportsTo, photoPath)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Employees sorted by the sort expression.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(employeeID As System.Nullable(Of Integer), lastName As String, firstName As String, title As String, titleOfCourtesy As String, birthDate As System.Nullable(Of Date), hireDate As System.Nullable(Of Date), address As String, city As String, region1 As String, postalCode As String, country As String, homePhone As String, extension As String, reportsTo As System.Nullable(Of Integer), photoPath As String, sortExpression As String) As EmployeesCollection
             Dim objEmployeesCol As EmployeesCollection = EmployeesDataLayer.SelectAllDynamicWhere(employeeID, lastName, firstName, title, titleOfCourtesy, birthDate, hireDate, address, city, region1, postalCode, country, homePhone, extension, reportsTo, photoPath)
             Return SortByExpression(objEmployeesCol, sortExpression)
         End Function

         ''' <summary>
         ''' Selects all Employees by Employees, related to column ReportsTo
         ''' </summary>
         Public Shared Function SelectEmployeesCollectionByReportsTo(employeeID As Integer) As EmployeesCollection
             Return EmployeesDataLayer.SelectEmployeesCollectionByReportsTo(employeeID)
         End Function

         ''' <summary>
         ''' Selects all Employees by Employees, related to column ReportsTo, sorted by the sort expression
         ''' </summary>
         Public Shared Function SelectEmployeesCollectionByReportsTo(employeeID As Integer, sortExpression As String) As EmployeesCollection
             Dim objEmployeesCol As EmployeesCollection = EmployeesDataLayer.SelectEmployeesCollectionByReportsTo(employeeID)
             Return SortByExpression(objEmployeesCol, sortExpression)
         End Function

         ''' <summary>
         ''' Selects EmployeeID and LastName columns for use with a DropDownList web control, ComboBox, CheckedBoxList, ListView, ListBox, etc
         ''' </summary>
         Public Shared Function SelectEmployeesDropDownListData() As EmployeesCollection
             Return EmployeesDataLayer.SelectEmployeesDropDownListData()
         End Function

         ''' <summary>
         ''' Sorts the EmployeesCollection by sort expression
         ''' </summary>
         Public Shared Function SortByExpression(objEmployeesCol As EmployeesCollection, sortExpression As String) As EmployeesCollection
             Dim isSortDescending As Boolean = sortExpression.ToLower().Contains(" desc")

             If isSortDescending Then
                 sortExpression = sortExpression.Replace(" DESC", "")
                 sortExpression = sortExpression.Replace(" desc", "")
             Else
                 sortExpression = sortExpression.Replace(" ASC", "")
                 sortExpression = sortExpression.Replace(" asc", "")
             End If

             Select Case sortExpression
                 Case "EmployeeID"
                     objEmployeesCol.Sort(Employees.ByEmployeeID)
                     Exit Select
                 Case "LastName"
                     objEmployeesCol.Sort(Employees.ByLastName)
                     Exit Select
                 Case "FirstName"
                     objEmployeesCol.Sort(Employees.ByFirstName)
                     Exit Select
                 Case "Title"
                     objEmployeesCol.Sort(Employees.ByTitle)
                     Exit Select
                 Case "TitleOfCourtesy"
                     objEmployeesCol.Sort(Employees.ByTitleOfCourtesy)
                     Exit Select
                 Case "BirthDate"
                     objEmployeesCol.Sort(Employees.ByBirthDate)
                     Exit Select
                 Case "HireDate"
                     objEmployeesCol.Sort(Employees.ByHireDate)
                     Exit Select
                 Case "Address"
                     objEmployeesCol.Sort(Employees.ByAddress)
                     Exit Select
                 Case "City"
                     objEmployeesCol.Sort(Employees.ByCity)
                     Exit Select
                 Case "Region1"
                     objEmployeesCol.Sort(Employees.ByRegion1)
                     Exit Select
                 Case "PostalCode"
                     objEmployeesCol.Sort(Employees.ByPostalCode)
                     Exit Select
                 Case "Country"
                     objEmployeesCol.Sort(Employees.ByCountry)
                     Exit Select
                 Case "HomePhone"
                     objEmployeesCol.Sort(Employees.ByHomePhone)
                     Exit Select
                 Case "Extension"
                     objEmployeesCol.Sort(Employees.ByExtension)
                     Exit Select
                 Case "Notes"
                     objEmployeesCol.Sort(Employees.ByNotes)
                     Exit Select
                 Case "ReportsTo"
                     objEmployeesCol.Sort(Employees.ByReportsTo)
                     Exit Select
                 Case "PhotoPath"
                     objEmployeesCol.Sort(Employees.ByPhotoPath)
                     Exit Select
                 Case Else
                     Exit Select
             End Select

             If isSortDescending Then
                 objEmployeesCol.Reverse()
             End If

             Return objEmployeesCol
         End Function

         ''' <summary>
         ''' Inserts a record
         ''' </summary>
         Public Function Insert() As Integer
             Dim objEmployees As Employees = DirectCast(Me, Employees)
             Return EmployeesDataLayer.Insert(objEmployees)
         End Function

         ''' <summary>
         ''' Updates a record
         ''' </summary>
         Public Sub Update()
             Dim objEmployees As Employees = DirectCast(Me, Employees)
             EmployeesDataLayer.Update(objEmployees)
         End Sub

         ''' <summary>
         ''' Deletes a record based on primary key(s)
         ''' </summary>
         Public Shared Sub Delete(ByVal employeeID As Integer)
             EmployeesDataLayer.Delete(employeeID)
         End Sub

         Private Shared Function GetSortExpression(sortByExpression As String) As String
             If [String].IsNullOrEmpty(sortByExpression) Or sortByExpression = " asc" Then
                 sortByExpression = "EmployeeID"
             Else If sortByExpression.Contains(" asc") Then
                 sortByExpression = sortByExpression.Replace(" asc", "")
             End If

             Return sortByExpression
         End Function

         ''' <summary>
         ''' Compares EmployeeID used for sorting
         ''' </summary>
         Public Shared ByEmployeeID As Comparison(Of Employees) =
         Function(ByVal x As Employees, ByVal y As Employees)
             Return x.EmployeeID.CompareTo(y.EmployeeID)
         End Function

         ''' <summary>
         ''' Compares LastName used for sorting
         ''' </summary>
         Public Shared ByLastName As Comparison(Of Employees) =
         Function(ByVal x As Employees, ByVal y As Employees)
             Dim value1 As String = If(x.LastName, String.Empty)
             Dim value2 As String = If(y.LastName, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares FirstName used for sorting
         ''' </summary>
         Public Shared ByFirstName As Comparison(Of Employees) =
         Function(ByVal x As Employees, ByVal y As Employees)
             Dim value1 As String = If(x.FirstName, String.Empty)
             Dim value2 As String = If(y.FirstName, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares Title used for sorting
         ''' </summary>
         Public Shared ByTitle As Comparison(Of Employees) =
         Function(ByVal x As Employees, ByVal y As Employees)
             Dim value1 As String = If(x.Title, String.Empty)
             Dim value2 As String = If(y.Title, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares TitleOfCourtesy used for sorting
         ''' </summary>
         Public Shared ByTitleOfCourtesy As Comparison(Of Employees) =
         Function(ByVal x As Employees, ByVal y As Employees)
             Dim value1 As String = If(x.TitleOfCourtesy, String.Empty)
             Dim value2 As String = If(y.TitleOfCourtesy, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares BirthDate used for sorting
         ''' </summary>
         Public Shared ByBirthDate As Comparison(Of Employees) =
         Function(ByVal x As Employees, ByVal y As Employees)
             Return Nullable.Compare(x.BirthDate, y.BirthDate)
         End Function

         ''' <summary>
         ''' Compares HireDate used for sorting
         ''' </summary>
         Public Shared ByHireDate As Comparison(Of Employees) =
         Function(ByVal x As Employees, ByVal y As Employees)
             Return Nullable.Compare(x.HireDate, y.HireDate)
         End Function

         ''' <summary>
         ''' Compares Address used for sorting
         ''' </summary>
         Public Shared ByAddress As Comparison(Of Employees) =
         Function(ByVal x As Employees, ByVal y As Employees)
             Dim value1 As String = If(x.Address, String.Empty)
             Dim value2 As String = If(y.Address, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares City used for sorting
         ''' </summary>
         Public Shared ByCity As Comparison(Of Employees) =
         Function(ByVal x As Employees, ByVal y As Employees)
             Dim value1 As String = If(x.City, String.Empty)
             Dim value2 As String = If(y.City, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares Region1 used for sorting
         ''' </summary>
         Public Shared ByRegion1 As Comparison(Of Employees) =
         Function(ByVal x As Employees, ByVal y As Employees)
             Dim value1 As String = If(x.Region1, String.Empty)
             Dim value2 As String = If(y.Region1, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares PostalCode used for sorting
         ''' </summary>
         Public Shared ByPostalCode As Comparison(Of Employees) =
         Function(ByVal x As Employees, ByVal y As Employees)
             Dim value1 As String = If(x.PostalCode, String.Empty)
             Dim value2 As String = If(y.PostalCode, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares Country used for sorting
         ''' </summary>
         Public Shared ByCountry As Comparison(Of Employees) =
         Function(ByVal x As Employees, ByVal y As Employees)
             Dim value1 As String = If(x.Country, String.Empty)
             Dim value2 As String = If(y.Country, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares HomePhone used for sorting
         ''' </summary>
         Public Shared ByHomePhone As Comparison(Of Employees) =
         Function(ByVal x As Employees, ByVal y As Employees)
             Dim value1 As String = If(x.HomePhone, String.Empty)
             Dim value2 As String = If(y.HomePhone, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares Extension used for sorting
         ''' </summary>
         Public Shared ByExtension As Comparison(Of Employees) =
         Function(ByVal x As Employees, ByVal y As Employees)
             Dim value1 As String = If(x.Extension, String.Empty)
             Dim value2 As String = If(y.Extension, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares Notes used for sorting
         ''' </summary>
         Public Shared ByNotes As Comparison(Of Employees) =
         Function(ByVal x As Employees, ByVal y As Employees)
             Dim value1 As String = If(x.Notes, String.Empty)
             Dim value2 As String = If(y.Notes, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares ReportsTo used for sorting
         ''' </summary>
         Public Shared ByReportsTo As Comparison(Of Employees) =
         Function(ByVal x As Employees, ByVal y As Employees)
             Return Nullable.Compare(x.ReportsTo, y.ReportsTo)
         End Function

         ''' <summary>
         ''' Compares PhotoPath used for sorting
         ''' </summary>
         Public Shared ByPhotoPath As Comparison(Of Employees) =
         Function(ByVal x As Employees, ByVal y As Employees)
             Dim value1 As String = If(x.PhotoPath, String.Empty)
             Dim value2 As String = If(y.PhotoPath, String.Empty)
             Return value1.CompareTo(value2)
         End Function

     End Class
End Namespace
