Imports System
Imports System.Data
Imports NorthwindWinFormDynamic_VB.DataLayer
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Models
Imports System.Runtime.InteropServices

Namespace BusinessObject.Base

     ''' <summary>
     ''' Base class for OrderDetails.  Do not make changes to this class,
     ''' instead, put additional code in the OrderDetails class
     ''' </summary>
     Public Class OrderDetailsBase
         ' private members
         Private _orderID As Integer
         Private _productID As Integer
         Private _unitPrice As Decimal
         Private _unitPriceTotal As Decimal
         Private _quantity As Short
         Private _discount As Single

         ''' <summary>
         ''' Gets or Sets OrderID
         ''' </summary>
         Public Property OrderID() As Integer
             Get
                 Return _orderID
             End Get
             Set(ByVal value As Integer)
                 _orderID = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets ProductID
         ''' </summary>
         Public Property ProductID() As Integer
             Get
                 Return _productID
             End Get
             Set(ByVal value As Integer)
                 _productID = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets UnitPrice
         ''' </summary>
         Public Property UnitPrice() As Decimal
             Get
                 Return _unitPrice
             End Get
             Set(ByVal value As Decimal)
                 _unitPrice = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets UnitPriceTotal
         ''' </summary>
         Public Property UnitPriceTotal() As Decimal
             Get
                 Return _unitPriceTotal
             End Get
             Set(ByVal value As Decimal)
                 _unitPriceTotal = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets Quantity
         ''' </summary>
         Public Property Quantity() As Short
             Get
                 Return _quantity
             End Get
             Set(ByVal value As Short)
                 _quantity = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets Discount
         ''' </summary>
         Public Property Discount() As Single
             Get
                 Return _discount
             End Get
             Set(ByVal value As Single)
                 _discount = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or sets the Related Orders.  Related to column OrderID
         ''' </summary>
         Public ReadOnly Property Orders() As Lazy(Of Orders)
             Get
                 Return New Lazy(Of Orders)(Function() BusinessObject.Orders.SelectByPrimaryKey(OrderID))
             End Get
         End Property

         ''' <summary>
         ''' Gets or sets the Related Products.  Related to column ProductID
         ''' </summary>
         Public ReadOnly Property Products() As Lazy(Of Products)
             Get
                 Return New Lazy(Of Products)(Function() BusinessObject.Products.SelectByPrimaryKey(ProductID))
             End Get
         End Property


         ''' <summary>
         ''' Constructor
         ''' </summary>
         Public Sub New()
         End Sub

         ''' <summary>
         ''' Selects a record by primary key(s)
         ''' </summary>
         Public Shared Function SelectByPrimaryKey(ByVal orderID As Integer, ByVal productID As Integer) As OrderDetails
             Return OrderDetailsDataLayer.SelectByPrimaryKey(orderID, productID)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the OrderDetails table
         ''' </summary>
         Public Shared Function GetRecordCount() As Integer
             Return OrderDetailsDataLayer.GetRecordCount()
         End Function

         ''' <summary>
         ''' Gets the total number of records in the OrderDetails table by OrderID
         ''' </summary>
         Public Shared Function GetRecordCountByOrderID(orderID As Integer) As Integer
             Return OrderDetailsDataLayer.GetRecordCountByOrderID(orderID)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the OrderDetails table by ProductID
         ''' </summary>
         Public Shared Function GetRecordCountByProductID(productID As Integer) As Integer
             Return OrderDetailsDataLayer.GetRecordCountByProductID(productID)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the OrderDetails table based on search parameters
         ''' </summary>
         Public Shared Function GetRecordCountDynamicWhere(orderID As Integer?, productID As Integer?, unitPrice As Decimal?, quantity As Short?, discount As Single?) As Integer
             Return OrderDetailsDataLayer.GetRecordCountDynamicWhere(orderID, productID, unitPrice, quantity, discount)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of OrderDetails sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared Function SelectSkipAndTake(rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer, sortByExpression As String) As OrderDetailsCollection
             totalRowCount = GetRecordCount()
             sortByExpression = GetSortExpression(sortByExpression)

             Return OrderDetailsDataLayer.SelectSkipAndTake(sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of OrderDetails sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex.  Use for non model-binded gridview control.  Does not return total row count.
         ''' </summary>
         Public Shared Function SelectSkipAndTake(rows As Integer, startRowIndex As Integer, sortByExpression As String) As OrderDetailsCollection
             sortByExpression = GetSortExpression(sortByExpression)
             Return OrderDetailsDataLayer.SelectSkipAndTake(sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records by OrderID as a collection (List) of OrderDetails sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared  Function SelectSkipAndTakeByOrderID(rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer,  sortByExpression As String, orderID As Integer) As OrderDetailsCollection
             totalRowCount = OrderDetailsDataLayer.GetRecordCountByOrderID(orderID)
             Dim ending As Integer = startRowIndex + rows
             sortByExpression = GetSortExpression(sortByExpression)
             Return OrderDetailsDataLayer.SelectSkipAndTakeByOrderID(sortByExpression, startRowIndex, ending, orderID)
         End Function

         ''' <summary>
         ''' Selects records by OrderID as a collection (List) of OrderDetails sorted by the sortByExpression starting from the startRowIndex
         ''' </summary>
         Public Shared  Function SelectSkipAndTakeByOrderID(rows As Integer, startRowIndex As Integer,  sortByExpression As String, orderID As Integer) As OrderDetailsCollection
             Dim ending As Integer = startRowIndex + rows
             sortByExpression = GetSortExpression(sortByExpression)
             Return OrderDetailsDataLayer.SelectSkipAndTakeByOrderID(sortByExpression, startRowIndex, ending, orderID)
         End Function

         ''' <summary>
         ''' Selects records by ProductID as a collection (List) of OrderDetails sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared  Function SelectSkipAndTakeByProductID(rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer,  sortByExpression As String, productID As Integer) As OrderDetailsCollection
             totalRowCount = OrderDetailsDataLayer.GetRecordCountByProductID(productID)
             Dim ending As Integer = startRowIndex + rows
             sortByExpression = GetSortExpression(sortByExpression)
             Return OrderDetailsDataLayer.SelectSkipAndTakeByProductID(sortByExpression, startRowIndex, ending, productID)
         End Function

         ''' <summary>
         ''' Selects records by ProductID as a collection (List) of OrderDetails sorted by the sortByExpression starting from the startRowIndex
         ''' </summary>
         Public Shared  Function SelectSkipAndTakeByProductID(rows As Integer, startRowIndex As Integer,  sortByExpression As String, productID As Integer) As OrderDetailsCollection
             Dim ending As Integer = startRowIndex + rows
             sortByExpression = GetSortExpression(sortByExpression)
             Return OrderDetailsDataLayer.SelectSkipAndTakeByProductID(sortByExpression, startRowIndex, ending, productID)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of OrderDetails sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex, based on the search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(orderID As Integer?, productID As Integer?, unitPrice As Decimal?, quantity As Short?, discount As Single?, rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer, sortByExpression As String) As OrderDetailsCollection
             totalRowCount = GetRecordCountDynamicWhere(orderID, productID, unitPrice, quantity, discount)
             sortByExpression = GetSortExpression(sortByExpression)
             Return OrderDetailsDataLayer.SelectSkipAndTakeDynamicWhere(orderID, productID, unitPrice, quantity, discount, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of OrderDetails sorted by the sortByExpression starting from the startRowIndex, based on the search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(orderID As Integer?, productID As Integer?, unitPrice As Decimal?, quantity As Short?, discount As Single?, rows As Integer, startRowIndex As Integer, sortByExpression As String) As OrderDetailsCollection
             sortByExpression = GetSortExpression(sortByExpression)
             Return OrderDetailsDataLayer.SelectSkipAndTakeDynamicWhere(orderID, productID, unitPrice, quantity, discount, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Gets the grand total or sum of fields with a money or decimal data type.  E.g. UnitPriceTotal
         ''' </summary>
         Public Shared Function SelectTotals() As OrderDetails
             Return OrderDetailsDataLayer.SelectTotals()
         End Function

         ''' <summary>
         ''' Selects all records as a collection (List) of OrderDetails
         ''' </summary>
         Public Shared Function SelectAll() As OrderDetailsCollection
             Return OrderDetailsDataLayer.SelectAll()
         End Function

         ''' <summary>
         ''' Selects all records as a collection (List) of OrderDetails sorted by the sort expression
         ''' </summary>
         Public Shared Function SelectAll(sortExpression As String) As OrderDetailsCollection
             Dim objOrderDetailsCol As OrderDetailsCollection = OrderDetailsDataLayer.SelectAll()
             Return SortByExpression(objOrderDetailsCol, sortExpression)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of OrderDetails.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(orderID As Integer?, productID As Integer?, unitPrice As Decimal?, quantity As Short?, discount As Single?) As OrderDetailsCollection
             Return OrderDetailsDataLayer.SelectAllDynamicWhere(orderID, productID, unitPrice, quantity, discount)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of OrderDetails sorted by the sort expression.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(orderID As System.Nullable(Of Integer), productID As System.Nullable(Of Integer), unitPrice As System.Nullable(Of Decimal), quantity As System.Nullable(Of Short), discount As System.Nullable(Of Single), sortExpression As String) As OrderDetailsCollection
             Dim objOrderDetailsCol As OrderDetailsCollection = OrderDetailsDataLayer.SelectAllDynamicWhere(orderID, productID, unitPrice, quantity, discount)
             Return SortByExpression(objOrderDetailsCol, sortExpression)
         End Function

         ''' <summary>
         ''' Selects all OrderDetails by Orders, related to column OrderID
         ''' </summary>
         Public Shared Function SelectOrderDetailsCollectionByOrderID(orderID As Integer) As OrderDetailsCollection
             Return OrderDetailsDataLayer.SelectOrderDetailsCollectionByOrderID(orderID)
         End Function

         ''' <summary>
         ''' Selects all OrderDetails by Orders, related to column OrderID, sorted by the sort expression
         ''' </summary>
         Public Shared Function SelectOrderDetailsCollectionByOrderID(orderID As Integer, sortExpression As String) As OrderDetailsCollection
             Dim objOrderDetailsCol As OrderDetailsCollection = OrderDetailsDataLayer.SelectOrderDetailsCollectionByOrderID(orderID)
             Return SortByExpression(objOrderDetailsCol, sortExpression)
         End Function

         ''' <summary>
         ''' Selects all OrderDetails by Products, related to column ProductID
         ''' </summary>
         Public Shared Function SelectOrderDetailsCollectionByProductID(productID As Integer) As OrderDetailsCollection
             Return OrderDetailsDataLayer.SelectOrderDetailsCollectionByProductID(productID)
         End Function

         ''' <summary>
         ''' Selects all OrderDetails by Products, related to column ProductID, sorted by the sort expression
         ''' </summary>
         Public Shared Function SelectOrderDetailsCollectionByProductID(productID As Integer, sortExpression As String) As OrderDetailsCollection
             Dim objOrderDetailsCol As OrderDetailsCollection = OrderDetailsDataLayer.SelectOrderDetailsCollectionByProductID(productID)
             Return SortByExpression(objOrderDetailsCol, sortExpression)
         End Function

         ''' <summary>
         ''' Sorts the OrderDetailsCollection by sort expression
         ''' </summary>
         Public Shared Function SortByExpression(objOrderDetailsCol As OrderDetailsCollection, sortExpression As String) As OrderDetailsCollection
             Dim isSortDescending As Boolean = sortExpression.ToLower().Contains(" desc")

             If isSortDescending Then
                 sortExpression = sortExpression.Replace(" DESC", "")
                 sortExpression = sortExpression.Replace(" desc", "")
             Else
                 sortExpression = sortExpression.Replace(" ASC", "")
                 sortExpression = sortExpression.Replace(" asc", "")
             End If

             Select Case sortExpression
                 Case "OrderID"
                     objOrderDetailsCol.Sort(OrderDetails.ByOrderID)
                     Exit Select
                 Case "ProductID"
                     objOrderDetailsCol.Sort(OrderDetails.ByProductID)
                     Exit Select
                 Case "UnitPrice"
                     objOrderDetailsCol.Sort(OrderDetails.ByUnitPrice)
                     Exit Select
                 Case "Quantity"
                     objOrderDetailsCol.Sort(OrderDetails.ByQuantity)
                     Exit Select
                 Case "Discount"
                     objOrderDetailsCol.Sort(OrderDetails.ByDiscount)
                     Exit Select
                 Case Else
                     Exit Select
             End Select

             If isSortDescending Then
                 objOrderDetailsCol.Reverse()
             End If

             Return objOrderDetailsCol
         End Function

         ''' <summary>
         ''' Inserts a record
         ''' </summary>
         Public Sub Insert()
             Dim objOrderDetails As OrderDetails = DirectCast(Me, OrderDetails)
              OrderDetailsDataLayer.Insert(objOrderDetails)
         End Sub

         ''' <summary>
         ''' Updates a record
         ''' </summary>
         Public Sub Update()
             Dim objOrderDetails As OrderDetails = DirectCast(Me, OrderDetails)
             OrderDetailsDataLayer.Update(objOrderDetails)
         End Sub

         ''' <summary>
         ''' Deletes a record based on primary key(s)
         ''' </summary>
         Public Shared Sub Delete(ByVal orderID As Integer, ByVal productID As Integer)
             OrderDetailsDataLayer.Delete(orderID, productID)
         End Sub

         Private Shared Function GetSortExpression(sortByExpression As String) As String
             If [String].IsNullOrEmpty(sortByExpression) Or sortByExpression = " asc" Then
                 sortByExpression = "OrderID"
             Else If sortByExpression.Contains(" asc") Then
                 sortByExpression = sortByExpression.Replace(" asc", "")
             End If

             Return sortByExpression
         End Function

         ''' <summary>
         ''' Compares OrderID used for sorting
         ''' </summary>
         Public Shared ByOrderID As Comparison(Of OrderDetails) =
         Function(ByVal x As OrderDetails, ByVal y As OrderDetails)
             Return x.OrderID.CompareTo(y.OrderID)
         End Function

         ''' <summary>
         ''' Compares ProductID used for sorting
         ''' </summary>
         Public Shared ByProductID As Comparison(Of OrderDetails) =
         Function(ByVal x As OrderDetails, ByVal y As OrderDetails)
             Return x.ProductID.CompareTo(y.ProductID)
         End Function

         ''' <summary>
         ''' Compares UnitPrice used for sorting
         ''' </summary>
         Public Shared ByUnitPrice As Comparison(Of OrderDetails) =
         Function(ByVal x As OrderDetails, ByVal y As OrderDetails)
             Return x.UnitPrice.CompareTo(y.UnitPrice)
         End Function

         ''' <summary>
         ''' Compares Quantity used for sorting
         ''' </summary>
         Public Shared ByQuantity As Comparison(Of OrderDetails) =
         Function(ByVal x As OrderDetails, ByVal y As OrderDetails)
             Return x.Quantity.CompareTo(y.Quantity)
         End Function

         ''' <summary>
         ''' Compares Discount used for sorting
         ''' </summary>
         Public Shared ByDiscount As Comparison(Of OrderDetails) =
         Function(ByVal x As OrderDetails, ByVal y As OrderDetails)
             Return x.Discount.CompareTo(y.Discount)
         End Function

     End Class
End Namespace
