Imports System
Imports System.Data
Imports NorthwindWinFormDynamic_VB.DataLayer
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Models
Imports System.Runtime.InteropServices

Namespace BusinessObject.Base

     ''' <summary>
     ''' Base class for Orders.  Do not make changes to this class,
     ''' instead, put additional code in the Orders class
     ''' </summary>
     Public Class OrdersBase
         ' private members
         Private _orderID As Integer
         Private _customerID As String
         Private _employeeID As System.Nullable(Of Integer)
         Private _orderDate As System.Nullable(Of Date)
         Private _requiredDate As System.Nullable(Of Date)
         Private _shippedDate As System.Nullable(Of Date)
         Private _shipVia As System.Nullable(Of Integer)
         Private _freight As System.Nullable(Of Decimal)
         Private _freightTotal As Decimal
         Private _shipName As String
         Private _shipAddress As String
         Private _shipCity As String
         Private _shipRegion As String
         Private _shipPostalCode As String
         Private _shipCountry As String

         ''' <summary>
         ''' Gets or Sets OrderID
         ''' </summary>
         Public Property OrderID() As Integer
             Get
                 Return _orderID
             End Get
             Set(ByVal value As Integer)
                 _orderID = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets CustomerID
         ''' </summary>
         Public Property CustomerID() As String
             Get
                 Return _customerID
             End Get
             Set(ByVal value As String)
                 _customerID = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets EmployeeID
         ''' </summary>
         Public Property EmployeeID() As System.Nullable(Of Integer)
             Get
                 Return _employeeID
             End Get
             Set(ByVal value As System.Nullable(Of Integer))
                 _employeeID = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets OrderDate
         ''' </summary>
         Public Property OrderDate() As System.Nullable(Of Date)
             Get
                 Return _orderDate
             End Get
             Set(ByVal value As System.Nullable(Of Date))
                 _orderDate = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets RequiredDate
         ''' </summary>
         Public Property RequiredDate() As System.Nullable(Of Date)
             Get
                 Return _requiredDate
             End Get
             Set(ByVal value As System.Nullable(Of Date))
                 _requiredDate = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets ShippedDate
         ''' </summary>
         Public Property ShippedDate() As System.Nullable(Of Date)
             Get
                 Return _shippedDate
             End Get
             Set(ByVal value As System.Nullable(Of Date))
                 _shippedDate = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets ShipVia
         ''' </summary>
         Public Property ShipVia() As System.Nullable(Of Integer)
             Get
                 Return _shipVia
             End Get
             Set(ByVal value As System.Nullable(Of Integer))
                 _shipVia = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets Freight
         ''' </summary>
         Public Property Freight() As System.Nullable(Of Decimal)
             Get
                 Return _freight
             End Get
             Set(ByVal value As System.Nullable(Of Decimal))
                 _freight = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets FreightTotal
         ''' </summary>
         Public Property FreightTotal() As Decimal
             Get
                 Return _freightTotal
             End Get
             Set(ByVal value As Decimal)
                 _freightTotal = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets ShipName
         ''' </summary>
         Public Property ShipName() As String
             Get
                 Return _shipName
             End Get
             Set(ByVal value As String)
                 _shipName = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets ShipAddress
         ''' </summary>
         Public Property ShipAddress() As String
             Get
                 Return _shipAddress
             End Get
             Set(ByVal value As String)
                 _shipAddress = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets ShipCity
         ''' </summary>
         Public Property ShipCity() As String
             Get
                 Return _shipCity
             End Get
             Set(ByVal value As String)
                 _shipCity = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets ShipRegion
         ''' </summary>
         Public Property ShipRegion() As String
             Get
                 Return _shipRegion
             End Get
             Set(ByVal value As String)
                 _shipRegion = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets ShipPostalCode
         ''' </summary>
         Public Property ShipPostalCode() As String
             Get
                 Return _shipPostalCode
             End Get
             Set(ByVal value As String)
                 _shipPostalCode = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets ShipCountry
         ''' </summary>
         Public Property ShipCountry() As String
             Get
                 Return _shipCountry
             End Get
             Set(ByVal value As String)
                 _shipCountry = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or sets the Related Customers.  Related to column CustomerID
         ''' </summary>
         Public ReadOnly Property Customers() As Lazy(Of Customers)
             Get
                 If Not String.IsNullOrEmpty(CustomerID) Then
                     Return New Lazy(Of Customers)(Function()BusinessObject.Customers.SelectByPrimaryKey(CustomerID))
                 Else
                     Return Nothing
                 End If
             End Get
         End Property

         ''' <summary>
         ''' Gets or sets the Related Employees.  Related to column EmployeeID
         ''' </summary>
         Public ReadOnly Property Employees() As Lazy(Of Employees)
             Get
                 Dim value As Integer
                 Dim hasValue As Boolean = Int32.TryParse(EmployeeID.ToString(), value)

                 If hasValue Then
                     Return New Lazy(Of Employees)(Function() BusinessObject.Employees.SelectByPrimaryKey(value))
                 Else
                     Return Nothing
                 End If
             End Get
         End Property

         ''' <summary>
         ''' Gets or sets the Related Shippers.  Related to column ShipVia
         ''' </summary>
         Public ReadOnly Property Shippers() As Lazy(Of Shippers)
             Get
                 Dim value As Integer
                 Dim hasValue As Boolean = Int32.TryParse(ShipVia.ToString(), value)

                 If hasValue Then
                     Return New Lazy(Of Shippers)(Function() BusinessObject.Shippers.SelectByPrimaryKey(value))
                 Else
                     Return Nothing
                 End If
             End Get
         End Property


         ''' <summary>
         ''' Constructor
         ''' </summary>
         Public Sub New()
         End Sub

         ''' <summary>
         ''' Selects a record by primary key(s)
         ''' </summary>
         Public Shared Function SelectByPrimaryKey(ByVal orderID As Integer) As Orders
             Return OrdersDataLayer.SelectByPrimaryKey(orderID)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Orders table
         ''' </summary>
         Public Shared Function GetRecordCount() As Integer
             Return OrdersDataLayer.GetRecordCount()
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Orders table by CustomerID
         ''' </summary>
         Public Shared Function GetRecordCountByCustomerID(customerID As String) As Integer
             Return OrdersDataLayer.GetRecordCountByCustomerID(customerID)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Orders table by EmployeeID
         ''' </summary>
         Public Shared Function GetRecordCountByEmployeeID(employeeID As Integer) As Integer
             Return OrdersDataLayer.GetRecordCountByEmployeeID(employeeID)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Orders table by ShipVia
         ''' </summary>
         Public Shared Function GetRecordCountByShipVia(shipVia As Integer) As Integer
             Return OrdersDataLayer.GetRecordCountByShipVia(shipVia)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Orders table based on search parameters
         ''' </summary>
         Public Shared Function GetRecordCountDynamicWhere(orderID As Integer?, customerID As String, employeeID As Integer?, orderDate As Date?, requiredDate As Date?, shippedDate As Date?, shipVia As Integer?, freight As Decimal?, shipName As String, shipAddress As String, shipCity As String, shipRegion As String, shipPostalCode As String, shipCountry As String) As Integer
             Return OrdersDataLayer.GetRecordCountDynamicWhere(orderID, customerID, employeeID, orderDate, requiredDate, shippedDate, shipVia, freight, shipName, shipAddress, shipCity, shipRegion, shipPostalCode, shipCountry)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Orders sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared Function SelectSkipAndTake(rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer, sortByExpression As String) As OrdersCollection
             totalRowCount = GetRecordCount()
             sortByExpression = GetSortExpression(sortByExpression)

             Return OrdersDataLayer.SelectSkipAndTake(sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Orders sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex.  Use for non model-binded gridview control.  Does not return total row count.
         ''' </summary>
         Public Shared Function SelectSkipAndTake(rows As Integer, startRowIndex As Integer, sortByExpression As String) As OrdersCollection
             sortByExpression = GetSortExpression(sortByExpression)
             Return OrdersDataLayer.SelectSkipAndTake(sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records by CustomerID as a collection (List) of Orders sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared  Function SelectSkipAndTakeByCustomerID(rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer,  sortByExpression As String, customerID As String) As OrdersCollection
             totalRowCount = OrdersDataLayer.GetRecordCountByCustomerID(customerID)
             Dim ending As Integer = startRowIndex + rows
             sortByExpression = GetSortExpression(sortByExpression)
             Return OrdersDataLayer.SelectSkipAndTakeByCustomerID(sortByExpression, startRowIndex, ending, customerID)
         End Function

         ''' <summary>
         ''' Selects records by CustomerID as a collection (List) of Orders sorted by the sortByExpression starting from the startRowIndex
         ''' </summary>
         Public Shared  Function SelectSkipAndTakeByCustomerID(rows As Integer, startRowIndex As Integer,  sortByExpression As String, customerID As String) As OrdersCollection
             Dim ending As Integer = startRowIndex + rows
             sortByExpression = GetSortExpression(sortByExpression)
             Return OrdersDataLayer.SelectSkipAndTakeByCustomerID(sortByExpression, startRowIndex, ending, customerID)
         End Function

         ''' <summary>
         ''' Selects records by EmployeeID as a collection (List) of Orders sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared  Function SelectSkipAndTakeByEmployeeID(rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer,  sortByExpression As String, employeeID As Integer) As OrdersCollection
             totalRowCount = OrdersDataLayer.GetRecordCountByEmployeeID(employeeID)
             Dim ending As Integer = startRowIndex + rows
             sortByExpression = GetSortExpression(sortByExpression)
             Return OrdersDataLayer.SelectSkipAndTakeByEmployeeID(sortByExpression, startRowIndex, ending, employeeID)
         End Function

         ''' <summary>
         ''' Selects records by EmployeeID as a collection (List) of Orders sorted by the sortByExpression starting from the startRowIndex
         ''' </summary>
         Public Shared  Function SelectSkipAndTakeByEmployeeID(rows As Integer, startRowIndex As Integer,  sortByExpression As String, employeeID As Integer) As OrdersCollection
             Dim ending As Integer = startRowIndex + rows
             sortByExpression = GetSortExpression(sortByExpression)
             Return OrdersDataLayer.SelectSkipAndTakeByEmployeeID(sortByExpression, startRowIndex, ending, employeeID)
         End Function

         ''' <summary>
         ''' Selects records by ShipVia as a collection (List) of Orders sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared  Function SelectSkipAndTakeByShipVia(rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer,  sortByExpression As String, shipVia As Integer) As OrdersCollection
             totalRowCount = OrdersDataLayer.GetRecordCountByShipVia(shipVia)
             Dim ending As Integer = startRowIndex + rows
             sortByExpression = GetSortExpression(sortByExpression)
             Return OrdersDataLayer.SelectSkipAndTakeByShipVia(sortByExpression, startRowIndex, ending, shipVia)
         End Function

         ''' <summary>
         ''' Selects records by ShipVia as a collection (List) of Orders sorted by the sortByExpression starting from the startRowIndex
         ''' </summary>
         Public Shared  Function SelectSkipAndTakeByShipVia(rows As Integer, startRowIndex As Integer,  sortByExpression As String, shipVia As Integer) As OrdersCollection
             Dim ending As Integer = startRowIndex + rows
             sortByExpression = GetSortExpression(sortByExpression)
             Return OrdersDataLayer.SelectSkipAndTakeByShipVia(sortByExpression, startRowIndex, ending, shipVia)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Orders sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex, based on the search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(orderID As Integer?, customerID As String, employeeID As Integer?, orderDate As Date?, requiredDate As Date?, shippedDate As Date?, shipVia As Integer?, freight As Decimal?, shipName As String, shipAddress As String, shipCity As String, shipRegion As String, shipPostalCode As String, shipCountry As String, rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer, sortByExpression As String) As OrdersCollection
             totalRowCount = GetRecordCountDynamicWhere(orderID, customerID, employeeID, orderDate, requiredDate, shippedDate, shipVia, freight, shipName, shipAddress, shipCity, shipRegion, shipPostalCode, shipCountry)
             sortByExpression = GetSortExpression(sortByExpression)
             Return OrdersDataLayer.SelectSkipAndTakeDynamicWhere(orderID, customerID, employeeID, orderDate, requiredDate, shippedDate, shipVia, freight, shipName, shipAddress, shipCity, shipRegion, shipPostalCode, shipCountry, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Orders sorted by the sortByExpression starting from the startRowIndex, based on the search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(orderID As Integer?, customerID As String, employeeID As Integer?, orderDate As Date?, requiredDate As Date?, shippedDate As Date?, shipVia As Integer?, freight As Decimal?, shipName As String, shipAddress As String, shipCity As String, shipRegion As String, shipPostalCode As String, shipCountry As String, rows As Integer, startRowIndex As Integer, sortByExpression As String) As OrdersCollection
             sortByExpression = GetSortExpression(sortByExpression)
             Return OrdersDataLayer.SelectSkipAndTakeDynamicWhere(orderID, customerID, employeeID, orderDate, requiredDate, shippedDate, shipVia, freight, shipName, shipAddress, shipCity, shipRegion, shipPostalCode, shipCountry, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Gets the grand total or sum of fields with a money or decimal data type.  E.g. FreightTotal
         ''' </summary>
         Public Shared Function SelectTotals() As Orders
             Return OrdersDataLayer.SelectTotals()
         End Function

         ''' <summary>
         ''' Selects all records as a collection (List) of Orders
         ''' </summary>
         Public Shared Function SelectAll() As OrdersCollection
             Return OrdersDataLayer.SelectAll()
         End Function

         ''' <summary>
         ''' Selects all records as a collection (List) of Orders sorted by the sort expression
         ''' </summary>
         Public Shared Function SelectAll(sortExpression As String) As OrdersCollection
             Dim objOrdersCol As OrdersCollection = OrdersDataLayer.SelectAll()
             Return SortByExpression(objOrdersCol, sortExpression)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Orders.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(orderID As Integer?, customerID As String, employeeID As Integer?, orderDate As Date?, requiredDate As Date?, shippedDate As Date?, shipVia As Integer?, freight As Decimal?, shipName As String, shipAddress As String, shipCity As String, shipRegion As String, shipPostalCode As String, shipCountry As String) As OrdersCollection
             Return OrdersDataLayer.SelectAllDynamicWhere(orderID, customerID, employeeID, orderDate, requiredDate, shippedDate, shipVia, freight, shipName, shipAddress, shipCity, shipRegion, shipPostalCode, shipCountry)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Orders sorted by the sort expression.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(orderID As System.Nullable(Of Integer), customerID As String, employeeID As System.Nullable(Of Integer), orderDate As System.Nullable(Of Date), requiredDate As System.Nullable(Of Date), shippedDate As System.Nullable(Of Date), shipVia As System.Nullable(Of Integer), freight As System.Nullable(Of Decimal), shipName As String, shipAddress As String, shipCity As String, shipRegion As String, shipPostalCode As String, shipCountry As String, sortExpression As String) As OrdersCollection
             Dim objOrdersCol As OrdersCollection = OrdersDataLayer.SelectAllDynamicWhere(orderID, customerID, employeeID, orderDate, requiredDate, shippedDate, shipVia, freight, shipName, shipAddress, shipCity, shipRegion, shipPostalCode, shipCountry)
             Return SortByExpression(objOrdersCol, sortExpression)
         End Function

         ''' <summary>
         ''' Selects all Orders by Customers, related to column CustomerID
         ''' </summary>
         Public Shared Function SelectOrdersCollectionByCustomerID(customerID As String) As OrdersCollection
             Return OrdersDataLayer.SelectOrdersCollectionByCustomerID(customerID)
         End Function

         ''' <summary>
         ''' Selects all Orders by Customers, related to column CustomerID, sorted by the sort expression
         ''' </summary>
         Public Shared Function SelectOrdersCollectionByCustomerID(customerID As String, sortExpression As String) As OrdersCollection
             Dim objOrdersCol As OrdersCollection = OrdersDataLayer.SelectOrdersCollectionByCustomerID(customerID)
             Return SortByExpression(objOrdersCol, sortExpression)
         End Function

         ''' <summary>
         ''' Selects all Orders by Employees, related to column EmployeeID
         ''' </summary>
         Public Shared Function SelectOrdersCollectionByEmployeeID(employeeID As Integer) As OrdersCollection
             Return OrdersDataLayer.SelectOrdersCollectionByEmployeeID(employeeID)
         End Function

         ''' <summary>
         ''' Selects all Orders by Employees, related to column EmployeeID, sorted by the sort expression
         ''' </summary>
         Public Shared Function SelectOrdersCollectionByEmployeeID(employeeID As Integer, sortExpression As String) As OrdersCollection
             Dim objOrdersCol As OrdersCollection = OrdersDataLayer.SelectOrdersCollectionByEmployeeID(employeeID)
             Return SortByExpression(objOrdersCol, sortExpression)
         End Function

         ''' <summary>
         ''' Selects all Orders by Shippers, related to column ShipVia
         ''' </summary>
         Public Shared Function SelectOrdersCollectionByShipVia(shipperID As Integer) As OrdersCollection
             Return OrdersDataLayer.SelectOrdersCollectionByShipVia(shipperID)
         End Function

         ''' <summary>
         ''' Selects all Orders by Shippers, related to column ShipVia, sorted by the sort expression
         ''' </summary>
         Public Shared Function SelectOrdersCollectionByShipVia(shipperID As Integer, sortExpression As String) As OrdersCollection
             Dim objOrdersCol As OrdersCollection = OrdersDataLayer.SelectOrdersCollectionByShipVia(shipperID)
             Return SortByExpression(objOrdersCol, sortExpression)
         End Function

         ''' <summary>
         ''' Selects OrderID and ShipName columns for use with a DropDownList web control, ComboBox, CheckedBoxList, ListView, ListBox, etc
         ''' </summary>
         Public Shared Function SelectOrdersDropDownListData() As OrdersCollection
             Return OrdersDataLayer.SelectOrdersDropDownListData()
         End Function

         ''' <summary>
         ''' Sorts the OrdersCollection by sort expression
         ''' </summary>
         Public Shared Function SortByExpression(objOrdersCol As OrdersCollection, sortExpression As String) As OrdersCollection
             Dim isSortDescending As Boolean = sortExpression.ToLower().Contains(" desc")

             If isSortDescending Then
                 sortExpression = sortExpression.Replace(" DESC", "")
                 sortExpression = sortExpression.Replace(" desc", "")
             Else
                 sortExpression = sortExpression.Replace(" ASC", "")
                 sortExpression = sortExpression.Replace(" asc", "")
             End If

             Select Case sortExpression
                 Case "OrderID"
                     objOrdersCol.Sort(Orders.ByOrderID)
                     Exit Select
                 Case "CustomerID"
                     objOrdersCol.Sort(Orders.ByCustomerID)
                     Exit Select
                 Case "EmployeeID"
                     objOrdersCol.Sort(Orders.ByEmployeeID)
                     Exit Select
                 Case "OrderDate"
                     objOrdersCol.Sort(Orders.ByOrderDate)
                     Exit Select
                 Case "RequiredDate"
                     objOrdersCol.Sort(Orders.ByRequiredDate)
                     Exit Select
                 Case "ShippedDate"
                     objOrdersCol.Sort(Orders.ByShippedDate)
                     Exit Select
                 Case "ShipVia"
                     objOrdersCol.Sort(Orders.ByShipVia)
                     Exit Select
                 Case "Freight"
                     objOrdersCol.Sort(Orders.ByFreight)
                     Exit Select
                 Case "ShipName"
                     objOrdersCol.Sort(Orders.ByShipName)
                     Exit Select
                 Case "ShipAddress"
                     objOrdersCol.Sort(Orders.ByShipAddress)
                     Exit Select
                 Case "ShipCity"
                     objOrdersCol.Sort(Orders.ByShipCity)
                     Exit Select
                 Case "ShipRegion"
                     objOrdersCol.Sort(Orders.ByShipRegion)
                     Exit Select
                 Case "ShipPostalCode"
                     objOrdersCol.Sort(Orders.ByShipPostalCode)
                     Exit Select
                 Case "ShipCountry"
                     objOrdersCol.Sort(Orders.ByShipCountry)
                     Exit Select
                 Case Else
                     Exit Select
             End Select

             If isSortDescending Then
                 objOrdersCol.Reverse()
             End If

             Return objOrdersCol
         End Function

         ''' <summary>
         ''' Inserts a record
         ''' </summary>
         Public Function Insert() As Integer
             Dim objOrders As Orders = DirectCast(Me, Orders)
             Return OrdersDataLayer.Insert(objOrders)
         End Function

         ''' <summary>
         ''' Updates a record
         ''' </summary>
         Public Sub Update()
             Dim objOrders As Orders = DirectCast(Me, Orders)
             OrdersDataLayer.Update(objOrders)
         End Sub

         ''' <summary>
         ''' Deletes a record based on primary key(s)
         ''' </summary>
         Public Shared Sub Delete(ByVal orderID As Integer)
             OrdersDataLayer.Delete(orderID)
         End Sub

         Private Shared Function GetSortExpression(sortByExpression As String) As String
             If [String].IsNullOrEmpty(sortByExpression) Or sortByExpression = " asc" Then
                 sortByExpression = "OrderID"
             Else If sortByExpression.Contains(" asc") Then
                 sortByExpression = sortByExpression.Replace(" asc", "")
             End If

             Return sortByExpression
         End Function

         ''' <summary>
         ''' Compares OrderID used for sorting
         ''' </summary>
         Public Shared ByOrderID As Comparison(Of Orders) =
         Function(ByVal x As Orders, ByVal y As Orders)
             Return x.OrderID.CompareTo(y.OrderID)
         End Function

         ''' <summary>
         ''' Compares CustomerID used for sorting
         ''' </summary>
         Public Shared ByCustomerID As Comparison(Of Orders) =
         Function(ByVal x As Orders, ByVal y As Orders)
             Dim value1 As String = If(x.CustomerID, String.Empty)
             Dim value2 As String = If(y.CustomerID, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares EmployeeID used for sorting
         ''' </summary>
         Public Shared ByEmployeeID As Comparison(Of Orders) =
         Function(ByVal x As Orders, ByVal y As Orders)
             Return Nullable.Compare(x.EmployeeID, y.EmployeeID)
         End Function

         ''' <summary>
         ''' Compares OrderDate used for sorting
         ''' </summary>
         Public Shared ByOrderDate As Comparison(Of Orders) =
         Function(ByVal x As Orders, ByVal y As Orders)
             Return Nullable.Compare(x.OrderDate, y.OrderDate)
         End Function

         ''' <summary>
         ''' Compares RequiredDate used for sorting
         ''' </summary>
         Public Shared ByRequiredDate As Comparison(Of Orders) =
         Function(ByVal x As Orders, ByVal y As Orders)
             Return Nullable.Compare(x.RequiredDate, y.RequiredDate)
         End Function

         ''' <summary>
         ''' Compares ShippedDate used for sorting
         ''' </summary>
         Public Shared ByShippedDate As Comparison(Of Orders) =
         Function(ByVal x As Orders, ByVal y As Orders)
             Return Nullable.Compare(x.ShippedDate, y.ShippedDate)
         End Function

         ''' <summary>
         ''' Compares ShipVia used for sorting
         ''' </summary>
         Public Shared ByShipVia As Comparison(Of Orders) =
         Function(ByVal x As Orders, ByVal y As Orders)
             Return Nullable.Compare(x.ShipVia, y.ShipVia)
         End Function

         ''' <summary>
         ''' Compares Freight used for sorting
         ''' </summary>
         Public Shared ByFreight As Comparison(Of Orders) =
         Function(ByVal x As Orders, ByVal y As Orders)
             Return Nullable.Compare(x.Freight, y.Freight)
         End Function

         ''' <summary>
         ''' Compares ShipName used for sorting
         ''' </summary>
         Public Shared ByShipName As Comparison(Of Orders) =
         Function(ByVal x As Orders, ByVal y As Orders)
             Dim value1 As String = If(x.ShipName, String.Empty)
             Dim value2 As String = If(y.ShipName, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares ShipAddress used for sorting
         ''' </summary>
         Public Shared ByShipAddress As Comparison(Of Orders) =
         Function(ByVal x As Orders, ByVal y As Orders)
             Dim value1 As String = If(x.ShipAddress, String.Empty)
             Dim value2 As String = If(y.ShipAddress, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares ShipCity used for sorting
         ''' </summary>
         Public Shared ByShipCity As Comparison(Of Orders) =
         Function(ByVal x As Orders, ByVal y As Orders)
             Dim value1 As String = If(x.ShipCity, String.Empty)
             Dim value2 As String = If(y.ShipCity, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares ShipRegion used for sorting
         ''' </summary>
         Public Shared ByShipRegion As Comparison(Of Orders) =
         Function(ByVal x As Orders, ByVal y As Orders)
             Dim value1 As String = If(x.ShipRegion, String.Empty)
             Dim value2 As String = If(y.ShipRegion, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares ShipPostalCode used for sorting
         ''' </summary>
         Public Shared ByShipPostalCode As Comparison(Of Orders) =
         Function(ByVal x As Orders, ByVal y As Orders)
             Dim value1 As String = If(x.ShipPostalCode, String.Empty)
             Dim value2 As String = If(y.ShipPostalCode, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares ShipCountry used for sorting
         ''' </summary>
         Public Shared ByShipCountry As Comparison(Of Orders) =
         Function(ByVal x As Orders, ByVal y As Orders)
             Dim value1 As String = If(x.ShipCountry, String.Empty)
             Dim value2 As String = If(y.ShipCountry, String.Empty)
             Return value1.CompareTo(value2)
         End Function

     End Class
End Namespace
