Imports System
Imports System.Data
Imports NorthwindWinFormDynamic_VB.DataLayer
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Models
Imports System.Runtime.InteropServices

Namespace BusinessObject.Base

     ''' <summary>
     ''' Base class for Region.  Do not make changes to this class,
     ''' instead, put additional code in the Region class
     ''' </summary>
     Public Class RegionBase
         ' private members
         Private _regionID As Integer
         Private _regionDescription As String

         ''' <summary>
         ''' Gets or Sets RegionID
         ''' </summary>
         Public Property RegionID() As Integer
             Get
                 Return _regionID
             End Get
             Set(ByVal value As Integer)
                 _regionID = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets RegionDescription
         ''' </summary>
         Public Property RegionDescription() As String
             Get
                 Return _regionDescription
             End Get
             Set(ByVal value As String)
                 _regionDescription = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or sets the related Territories(s) by RegionID
         ''' </summary>
         Public ReadOnly Property TerritoriesCollection() As Lazy(Of TerritoriesCollection)
             Get
                 Dim value As Integer
                 Dim hasValue As Boolean = Int32.TryParse(RegionID.ToString(), value)

                 If hasValue Then
                     Return New Lazy(Of TerritoriesCollection)(Function() Territories.SelectTerritoriesCollectionByRegionID(value))
                 Else
                     Return Nothing
                 End If
             End Get
         End Property


         ''' <summary>
         ''' Constructor
         ''' </summary>
         Public Sub New()
         End Sub

         ''' <summary>
         ''' Selects a record by primary key(s)
         ''' </summary>
         Public Shared Function SelectByPrimaryKey(ByVal regionID As Integer) As Region
             Return RegionDataLayer.SelectByPrimaryKey(regionID)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Region table
         ''' </summary>
         Public Shared Function GetRecordCount() As Integer
             Return RegionDataLayer.GetRecordCount()
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Region table based on search parameters
         ''' </summary>
         Public Shared Function GetRecordCountDynamicWhere(regionID As Integer?, regionDescription As String) As Integer
             Return RegionDataLayer.GetRecordCountDynamicWhere(regionID, regionDescription)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Region sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared Function SelectSkipAndTake(rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer, sortByExpression As String) As RegionCollection
             totalRowCount = GetRecordCount()
             sortByExpression = GetSortExpression(sortByExpression)

             Return RegionDataLayer.SelectSkipAndTake(sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Region sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex.  Use for non model-binded gridview control.  Does not return total row count.
         ''' </summary>
         Public Shared Function SelectSkipAndTake(rows As Integer, startRowIndex As Integer, sortByExpression As String) As RegionCollection
             sortByExpression = GetSortExpression(sortByExpression)
             Return RegionDataLayer.SelectSkipAndTake(sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Region sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex, based on the search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(regionID As Integer?, regionDescription As String, rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer, sortByExpression As String) As RegionCollection
             totalRowCount = GetRecordCountDynamicWhere(regionID, regionDescription)
             sortByExpression = GetSortExpression(sortByExpression)
             Return RegionDataLayer.SelectSkipAndTakeDynamicWhere(regionID, regionDescription, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Region sorted by the sortByExpression starting from the startRowIndex, based on the search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(regionID As Integer?, regionDescription As String, rows As Integer, startRowIndex As Integer, sortByExpression As String) As RegionCollection
             sortByExpression = GetSortExpression(sortByExpression)
             Return RegionDataLayer.SelectSkipAndTakeDynamicWhere(regionID, regionDescription, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects all records as a collection (List) of Region
         ''' </summary>
         Public Shared Function SelectAll() As RegionCollection
             Return RegionDataLayer.SelectAll()
         End Function

         ''' <summary>
         ''' Selects all records as a collection (List) of Region sorted by the sort expression
         ''' </summary>
         Public Shared Function SelectAll(sortExpression As String) As RegionCollection
             Dim objRegionCol As RegionCollection = RegionDataLayer.SelectAll()
             Return SortByExpression(objRegionCol, sortExpression)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Region.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(regionID As Integer?, regionDescription As String) As RegionCollection
             Return RegionDataLayer.SelectAllDynamicWhere(regionID, regionDescription)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Region sorted by the sort expression.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(regionID As System.Nullable(Of Integer), regionDescription As String, sortExpression As String) As RegionCollection
             Dim objRegionCol As RegionCollection = RegionDataLayer.SelectAllDynamicWhere(regionID, regionDescription)
             Return SortByExpression(objRegionCol, sortExpression)
         End Function

         ''' <summary>
         ''' Selects RegionID and RegionDescription columns for use with a DropDownList web control, ComboBox, CheckedBoxList, ListView, ListBox, etc
         ''' </summary>
         Public Shared Function SelectRegionDropDownListData() As RegionCollection
             Return RegionDataLayer.SelectRegionDropDownListData()
         End Function

         ''' <summary>
         ''' Sorts the RegionCollection by sort expression
         ''' </summary>
         Public Shared Function SortByExpression(objRegionCol As RegionCollection, sortExpression As String) As RegionCollection
             Dim isSortDescending As Boolean = sortExpression.ToLower().Contains(" desc")

             If isSortDescending Then
                 sortExpression = sortExpression.Replace(" DESC", "")
                 sortExpression = sortExpression.Replace(" desc", "")
             Else
                 sortExpression = sortExpression.Replace(" ASC", "")
                 sortExpression = sortExpression.Replace(" asc", "")
             End If

             Select Case sortExpression
                 Case "RegionID"
                     objRegionCol.Sort(BusinessObject.Region.ByRegionID)
                     Exit Select
                 Case "RegionDescription"
                     objRegionCol.Sort(BusinessObject.Region.ByRegionDescription)
                     Exit Select
                 Case Else
                     Exit Select
             End Select

             If isSortDescending Then
                 objRegionCol.Reverse()
             End If

             Return objRegionCol
         End Function

         ''' <summary>
         ''' Inserts a record
         ''' </summary>
         Public Function Insert() As Integer
             Dim objRegion As Region = DirectCast(Me, Region)
             Return RegionDataLayer.Insert(objRegion)
         End Function

         ''' <summary>
         ''' Updates a record
         ''' </summary>
         Public Sub Update()
             Dim objRegion As Region = DirectCast(Me, Region)
             RegionDataLayer.Update(objRegion)
         End Sub

         ''' <summary>
         ''' Deletes a record based on primary key(s)
         ''' </summary>
         Public Shared Sub Delete(ByVal regionID As Integer)
             RegionDataLayer.Delete(regionID)
         End Sub

         Private Shared Function GetSortExpression(sortByExpression As String) As String
             If [String].IsNullOrEmpty(sortByExpression) Or sortByExpression = " asc" Then
                 sortByExpression = "RegionID"
             Else If sortByExpression.Contains(" asc") Then
                 sortByExpression = sortByExpression.Replace(" asc", "")
             End If

             Return sortByExpression
         End Function

         ''' <summary>
         ''' Compares RegionID used for sorting
         ''' </summary>
         Public Shared ByRegionID As Comparison(Of Region) =
         Function(ByVal x As Region, ByVal y As Region)
             Return x.RegionID.CompareTo(y.RegionID)
         End Function

         ''' <summary>
         ''' Compares RegionDescription used for sorting
         ''' </summary>
         Public Shared ByRegionDescription As Comparison(Of Region) =
         Function(ByVal x As Region, ByVal y As Region)
             Dim value1 As String = If(x.RegionDescription, String.Empty)
             Dim value2 As String = If(y.RegionDescription, String.Empty)
             Return value1.CompareTo(value2)
         End Function

     End Class
End Namespace
