Imports System
Imports System.Data
Imports NorthwindWinFormDynamic_VB.DataLayer
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Models
Imports System.Runtime.InteropServices

Namespace BusinessObject.Base

     ''' <summary>
     ''' Base class for Shippers.  Do not make changes to this class,
     ''' instead, put additional code in the Shippers class
     ''' </summary>
     Public Class ShippersBase
         ' private members
         Private _shipperID As Integer
         Private _companyName As String
         Private _phone As String

         ''' <summary>
         ''' Gets or Sets ShipperID
         ''' </summary>
         Public Property ShipperID() As Integer
             Get
                 Return _shipperID
             End Get
             Set(ByVal value As Integer)
                 _shipperID = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets CompanyName
         ''' </summary>
         Public Property CompanyName() As String
             Get
                 Return _companyName
             End Get
             Set(ByVal value As String)
                 _companyName = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets Phone
         ''' </summary>
         Public Property Phone() As String
             Get
                 Return _phone
             End Get
             Set(ByVal value As String)
                 _phone = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or sets the related Orders(s) by ShipperID
         ''' </summary>
         Public ReadOnly Property OrdersCollection() As Lazy(Of OrdersCollection)
             Get
                 Dim value As Integer
                 Dim hasValue As Boolean = Int32.TryParse(ShipperID.ToString(), value)

                 If hasValue Then
                     Return New Lazy(Of OrdersCollection)(Function() Orders.SelectOrdersCollectionByShipVia(value))
                 Else
                     Return Nothing
                 End If
             End Get
         End Property


         ''' <summary>
         ''' Constructor
         ''' </summary>
         Public Sub New()
         End Sub

         ''' <summary>
         ''' Selects a record by primary key(s)
         ''' </summary>
         Public Shared Function SelectByPrimaryKey(ByVal shipperID As Integer) As Shippers
             Return ShippersDataLayer.SelectByPrimaryKey(shipperID)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Shippers table
         ''' </summary>
         Public Shared Function GetRecordCount() As Integer
             Return ShippersDataLayer.GetRecordCount()
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Shippers table based on search parameters
         ''' </summary>
         Public Shared Function GetRecordCountDynamicWhere(shipperID As Integer?, companyName As String, phone As String) As Integer
             Return ShippersDataLayer.GetRecordCountDynamicWhere(shipperID, companyName, phone)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Shippers sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared Function SelectSkipAndTake(rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer, sortByExpression As String) As ShippersCollection
             totalRowCount = GetRecordCount()
             sortByExpression = GetSortExpression(sortByExpression)

             Return ShippersDataLayer.SelectSkipAndTake(sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Shippers sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex.  Use for non model-binded gridview control.  Does not return total row count.
         ''' </summary>
         Public Shared Function SelectSkipAndTake(rows As Integer, startRowIndex As Integer, sortByExpression As String) As ShippersCollection
             sortByExpression = GetSortExpression(sortByExpression)
             Return ShippersDataLayer.SelectSkipAndTake(sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Shippers sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex, based on the search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(shipperID As Integer?, companyName As String, phone As String, rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer, sortByExpression As String) As ShippersCollection
             totalRowCount = GetRecordCountDynamicWhere(shipperID, companyName, phone)
             sortByExpression = GetSortExpression(sortByExpression)
             Return ShippersDataLayer.SelectSkipAndTakeDynamicWhere(shipperID, companyName, phone, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Shippers sorted by the sortByExpression starting from the startRowIndex, based on the search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(shipperID As Integer?, companyName As String, phone As String, rows As Integer, startRowIndex As Integer, sortByExpression As String) As ShippersCollection
             sortByExpression = GetSortExpression(sortByExpression)
             Return ShippersDataLayer.SelectSkipAndTakeDynamicWhere(shipperID, companyName, phone, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects all records as a collection (List) of Shippers
         ''' </summary>
         Public Shared Function SelectAll() As ShippersCollection
             Return ShippersDataLayer.SelectAll()
         End Function

         ''' <summary>
         ''' Selects all records as a collection (List) of Shippers sorted by the sort expression
         ''' </summary>
         Public Shared Function SelectAll(sortExpression As String) As ShippersCollection
             Dim objShippersCol As ShippersCollection = ShippersDataLayer.SelectAll()
             Return SortByExpression(objShippersCol, sortExpression)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Shippers.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(shipperID As Integer?, companyName As String, phone As String) As ShippersCollection
             Return ShippersDataLayer.SelectAllDynamicWhere(shipperID, companyName, phone)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Shippers sorted by the sort expression.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(shipperID As System.Nullable(Of Integer), companyName As String, phone As String, sortExpression As String) As ShippersCollection
             Dim objShippersCol As ShippersCollection = ShippersDataLayer.SelectAllDynamicWhere(shipperID, companyName, phone)
             Return SortByExpression(objShippersCol, sortExpression)
         End Function

         ''' <summary>
         ''' Selects ShipperID and CompanyName columns for use with a DropDownList web control, ComboBox, CheckedBoxList, ListView, ListBox, etc
         ''' </summary>
         Public Shared Function SelectShippersDropDownListData() As ShippersCollection
             Return ShippersDataLayer.SelectShippersDropDownListData()
         End Function

         ''' <summary>
         ''' Sorts the ShippersCollection by sort expression
         ''' </summary>
         Public Shared Function SortByExpression(objShippersCol As ShippersCollection, sortExpression As String) As ShippersCollection
             Dim isSortDescending As Boolean = sortExpression.ToLower().Contains(" desc")

             If isSortDescending Then
                 sortExpression = sortExpression.Replace(" DESC", "")
                 sortExpression = sortExpression.Replace(" desc", "")
             Else
                 sortExpression = sortExpression.Replace(" ASC", "")
                 sortExpression = sortExpression.Replace(" asc", "")
             End If

             Select Case sortExpression
                 Case "ShipperID"
                     objShippersCol.Sort(Shippers.ByShipperID)
                     Exit Select
                 Case "CompanyName"
                     objShippersCol.Sort(Shippers.ByCompanyName)
                     Exit Select
                 Case "Phone"
                     objShippersCol.Sort(Shippers.ByPhone)
                     Exit Select
                 Case Else
                     Exit Select
             End Select

             If isSortDescending Then
                 objShippersCol.Reverse()
             End If

             Return objShippersCol
         End Function

         ''' <summary>
         ''' Inserts a record
         ''' </summary>
         Public Function Insert() As Integer
             Dim objShippers As Shippers = DirectCast(Me, Shippers)
             Return ShippersDataLayer.Insert(objShippers)
         End Function

         ''' <summary>
         ''' Updates a record
         ''' </summary>
         Public Sub Update()
             Dim objShippers As Shippers = DirectCast(Me, Shippers)
             ShippersDataLayer.Update(objShippers)
         End Sub

         ''' <summary>
         ''' Deletes a record based on primary key(s)
         ''' </summary>
         Public Shared Sub Delete(ByVal shipperID As Integer)
             ShippersDataLayer.Delete(shipperID)
         End Sub

         Private Shared Function GetSortExpression(sortByExpression As String) As String
             If [String].IsNullOrEmpty(sortByExpression) Or sortByExpression = " asc" Then
                 sortByExpression = "ShipperID"
             Else If sortByExpression.Contains(" asc") Then
                 sortByExpression = sortByExpression.Replace(" asc", "")
             End If

             Return sortByExpression
         End Function

         ''' <summary>
         ''' Compares ShipperID used for sorting
         ''' </summary>
         Public Shared ByShipperID As Comparison(Of Shippers) =
         Function(ByVal x As Shippers, ByVal y As Shippers)
             Return x.ShipperID.CompareTo(y.ShipperID)
         End Function

         ''' <summary>
         ''' Compares CompanyName used for sorting
         ''' </summary>
         Public Shared ByCompanyName As Comparison(Of Shippers) =
         Function(ByVal x As Shippers, ByVal y As Shippers)
             Dim value1 As String = If(x.CompanyName, String.Empty)
             Dim value2 As String = If(y.CompanyName, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares Phone used for sorting
         ''' </summary>
         Public Shared ByPhone As Comparison(Of Shippers) =
         Function(ByVal x As Shippers, ByVal y As Shippers)
             Dim value1 As String = If(x.Phone, String.Empty)
             Dim value2 As String = If(y.Phone, String.Empty)
             Return value1.CompareTo(value2)
         End Function

     End Class
End Namespace
