Imports System
Imports System.Data
Imports NorthwindWinFormDynamic_VB.DataLayer
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Models
Imports System.Runtime.InteropServices

Namespace BusinessObject.Base

     ''' <summary>
     ''' Base class for Territories.  Do not make changes to this class,
     ''' instead, put additional code in the Territories class
     ''' </summary>
     Public Class TerritoriesBase
         ' private members
         Private _territoryID As String
         Private _territoryDescription As String
         Private _regionID As Integer

         ''' <summary>
         ''' Gets or Sets TerritoryID
         ''' </summary>
         Public Property TerritoryID() As String
             Get
                 Return _territoryID
             End Get
             Set(ByVal value As String)
                 _territoryID = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets TerritoryDescription
         ''' </summary>
         Public Property TerritoryDescription() As String
             Get
                 Return _territoryDescription
             End Get
             Set(ByVal value As String)
                 _territoryDescription = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or Sets RegionID
         ''' </summary>
         Public Property RegionID() As Integer
             Get
                 Return _regionID
             End Get
             Set(ByVal value As Integer)
                 _regionID = value
             End Set
         End Property

         ''' <summary>
         ''' Gets or sets the Related Region.  Related to column RegionID
         ''' </summary>
         Public ReadOnly Property Region() As Lazy(Of Region)
             Get
                 Return New Lazy(Of Region)(Function() BusinessObject.Region.SelectByPrimaryKey(RegionID))
             End Get
         End Property


         ''' <summary>
         ''' Constructor
         ''' </summary>
         Public Sub New()
         End Sub

         ''' <summary>
         ''' Selects a record by primary key(s)
         ''' </summary>
         Public Shared Function SelectByPrimaryKey(ByVal territoryID As String) As Territories
             Return TerritoriesDataLayer.SelectByPrimaryKey(territoryID)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Territories table
         ''' </summary>
         Public Shared Function GetRecordCount() As Integer
             Return TerritoriesDataLayer.GetRecordCount()
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Territories table by RegionID
         ''' </summary>
         Public Shared Function GetRecordCountByRegionID(regionID As Integer) As Integer
             Return TerritoriesDataLayer.GetRecordCountByRegionID(regionID)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Territories table based on search parameters
         ''' </summary>
         Public Shared Function GetRecordCountDynamicWhere(territoryID As String, territoryDescription As String, regionID As Integer?) As Integer
             Return TerritoriesDataLayer.GetRecordCountDynamicWhere(territoryID, territoryDescription, regionID)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Territories sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared Function SelectSkipAndTake(rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer, sortByExpression As String) As TerritoriesCollection
             totalRowCount = GetRecordCount()
             sortByExpression = GetSortExpression(sortByExpression)

             Return TerritoriesDataLayer.SelectSkipAndTake(sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Territories sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex.  Use for non model-binded gridview control.  Does not return total row count.
         ''' </summary>
         Public Shared Function SelectSkipAndTake(rows As Integer, startRowIndex As Integer, sortByExpression As String) As TerritoriesCollection
             sortByExpression = GetSortExpression(sortByExpression)
             Return TerritoriesDataLayer.SelectSkipAndTake(sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records by RegionID as a collection (List) of Territories sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared  Function SelectSkipAndTakeByRegionID(rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer,  sortByExpression As String, regionID As Integer) As TerritoriesCollection
             totalRowCount = TerritoriesDataLayer.GetRecordCountByRegionID(regionID)
             Dim ending As Integer = startRowIndex + rows
             sortByExpression = GetSortExpression(sortByExpression)
             Return TerritoriesDataLayer.SelectSkipAndTakeByRegionID(sortByExpression, startRowIndex, ending, regionID)
         End Function

         ''' <summary>
         ''' Selects records by RegionID as a collection (List) of Territories sorted by the sortByExpression starting from the startRowIndex
         ''' </summary>
         Public Shared  Function SelectSkipAndTakeByRegionID(rows As Integer, startRowIndex As Integer,  sortByExpression As String, regionID As Integer) As TerritoriesCollection
             Dim ending As Integer = startRowIndex + rows
             sortByExpression = GetSortExpression(sortByExpression)
             Return TerritoriesDataLayer.SelectSkipAndTakeByRegionID(sortByExpression, startRowIndex, ending, regionID)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Territories sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex, based on the search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(territoryID As String, territoryDescription As String, regionID As Integer?, rows As Integer, startRowIndex As Integer, <Out()> ByRef totalRowCount As Integer, sortByExpression As String) As TerritoriesCollection
             totalRowCount = GetRecordCountDynamicWhere(territoryID, territoryDescription, regionID)
             sortByExpression = GetSortExpression(sortByExpression)
             Return TerritoriesDataLayer.SelectSkipAndTakeDynamicWhere(territoryID, territoryDescription, regionID, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records as a collection (List) of Territories sorted by the sortByExpression starting from the startRowIndex, based on the search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(territoryID As String, territoryDescription As String, regionID As Integer?, rows As Integer, startRowIndex As Integer, sortByExpression As String) As TerritoriesCollection
             sortByExpression = GetSortExpression(sortByExpression)
             Return TerritoriesDataLayer.SelectSkipAndTakeDynamicWhere(territoryID, territoryDescription, regionID, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects all records as a collection (List) of Territories
         ''' </summary>
         Public Shared Function SelectAll() As TerritoriesCollection
             Return TerritoriesDataLayer.SelectAll()
         End Function

         ''' <summary>
         ''' Selects all records as a collection (List) of Territories sorted by the sort expression
         ''' </summary>
         Public Shared Function SelectAll(sortExpression As String) As TerritoriesCollection
             Dim objTerritoriesCol As TerritoriesCollection = TerritoriesDataLayer.SelectAll()
             Return SortByExpression(objTerritoriesCol, sortExpression)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Territories.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(territoryID As String, territoryDescription As String, regionID As Integer?) As TerritoriesCollection
             Return TerritoriesDataLayer.SelectAllDynamicWhere(territoryID, territoryDescription, regionID)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Territories sorted by the sort expression.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(territoryID As String, territoryDescription As String, regionID As System.Nullable(Of Integer), sortExpression As String) As TerritoriesCollection
             Dim objTerritoriesCol As TerritoriesCollection = TerritoriesDataLayer.SelectAllDynamicWhere(territoryID, territoryDescription, regionID)
             Return SortByExpression(objTerritoriesCol, sortExpression)
         End Function

         ''' <summary>
         ''' Selects all Territories by Region, related to column RegionID
         ''' </summary>
         Public Shared Function SelectTerritoriesCollectionByRegionID(regionID As Integer) As TerritoriesCollection
             Return TerritoriesDataLayer.SelectTerritoriesCollectionByRegionID(regionID)
         End Function

         ''' <summary>
         ''' Selects all Territories by Region, related to column RegionID, sorted by the sort expression
         ''' </summary>
         Public Shared Function SelectTerritoriesCollectionByRegionID(regionID As Integer, sortExpression As String) As TerritoriesCollection
             Dim objTerritoriesCol As TerritoriesCollection = TerritoriesDataLayer.SelectTerritoriesCollectionByRegionID(regionID)
             Return SortByExpression(objTerritoriesCol, sortExpression)
         End Function

         ''' <summary>
         ''' Selects TerritoryID and TerritoryDescription columns for use with a DropDownList web control, ComboBox, CheckedBoxList, ListView, ListBox, etc
         ''' </summary>
         Public Shared Function SelectTerritoriesDropDownListData() As TerritoriesCollection
             Return TerritoriesDataLayer.SelectTerritoriesDropDownListData()
         End Function

         ''' <summary>
         ''' Sorts the TerritoriesCollection by sort expression
         ''' </summary>
         Public Shared Function SortByExpression(objTerritoriesCol As TerritoriesCollection, sortExpression As String) As TerritoriesCollection
             Dim isSortDescending As Boolean = sortExpression.ToLower().Contains(" desc")

             If isSortDescending Then
                 sortExpression = sortExpression.Replace(" DESC", "")
                 sortExpression = sortExpression.Replace(" desc", "")
             Else
                 sortExpression = sortExpression.Replace(" ASC", "")
                 sortExpression = sortExpression.Replace(" asc", "")
             End If

             Select Case sortExpression
                 Case "TerritoryID"
                     objTerritoriesCol.Sort(Territories.ByTerritoryID)
                     Exit Select
                 Case "TerritoryDescription"
                     objTerritoriesCol.Sort(Territories.ByTerritoryDescription)
                     Exit Select
                 Case "RegionID"
                     objTerritoriesCol.Sort(Territories.ByRegionID)
                     Exit Select
                 Case Else
                     Exit Select
             End Select

             If isSortDescending Then
                 objTerritoriesCol.Reverse()
             End If

             Return objTerritoriesCol
         End Function

         ''' <summary>
         ''' Inserts a record
         ''' </summary>
         Public Function Insert() As String
             Dim objTerritories As Territories = DirectCast(Me, Territories)
             Return TerritoriesDataLayer.Insert(objTerritories)
         End Function

         ''' <summary>
         ''' Updates a record
         ''' </summary>
         Public Sub Update()
             Dim objTerritories As Territories = DirectCast(Me, Territories)
             TerritoriesDataLayer.Update(objTerritories)
         End Sub

         ''' <summary>
         ''' Deletes a record based on primary key(s)
         ''' </summary>
         Public Shared Sub Delete(ByVal territoryID As String)
             TerritoriesDataLayer.Delete(territoryID)
         End Sub

         Private Shared Function GetSortExpression(sortByExpression As String) As String
             If [String].IsNullOrEmpty(sortByExpression) Or sortByExpression = " asc" Then
                 sortByExpression = "TerritoryID"
             Else If sortByExpression.Contains(" asc") Then
                 sortByExpression = sortByExpression.Replace(" asc", "")
             End If

             Return sortByExpression
         End Function

         ''' <summary>
         ''' Compares TerritoryID used for sorting
         ''' </summary>
         Public Shared ByTerritoryID As Comparison(Of Territories) =
         Function(ByVal x As Territories, ByVal y As Territories)
             Dim value1 As String = If(x.TerritoryID, String.Empty)
             Dim value2 As String = If(y.TerritoryID, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares TerritoryDescription used for sorting
         ''' </summary>
         Public Shared ByTerritoryDescription As Comparison(Of Territories) =
         Function(ByVal x As Territories, ByVal y As Territories)
             Dim value1 As String = If(x.TerritoryDescription, String.Empty)
             Dim value2 As String = If(y.TerritoryDescription, String.Empty)
             Return value1.CompareTo(value2)
         End Function

         ''' <summary>
         ''' Compares RegionID used for sorting
         ''' </summary>
         Public Shared ByRegionID As Comparison(Of Territories) =
         Function(ByVal x As Territories, ByVal y As Territories)
             Return x.RegionID.CompareTo(y.RegionID)
         End Function

     End Class
End Namespace
