Imports System
Imports System.Data
Imports System.Data.SqlClient
Imports System.Configuration
Imports NorthwindWinFormDynamic_VB.BusinessObject

Namespace DataLayer.Base
     ''' <summary>
     ''' Base class for CategoriesDataLayer.  Do not make changes to this class,
     ''' instead, put additional code in the CategoriesDataLayer class
     ''' </summary>
     Public Class CategoriesDataLayerBase

         ' constructor
         Public Sub New()
         End Sub

         ''' <summary>
         ''' Selects a record by primary key(s)
         ''' </summary>
         Public Shared Function SelectByPrimaryKey(ByVal categoryID As Integer) As Categories
              Dim objCategories As Categories = Nothing
              Dim dynamicSqlScript As String = CategoriesSql.SelectByPrimaryKey()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@categoryID", categoryID)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objCategories = CreateCategoriesFromDataRowShared(dt.Rows(0))
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objCategories
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Categories table
         ''' </summary>
         Public Shared Function GetRecordCount() As Integer
             Dim sql As String = CategoriesSql.GetRecordCount()
             Return GetRecordCountShared(Nothing, Nothing, Nothing, False, sql)
         End Function

         Public Shared Function GetRecordCountShared(Optional ByVal storedProcName As String = Nothing, Optional ByVal param As String = Nothing, Optional ByVal paramValue As Object = Nothing, Optional ByVal isUseStoredProc As Boolean = True, Optional ByVal dynamicSqlScript As String = Nothing) As Integer
              Dim recordCount As Integer = 0

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  recordCount = DirectCast(dt.Rows(0)("RecordCount"), Integer)
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return recordCount
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Categories table based on search parameters
         ''' </summary>
         Public Shared Function GetRecordCountDynamicWhere(categoryID As Integer?, categoryName As String) As Integer
              Dim recordCount As Integer = 0
              Dim dynamicSqlScript As String = CategoriesSql.GetRecordCountDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' search parameters
                      AddSearchCommandParamsShared(command, categoryID, categoryName)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  recordCount = DirectCast(dt.Rows(0)("RecordCount"), Integer)
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return recordCount
         End Function

         ''' <summary>
         ''' Selects Categories records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of records)
         ''' </summary>
         Public Shared Function SelectSkipAndTake(sortByExpression As String, startRowIndex As Integer, rows As Integer) As CategoriesCollection
             Dim dynamicSqlScript As String = CategoriesSql.SelectSkipAndTake()
             Return SelectShared(dynamicSqlScript, Nothing, Nothing, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects Categories records sorted by the sortByExpression and returns records between the start and end based on search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(categoryID As Integer?, categoryName As String, sortByExpression As String, startRowIndex As Integer, rows  As Integer) As CategoriesCollection
              Dim  objCategoriesCol As CategoriesCollection = Nothing
              Dim dynamicSqlScript As String = CategoriesSql.SelectSkipAndTakeDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' select, skip, take, sort parameters
                      command.Parameters.AddWithValue("@start", startRowIndex)
                      command.Parameters.AddWithValue("@numberOfRows", rows)
                      command.Parameters.AddWithValue("@sortByExpression", sortByExpression)

                      ' search parameters
                      AddSearchCommandParamsShared(command, categoryID, categoryName)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objCategoriesCol = New CategoriesCollection()

                                  For Each dr As DataRow in dt.Rows
                                      Dim objCategories As Categories = CreateCategoriesFromDataRowShared(dr)
                                      objCategoriesCol.Add(objCategories)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objCategoriesCol
         End Function

         ''' <summary>
         ''' Selects all Categories
         ''' </summary>
         Public Shared Function SelectAll() As CategoriesCollection
             Dim dynamicSqlScript As String = CategoriesSql.SelectAll()
             Return SelectShared(dynamicSqlScript, String.Empty, Nothing)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Categories.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(categoryID As Integer?, categoryName As String) As CategoriesCollection
              Dim objCategoriesCol As CategoriesCollection = Nothing
              Dim dynamicSqlScript As String = CategoriesSql.SelectAllDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' search parameters
                      AddSearchCommandParamsShared(command, categoryID, categoryName)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objCategoriesCol = New CategoriesCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objCategories As Categories = CreateCategoriesFromDataRowShared(dr)
                                      objCategoriesCol.Add(objCategories)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objCategoriesCol
         End Function

         ''' <summary>
         ''' Selects CategoryID and CategoryName columns for use with a DropDownList web control
         ''' </summary>
         Public Shared Function SelectCategoriesDropDownListData() As CategoriesCollection
              Dim objCategoriesCol As CategoriesCollection = Nothing
              Dim dynamicSqlScript As String = CategoriesSql.SelectCategoriesDropDownListData()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objCategoriesCol = New CategoriesCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objCategories As Categories  = New Categories()
                                      objCategories.CategoryID = DirectCast(dr("CategoryID"), Integer)
                                      objCategories.CategoryName = DirectCast(dr("CategoryName"), String)

                                      objCategoriesCol.Add(objCategories)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objCategoriesCol
         End Function

         Public Shared Function SelectShared(dynamicSqlScript As String, param As String, paramValue As Object, Optional ByVal sortByExpression As String = Nothing, Optional ByVal startRowIndex As Integer? = Nothing, Optional ByVal rows As Integer? = Nothing) As CategoriesCollection
              Dim objCategoriesCol As CategoriesCollection = Nothing

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' select, skip, take, sort parameters
                      If Not [String].IsNullOrEmpty(sortByExpression) AndAlso startRowIndex IsNot Nothing AndAlso rows IsNot Nothing Then
                          command.Parameters.AddWithValue("@start", startRowIndex.Value)
                          command.Parameters.AddWithValue("@numberOfRows", rows.Value)
                          command.Parameters.AddWithValue("@sortByExpression", sortByExpression)
                      End If

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objCategoriesCol = New CategoriesCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objCategories As Categories = CreateCategoriesFromDataRowShared(dr)
                                      objCategoriesCol.Add(objCategories)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objCategoriesCol
         End Function

         ''' <summary>
         ''' Inserts a record
         ''' </summary>
         Public Shared Function Insert(objCategories As Categories) As Integer
             Return InsertUpdate(objCategories, False)
         End Function

         ''' <summary>
         ''' Updates a record
         ''' </summary>
         Public Shared Sub Update(objCategories As Categories)
             InsertUpdate(objCategories, True)
         End Sub

         Private Shared Function InsertUpdate(objCategories As Categories, isUpdate As Boolean) As Integer
              Dim newlyCreatedCategoryID As Integer = objCategories.CategoryID

              Dim dynamicSqlScript As String = String.Empty

              If isUpdate Then
                  dynamicSqlScript = CategoriesSql.Update()
              Else
                  dynamicSqlScript = CategoriesSql.Insert()
              End If
              Dim description As Object = objCategories.Description

              If [String].IsNullOrEmpty(objCategories.Description) Then
                  description = System.DBNull.Value
              End If

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      If isUpdate Then
                          ' for update only
                          command.Parameters.AddWithValue("@categoryID", objCategories.CategoryID)
                      End If

                      command.Parameters.AddWithValue("@categoryName", objCategories.CategoryName)
                      command.Parameters.AddWithValue("@description", description)

                      If isUpdate Then
                          command.ExecuteNonQuery()
                      Else
                          newlyCreatedCategoryID = DirectCast(command.ExecuteScalar(), Integer)
                      End If
                  End Using
              End Using

              Return newlyCreatedCategoryID
         End Function

         ''' <summary>
         ''' Deletes a record based on primary key(s)
         ''' </summary>
         Public Shared Sub Delete(ByVal categoryID As Integer)
              Dim dynamicSqlScript As String = CategoriesSql.Delete()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@categoryID", categoryID)

                      ' execute
                      command.ExecuteNonQuery()
                  End Using
              End Using
         End Sub

         ''' <summary>
         ''' Adds search parameters to the Command object
         ''' </summary>
         Private Shared Sub AddSearchCommandParamsShared(command As SqlCommand, categoryID As Integer?, categoryName As String)
              If Not categoryID Is Nothing Then
                  command.Parameters.AddWithValue("@categoryID", categoryID)
              Else
                  command.Parameters.AddWithValue("@categoryID", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(categoryName) Then
                  command.Parameters.AddWithValue("@categoryName", categoryName)
              Else
                  command.Parameters.AddWithValue("@categoryName", System.DBNull.Value)
              End If

         End Sub

         ''' <summary>
         ''' Creates a Categories object from the passed data row
         ''' </summary>
         Private Shared Function CreateCategoriesFromDataRowShared(dr As DataRow) As Categories
             Dim objCategories As Categories = New Categories()


             If Not DbNull.Value.Equals(dr("CategoryID")) Then
                 objCategories.CategoryID = DirectCast(dr("CategoryID"), Integer)
             Else
                 objCategories.CategoryID = Nothing
             End If

             If Not DbNull.Value.Equals(dr("CategoryName")) Then
                 objCategories.CategoryName = dr("CategoryName").ToString()
             Else
                 objCategories.CategoryName = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Description")) Then
                 objCategories.Description = dr("Description").ToString()
             Else
                 objCategories.Description = Nothing
             End If

             Return objCategories
         End Function
     End Class
End Namespace
