Imports System
Imports System.Data
Imports System.Data.SqlClient
Imports System.Configuration
Imports NorthwindWinFormDynamic_VB.BusinessObject

Namespace DataLayer.Base
     ''' <summary>
     ''' Base class for CustomersDataLayer.  Do not make changes to this class,
     ''' instead, put additional code in the CustomersDataLayer class
     ''' </summary>
     Public Class CustomersDataLayerBase

         ' constructor
         Public Sub New()
         End Sub

         ''' <summary>
         ''' Selects a record by primary key(s)
         ''' </summary>
         Public Shared Function SelectByPrimaryKey(ByVal customerID As String) As Customers
              Dim objCustomers As Customers = Nothing
              Dim dynamicSqlScript As String = CustomersSql.SelectByPrimaryKey()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@customerID", customerID)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objCustomers = CreateCustomersFromDataRowShared(dt.Rows(0))
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objCustomers
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Customers table
         ''' </summary>
         Public Shared Function GetRecordCount() As Integer
             Dim sql As String = CustomersSql.GetRecordCount()
             Return GetRecordCountShared(Nothing, Nothing, Nothing, False, sql)
         End Function

         Public Shared Function GetRecordCountShared(Optional ByVal storedProcName As String = Nothing, Optional ByVal param As String = Nothing, Optional ByVal paramValue As Object = Nothing, Optional ByVal isUseStoredProc As Boolean = True, Optional ByVal dynamicSqlScript As String = Nothing) As Integer
              Dim recordCount As Integer = 0

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  recordCount = DirectCast(dt.Rows(0)("RecordCount"), Integer)
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return recordCount
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Customers table based on search parameters
         ''' </summary>
         Public Shared Function GetRecordCountDynamicWhere(customerID As String, companyName As String, contactName As String, contactTitle As String, address As String, city As String, region1 As String, postalCode As String, country As String, phone As String, fax As String) As Integer
              Dim recordCount As Integer = 0
              Dim dynamicSqlScript As String = CustomersSql.GetRecordCountDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' search parameters
                      AddSearchCommandParamsShared(command, customerID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  recordCount = DirectCast(dt.Rows(0)("RecordCount"), Integer)
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return recordCount
         End Function

         ''' <summary>
         ''' Selects Customers records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of records)
         ''' </summary>
         Public Shared Function SelectSkipAndTake(sortByExpression As String, startRowIndex As Integer, rows As Integer) As CustomersCollection
             Dim dynamicSqlScript As String = CustomersSql.SelectSkipAndTake()
             Return SelectShared(dynamicSqlScript, Nothing, Nothing, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects Customers records sorted by the sortByExpression and returns records between the start and end based on search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(customerID As String, companyName As String, contactName As String, contactTitle As String, address As String, city As String, region1 As String, postalCode As String, country As String, phone As String, fax As String, sortByExpression As String, startRowIndex As Integer, rows  As Integer) As CustomersCollection
              Dim  objCustomersCol As CustomersCollection = Nothing
              Dim dynamicSqlScript As String = CustomersSql.SelectSkipAndTakeDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' select, skip, take, sort parameters
                      command.Parameters.AddWithValue("@start", startRowIndex)
                      command.Parameters.AddWithValue("@numberOfRows", rows)
                      command.Parameters.AddWithValue("@sortByExpression", sortByExpression)

                      ' search parameters
                      AddSearchCommandParamsShared(command, customerID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objCustomersCol = New CustomersCollection()

                                  For Each dr As DataRow in dt.Rows
                                      Dim objCustomers As Customers = CreateCustomersFromDataRowShared(dr)
                                      objCustomersCol.Add(objCustomers)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objCustomersCol
         End Function

         ''' <summary>
         ''' Selects all Customers
         ''' </summary>
         Public Shared Function SelectAll() As CustomersCollection
             Dim dynamicSqlScript As String = CustomersSql.SelectAll()
             Return SelectShared(dynamicSqlScript, String.Empty, Nothing)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Customers.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(customerID As String, companyName As String, contactName As String, contactTitle As String, address As String, city As String, region1 As String, postalCode As String, country As String, phone As String, fax As String) As CustomersCollection
              Dim objCustomersCol As CustomersCollection = Nothing
              Dim dynamicSqlScript As String = CustomersSql.SelectAllDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' search parameters
                      AddSearchCommandParamsShared(command, customerID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objCustomersCol = New CustomersCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objCustomers As Customers = CreateCustomersFromDataRowShared(dr)
                                      objCustomersCol.Add(objCustomers)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objCustomersCol
         End Function

         ''' <summary>
         ''' Selects CustomerID and CompanyName columns for use with a DropDownList web control
         ''' </summary>
         Public Shared Function SelectCustomersDropDownListData() As CustomersCollection
              Dim objCustomersCol As CustomersCollection = Nothing
              Dim dynamicSqlScript As String = CustomersSql.SelectCustomersDropDownListData()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objCustomersCol = New CustomersCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objCustomers As Customers  = New Customers()
                                      objCustomers.CustomerID = DirectCast(dr("CustomerID"), String)
                                      objCustomers.CompanyName = DirectCast(dr("CompanyName"), String)

                                      objCustomersCol.Add(objCustomers)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objCustomersCol
         End Function

         Public Shared Function SelectShared(dynamicSqlScript As String, param As String, paramValue As Object, Optional ByVal sortByExpression As String = Nothing, Optional ByVal startRowIndex As Integer? = Nothing, Optional ByVal rows As Integer? = Nothing) As CustomersCollection
              Dim objCustomersCol As CustomersCollection = Nothing

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' select, skip, take, sort parameters
                      If Not [String].IsNullOrEmpty(sortByExpression) AndAlso startRowIndex IsNot Nothing AndAlso rows IsNot Nothing Then
                          command.Parameters.AddWithValue("@start", startRowIndex.Value)
                          command.Parameters.AddWithValue("@numberOfRows", rows.Value)
                          command.Parameters.AddWithValue("@sortByExpression", sortByExpression)
                      End If

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objCustomersCol = New CustomersCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objCustomers As Customers = CreateCustomersFromDataRowShared(dr)
                                      objCustomersCol.Add(objCustomers)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objCustomersCol
         End Function

         ''' <summary>
         ''' Inserts a record
         ''' </summary>
         Public Shared Function Insert(objCustomers As Customers) As String
             Return InsertUpdate(objCustomers, False)
         End Function

         ''' <summary>
         ''' Updates a record
         ''' </summary>
         Public Shared Sub Update(objCustomers As Customers)
             InsertUpdate(objCustomers, True)
         End Sub

         Private Shared Function InsertUpdate(objCustomers As Customers, isUpdate As Boolean) As String
              Dim newlyCreatedCustomerID As String = objCustomers.CustomerID

              Dim dynamicSqlScript As String = String.Empty

              If isUpdate Then
                  dynamicSqlScript = CustomersSql.Update()
              Else
                  dynamicSqlScript = CustomersSql.Insert()
              End If
              Dim contactName As Object = objCustomers.ContactName
              Dim contactTitle As Object = objCustomers.ContactTitle
              Dim address As Object = objCustomers.Address
              Dim city As Object = objCustomers.City
              Dim region1 As Object = objCustomers.Region1
              Dim postalCode As Object = objCustomers.PostalCode
              Dim country As Object = objCustomers.Country
              Dim phone As Object = objCustomers.Phone
              Dim fax As Object = objCustomers.Fax

              If [String].IsNullOrEmpty(objCustomers.ContactName) Then
                  contactName = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objCustomers.ContactTitle) Then
                  contactTitle = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objCustomers.Address) Then
                  address = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objCustomers.City) Then
                  city = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objCustomers.Region1) Then
                  region1 = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objCustomers.PostalCode) Then
                  postalCode = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objCustomers.Country) Then
                  country = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objCustomers.Phone) Then
                  phone = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objCustomers.Fax) Then
                  fax = System.DBNull.Value
              End If

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@customerID", objCustomers.CustomerID)
                      command.Parameters.AddWithValue("@companyName", objCustomers.CompanyName)
                      command.Parameters.AddWithValue("@contactName", contactName)
                      command.Parameters.AddWithValue("@contactTitle", contactTitle)
                      command.Parameters.AddWithValue("@address", address)
                      command.Parameters.AddWithValue("@city", city)
                      command.Parameters.AddWithValue("@region1", region1)
                      command.Parameters.AddWithValue("@postalCode", postalCode)
                      command.Parameters.AddWithValue("@country", country)
                      command.Parameters.AddWithValue("@phone", phone)
                      command.Parameters.AddWithValue("@fax", fax)

                      If isUpdate Then
                          command.ExecuteNonQuery()
                      Else
                          newlyCreatedCustomerID = DirectCast(command.ExecuteScalar(), String)
                      End If
                  End Using
              End Using

              Return newlyCreatedCustomerID
         End Function

         ''' <summary>
         ''' Deletes a record based on primary key(s)
         ''' </summary>
         Public Shared Sub Delete(ByVal customerID As String)
              Dim dynamicSqlScript As String = CustomersSql.Delete()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@customerID", customerID)

                      ' execute
                      command.ExecuteNonQuery()
                  End Using
              End Using
         End Sub

         ''' <summary>
         ''' Adds search parameters to the Command object
         ''' </summary>
         Private Shared Sub AddSearchCommandParamsShared(command As SqlCommand, customerID As String, companyName As String, contactName As String, contactTitle As String, address As String, city As String, region1 As String, postalCode As String, country As String, phone As String, fax As String)
              If Not [String].IsNullOrEmpty(customerID) Then
                  command.Parameters.AddWithValue("@customerID", customerID)
              Else
                  command.Parameters.AddWithValue("@customerID", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(companyName) Then
                  command.Parameters.AddWithValue("@companyName", companyName)
              Else
                  command.Parameters.AddWithValue("@companyName", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(contactName) Then
                  command.Parameters.AddWithValue("@contactName", contactName)
              Else
                  command.Parameters.AddWithValue("@contactName", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(contactTitle) Then
                  command.Parameters.AddWithValue("@contactTitle", contactTitle)
              Else
                  command.Parameters.AddWithValue("@contactTitle", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(address) Then
                  command.Parameters.AddWithValue("@address", address)
              Else
                  command.Parameters.AddWithValue("@address", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(city) Then
                  command.Parameters.AddWithValue("@city", city)
              Else
                  command.Parameters.AddWithValue("@city", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(region1) Then
                  command.Parameters.AddWithValue("@region1", region1)
              Else
                  command.Parameters.AddWithValue("@region1", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(postalCode) Then
                  command.Parameters.AddWithValue("@postalCode", postalCode)
              Else
                  command.Parameters.AddWithValue("@postalCode", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(country) Then
                  command.Parameters.AddWithValue("@country", country)
              Else
                  command.Parameters.AddWithValue("@country", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(phone) Then
                  command.Parameters.AddWithValue("@phone", phone)
              Else
                  command.Parameters.AddWithValue("@phone", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(fax) Then
                  command.Parameters.AddWithValue("@fax", fax)
              Else
                  command.Parameters.AddWithValue("@fax", System.DBNull.Value)
              End If

         End Sub

         ''' <summary>
         ''' Creates a Customers object from the passed data row
         ''' </summary>
         Private Shared Function CreateCustomersFromDataRowShared(dr As DataRow) As Customers
             Dim objCustomers As Customers = New Customers()


             If Not DbNull.Value.Equals(dr("CustomerID")) Then
                 objCustomers.CustomerID = dr("CustomerID").ToString()
             Else
                 objCustomers.CustomerID = Nothing
             End If

             If Not DbNull.Value.Equals(dr("CompanyName")) Then
                 objCustomers.CompanyName = dr("CompanyName").ToString()
             Else
                 objCustomers.CompanyName = Nothing
             End If

             If Not DbNull.Value.Equals(dr("ContactName")) Then
                 objCustomers.ContactName = dr("ContactName").ToString()
             Else
                 objCustomers.ContactName = Nothing
             End If

             If Not DbNull.Value.Equals(dr("ContactTitle")) Then
                 objCustomers.ContactTitle = dr("ContactTitle").ToString()
             Else
                 objCustomers.ContactTitle = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Address")) Then
                 objCustomers.Address = dr("Address").ToString()
             Else
                 objCustomers.Address = Nothing
             End If

             If Not DbNull.Value.Equals(dr("City")) Then
                 objCustomers.City = dr("City").ToString()
             Else
                 objCustomers.City = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Region1")) Then
                 objCustomers.Region1 = dr("Region1").ToString()
             Else
                 objCustomers.Region1 = Nothing
             End If

             If Not DbNull.Value.Equals(dr("PostalCode")) Then
                 objCustomers.PostalCode = dr("PostalCode").ToString()
             Else
                 objCustomers.PostalCode = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Country")) Then
                 objCustomers.Country = dr("Country").ToString()
             Else
                 objCustomers.Country = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Phone")) Then
                 objCustomers.Phone = dr("Phone").ToString()
             Else
                 objCustomers.Phone = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Fax")) Then
                 objCustomers.Fax = dr("Fax").ToString()
             Else
                 objCustomers.Fax = Nothing
             End If

             Return objCustomers
         End Function
     End Class
End Namespace
