Imports System
Imports System.Data
Imports System.Data.SqlClient
Imports System.Configuration
Imports NorthwindWinFormDynamic_VB.BusinessObject

Namespace DataLayer.Base
     ''' <summary>
     ''' Base class for EmployeesDataLayer.  Do not make changes to this class,
     ''' instead, put additional code in the EmployeesDataLayer class
     ''' </summary>
     Public Class EmployeesDataLayerBase

         ' constructor
         Public Sub New()
         End Sub

         ''' <summary>
         ''' Selects a record by primary key(s)
         ''' </summary>
         Public Shared Function SelectByPrimaryKey(ByVal employeeID As Integer) As Employees
              Dim objEmployees As Employees = Nothing
              Dim dynamicSqlScript As String = EmployeesSql.SelectByPrimaryKey()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@employeeID", employeeID)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objEmployees = CreateEmployeesFromDataRowShared(dt.Rows(0))
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objEmployees
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Employees table
         ''' </summary>
         Public Shared Function GetRecordCount() As Integer
             Dim sql As String = EmployeesSql.GetRecordCount()
             Return GetRecordCountShared(Nothing, Nothing, Nothing, False, sql)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Employees table by ReportsTo
         ''' </summary>
         Public Shared Function GetRecordCountByReportsTo(reportsTo As Integer) As Integer
             Dim sql As String = EmployeesSql.GetRecordCountByReportsTo(reportsTo)
             Return GetRecordCountShared(Nothing, "reportsTo", reportsTo, False, sql)
         End Function

         Public Shared Function GetRecordCountShared(Optional ByVal storedProcName As String = Nothing, Optional ByVal param As String = Nothing, Optional ByVal paramValue As Object = Nothing, Optional ByVal isUseStoredProc As Boolean = True, Optional ByVal dynamicSqlScript As String = Nothing) As Integer
              Dim recordCount As Integer = 0

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      Select Case param
                          Case "reportsTo"
                              command.Parameters.AddWithValue("@reportsTo", paramValue)
                          Case Else
                              Exit Select
                      End Select

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  recordCount = DirectCast(dt.Rows(0)("RecordCount"), Integer)
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return recordCount
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Employees table based on search parameters
         ''' </summary>
         Public Shared Function GetRecordCountDynamicWhere(employeeID As Integer?, lastName As String, firstName As String, title As String, titleOfCourtesy As String, birthDate As Date?, hireDate As Date?, address As String, city As String, region1 As String, postalCode As String, country As String, homePhone As String, extension As String, reportsTo As Integer?, photoPath As String) As Integer
              Dim recordCount As Integer = 0
              Dim dynamicSqlScript As String = EmployeesSql.GetRecordCountDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' search parameters
                      AddSearchCommandParamsShared(command, employeeID, lastName, firstName, title, titleOfCourtesy, birthDate, hireDate, address, city, region1, postalCode, country, homePhone, extension, reportsTo, photoPath)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  recordCount = DirectCast(dt.Rows(0)("RecordCount"), Integer)
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return recordCount
         End Function

         ''' <summary>
         ''' Selects Employees records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of records)
         ''' </summary>
         Public Shared Function SelectSkipAndTake(sortByExpression As String, startRowIndex As Integer, rows As Integer) As EmployeesCollection
             Dim dynamicSqlScript As String = EmployeesSql.SelectSkipAndTake()
             Return SelectShared(dynamicSqlScript, Nothing, Nothing, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records by ReportsTo as a collection (List) of Employees sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared Function SelectSkipAndTakeByReportsTo(sortByExpression As String, startRowIndex As Integer, rows As Integer, reportsTo As Integer) As EmployeesCollection
             Dim dynamicSqlScript As String = EmployeesSql.SelectSkipAndTakeByReportsTo(reportsTo)
             Return SelectShared(dynamicSqlScript, "reportsTo", reportsTo, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects Employees records sorted by the sortByExpression and returns records between the start and end based on search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(employeeID As Integer?, lastName As String, firstName As String, title As String, titleOfCourtesy As String, birthDate As Date?, hireDate As Date?, address As String, city As String, region1 As String, postalCode As String, country As String, homePhone As String, extension As String, reportsTo As Integer?, photoPath As String, sortByExpression As String, startRowIndex As Integer, rows  As Integer) As EmployeesCollection
              Dim  objEmployeesCol As EmployeesCollection = Nothing
              Dim dynamicSqlScript As String = EmployeesSql.SelectSkipAndTakeDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' select, skip, take, sort parameters
                      command.Parameters.AddWithValue("@start", startRowIndex)
                      command.Parameters.AddWithValue("@numberOfRows", rows)
                      command.Parameters.AddWithValue("@sortByExpression", sortByExpression)

                      ' search parameters
                      AddSearchCommandParamsShared(command, employeeID, lastName, firstName, title, titleOfCourtesy, birthDate, hireDate, address, city, region1, postalCode, country, homePhone, extension, reportsTo, photoPath)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objEmployeesCol = New EmployeesCollection()

                                  For Each dr As DataRow in dt.Rows
                                      Dim objEmployees As Employees = CreateEmployeesFromDataRowShared(dr)
                                      objEmployeesCol.Add(objEmployees)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objEmployeesCol
         End Function

         ''' <summary>
         ''' Selects all Employees
         ''' </summary>
         Public Shared Function SelectAll() As EmployeesCollection
             Dim dynamicSqlScript As String = EmployeesSql.SelectAll()
             Return SelectShared(dynamicSqlScript, String.Empty, Nothing)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Employees.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(employeeID As Integer?, lastName As String, firstName As String, title As String, titleOfCourtesy As String, birthDate As Date?, hireDate As Date?, address As String, city As String, region1 As String, postalCode As String, country As String, homePhone As String, extension As String, reportsTo As Integer?, photoPath As String) As EmployeesCollection
              Dim objEmployeesCol As EmployeesCollection = Nothing
              Dim dynamicSqlScript As String = EmployeesSql.SelectAllDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' search parameters
                      AddSearchCommandParamsShared(command, employeeID, lastName, firstName, title, titleOfCourtesy, birthDate, hireDate, address, city, region1, postalCode, country, homePhone, extension, reportsTo, photoPath)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objEmployeesCol = New EmployeesCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objEmployees As Employees = CreateEmployeesFromDataRowShared(dr)
                                      objEmployeesCol.Add(objEmployees)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objEmployeesCol
         End Function

         ''' <summary>
         ''' Selects all Employees by Employees, related to column ReportsTo
         ''' </summary>
         Public Shared Function SelectEmployeesCollectionByReportsTo(employeeID As Integer) As EmployeesCollection
             Dim dynamicSqlScript As String = EmployeesSql.SelectAllByReportsTo(employeeID)
             Return SelectShared(dynamicSqlScript, "employeeID", employeeID)
         End Function

         ''' <summary>
         ''' Selects EmployeeID and LastName columns for use with a DropDownList web control
         ''' </summary>
         Public Shared Function SelectEmployeesDropDownListData() As EmployeesCollection
              Dim objEmployeesCol As EmployeesCollection = Nothing
              Dim dynamicSqlScript As String = EmployeesSql.SelectEmployeesDropDownListData()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objEmployeesCol = New EmployeesCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objEmployees As Employees  = New Employees()
                                      objEmployees.EmployeeID = DirectCast(dr("EmployeeID"), Integer)
                                      objEmployees.LastName = DirectCast(dr("LastName"), String)

                                      objEmployeesCol.Add(objEmployees)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objEmployeesCol
         End Function

         Public Shared Function SelectShared(dynamicSqlScript As String, param As String, paramValue As Object, Optional ByVal sortByExpression As String = Nothing, Optional ByVal startRowIndex As Integer? = Nothing, Optional ByVal rows As Integer? = Nothing) As EmployeesCollection
              Dim objEmployeesCol As EmployeesCollection = Nothing

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' select, skip, take, sort parameters
                      If Not [String].IsNullOrEmpty(sortByExpression) AndAlso startRowIndex IsNot Nothing AndAlso rows IsNot Nothing Then
                          command.Parameters.AddWithValue("@start", startRowIndex.Value)
                          command.Parameters.AddWithValue("@numberOfRows", rows.Value)
                          command.Parameters.AddWithValue("@sortByExpression", sortByExpression)
                      End If

                      ' parameters
                      Select Case param
                          Case "reportsTo"
                              command.Parameters.AddWithValue("@reportsTo", paramValue)
                              Exit Select
                          Case Else
                              Exit Select
                      End Select

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objEmployeesCol = New EmployeesCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objEmployees As Employees = CreateEmployeesFromDataRowShared(dr)
                                      objEmployeesCol.Add(objEmployees)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objEmployeesCol
         End Function

         ''' <summary>
         ''' Inserts a record
         ''' </summary>
         Public Shared Function Insert(objEmployees As Employees) As Integer
             Return InsertUpdate(objEmployees, False)
         End Function

         ''' <summary>
         ''' Updates a record
         ''' </summary>
         Public Shared Sub Update(objEmployees As Employees)
             InsertUpdate(objEmployees, True)
         End Sub

         Private Shared Function InsertUpdate(objEmployees As Employees, isUpdate As Boolean) As Integer
              Dim newlyCreatedEmployeeID As Integer = objEmployees.EmployeeID

              Dim dynamicSqlScript As String = String.Empty

              If isUpdate Then
                  dynamicSqlScript = EmployeesSql.Update()
              Else
                  dynamicSqlScript = EmployeesSql.Insert()
              End If
              Dim title As Object = objEmployees.Title
              Dim titleOfCourtesy As Object = objEmployees.TitleOfCourtesy
              Dim birthDate As Object = objEmployees.BirthDate
              Dim hireDate As Object = objEmployees.HireDate
              Dim address As Object = objEmployees.Address
              Dim city As Object = objEmployees.City
              Dim region1 As Object = objEmployees.Region1
              Dim postalCode As Object = objEmployees.PostalCode
              Dim country As Object = objEmployees.Country
              Dim homePhone As Object = objEmployees.HomePhone
              Dim extension As Object = objEmployees.Extension
              Dim notes As Object = objEmployees.Notes
              Dim reportsTo As Object = objEmployees.ReportsTo
              Dim photoPath As Object = objEmployees.PhotoPath

              If [String].IsNullOrEmpty(objEmployees.Title) Then
                  title = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objEmployees.TitleOfCourtesy) Then
                  titleOfCourtesy = System.DBNull.Value
              End If

              If objEmployees.BirthDate Is Nothing Then
                  birthDate = System.DBNull.Value
              End If

              If objEmployees.HireDate Is Nothing Then
                  hireDate = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objEmployees.Address) Then
                  address = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objEmployees.City) Then
                  city = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objEmployees.Region1) Then
                  region1 = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objEmployees.PostalCode) Then
                  postalCode = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objEmployees.Country) Then
                  country = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objEmployees.HomePhone) Then
                  homePhone = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objEmployees.Extension) Then
                  extension = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objEmployees.Notes) Then
                  notes = System.DBNull.Value
              End If

              If objEmployees.ReportsTo Is Nothing Then
                  reportsTo = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objEmployees.PhotoPath) Then
                  photoPath = System.DBNull.Value
              End If

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      If isUpdate Then
                          ' for update only
                          command.Parameters.AddWithValue("@employeeID", objEmployees.EmployeeID)
                      End If

                      command.Parameters.AddWithValue("@lastName", objEmployees.LastName)
                      command.Parameters.AddWithValue("@firstName", objEmployees.FirstName)
                      command.Parameters.AddWithValue("@title", title)
                      command.Parameters.AddWithValue("@titleOfCourtesy", titleOfCourtesy)
                      command.Parameters.AddWithValue("@birthDate", birthDate)
                      command.Parameters.AddWithValue("@hireDate", hireDate)
                      command.Parameters.AddWithValue("@address", address)
                      command.Parameters.AddWithValue("@city", city)
                      command.Parameters.AddWithValue("@region1", region1)
                      command.Parameters.AddWithValue("@postalCode", postalCode)
                      command.Parameters.AddWithValue("@country", country)
                      command.Parameters.AddWithValue("@homePhone", homePhone)
                      command.Parameters.AddWithValue("@extension", extension)
                      command.Parameters.AddWithValue("@notes", notes)
                      command.Parameters.AddWithValue("@reportsTo", reportsTo)
                      command.Parameters.AddWithValue("@photoPath", photoPath)

                      If isUpdate Then
                          command.ExecuteNonQuery()
                      Else
                          newlyCreatedEmployeeID = DirectCast(command.ExecuteScalar(), Integer)
                      End If
                  End Using
              End Using

              Return newlyCreatedEmployeeID
         End Function

         ''' <summary>
         ''' Deletes a record based on primary key(s)
         ''' </summary>
         Public Shared Sub Delete(ByVal employeeID As Integer)
              Dim dynamicSqlScript As String = EmployeesSql.Delete()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@employeeID", employeeID)

                      ' execute
                      command.ExecuteNonQuery()
                  End Using
              End Using
         End Sub

         ''' <summary>
         ''' Adds search parameters to the Command object
         ''' </summary>
         Private Shared Sub AddSearchCommandParamsShared(command As SqlCommand, employeeID As Integer?, lastName As String, firstName As String, title As String, titleOfCourtesy As String, birthDate As Date?, hireDate As Date?, address As String, city As String, region1 As String, postalCode As String, country As String, homePhone As String, extension As String, reportsTo As Integer?, photoPath As String)
              If Not employeeID Is Nothing Then
                  command.Parameters.AddWithValue("@employeeID", employeeID)
              Else
                  command.Parameters.AddWithValue("@employeeID", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(lastName) Then
                  command.Parameters.AddWithValue("@lastName", lastName)
              Else
                  command.Parameters.AddWithValue("@lastName", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(firstName) Then
                  command.Parameters.AddWithValue("@firstName", firstName)
              Else
                  command.Parameters.AddWithValue("@firstName", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(title) Then
                  command.Parameters.AddWithValue("@title", title)
              Else
                  command.Parameters.AddWithValue("@title", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(titleOfCourtesy) Then
                  command.Parameters.AddWithValue("@titleOfCourtesy", titleOfCourtesy)
              Else
                  command.Parameters.AddWithValue("@titleOfCourtesy", System.DBNull.Value)
              End If

              If Not birthDate Is Nothing Then
                  command.Parameters.AddWithValue("@birthDate", birthDate)
              Else
                  command.Parameters.AddWithValue("@birthDate", System.DBNull.Value)
              End If

              If Not hireDate Is Nothing Then
                  command.Parameters.AddWithValue("@hireDate", hireDate)
              Else
                  command.Parameters.AddWithValue("@hireDate", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(address) Then
                  command.Parameters.AddWithValue("@address", address)
              Else
                  command.Parameters.AddWithValue("@address", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(city) Then
                  command.Parameters.AddWithValue("@city", city)
              Else
                  command.Parameters.AddWithValue("@city", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(region1) Then
                  command.Parameters.AddWithValue("@region1", region1)
              Else
                  command.Parameters.AddWithValue("@region1", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(postalCode) Then
                  command.Parameters.AddWithValue("@postalCode", postalCode)
              Else
                  command.Parameters.AddWithValue("@postalCode", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(country) Then
                  command.Parameters.AddWithValue("@country", country)
              Else
                  command.Parameters.AddWithValue("@country", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(homePhone) Then
                  command.Parameters.AddWithValue("@homePhone", homePhone)
              Else
                  command.Parameters.AddWithValue("@homePhone", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(extension) Then
                  command.Parameters.AddWithValue("@extension", extension)
              Else
                  command.Parameters.AddWithValue("@extension", System.DBNull.Value)
              End If

              If Not reportsTo Is Nothing Then
                  command.Parameters.AddWithValue("@reportsTo", reportsTo)
              Else
                  command.Parameters.AddWithValue("@reportsTo", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(photoPath) Then
                  command.Parameters.AddWithValue("@photoPath", photoPath)
              Else
                  command.Parameters.AddWithValue("@photoPath", System.DBNull.Value)
              End If

         End Sub

         ''' <summary>
         ''' Creates a Employees object from the passed data row
         ''' </summary>
         Private Shared Function CreateEmployeesFromDataRowShared(dr As DataRow) As Employees
             Dim objEmployees As Employees = New Employees()


             If Not DbNull.Value.Equals(dr("EmployeeID")) Then
                 objEmployees.EmployeeID = DirectCast(dr("EmployeeID"), Integer)
             Else
                 objEmployees.EmployeeID = Nothing
             End If

             If Not DbNull.Value.Equals(dr("LastName")) Then
                 objEmployees.LastName = dr("LastName").ToString()
             Else
                 objEmployees.LastName = Nothing
             End If

             If Not DbNull.Value.Equals(dr("FirstName")) Then
                 objEmployees.FirstName = dr("FirstName").ToString()
             Else
                 objEmployees.FirstName = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Title")) Then
                 objEmployees.Title = dr("Title").ToString()
             Else
                 objEmployees.Title = Nothing
             End If

             If Not DbNull.Value.Equals(dr("TitleOfCourtesy")) Then
                 objEmployees.TitleOfCourtesy = dr("TitleOfCourtesy").ToString()
             Else
                 objEmployees.TitleOfCourtesy = Nothing
             End If

             If Not DbNull.Value.Equals(dr("BirthDate")) Then
                 objEmployees.BirthDate = DirectCast(dr("BirthDate"), Date)
             Else
                 objEmployees.BirthDate = Nothing
             End If

             If Not DbNull.Value.Equals(dr("HireDate")) Then
                 objEmployees.HireDate = DirectCast(dr("HireDate"), Date)
             Else
                 objEmployees.HireDate = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Address")) Then
                 objEmployees.Address = dr("Address").ToString()
             Else
                 objEmployees.Address = Nothing
             End If

             If Not DbNull.Value.Equals(dr("City")) Then
                 objEmployees.City = dr("City").ToString()
             Else
                 objEmployees.City = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Region1")) Then
                 objEmployees.Region1 = dr("Region1").ToString()
             Else
                 objEmployees.Region1 = Nothing
             End If

             If Not DbNull.Value.Equals(dr("PostalCode")) Then
                 objEmployees.PostalCode = dr("PostalCode").ToString()
             Else
                 objEmployees.PostalCode = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Country")) Then
                 objEmployees.Country = dr("Country").ToString()
             Else
                 objEmployees.Country = Nothing
             End If

             If Not DbNull.Value.Equals(dr("HomePhone")) Then
                 objEmployees.HomePhone = dr("HomePhone").ToString()
             Else
                 objEmployees.HomePhone = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Extension")) Then
                 objEmployees.Extension = dr("Extension").ToString()
             Else
                 objEmployees.Extension = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Notes")) Then
                 objEmployees.Notes = dr("Notes").ToString()
             Else
                 objEmployees.Notes = Nothing
             End If

             If Not DbNull.Value.Equals(dr("ReportsTo")) Then
                 objEmployees.ReportsTo = DirectCast(dr("ReportsTo"), Integer)
             Else
                 objEmployees.ReportsTo = Nothing
             End If


             If Not DbNull.Value.Equals(dr("PhotoPath")) Then
                 objEmployees.PhotoPath = dr("PhotoPath").ToString()
             Else
                 objEmployees.PhotoPath = Nothing
             End If

             Return objEmployees
         End Function
     End Class
End Namespace
