Imports System
Imports System.Data
Imports System.Data.SqlClient
Imports System.Configuration
Imports NorthwindWinFormDynamic_VB.BusinessObject

Namespace DataLayer.Base
     ''' <summary>
     ''' Base class for OrderDetailsDataLayer.  Do not make changes to this class,
     ''' instead, put additional code in the OrderDetailsDataLayer class
     ''' </summary>
     Public Class OrderDetailsDataLayerBase

         ' constructor
         Public Sub New()
         End Sub

         ''' <summary>
         ''' Selects a record by primary key(s)
         ''' </summary>
         Public Shared Function SelectByPrimaryKey(ByVal orderID As Integer, ByVal productID As Integer) As OrderDetails
              Dim objOrderDetails As OrderDetails = Nothing
              Dim dynamicSqlScript As String = OrderDetailsSql.SelectByPrimaryKey()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@orderID", orderID)
                      command.Parameters.AddWithValue("@productID", productID)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objOrderDetails = CreateOrderDetailsFromDataRowShared(dt.Rows(0))
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objOrderDetails
         End Function

         ''' <summary>
         ''' Gets the total number of records in the OrderDetails table
         ''' </summary>
         Public Shared Function GetRecordCount() As Integer
             Dim sql As String = OrderDetailsSql.GetRecordCount()
             Return GetRecordCountShared(Nothing, Nothing, Nothing, False, sql)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the OrderDetails table by OrderID
         ''' </summary>
         Public Shared Function GetRecordCountByOrderID(orderID As Integer) As Integer
             Dim sql As String = OrderDetailsSql.GetRecordCountByOrderID(orderID)
             Return GetRecordCountShared(Nothing, "orderID", orderID, False, sql)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the OrderDetails table by ProductID
         ''' </summary>
         Public Shared Function GetRecordCountByProductID(productID As Integer) As Integer
             Dim sql As String = OrderDetailsSql.GetRecordCountByProductID(productID)
             Return GetRecordCountShared(Nothing, "productID", productID, False, sql)
         End Function

         Public Shared Function GetRecordCountShared(Optional ByVal storedProcName As String = Nothing, Optional ByVal param As String = Nothing, Optional ByVal paramValue As Object = Nothing, Optional ByVal isUseStoredProc As Boolean = True, Optional ByVal dynamicSqlScript As String = Nothing) As Integer
              Dim recordCount As Integer = 0

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      Select Case param
                          Case "orderID"
                              command.Parameters.AddWithValue("@orderID", paramValue)
                          Case "productID"
                              command.Parameters.AddWithValue("@productID", paramValue)
                          Case Else
                              Exit Select
                      End Select

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  recordCount = DirectCast(dt.Rows(0)("RecordCount"), Integer)
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return recordCount
         End Function

         ''' <summary>
         ''' Gets the total number of records in the OrderDetails table based on search parameters
         ''' </summary>
         Public Shared Function GetRecordCountDynamicWhere(orderID As Integer?, productID As Integer?, unitPrice As Decimal?, quantity As Short?, discount As Single?) As Integer
              Dim recordCount As Integer = 0
              Dim dynamicSqlScript As String = OrderDetailsSql.GetRecordCountDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' search parameters
                      AddSearchCommandParamsShared(command, orderID, productID, unitPrice, quantity, discount)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  recordCount = DirectCast(dt.Rows(0)("RecordCount"), Integer)
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return recordCount
         End Function

         ''' <summary>
         ''' Selects OrderDetails records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of records)
         ''' </summary>
         Public Shared Function SelectSkipAndTake(sortByExpression As String, startRowIndex As Integer, rows As Integer) As OrderDetailsCollection
             Dim dynamicSqlScript As String = OrderDetailsSql.SelectSkipAndTake()
             Return SelectShared(dynamicSqlScript, Nothing, Nothing, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records by OrderID as a collection (List) of OrderDetails sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared Function SelectSkipAndTakeByOrderID(sortByExpression As String, startRowIndex As Integer, rows As Integer, orderID As Integer) As OrderDetailsCollection
             Dim dynamicSqlScript As String = OrderDetailsSql.SelectSkipAndTakeByOrderID(orderID)
             Return SelectShared(dynamicSqlScript, "orderID", orderID, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records by ProductID as a collection (List) of OrderDetails sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared Function SelectSkipAndTakeByProductID(sortByExpression As String, startRowIndex As Integer, rows As Integer, productID As Integer) As OrderDetailsCollection
             Dim dynamicSqlScript As String = OrderDetailsSql.SelectSkipAndTakeByProductID(productID)
             Return SelectShared(dynamicSqlScript, "productID", productID, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects OrderDetails records sorted by the sortByExpression and returns records between the start and end based on search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(orderID As Integer?, productID As Integer?, unitPrice As Decimal?, quantity As Short?, discount As Single?, sortByExpression As String, startRowIndex As Integer, rows  As Integer) As OrderDetailsCollection
              Dim  objOrderDetailsCol As OrderDetailsCollection = Nothing
              Dim dynamicSqlScript As String = OrderDetailsSql.SelectSkipAndTakeDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' select, skip, take, sort parameters
                      command.Parameters.AddWithValue("@start", startRowIndex)
                      command.Parameters.AddWithValue("@numberOfRows", rows)
                      command.Parameters.AddWithValue("@sortByExpression", sortByExpression)

                      ' search parameters
                      AddSearchCommandParamsShared(command, orderID, productID, unitPrice, quantity, discount)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objOrderDetailsCol = New OrderDetailsCollection()

                                  For Each dr As DataRow in dt.Rows
                                      Dim objOrderDetails As OrderDetails = CreateOrderDetailsFromDataRowShared(dr)
                                      objOrderDetailsCol.Add(objOrderDetails)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objOrderDetailsCol
         End Function

         ''' <summary>
         ''' Gets the grand total or sum of fields with a money of decimal data type
         ''' </summary>
         Public Shared Function SelectTotals() As OrderDetails
              Dim objOrderDetails As OrderDetails = Nothing
              Dim dynamicSqlScript As String = OrderDetailsSql.SelectTotals()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  If Not DbNull.Value.Equals(dt.Rows(0)("UnitPriceTotal")) Then
                                       objOrderDetails.UnitPriceTotal = DirectCast(dt.Rows(0)("UnitPriceTotal"), Decimal)
                                  End If
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objOrderDetails
         End Function

         ''' <summary>
         ''' Selects all OrderDetails
         ''' </summary>
         Public Shared Function SelectAll() As OrderDetailsCollection
             Dim dynamicSqlScript As String = OrderDetailsSql.SelectAll()
             Return SelectShared(dynamicSqlScript, String.Empty, Nothing)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of OrderDetails.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(orderID As Integer?, productID As Integer?, unitPrice As Decimal?, quantity As Short?, discount As Single?) As OrderDetailsCollection
              Dim objOrderDetailsCol As OrderDetailsCollection = Nothing
              Dim dynamicSqlScript As String = OrderDetailsSql.SelectAllDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' search parameters
                      AddSearchCommandParamsShared(command, orderID, productID, unitPrice, quantity, discount)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objOrderDetailsCol = New OrderDetailsCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objOrderDetails As OrderDetails = CreateOrderDetailsFromDataRowShared(dr)
                                      objOrderDetailsCol.Add(objOrderDetails)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objOrderDetailsCol
         End Function

         ''' <summary>
         ''' Selects all OrderDetails by Orders, related to column OrderID
         ''' </summary>
         Public Shared Function SelectOrderDetailsCollectionByOrderID(orderID As Integer) As OrderDetailsCollection
             Dim dynamicSqlScript As String = OrderDetailsSql.SelectAllByOrderID(orderID)
             Return SelectShared(dynamicSqlScript, "orderID", orderID)
         End Function

         ''' <summary>
         ''' Selects all OrderDetails by Products, related to column ProductID
         ''' </summary>
         Public Shared Function SelectOrderDetailsCollectionByProductID(productID As Integer) As OrderDetailsCollection
             Dim dynamicSqlScript As String = OrderDetailsSql.SelectAllByProductID(productID)
             Return SelectShared(dynamicSqlScript, "productID", productID)
         End Function

         Public Shared Function SelectShared(dynamicSqlScript As String, param As String, paramValue As Object, Optional ByVal sortByExpression As String = Nothing, Optional ByVal startRowIndex As Integer? = Nothing, Optional ByVal rows As Integer? = Nothing) As OrderDetailsCollection
              Dim objOrderDetailsCol As OrderDetailsCollection = Nothing

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' select, skip, take, sort parameters
                      If Not [String].IsNullOrEmpty(sortByExpression) AndAlso startRowIndex IsNot Nothing AndAlso rows IsNot Nothing Then
                          command.Parameters.AddWithValue("@start", startRowIndex.Value)
                          command.Parameters.AddWithValue("@numberOfRows", rows.Value)
                          command.Parameters.AddWithValue("@sortByExpression", sortByExpression)
                      End If

                      ' parameters
                      Select Case param
                          Case "orderID"
                              command.Parameters.AddWithValue("@orderID", paramValue)
                              Exit Select
                          Case "productID"
                              command.Parameters.AddWithValue("@productID", paramValue)
                              Exit Select
                          Case Else
                              Exit Select
                      End Select

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objOrderDetailsCol = New OrderDetailsCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objOrderDetails As OrderDetails = CreateOrderDetailsFromDataRowShared(dr)
                                      objOrderDetailsCol.Add(objOrderDetails)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objOrderDetailsCol
         End Function

         ''' <summary>
         ''' Inserts a record
         ''' </summary>
         Public Shared Sub Insert(objOrderDetails As OrderDetails)
             InsertUpdate(objOrderDetails, False)
         End Sub

         ''' <summary>
         ''' Updates a record
         ''' </summary>
         Public Shared Sub Update(objOrderDetails As OrderDetails)
             InsertUpdate(objOrderDetails, True)
         End Sub

         Private Shared Sub InsertUpdate(objOrderDetails As OrderDetails, isUpdate As Boolean)
              Dim dynamicSqlScript As String = String.Empty

              If isUpdate Then
                  dynamicSqlScript = OrderDetailsSql.Update()
              Else
                  dynamicSqlScript = OrderDetailsSql.Insert()
              End If
              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@orderID", objOrderDetails.OrderID)
                      command.Parameters.AddWithValue("@productID", objOrderDetails.ProductID)
                      command.Parameters.AddWithValue("@unitPrice", objOrderDetails.UnitPrice)
                      command.Parameters.AddWithValue("@quantity", objOrderDetails.Quantity)
                      command.Parameters.AddWithValue("@discount", objOrderDetails.Discount)

                      command.ExecuteNonQuery()
                  End Using
              End Using
         End Sub

         ''' <summary>
         ''' Deletes a record based on primary key(s)
         ''' </summary>
         Public Shared Sub Delete(ByVal orderID As Integer, ByVal productID As Integer)
              Dim dynamicSqlScript As String = OrderDetailsSql.Delete()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@orderID", orderID)
                      command.Parameters.AddWithValue("@productID", productID)

                      ' execute
                      command.ExecuteNonQuery()
                  End Using
              End Using
         End Sub

         ''' <summary>
         ''' Adds search parameters to the Command object
         ''' </summary>
         Private Shared Sub AddSearchCommandParamsShared(command As SqlCommand, orderID As Integer?, productID As Integer?, unitPrice As Decimal?, quantity As Short?, discount As Single?)
              If Not orderID Is Nothing Then
                  command.Parameters.AddWithValue("@orderID", orderID)
              Else
                  command.Parameters.AddWithValue("@orderID", System.DBNull.Value)
              End If

              If Not productID Is Nothing Then
                  command.Parameters.AddWithValue("@productID", productID)
              Else
                  command.Parameters.AddWithValue("@productID", System.DBNull.Value)
              End If

              If Not unitPrice Is Nothing Then
                  command.Parameters.AddWithValue("@unitPrice", unitPrice)
              Else
                  command.Parameters.AddWithValue("@unitPrice", System.DBNull.Value)
              End If

              If Not quantity Is Nothing Then
                  command.Parameters.AddWithValue("@quantity", quantity)
              Else
                  command.Parameters.AddWithValue("@quantity", System.DBNull.Value)
              End If

              If Not discount Is Nothing Then
                  command.Parameters.AddWithValue("@discount", discount)
              Else
                  command.Parameters.AddWithValue("@discount", System.DBNull.Value)
              End If

         End Sub

         ''' <summary>
         ''' Creates a OrderDetails object from the passed data row
         ''' </summary>
         Private Shared Function CreateOrderDetailsFromDataRowShared(dr As DataRow) As OrderDetails
             Dim objOrderDetails As OrderDetails = New OrderDetails()


             If Not DbNull.Value.Equals(dr("OrderID")) Then
                 objOrderDetails.OrderID = DirectCast(dr("OrderID"), Integer)
             Else
                 objOrderDetails.OrderID = Nothing
             End If


             If Not DbNull.Value.Equals(dr("ProductID")) Then
                 objOrderDetails.ProductID = DirectCast(dr("ProductID"), Integer)
             Else
                 objOrderDetails.ProductID = Nothing
             End If


             If Not DbNull.Value.Equals(dr("UnitPrice")) Then
                 objOrderDetails.UnitPrice = DirectCast(dr("UnitPrice"), Decimal)
             Else
                 objOrderDetails.UnitPrice = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Quantity")) Then
                 objOrderDetails.Quantity = DirectCast(dr("Quantity"), Short)
             Else
                 objOrderDetails.Quantity = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Discount")) Then
                 objOrderDetails.Discount = DirectCast(dr("Discount"), Single)
             Else
                 objOrderDetails.Discount = Nothing
             End If

             Return objOrderDetails
         End Function
     End Class
End Namespace
