Imports System
Imports System.Data
Imports System.Data.SqlClient
Imports System.Configuration
Imports NorthwindWinFormDynamic_VB.BusinessObject

Namespace DataLayer.Base
     ''' <summary>
     ''' Base class for OrdersDataLayer.  Do not make changes to this class,
     ''' instead, put additional code in the OrdersDataLayer class
     ''' </summary>
     Public Class OrdersDataLayerBase

         ' constructor
         Public Sub New()
         End Sub

         ''' <summary>
         ''' Selects a record by primary key(s)
         ''' </summary>
         Public Shared Function SelectByPrimaryKey(ByVal orderID As Integer) As Orders
              Dim objOrders As Orders = Nothing
              Dim dynamicSqlScript As String = OrdersSql.SelectByPrimaryKey()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@orderID", orderID)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objOrders = CreateOrdersFromDataRowShared(dt.Rows(0))
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objOrders
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Orders table
         ''' </summary>
         Public Shared Function GetRecordCount() As Integer
             Dim sql As String = OrdersSql.GetRecordCount()
             Return GetRecordCountShared(Nothing, Nothing, Nothing, False, sql)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Orders table by CustomerID
         ''' </summary>
         Public Shared Function GetRecordCountByCustomerID(customerID As String) As Integer
             Dim sql As String = OrdersSql.GetRecordCountByCustomerID(customerID)
             Return GetRecordCountShared(Nothing, "customerID", customerID, False, sql)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Orders table by EmployeeID
         ''' </summary>
         Public Shared Function GetRecordCountByEmployeeID(employeeID As Integer) As Integer
             Dim sql As String = OrdersSql.GetRecordCountByEmployeeID(employeeID)
             Return GetRecordCountShared(Nothing, "employeeID", employeeID, False, sql)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Orders table by ShipVia
         ''' </summary>
         Public Shared Function GetRecordCountByShipVia(shipVia As Integer) As Integer
             Dim sql As String = OrdersSql.GetRecordCountByShipVia(shipVia)
             Return GetRecordCountShared(Nothing, "shipVia", shipVia, False, sql)
         End Function

         Public Shared Function GetRecordCountShared(Optional ByVal storedProcName As String = Nothing, Optional ByVal param As String = Nothing, Optional ByVal paramValue As Object = Nothing, Optional ByVal isUseStoredProc As Boolean = True, Optional ByVal dynamicSqlScript As String = Nothing) As Integer
              Dim recordCount As Integer = 0

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      Select Case param
                          Case "customerID"
                              command.Parameters.AddWithValue("@customerID", paramValue)
                          Case "employeeID"
                              command.Parameters.AddWithValue("@employeeID", paramValue)
                          Case "shipVia"
                              command.Parameters.AddWithValue("@shipVia", paramValue)
                          Case Else
                              Exit Select
                      End Select

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  recordCount = DirectCast(dt.Rows(0)("RecordCount"), Integer)
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return recordCount
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Orders table based on search parameters
         ''' </summary>
         Public Shared Function GetRecordCountDynamicWhere(orderID As Integer?, customerID As String, employeeID As Integer?, orderDate As Date?, requiredDate As Date?, shippedDate As Date?, shipVia As Integer?, freight As Decimal?, shipName As String, shipAddress As String, shipCity As String, shipRegion As String, shipPostalCode As String, shipCountry As String) As Integer
              Dim recordCount As Integer = 0
              Dim dynamicSqlScript As String = OrdersSql.GetRecordCountDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' search parameters
                      AddSearchCommandParamsShared(command, orderID, customerID, employeeID, orderDate, requiredDate, shippedDate, shipVia, freight, shipName, shipAddress, shipCity, shipRegion, shipPostalCode, shipCountry)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  recordCount = DirectCast(dt.Rows(0)("RecordCount"), Integer)
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return recordCount
         End Function

         ''' <summary>
         ''' Selects Orders records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of records)
         ''' </summary>
         Public Shared Function SelectSkipAndTake(sortByExpression As String, startRowIndex As Integer, rows As Integer) As OrdersCollection
             Dim dynamicSqlScript As String = OrdersSql.SelectSkipAndTake()
             Return SelectShared(dynamicSqlScript, Nothing, Nothing, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records by CustomerID as a collection (List) of Orders sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared Function SelectSkipAndTakeByCustomerID(sortByExpression As String, startRowIndex As Integer, rows As Integer, customerID As String) As OrdersCollection
             Dim dynamicSqlScript As String = OrdersSql.SelectSkipAndTakeByCustomerID(customerID)
             Return SelectShared(dynamicSqlScript, "customerID", customerID, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records by EmployeeID as a collection (List) of Orders sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared Function SelectSkipAndTakeByEmployeeID(sortByExpression As String, startRowIndex As Integer, rows As Integer, employeeID As Integer) As OrdersCollection
             Dim dynamicSqlScript As String = OrdersSql.SelectSkipAndTakeByEmployeeID(employeeID)
             Return SelectShared(dynamicSqlScript, "employeeID", employeeID, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records by ShipVia as a collection (List) of Orders sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared Function SelectSkipAndTakeByShipVia(sortByExpression As String, startRowIndex As Integer, rows As Integer, shipVia As Integer) As OrdersCollection
             Dim dynamicSqlScript As String = OrdersSql.SelectSkipAndTakeByShipVia(shipVia)
             Return SelectShared(dynamicSqlScript, "shipVia", shipVia, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects Orders records sorted by the sortByExpression and returns records between the start and end based on search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(orderID As Integer?, customerID As String, employeeID As Integer?, orderDate As Date?, requiredDate As Date?, shippedDate As Date?, shipVia As Integer?, freight As Decimal?, shipName As String, shipAddress As String, shipCity As String, shipRegion As String, shipPostalCode As String, shipCountry As String, sortByExpression As String, startRowIndex As Integer, rows  As Integer) As OrdersCollection
              Dim  objOrdersCol As OrdersCollection = Nothing
              Dim dynamicSqlScript As String = OrdersSql.SelectSkipAndTakeDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' select, skip, take, sort parameters
                      command.Parameters.AddWithValue("@start", startRowIndex)
                      command.Parameters.AddWithValue("@numberOfRows", rows)
                      command.Parameters.AddWithValue("@sortByExpression", sortByExpression)

                      ' search parameters
                      AddSearchCommandParamsShared(command, orderID, customerID, employeeID, orderDate, requiredDate, shippedDate, shipVia, freight, shipName, shipAddress, shipCity, shipRegion, shipPostalCode, shipCountry)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objOrdersCol = New OrdersCollection()

                                  For Each dr As DataRow in dt.Rows
                                      Dim objOrders As Orders = CreateOrdersFromDataRowShared(dr)
                                      objOrdersCol.Add(objOrders)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objOrdersCol
         End Function

         ''' <summary>
         ''' Gets the grand total or sum of fields with a money of decimal data type
         ''' </summary>
         Public Shared Function SelectTotals() As Orders
              Dim objOrders As Orders = Nothing
              Dim dynamicSqlScript As String = OrdersSql.SelectTotals()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  If Not DbNull.Value.Equals(dt.Rows(0)("FreightTotal")) Then
                                       objOrders.FreightTotal = DirectCast(dt.Rows(0)("FreightTotal"), Decimal)
                                  End If
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objOrders
         End Function

         ''' <summary>
         ''' Selects all Orders
         ''' </summary>
         Public Shared Function SelectAll() As OrdersCollection
             Dim dynamicSqlScript As String = OrdersSql.SelectAll()
             Return SelectShared(dynamicSqlScript, String.Empty, Nothing)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Orders.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(orderID As Integer?, customerID As String, employeeID As Integer?, orderDate As Date?, requiredDate As Date?, shippedDate As Date?, shipVia As Integer?, freight As Decimal?, shipName As String, shipAddress As String, shipCity As String, shipRegion As String, shipPostalCode As String, shipCountry As String) As OrdersCollection
              Dim objOrdersCol As OrdersCollection = Nothing
              Dim dynamicSqlScript As String = OrdersSql.SelectAllDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' search parameters
                      AddSearchCommandParamsShared(command, orderID, customerID, employeeID, orderDate, requiredDate, shippedDate, shipVia, freight, shipName, shipAddress, shipCity, shipRegion, shipPostalCode, shipCountry)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objOrdersCol = New OrdersCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objOrders As Orders = CreateOrdersFromDataRowShared(dr)
                                      objOrdersCol.Add(objOrders)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objOrdersCol
         End Function

         ''' <summary>
         ''' Selects all Orders by Customers, related to column CustomerID
         ''' </summary>
         Public Shared Function SelectOrdersCollectionByCustomerID(customerID As String) As OrdersCollection
             Dim dynamicSqlScript As String = OrdersSql.SelectAllByCustomerID(customerID)
             Return SelectShared(dynamicSqlScript, "customerID", customerID)
         End Function

         ''' <summary>
         ''' Selects all Orders by Employees, related to column EmployeeID
         ''' </summary>
         Public Shared Function SelectOrdersCollectionByEmployeeID(employeeID As Integer) As OrdersCollection
             Dim dynamicSqlScript As String = OrdersSql.SelectAllByEmployeeID(employeeID)
             Return SelectShared(dynamicSqlScript, "employeeID", employeeID)
         End Function

         ''' <summary>
         ''' Selects all Orders by Shippers, related to column ShipVia
         ''' </summary>
         Public Shared Function SelectOrdersCollectionByShipVia(shipperID As Integer) As OrdersCollection
             Dim dynamicSqlScript As String = OrdersSql.SelectAllByShipVia(shipperID)
             Return SelectShared(dynamicSqlScript, "shipperID", shipperID)
         End Function

         ''' <summary>
         ''' Selects OrderID and ShipName columns for use with a DropDownList web control
         ''' </summary>
         Public Shared Function SelectOrdersDropDownListData() As OrdersCollection
              Dim objOrdersCol As OrdersCollection = Nothing
              Dim dynamicSqlScript As String = OrdersSql.SelectOrdersDropDownListData()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objOrdersCol = New OrdersCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objOrders As Orders  = New Orders()
                                      objOrders.OrderID = DirectCast(dr("OrderID"), Integer)

                                      If Not dr("ShipName").Equals(System.DBNull.Value) Then
                                          objOrders.ShipName = DirectCast(dr("ShipName"), String)
                                      Else
                                          objOrders.ShipName = Nothing
                                      End If

                                      objOrdersCol.Add(objOrders)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objOrdersCol
         End Function

         Public Shared Function SelectShared(dynamicSqlScript As String, param As String, paramValue As Object, Optional ByVal sortByExpression As String = Nothing, Optional ByVal startRowIndex As Integer? = Nothing, Optional ByVal rows As Integer? = Nothing) As OrdersCollection
              Dim objOrdersCol As OrdersCollection = Nothing

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' select, skip, take, sort parameters
                      If Not [String].IsNullOrEmpty(sortByExpression) AndAlso startRowIndex IsNot Nothing AndAlso rows IsNot Nothing Then
                          command.Parameters.AddWithValue("@start", startRowIndex.Value)
                          command.Parameters.AddWithValue("@numberOfRows", rows.Value)
                          command.Parameters.AddWithValue("@sortByExpression", sortByExpression)
                      End If

                      ' parameters
                      Select Case param
                          Case "customerID"
                              command.Parameters.AddWithValue("@customerID", paramValue)
                              Exit Select
                          Case "employeeID"
                              command.Parameters.AddWithValue("@employeeID", paramValue)
                              Exit Select
                          Case "shipVia"
                              command.Parameters.AddWithValue("@shipVia", paramValue)
                              Exit Select
                          Case Else
                              Exit Select
                      End Select

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objOrdersCol = New OrdersCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objOrders As Orders = CreateOrdersFromDataRowShared(dr)
                                      objOrdersCol.Add(objOrders)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objOrdersCol
         End Function

         ''' <summary>
         ''' Inserts a record
         ''' </summary>
         Public Shared Function Insert(objOrders As Orders) As Integer
             Return InsertUpdate(objOrders, False)
         End Function

         ''' <summary>
         ''' Updates a record
         ''' </summary>
         Public Shared Sub Update(objOrders As Orders)
             InsertUpdate(objOrders, True)
         End Sub

         Private Shared Function InsertUpdate(objOrders As Orders, isUpdate As Boolean) As Integer
              Dim newlyCreatedOrderID As Integer = objOrders.OrderID

              Dim dynamicSqlScript As String = String.Empty

              If isUpdate Then
                  dynamicSqlScript = OrdersSql.Update()
              Else
                  dynamicSqlScript = OrdersSql.Insert()
              End If
              Dim customerID As Object = objOrders.CustomerID
              Dim employeeID As Object = objOrders.EmployeeID
              Dim orderDate As Object = objOrders.OrderDate
              Dim requiredDate As Object = objOrders.RequiredDate
              Dim shippedDate As Object = objOrders.ShippedDate
              Dim shipVia As Object = objOrders.ShipVia
              Dim freight As Object = objOrders.Freight
              Dim shipName As Object = objOrders.ShipName
              Dim shipAddress As Object = objOrders.ShipAddress
              Dim shipCity As Object = objOrders.ShipCity
              Dim shipRegion As Object = objOrders.ShipRegion
              Dim shipPostalCode As Object = objOrders.ShipPostalCode
              Dim shipCountry As Object = objOrders.ShipCountry

              If [String].IsNullOrEmpty(objOrders.CustomerID) Then
                  customerID = System.DBNull.Value
              End If

              If objOrders.EmployeeID Is Nothing Then
                  employeeID = System.DBNull.Value
              End If

              If objOrders.OrderDate Is Nothing Then
                  orderDate = System.DBNull.Value
              End If

              If objOrders.RequiredDate Is Nothing Then
                  requiredDate = System.DBNull.Value
              End If

              If objOrders.ShippedDate Is Nothing Then
                  shippedDate = System.DBNull.Value
              End If

              If objOrders.ShipVia Is Nothing Then
                  shipVia = System.DBNull.Value
              End If

              If objOrders.Freight Is Nothing Then
                  freight = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objOrders.ShipName) Then
                  shipName = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objOrders.ShipAddress) Then
                  shipAddress = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objOrders.ShipCity) Then
                  shipCity = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objOrders.ShipRegion) Then
                  shipRegion = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objOrders.ShipPostalCode) Then
                  shipPostalCode = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objOrders.ShipCountry) Then
                  shipCountry = System.DBNull.Value
              End If

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      If isUpdate Then
                          ' for update only
                          command.Parameters.AddWithValue("@orderID", objOrders.OrderID)
                      End If

                      command.Parameters.AddWithValue("@customerID", customerID)
                      command.Parameters.AddWithValue("@employeeID", employeeID)
                      command.Parameters.AddWithValue("@orderDate", orderDate)
                      command.Parameters.AddWithValue("@requiredDate", requiredDate)
                      command.Parameters.AddWithValue("@shippedDate", shippedDate)
                      command.Parameters.AddWithValue("@shipVia", shipVia)
                      command.Parameters.AddWithValue("@freight", freight)
                      command.Parameters.AddWithValue("@shipName", shipName)
                      command.Parameters.AddWithValue("@shipAddress", shipAddress)
                      command.Parameters.AddWithValue("@shipCity", shipCity)
                      command.Parameters.AddWithValue("@shipRegion", shipRegion)
                      command.Parameters.AddWithValue("@shipPostalCode", shipPostalCode)
                      command.Parameters.AddWithValue("@shipCountry", shipCountry)

                      If isUpdate Then
                          command.ExecuteNonQuery()
                      Else
                          newlyCreatedOrderID = DirectCast(command.ExecuteScalar(), Integer)
                      End If
                  End Using
              End Using

              Return newlyCreatedOrderID
         End Function

         ''' <summary>
         ''' Deletes a record based on primary key(s)
         ''' </summary>
         Public Shared Sub Delete(ByVal orderID As Integer)
              Dim dynamicSqlScript As String = OrdersSql.Delete()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@orderID", orderID)

                      ' execute
                      command.ExecuteNonQuery()
                  End Using
              End Using
         End Sub

         ''' <summary>
         ''' Adds search parameters to the Command object
         ''' </summary>
         Private Shared Sub AddSearchCommandParamsShared(command As SqlCommand, orderID As Integer?, customerID As String, employeeID As Integer?, orderDate As Date?, requiredDate As Date?, shippedDate As Date?, shipVia As Integer?, freight As Decimal?, shipName As String, shipAddress As String, shipCity As String, shipRegion As String, shipPostalCode As String, shipCountry As String)
              If Not orderID Is Nothing Then
                  command.Parameters.AddWithValue("@orderID", orderID)
              Else
                  command.Parameters.AddWithValue("@orderID", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(customerID) Then
                  command.Parameters.AddWithValue("@customerID", customerID)
              Else
                  command.Parameters.AddWithValue("@customerID", System.DBNull.Value)
              End If

              If Not employeeID Is Nothing Then
                  command.Parameters.AddWithValue("@employeeID", employeeID)
              Else
                  command.Parameters.AddWithValue("@employeeID", System.DBNull.Value)
              End If

              If Not orderDate Is Nothing Then
                  command.Parameters.AddWithValue("@orderDate", orderDate)
              Else
                  command.Parameters.AddWithValue("@orderDate", System.DBNull.Value)
              End If

              If Not requiredDate Is Nothing Then
                  command.Parameters.AddWithValue("@requiredDate", requiredDate)
              Else
                  command.Parameters.AddWithValue("@requiredDate", System.DBNull.Value)
              End If

              If Not shippedDate Is Nothing Then
                  command.Parameters.AddWithValue("@shippedDate", shippedDate)
              Else
                  command.Parameters.AddWithValue("@shippedDate", System.DBNull.Value)
              End If

              If Not shipVia Is Nothing Then
                  command.Parameters.AddWithValue("@shipVia", shipVia)
              Else
                  command.Parameters.AddWithValue("@shipVia", System.DBNull.Value)
              End If

              If Not freight Is Nothing Then
                  command.Parameters.AddWithValue("@freight", freight)
              Else
                  command.Parameters.AddWithValue("@freight", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(shipName) Then
                  command.Parameters.AddWithValue("@shipName", shipName)
              Else
                  command.Parameters.AddWithValue("@shipName", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(shipAddress) Then
                  command.Parameters.AddWithValue("@shipAddress", shipAddress)
              Else
                  command.Parameters.AddWithValue("@shipAddress", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(shipCity) Then
                  command.Parameters.AddWithValue("@shipCity", shipCity)
              Else
                  command.Parameters.AddWithValue("@shipCity", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(shipRegion) Then
                  command.Parameters.AddWithValue("@shipRegion", shipRegion)
              Else
                  command.Parameters.AddWithValue("@shipRegion", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(shipPostalCode) Then
                  command.Parameters.AddWithValue("@shipPostalCode", shipPostalCode)
              Else
                  command.Parameters.AddWithValue("@shipPostalCode", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(shipCountry) Then
                  command.Parameters.AddWithValue("@shipCountry", shipCountry)
              Else
                  command.Parameters.AddWithValue("@shipCountry", System.DBNull.Value)
              End If

         End Sub

         ''' <summary>
         ''' Creates a Orders object from the passed data row
         ''' </summary>
         Private Shared Function CreateOrdersFromDataRowShared(dr As DataRow) As Orders
             Dim objOrders As Orders = New Orders()


             If Not DbNull.Value.Equals(dr("OrderID")) Then
                 objOrders.OrderID = DirectCast(dr("OrderID"), Integer)
             Else
                 objOrders.OrderID = Nothing
             End If

             If Not DbNull.Value.Equals(dr("CustomerID")) Then
                 objOrders.CustomerID = dr("CustomerID").ToString()
             Else
                 objOrders.CustomerID = Nothing
             End If


             If Not DbNull.Value.Equals(dr("EmployeeID")) Then
                 objOrders.EmployeeID = DirectCast(dr("EmployeeID"), Integer)
             Else
                 objOrders.EmployeeID = Nothing
             End If


             If Not DbNull.Value.Equals(dr("OrderDate")) Then
                 objOrders.OrderDate = DirectCast(dr("OrderDate"), Date)
             Else
                 objOrders.OrderDate = Nothing
             End If

             If Not DbNull.Value.Equals(dr("RequiredDate")) Then
                 objOrders.RequiredDate = DirectCast(dr("RequiredDate"), Date)
             Else
                 objOrders.RequiredDate = Nothing
             End If

             If Not DbNull.Value.Equals(dr("ShippedDate")) Then
                 objOrders.ShippedDate = DirectCast(dr("ShippedDate"), Date)
             Else
                 objOrders.ShippedDate = Nothing
             End If

             If Not DbNull.Value.Equals(dr("ShipVia")) Then
                 objOrders.ShipVia = DirectCast(dr("ShipVia"), Integer)
             Else
                 objOrders.ShipVia = Nothing
             End If


             If Not DbNull.Value.Equals(dr("Freight")) Then
                 objOrders.Freight = DirectCast(dr("Freight"), Decimal)
             Else
                 objOrders.Freight = Nothing
             End If

             If Not DbNull.Value.Equals(dr("ShipName")) Then
                 objOrders.ShipName = dr("ShipName").ToString()
             Else
                 objOrders.ShipName = Nothing
             End If

             If Not DbNull.Value.Equals(dr("ShipAddress")) Then
                 objOrders.ShipAddress = dr("ShipAddress").ToString()
             Else
                 objOrders.ShipAddress = Nothing
             End If

             If Not DbNull.Value.Equals(dr("ShipCity")) Then
                 objOrders.ShipCity = dr("ShipCity").ToString()
             Else
                 objOrders.ShipCity = Nothing
             End If

             If Not DbNull.Value.Equals(dr("ShipRegion")) Then
                 objOrders.ShipRegion = dr("ShipRegion").ToString()
             Else
                 objOrders.ShipRegion = Nothing
             End If

             If Not DbNull.Value.Equals(dr("ShipPostalCode")) Then
                 objOrders.ShipPostalCode = dr("ShipPostalCode").ToString()
             Else
                 objOrders.ShipPostalCode = Nothing
             End If

             If Not DbNull.Value.Equals(dr("ShipCountry")) Then
                 objOrders.ShipCountry = dr("ShipCountry").ToString()
             Else
                 objOrders.ShipCountry = Nothing
             End If

             Return objOrders
         End Function
     End Class
End Namespace
