Imports System
Imports System.Data
Imports System.Data.SqlClient
Imports System.Configuration
Imports NorthwindWinFormDynamic_VB.BusinessObject

Namespace DataLayer.Base
     ''' <summary>
     ''' Base class for ProductsDataLayer.  Do not make changes to this class,
     ''' instead, put additional code in the ProductsDataLayer class
     ''' </summary>
     Public Class ProductsDataLayerBase

         ' constructor
         Public Sub New()
         End Sub

         ''' <summary>
         ''' Selects a record by primary key(s)
         ''' </summary>
         Public Shared Function SelectByPrimaryKey(ByVal productID As Integer) As Products
              Dim objProducts As Products = Nothing
              Dim dynamicSqlScript As String = ProductsSql.SelectByPrimaryKey()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@productID", productID)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objProducts = CreateProductsFromDataRowShared(dt.Rows(0))
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objProducts
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Products table
         ''' </summary>
         Public Shared Function GetRecordCount() As Integer
             Dim sql As String = ProductsSql.GetRecordCount()
             Return GetRecordCountShared(Nothing, Nothing, Nothing, False, sql)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Products table by SupplierID
         ''' </summary>
         Public Shared Function GetRecordCountBySupplierID(supplierID As Integer) As Integer
             Dim sql As String = ProductsSql.GetRecordCountBySupplierID(supplierID)
             Return GetRecordCountShared(Nothing, "supplierID", supplierID, False, sql)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Products table by CategoryID
         ''' </summary>
         Public Shared Function GetRecordCountByCategoryID(categoryID As Integer) As Integer
             Dim sql As String = ProductsSql.GetRecordCountByCategoryID(categoryID)
             Return GetRecordCountShared(Nothing, "categoryID", categoryID, False, sql)
         End Function

         Public Shared Function GetRecordCountShared(Optional ByVal storedProcName As String = Nothing, Optional ByVal param As String = Nothing, Optional ByVal paramValue As Object = Nothing, Optional ByVal isUseStoredProc As Boolean = True, Optional ByVal dynamicSqlScript As String = Nothing) As Integer
              Dim recordCount As Integer = 0

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      Select Case param
                          Case "supplierID"
                              command.Parameters.AddWithValue("@supplierID", paramValue)
                          Case "categoryID"
                              command.Parameters.AddWithValue("@categoryID", paramValue)
                          Case Else
                              Exit Select
                      End Select

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  recordCount = DirectCast(dt.Rows(0)("RecordCount"), Integer)
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return recordCount
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Products table based on search parameters
         ''' </summary>
         Public Shared Function GetRecordCountDynamicWhere(productID As Integer?, productName As String, supplierID As Integer?, categoryID As Integer?, quantityPerUnit As String, unitPrice As Decimal?, unitsInStock As Short?, unitsOnOrder As Short?, reorderLevel As Short?, discontinued As Boolean?) As Integer
              Dim recordCount As Integer = 0
              Dim dynamicSqlScript As String = ProductsSql.GetRecordCountDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' search parameters
                      AddSearchCommandParamsShared(command, productID, productName, supplierID, categoryID, quantityPerUnit, unitPrice, unitsInStock, unitsOnOrder, reorderLevel, discontinued)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  recordCount = DirectCast(dt.Rows(0)("RecordCount"), Integer)
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return recordCount
         End Function

         ''' <summary>
         ''' Selects Products records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of records)
         ''' </summary>
         Public Shared Function SelectSkipAndTake(sortByExpression As String, startRowIndex As Integer, rows As Integer) As ProductsCollection
             Dim dynamicSqlScript As String = ProductsSql.SelectSkipAndTake()
             Return SelectShared(dynamicSqlScript, Nothing, Nothing, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records by SupplierID as a collection (List) of Products sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared Function SelectSkipAndTakeBySupplierID(sortByExpression As String, startRowIndex As Integer, rows As Integer, supplierID As Integer) As ProductsCollection
             Dim dynamicSqlScript As String = ProductsSql.SelectSkipAndTakeBySupplierID(supplierID)
             Return SelectShared(dynamicSqlScript, "supplierID", supplierID, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records by CategoryID as a collection (List) of Products sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared Function SelectSkipAndTakeByCategoryID(sortByExpression As String, startRowIndex As Integer, rows As Integer, categoryID As Integer) As ProductsCollection
             Dim dynamicSqlScript As String = ProductsSql.SelectSkipAndTakeByCategoryID(categoryID)
             Return SelectShared(dynamicSqlScript, "categoryID", categoryID, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects Products records sorted by the sortByExpression and returns records between the start and end based on search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(productID As Integer?, productName As String, supplierID As Integer?, categoryID As Integer?, quantityPerUnit As String, unitPrice As Decimal?, unitsInStock As Short?, unitsOnOrder As Short?, reorderLevel As Short?, discontinued As Boolean?, sortByExpression As String, startRowIndex As Integer, rows  As Integer) As ProductsCollection
              Dim  objProductsCol As ProductsCollection = Nothing
              Dim dynamicSqlScript As String = ProductsSql.SelectSkipAndTakeDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' select, skip, take, sort parameters
                      command.Parameters.AddWithValue("@start", startRowIndex)
                      command.Parameters.AddWithValue("@numberOfRows", rows)
                      command.Parameters.AddWithValue("@sortByExpression", sortByExpression)

                      ' search parameters
                      AddSearchCommandParamsShared(command, productID, productName, supplierID, categoryID, quantityPerUnit, unitPrice, unitsInStock, unitsOnOrder, reorderLevel, discontinued)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objProductsCol = New ProductsCollection()

                                  For Each dr As DataRow in dt.Rows
                                      Dim objProducts As Products = CreateProductsFromDataRowShared(dr)
                                      objProductsCol.Add(objProducts)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objProductsCol
         End Function

         ''' <summary>
         ''' Gets the grand total or sum of fields with a money of decimal data type
         ''' </summary>
         Public Shared Function SelectTotals() As Products
              Dim objProducts As Products = Nothing
              Dim dynamicSqlScript As String = ProductsSql.SelectTotals()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  If Not DbNull.Value.Equals(dt.Rows(0)("UnitPriceTotal")) Then
                                       objProducts.UnitPriceTotal = DirectCast(dt.Rows(0)("UnitPriceTotal"), Decimal)
                                  End If
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objProducts
         End Function

         ''' <summary>
         ''' Selects all Products
         ''' </summary>
         Public Shared Function SelectAll() As ProductsCollection
             Dim dynamicSqlScript As String = ProductsSql.SelectAll()
             Return SelectShared(dynamicSqlScript, String.Empty, Nothing)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Products.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(productID As Integer?, productName As String, supplierID As Integer?, categoryID As Integer?, quantityPerUnit As String, unitPrice As Decimal?, unitsInStock As Short?, unitsOnOrder As Short?, reorderLevel As Short?, discontinued As Boolean?) As ProductsCollection
              Dim objProductsCol As ProductsCollection = Nothing
              Dim dynamicSqlScript As String = ProductsSql.SelectAllDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' search parameters
                      AddSearchCommandParamsShared(command, productID, productName, supplierID, categoryID, quantityPerUnit, unitPrice, unitsInStock, unitsOnOrder, reorderLevel, discontinued)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objProductsCol = New ProductsCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objProducts As Products = CreateProductsFromDataRowShared(dr)
                                      objProductsCol.Add(objProducts)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objProductsCol
         End Function

         ''' <summary>
         ''' Selects all Products by Suppliers, related to column SupplierID
         ''' </summary>
         Public Shared Function SelectProductsCollectionBySupplierID(supplierID As Integer) As ProductsCollection
             Dim dynamicSqlScript As String = ProductsSql.SelectAllBySupplierID(supplierID)
             Return SelectShared(dynamicSqlScript, "supplierID", supplierID)
         End Function

         ''' <summary>
         ''' Selects all Products by Categories, related to column CategoryID
         ''' </summary>
         Public Shared Function SelectProductsCollectionByCategoryID(categoryID As Integer) As ProductsCollection
             Dim dynamicSqlScript As String = ProductsSql.SelectAllByCategoryID(categoryID)
             Return SelectShared(dynamicSqlScript, "categoryID", categoryID)
         End Function

         ''' <summary>
         ''' Selects ProductID and ProductName columns for use with a DropDownList web control
         ''' </summary>
         Public Shared Function SelectProductsDropDownListData() As ProductsCollection
              Dim objProductsCol As ProductsCollection = Nothing
              Dim dynamicSqlScript As String = ProductsSql.SelectProductsDropDownListData()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objProductsCol = New ProductsCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objProducts As Products  = New Products()
                                      objProducts.ProductID = DirectCast(dr("ProductID"), Integer)
                                      objProducts.ProductName = DirectCast(dr("ProductName"), String)

                                      objProductsCol.Add(objProducts)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objProductsCol
         End Function

         Public Shared Function SelectShared(dynamicSqlScript As String, param As String, paramValue As Object, Optional ByVal sortByExpression As String = Nothing, Optional ByVal startRowIndex As Integer? = Nothing, Optional ByVal rows As Integer? = Nothing) As ProductsCollection
              Dim objProductsCol As ProductsCollection = Nothing

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' select, skip, take, sort parameters
                      If Not [String].IsNullOrEmpty(sortByExpression) AndAlso startRowIndex IsNot Nothing AndAlso rows IsNot Nothing Then
                          command.Parameters.AddWithValue("@start", startRowIndex.Value)
                          command.Parameters.AddWithValue("@numberOfRows", rows.Value)
                          command.Parameters.AddWithValue("@sortByExpression", sortByExpression)
                      End If

                      ' parameters
                      Select Case param
                          Case "supplierID"
                              command.Parameters.AddWithValue("@supplierID", paramValue)
                              Exit Select
                          Case "categoryID"
                              command.Parameters.AddWithValue("@categoryID", paramValue)
                              Exit Select
                          Case Else
                              Exit Select
                      End Select

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objProductsCol = New ProductsCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objProducts As Products = CreateProductsFromDataRowShared(dr)
                                      objProductsCol.Add(objProducts)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objProductsCol
         End Function

         ''' <summary>
         ''' Inserts a record
         ''' </summary>
         Public Shared Function Insert(objProducts As Products) As Integer
             Return InsertUpdate(objProducts, False)
         End Function

         ''' <summary>
         ''' Updates a record
         ''' </summary>
         Public Shared Sub Update(objProducts As Products)
             InsertUpdate(objProducts, True)
         End Sub

         Private Shared Function InsertUpdate(objProducts As Products, isUpdate As Boolean) As Integer
              Dim newlyCreatedProductID As Integer = objProducts.ProductID

              Dim dynamicSqlScript As String = String.Empty

              If isUpdate Then
                  dynamicSqlScript = ProductsSql.Update()
              Else
                  dynamicSqlScript = ProductsSql.Insert()
              End If
              Dim supplierID As Object = objProducts.SupplierID
              Dim categoryID As Object = objProducts.CategoryID
              Dim quantityPerUnit As Object = objProducts.QuantityPerUnit
              Dim unitPrice As Object = objProducts.UnitPrice
              Dim unitsInStock As Object = objProducts.UnitsInStock
              Dim unitsOnOrder As Object = objProducts.UnitsOnOrder
              Dim reorderLevel As Object = objProducts.ReorderLevel

              If objProducts.SupplierID Is Nothing Then
                  supplierID = System.DBNull.Value
              End If

              If objProducts.CategoryID Is Nothing Then
                  categoryID = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objProducts.QuantityPerUnit) Then
                  quantityPerUnit = System.DBNull.Value
              End If

              If objProducts.UnitPrice Is Nothing Then
                  unitPrice = System.DBNull.Value
              End If

              If objProducts.UnitsInStock Is Nothing Then
                  unitsInStock = System.DBNull.Value
              End If

              If objProducts.UnitsOnOrder Is Nothing Then
                  unitsOnOrder = System.DBNull.Value
              End If

              If objProducts.ReorderLevel Is Nothing Then
                  reorderLevel = System.DBNull.Value
              End If

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      If isUpdate Then
                          ' for update only
                          command.Parameters.AddWithValue("@productID", objProducts.ProductID)
                      End If

                      command.Parameters.AddWithValue("@productName", objProducts.ProductName)
                      command.Parameters.AddWithValue("@supplierID", supplierID)
                      command.Parameters.AddWithValue("@categoryID", categoryID)
                      command.Parameters.AddWithValue("@quantityPerUnit", quantityPerUnit)
                      command.Parameters.AddWithValue("@unitPrice", unitPrice)
                      command.Parameters.AddWithValue("@unitsInStock", unitsInStock)
                      command.Parameters.AddWithValue("@unitsOnOrder", unitsOnOrder)
                      command.Parameters.AddWithValue("@reorderLevel", reorderLevel)
                      command.Parameters.AddWithValue("@discontinued", objProducts.Discontinued)

                      If isUpdate Then
                          command.ExecuteNonQuery()
                      Else
                          newlyCreatedProductID = DirectCast(command.ExecuteScalar(), Integer)
                      End If
                  End Using
              End Using

              Return newlyCreatedProductID
         End Function

         ''' <summary>
         ''' Deletes a record based on primary key(s)
         ''' </summary>
         Public Shared Sub Delete(ByVal productID As Integer)
              Dim dynamicSqlScript As String = ProductsSql.Delete()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@productID", productID)

                      ' execute
                      command.ExecuteNonQuery()
                  End Using
              End Using
         End Sub

         ''' <summary>
         ''' Adds search parameters to the Command object
         ''' </summary>
         Private Shared Sub AddSearchCommandParamsShared(command As SqlCommand, productID As Integer?, productName As String, supplierID As Integer?, categoryID As Integer?, quantityPerUnit As String, unitPrice As Decimal?, unitsInStock As Short?, unitsOnOrder As Short?, reorderLevel As Short?, discontinued As Boolean?)
              If Not productID Is Nothing Then
                  command.Parameters.AddWithValue("@productID", productID)
              Else
                  command.Parameters.AddWithValue("@productID", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(productName) Then
                  command.Parameters.AddWithValue("@productName", productName)
              Else
                  command.Parameters.AddWithValue("@productName", System.DBNull.Value)
              End If

              If Not supplierID Is Nothing Then
                  command.Parameters.AddWithValue("@supplierID", supplierID)
              Else
                  command.Parameters.AddWithValue("@supplierID", System.DBNull.Value)
              End If

              If Not categoryID Is Nothing Then
                  command.Parameters.AddWithValue("@categoryID", categoryID)
              Else
                  command.Parameters.AddWithValue("@categoryID", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(quantityPerUnit) Then
                  command.Parameters.AddWithValue("@quantityPerUnit", quantityPerUnit)
              Else
                  command.Parameters.AddWithValue("@quantityPerUnit", System.DBNull.Value)
              End If

              If Not unitPrice Is Nothing Then
                  command.Parameters.AddWithValue("@unitPrice", unitPrice)
              Else
                  command.Parameters.AddWithValue("@unitPrice", System.DBNull.Value)
              End If

              If Not unitsInStock Is Nothing Then
                  command.Parameters.AddWithValue("@unitsInStock", unitsInStock)
              Else
                  command.Parameters.AddWithValue("@unitsInStock", System.DBNull.Value)
              End If

              If Not unitsOnOrder Is Nothing Then
                  command.Parameters.AddWithValue("@unitsOnOrder", unitsOnOrder)
              Else
                  command.Parameters.AddWithValue("@unitsOnOrder", System.DBNull.Value)
              End If

              If Not reorderLevel Is Nothing Then
                  command.Parameters.AddWithValue("@reorderLevel", reorderLevel)
              Else
                  command.Parameters.AddWithValue("@reorderLevel", System.DBNull.Value)
              End If

              If Not discontinued Is Nothing Then
                  command.Parameters.AddWithValue("@discontinued", discontinued)
              Else
                  command.Parameters.AddWithValue("@discontinued", System.DBNull.Value)
              End If

         End Sub

         ''' <summary>
         ''' Creates a Products object from the passed data row
         ''' </summary>
         Private Shared Function CreateProductsFromDataRowShared(dr As DataRow) As Products
             Dim objProducts As Products = New Products()


             If Not DbNull.Value.Equals(dr("ProductID")) Then
                 objProducts.ProductID = DirectCast(dr("ProductID"), Integer)
             Else
                 objProducts.ProductID = Nothing
             End If

             If Not DbNull.Value.Equals(dr("ProductName")) Then
                 objProducts.ProductName = dr("ProductName").ToString()
             Else
                 objProducts.ProductName = Nothing
             End If

             If Not DbNull.Value.Equals(dr("SupplierID")) Then
                 objProducts.SupplierID = DirectCast(dr("SupplierID"), Integer)
             Else
                 objProducts.SupplierID = Nothing
             End If


             If Not DbNull.Value.Equals(dr("CategoryID")) Then
                 objProducts.CategoryID = DirectCast(dr("CategoryID"), Integer)
             Else
                 objProducts.CategoryID = Nothing
             End If


             If Not DbNull.Value.Equals(dr("QuantityPerUnit")) Then
                 objProducts.QuantityPerUnit = dr("QuantityPerUnit").ToString()
             Else
                 objProducts.QuantityPerUnit = Nothing
             End If

             If Not DbNull.Value.Equals(dr("UnitPrice")) Then
                 objProducts.UnitPrice = DirectCast(dr("UnitPrice"), Decimal)
             Else
                 objProducts.UnitPrice = Nothing
             End If

             If Not DbNull.Value.Equals(dr("UnitsInStock")) Then
                 objProducts.UnitsInStock = DirectCast(dr("UnitsInStock"), Short)
             Else
                 objProducts.UnitsInStock = Nothing
             End If

             If Not DbNull.Value.Equals(dr("UnitsOnOrder")) Then
                 objProducts.UnitsOnOrder = DirectCast(dr("UnitsOnOrder"), Short)
             Else
                 objProducts.UnitsOnOrder = Nothing
             End If

             If Not DbNull.Value.Equals(dr("ReorderLevel")) Then
                 objProducts.ReorderLevel = DirectCast(dr("ReorderLevel"), Short)
             Else
                 objProducts.ReorderLevel = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Discontinued")) Then
                 objProducts.Discontinued = DirectCast(dr("Discontinued"), Boolean)
             Else
                 objProducts.Discontinued = Nothing
             End If

             Return objProducts
         End Function
     End Class
End Namespace
