Imports System
Imports System.Data
Imports System.Data.SqlClient
Imports System.Configuration
Imports NorthwindWinFormDynamic_VB.BusinessObject

Namespace DataLayer.Base
     ''' <summary>
     ''' Base class for RegionDataLayer.  Do not make changes to this class,
     ''' instead, put additional code in the RegionDataLayer class
     ''' </summary>
     Public Class RegionDataLayerBase

         ' constructor
         Public Sub New()
         End Sub

         ''' <summary>
         ''' Selects a record by primary key(s)
         ''' </summary>
         Public Shared Function SelectByPrimaryKey(ByVal regionID As Integer) As Region
              Dim objRegion As Region = Nothing
              Dim dynamicSqlScript As String = RegionSql.SelectByPrimaryKey()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@regionID", regionID)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objRegion = CreateRegionFromDataRowShared(dt.Rows(0))
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objRegion
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Region table
         ''' </summary>
         Public Shared Function GetRecordCount() As Integer
             Dim sql As String = RegionSql.GetRecordCount()
             Return GetRecordCountShared(Nothing, Nothing, Nothing, False, sql)
         End Function

         Public Shared Function GetRecordCountShared(Optional ByVal storedProcName As String = Nothing, Optional ByVal param As String = Nothing, Optional ByVal paramValue As Object = Nothing, Optional ByVal isUseStoredProc As Boolean = True, Optional ByVal dynamicSqlScript As String = Nothing) As Integer
              Dim recordCount As Integer = 0

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  recordCount = DirectCast(dt.Rows(0)("RecordCount"), Integer)
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return recordCount
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Region table based on search parameters
         ''' </summary>
         Public Shared Function GetRecordCountDynamicWhere(regionID As Integer?, regionDescription As String) As Integer
              Dim recordCount As Integer = 0
              Dim dynamicSqlScript As String = RegionSql.GetRecordCountDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' search parameters
                      AddSearchCommandParamsShared(command, regionID, regionDescription)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  recordCount = DirectCast(dt.Rows(0)("RecordCount"), Integer)
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return recordCount
         End Function

         ''' <summary>
         ''' Selects Region records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of records)
         ''' </summary>
         Public Shared Function SelectSkipAndTake(sortByExpression As String, startRowIndex As Integer, rows As Integer) As RegionCollection
             Dim dynamicSqlScript As String = RegionSql.SelectSkipAndTake()
             Return SelectShared(dynamicSqlScript, Nothing, Nothing, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects Region records sorted by the sortByExpression and returns records between the start and end based on search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(regionID As Integer?, regionDescription As String, sortByExpression As String, startRowIndex As Integer, rows  As Integer) As RegionCollection
              Dim  objRegionCol As RegionCollection = Nothing
              Dim dynamicSqlScript As String = RegionSql.SelectSkipAndTakeDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' select, skip, take, sort parameters
                      command.Parameters.AddWithValue("@start", startRowIndex)
                      command.Parameters.AddWithValue("@numberOfRows", rows)
                      command.Parameters.AddWithValue("@sortByExpression", sortByExpression)

                      ' search parameters
                      AddSearchCommandParamsShared(command, regionID, regionDescription)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objRegionCol = New RegionCollection()

                                  For Each dr As DataRow in dt.Rows
                                      Dim objRegion As Region = CreateRegionFromDataRowShared(dr)
                                      objRegionCol.Add(objRegion)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objRegionCol
         End Function

         ''' <summary>
         ''' Selects all Region
         ''' </summary>
         Public Shared Function SelectAll() As RegionCollection
             Dim dynamicSqlScript As String = RegionSql.SelectAll()
             Return SelectShared(dynamicSqlScript, String.Empty, Nothing)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Region.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(regionID As Integer?, regionDescription As String) As RegionCollection
              Dim objRegionCol As RegionCollection = Nothing
              Dim dynamicSqlScript As String = RegionSql.SelectAllDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' search parameters
                      AddSearchCommandParamsShared(command, regionID, regionDescription)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objRegionCol = New RegionCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objRegion As Region = CreateRegionFromDataRowShared(dr)
                                      objRegionCol.Add(objRegion)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objRegionCol
         End Function

         ''' <summary>
         ''' Selects RegionID and RegionDescription columns for use with a DropDownList web control
         ''' </summary>
         Public Shared Function SelectRegionDropDownListData() As RegionCollection
              Dim objRegionCol As RegionCollection = Nothing
              Dim dynamicSqlScript As String = RegionSql.SelectRegionDropDownListData()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objRegionCol = New RegionCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objRegion As Region  = New Region()
                                      objRegion.RegionID = DirectCast(dr("RegionID"), Integer)
                                      objRegion.RegionDescription = DirectCast(dr("RegionDescription"), String)

                                      objRegionCol.Add(objRegion)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objRegionCol
         End Function

         Public Shared Function SelectShared(dynamicSqlScript As String, param As String, paramValue As Object, Optional ByVal sortByExpression As String = Nothing, Optional ByVal startRowIndex As Integer? = Nothing, Optional ByVal rows As Integer? = Nothing) As RegionCollection
              Dim objRegionCol As RegionCollection = Nothing

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' select, skip, take, sort parameters
                      If Not [String].IsNullOrEmpty(sortByExpression) AndAlso startRowIndex IsNot Nothing AndAlso rows IsNot Nothing Then
                          command.Parameters.AddWithValue("@start", startRowIndex.Value)
                          command.Parameters.AddWithValue("@numberOfRows", rows.Value)
                          command.Parameters.AddWithValue("@sortByExpression", sortByExpression)
                      End If

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objRegionCol = New RegionCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objRegion As Region = CreateRegionFromDataRowShared(dr)
                                      objRegionCol.Add(objRegion)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objRegionCol
         End Function

         ''' <summary>
         ''' Inserts a record
         ''' </summary>
         Public Shared Function Insert(objRegion As Region) As Integer
             Return InsertUpdate(objRegion, False)
         End Function

         ''' <summary>
         ''' Updates a record
         ''' </summary>
         Public Shared Sub Update(objRegion As Region)
             InsertUpdate(objRegion, True)
         End Sub

         Private Shared Function InsertUpdate(objRegion As Region, isUpdate As Boolean) As Integer
              Dim newlyCreatedRegionID As Integer = objRegion.RegionID

              Dim dynamicSqlScript As String = String.Empty

              If isUpdate Then
                  dynamicSqlScript = RegionSql.Update()
              Else
                  dynamicSqlScript = RegionSql.Insert()
              End If
              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@regionID", objRegion.RegionID)
                      command.Parameters.AddWithValue("@regionDescription", objRegion.RegionDescription)

                      If isUpdate Then
                          command.ExecuteNonQuery()
                      Else
                          newlyCreatedRegionID = DirectCast(command.ExecuteScalar(), Integer)
                      End If
                  End Using
              End Using

              Return newlyCreatedRegionID
         End Function

         ''' <summary>
         ''' Deletes a record based on primary key(s)
         ''' </summary>
         Public Shared Sub Delete(ByVal regionID As Integer)
              Dim dynamicSqlScript As String = RegionSql.Delete()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@regionID", regionID)

                      ' execute
                      command.ExecuteNonQuery()
                  End Using
              End Using
         End Sub

         ''' <summary>
         ''' Adds search parameters to the Command object
         ''' </summary>
         Private Shared Sub AddSearchCommandParamsShared(command As SqlCommand, regionID As Integer?, regionDescription As String)
              If Not regionID Is Nothing Then
                  command.Parameters.AddWithValue("@regionID", regionID)
              Else
                  command.Parameters.AddWithValue("@regionID", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(regionDescription) Then
                  command.Parameters.AddWithValue("@regionDescription", regionDescription)
              Else
                  command.Parameters.AddWithValue("@regionDescription", System.DBNull.Value)
              End If

         End Sub

         ''' <summary>
         ''' Creates a Region object from the passed data row
         ''' </summary>
         Private Shared Function CreateRegionFromDataRowShared(dr As DataRow) As Region
             Dim objRegion As Region = New Region()


             If Not DbNull.Value.Equals(dr("RegionID")) Then
                 objRegion.RegionID = DirectCast(dr("RegionID"), Integer)
             Else
                 objRegion.RegionID = Nothing
             End If

             If Not DbNull.Value.Equals(dr("RegionDescription")) Then
                 objRegion.RegionDescription = dr("RegionDescription").ToString()
             Else
                 objRegion.RegionDescription = Nothing
             End If

             Return objRegion
         End Function
     End Class
End Namespace
