Imports System
Imports System.Data
Imports System.Data.SqlClient
Imports System.Configuration
Imports NorthwindWinFormDynamic_VB.BusinessObject

Namespace DataLayer.Base
     ''' <summary>
     ''' Base class for ShippersDataLayer.  Do not make changes to this class,
     ''' instead, put additional code in the ShippersDataLayer class
     ''' </summary>
     Public Class ShippersDataLayerBase

         ' constructor
         Public Sub New()
         End Sub

         ''' <summary>
         ''' Selects a record by primary key(s)
         ''' </summary>
         Public Shared Function SelectByPrimaryKey(ByVal shipperID As Integer) As Shippers
              Dim objShippers As Shippers = Nothing
              Dim dynamicSqlScript As String = ShippersSql.SelectByPrimaryKey()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@shipperID", shipperID)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objShippers = CreateShippersFromDataRowShared(dt.Rows(0))
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objShippers
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Shippers table
         ''' </summary>
         Public Shared Function GetRecordCount() As Integer
             Dim sql As String = ShippersSql.GetRecordCount()
             Return GetRecordCountShared(Nothing, Nothing, Nothing, False, sql)
         End Function

         Public Shared Function GetRecordCountShared(Optional ByVal storedProcName As String = Nothing, Optional ByVal param As String = Nothing, Optional ByVal paramValue As Object = Nothing, Optional ByVal isUseStoredProc As Boolean = True, Optional ByVal dynamicSqlScript As String = Nothing) As Integer
              Dim recordCount As Integer = 0

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  recordCount = DirectCast(dt.Rows(0)("RecordCount"), Integer)
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return recordCount
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Shippers table based on search parameters
         ''' </summary>
         Public Shared Function GetRecordCountDynamicWhere(shipperID As Integer?, companyName As String, phone As String) As Integer
              Dim recordCount As Integer = 0
              Dim dynamicSqlScript As String = ShippersSql.GetRecordCountDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' search parameters
                      AddSearchCommandParamsShared(command, shipperID, companyName, phone)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  recordCount = DirectCast(dt.Rows(0)("RecordCount"), Integer)
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return recordCount
         End Function

         ''' <summary>
         ''' Selects Shippers records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of records)
         ''' </summary>
         Public Shared Function SelectSkipAndTake(sortByExpression As String, startRowIndex As Integer, rows As Integer) As ShippersCollection
             Dim dynamicSqlScript As String = ShippersSql.SelectSkipAndTake()
             Return SelectShared(dynamicSqlScript, Nothing, Nothing, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects Shippers records sorted by the sortByExpression and returns records between the start and end based on search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(shipperID As Integer?, companyName As String, phone As String, sortByExpression As String, startRowIndex As Integer, rows  As Integer) As ShippersCollection
              Dim  objShippersCol As ShippersCollection = Nothing
              Dim dynamicSqlScript As String = ShippersSql.SelectSkipAndTakeDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' select, skip, take, sort parameters
                      command.Parameters.AddWithValue("@start", startRowIndex)
                      command.Parameters.AddWithValue("@numberOfRows", rows)
                      command.Parameters.AddWithValue("@sortByExpression", sortByExpression)

                      ' search parameters
                      AddSearchCommandParamsShared(command, shipperID, companyName, phone)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objShippersCol = New ShippersCollection()

                                  For Each dr As DataRow in dt.Rows
                                      Dim objShippers As Shippers = CreateShippersFromDataRowShared(dr)
                                      objShippersCol.Add(objShippers)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objShippersCol
         End Function

         ''' <summary>
         ''' Selects all Shippers
         ''' </summary>
         Public Shared Function SelectAll() As ShippersCollection
             Dim dynamicSqlScript As String = ShippersSql.SelectAll()
             Return SelectShared(dynamicSqlScript, String.Empty, Nothing)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Shippers.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(shipperID As Integer?, companyName As String, phone As String) As ShippersCollection
              Dim objShippersCol As ShippersCollection = Nothing
              Dim dynamicSqlScript As String = ShippersSql.SelectAllDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' search parameters
                      AddSearchCommandParamsShared(command, shipperID, companyName, phone)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objShippersCol = New ShippersCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objShippers As Shippers = CreateShippersFromDataRowShared(dr)
                                      objShippersCol.Add(objShippers)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objShippersCol
         End Function

         ''' <summary>
         ''' Selects ShipperID and CompanyName columns for use with a DropDownList web control
         ''' </summary>
         Public Shared Function SelectShippersDropDownListData() As ShippersCollection
              Dim objShippersCol As ShippersCollection = Nothing
              Dim dynamicSqlScript As String = ShippersSql.SelectShippersDropDownListData()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objShippersCol = New ShippersCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objShippers As Shippers  = New Shippers()
                                      objShippers.ShipperID = DirectCast(dr("ShipperID"), Integer)
                                      objShippers.CompanyName = DirectCast(dr("CompanyName"), String)

                                      objShippersCol.Add(objShippers)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objShippersCol
         End Function

         Public Shared Function SelectShared(dynamicSqlScript As String, param As String, paramValue As Object, Optional ByVal sortByExpression As String = Nothing, Optional ByVal startRowIndex As Integer? = Nothing, Optional ByVal rows As Integer? = Nothing) As ShippersCollection
              Dim objShippersCol As ShippersCollection = Nothing

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' select, skip, take, sort parameters
                      If Not [String].IsNullOrEmpty(sortByExpression) AndAlso startRowIndex IsNot Nothing AndAlso rows IsNot Nothing Then
                          command.Parameters.AddWithValue("@start", startRowIndex.Value)
                          command.Parameters.AddWithValue("@numberOfRows", rows.Value)
                          command.Parameters.AddWithValue("@sortByExpression", sortByExpression)
                      End If

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objShippersCol = New ShippersCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objShippers As Shippers = CreateShippersFromDataRowShared(dr)
                                      objShippersCol.Add(objShippers)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objShippersCol
         End Function

         ''' <summary>
         ''' Inserts a record
         ''' </summary>
         Public Shared Function Insert(objShippers As Shippers) As Integer
             Return InsertUpdate(objShippers, False)
         End Function

         ''' <summary>
         ''' Updates a record
         ''' </summary>
         Public Shared Sub Update(objShippers As Shippers)
             InsertUpdate(objShippers, True)
         End Sub

         Private Shared Function InsertUpdate(objShippers As Shippers, isUpdate As Boolean) As Integer
              Dim newlyCreatedShipperID As Integer = objShippers.ShipperID

              Dim dynamicSqlScript As String = String.Empty

              If isUpdate Then
                  dynamicSqlScript = ShippersSql.Update()
              Else
                  dynamicSqlScript = ShippersSql.Insert()
              End If
              Dim phone As Object = objShippers.Phone

              If [String].IsNullOrEmpty(objShippers.Phone) Then
                  phone = System.DBNull.Value
              End If

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      If isUpdate Then
                          ' for update only
                          command.Parameters.AddWithValue("@shipperID", objShippers.ShipperID)
                      End If

                      command.Parameters.AddWithValue("@companyName", objShippers.CompanyName)
                      command.Parameters.AddWithValue("@phone", phone)

                      If isUpdate Then
                          command.ExecuteNonQuery()
                      Else
                          newlyCreatedShipperID = DirectCast(command.ExecuteScalar(), Integer)
                      End If
                  End Using
              End Using

              Return newlyCreatedShipperID
         End Function

         ''' <summary>
         ''' Deletes a record based on primary key(s)
         ''' </summary>
         Public Shared Sub Delete(ByVal shipperID As Integer)
              Dim dynamicSqlScript As String = ShippersSql.Delete()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@shipperID", shipperID)

                      ' execute
                      command.ExecuteNonQuery()
                  End Using
              End Using
         End Sub

         ''' <summary>
         ''' Adds search parameters to the Command object
         ''' </summary>
         Private Shared Sub AddSearchCommandParamsShared(command As SqlCommand, shipperID As Integer?, companyName As String, phone As String)
              If Not shipperID Is Nothing Then
                  command.Parameters.AddWithValue("@shipperID", shipperID)
              Else
                  command.Parameters.AddWithValue("@shipperID", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(companyName) Then
                  command.Parameters.AddWithValue("@companyName", companyName)
              Else
                  command.Parameters.AddWithValue("@companyName", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(phone) Then
                  command.Parameters.AddWithValue("@phone", phone)
              Else
                  command.Parameters.AddWithValue("@phone", System.DBNull.Value)
              End If

         End Sub

         ''' <summary>
         ''' Creates a Shippers object from the passed data row
         ''' </summary>
         Private Shared Function CreateShippersFromDataRowShared(dr As DataRow) As Shippers
             Dim objShippers As Shippers = New Shippers()


             If Not DbNull.Value.Equals(dr("ShipperID")) Then
                 objShippers.ShipperID = DirectCast(dr("ShipperID"), Integer)
             Else
                 objShippers.ShipperID = Nothing
             End If

             If Not DbNull.Value.Equals(dr("CompanyName")) Then
                 objShippers.CompanyName = dr("CompanyName").ToString()
             Else
                 objShippers.CompanyName = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Phone")) Then
                 objShippers.Phone = dr("Phone").ToString()
             Else
                 objShippers.Phone = Nothing
             End If

             Return objShippers
         End Function
     End Class
End Namespace
