Imports System
Imports System.Data
Imports System.Data.SqlClient
Imports System.Configuration
Imports NorthwindWinFormDynamic_VB.BusinessObject

Namespace DataLayer.Base
     ''' <summary>
     ''' Base class for SuppliersDataLayer.  Do not make changes to this class,
     ''' instead, put additional code in the SuppliersDataLayer class
     ''' </summary>
     Public Class SuppliersDataLayerBase

         ' constructor
         Public Sub New()
         End Sub

         ''' <summary>
         ''' Selects a record by primary key(s)
         ''' </summary>
         Public Shared Function SelectByPrimaryKey(ByVal supplierID As Integer) As Suppliers
              Dim objSuppliers As Suppliers = Nothing
              Dim dynamicSqlScript As String = SuppliersSql.SelectByPrimaryKey()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@supplierID", supplierID)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objSuppliers = CreateSuppliersFromDataRowShared(dt.Rows(0))
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objSuppliers
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Suppliers table
         ''' </summary>
         Public Shared Function GetRecordCount() As Integer
             Dim sql As String = SuppliersSql.GetRecordCount()
             Return GetRecordCountShared(Nothing, Nothing, Nothing, False, sql)
         End Function

         Public Shared Function GetRecordCountShared(Optional ByVal storedProcName As String = Nothing, Optional ByVal param As String = Nothing, Optional ByVal paramValue As Object = Nothing, Optional ByVal isUseStoredProc As Boolean = True, Optional ByVal dynamicSqlScript As String = Nothing) As Integer
              Dim recordCount As Integer = 0

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  recordCount = DirectCast(dt.Rows(0)("RecordCount"), Integer)
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return recordCount
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Suppliers table based on search parameters
         ''' </summary>
         Public Shared Function GetRecordCountDynamicWhere(supplierID As Integer?, companyName As String, contactName As String, contactTitle As String, address As String, city As String, region1 As String, postalCode As String, country As String, phone As String, fax As String) As Integer
              Dim recordCount As Integer = 0
              Dim dynamicSqlScript As String = SuppliersSql.GetRecordCountDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' search parameters
                      AddSearchCommandParamsShared(command, supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  recordCount = DirectCast(dt.Rows(0)("RecordCount"), Integer)
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return recordCount
         End Function

         ''' <summary>
         ''' Selects Suppliers records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of records)
         ''' </summary>
         Public Shared Function SelectSkipAndTake(sortByExpression As String, startRowIndex As Integer, rows As Integer) As SuppliersCollection
             Dim dynamicSqlScript As String = SuppliersSql.SelectSkipAndTake()
             Return SelectShared(dynamicSqlScript, Nothing, Nothing, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects Suppliers records sorted by the sortByExpression and returns records between the start and end based on search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(supplierID As Integer?, companyName As String, contactName As String, contactTitle As String, address As String, city As String, region1 As String, postalCode As String, country As String, phone As String, fax As String, sortByExpression As String, startRowIndex As Integer, rows  As Integer) As SuppliersCollection
              Dim  objSuppliersCol As SuppliersCollection = Nothing
              Dim dynamicSqlScript As String = SuppliersSql.SelectSkipAndTakeDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' select, skip, take, sort parameters
                      command.Parameters.AddWithValue("@start", startRowIndex)
                      command.Parameters.AddWithValue("@numberOfRows", rows)
                      command.Parameters.AddWithValue("@sortByExpression", sortByExpression)

                      ' search parameters
                      AddSearchCommandParamsShared(command, supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objSuppliersCol = New SuppliersCollection()

                                  For Each dr As DataRow in dt.Rows
                                      Dim objSuppliers As Suppliers = CreateSuppliersFromDataRowShared(dr)
                                      objSuppliersCol.Add(objSuppliers)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objSuppliersCol
         End Function

         ''' <summary>
         ''' Selects all Suppliers
         ''' </summary>
         Public Shared Function SelectAll() As SuppliersCollection
             Dim dynamicSqlScript As String = SuppliersSql.SelectAll()
             Return SelectShared(dynamicSqlScript, String.Empty, Nothing)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Suppliers.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(supplierID As Integer?, companyName As String, contactName As String, contactTitle As String, address As String, city As String, region1 As String, postalCode As String, country As String, phone As String, fax As String) As SuppliersCollection
              Dim objSuppliersCol As SuppliersCollection = Nothing
              Dim dynamicSqlScript As String = SuppliersSql.SelectAllDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' search parameters
                      AddSearchCommandParamsShared(command, supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objSuppliersCol = New SuppliersCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objSuppliers As Suppliers = CreateSuppliersFromDataRowShared(dr)
                                      objSuppliersCol.Add(objSuppliers)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objSuppliersCol
         End Function

         ''' <summary>
         ''' Selects SupplierID and CompanyName columns for use with a DropDownList web control
         ''' </summary>
         Public Shared Function SelectSuppliersDropDownListData() As SuppliersCollection
              Dim objSuppliersCol As SuppliersCollection = Nothing
              Dim dynamicSqlScript As String = SuppliersSql.SelectSuppliersDropDownListData()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objSuppliersCol = New SuppliersCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objSuppliers As Suppliers  = New Suppliers()
                                      objSuppliers.SupplierID = DirectCast(dr("SupplierID"), Integer)
                                      objSuppliers.CompanyName = DirectCast(dr("CompanyName"), String)

                                      objSuppliersCol.Add(objSuppliers)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objSuppliersCol
         End Function

         Public Shared Function SelectShared(dynamicSqlScript As String, param As String, paramValue As Object, Optional ByVal sortByExpression As String = Nothing, Optional ByVal startRowIndex As Integer? = Nothing, Optional ByVal rows As Integer? = Nothing) As SuppliersCollection
              Dim objSuppliersCol As SuppliersCollection = Nothing

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' select, skip, take, sort parameters
                      If Not [String].IsNullOrEmpty(sortByExpression) AndAlso startRowIndex IsNot Nothing AndAlso rows IsNot Nothing Then
                          command.Parameters.AddWithValue("@start", startRowIndex.Value)
                          command.Parameters.AddWithValue("@numberOfRows", rows.Value)
                          command.Parameters.AddWithValue("@sortByExpression", sortByExpression)
                      End If

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objSuppliersCol = New SuppliersCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objSuppliers As Suppliers = CreateSuppliersFromDataRowShared(dr)
                                      objSuppliersCol.Add(objSuppliers)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objSuppliersCol
         End Function

         ''' <summary>
         ''' Inserts a record
         ''' </summary>
         Public Shared Function Insert(objSuppliers As Suppliers) As Integer
             Return InsertUpdate(objSuppliers, False)
         End Function

         ''' <summary>
         ''' Updates a record
         ''' </summary>
         Public Shared Sub Update(objSuppliers As Suppliers)
             InsertUpdate(objSuppliers, True)
         End Sub

         Private Shared Function InsertUpdate(objSuppliers As Suppliers, isUpdate As Boolean) As Integer
              Dim newlyCreatedSupplierID As Integer = objSuppliers.SupplierID

              Dim dynamicSqlScript As String = String.Empty

              If isUpdate Then
                  dynamicSqlScript = SuppliersSql.Update()
              Else
                  dynamicSqlScript = SuppliersSql.Insert()
              End If
              Dim contactName As Object = objSuppliers.ContactName
              Dim contactTitle As Object = objSuppliers.ContactTitle
              Dim address As Object = objSuppliers.Address
              Dim city As Object = objSuppliers.City
              Dim region1 As Object = objSuppliers.Region1
              Dim postalCode As Object = objSuppliers.PostalCode
              Dim country As Object = objSuppliers.Country
              Dim phone As Object = objSuppliers.Phone
              Dim fax As Object = objSuppliers.Fax
              Dim homePage As Object = objSuppliers.HomePage

              If [String].IsNullOrEmpty(objSuppliers.ContactName) Then
                  contactName = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objSuppliers.ContactTitle) Then
                  contactTitle = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objSuppliers.Address) Then
                  address = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objSuppliers.City) Then
                  city = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objSuppliers.Region1) Then
                  region1 = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objSuppliers.PostalCode) Then
                  postalCode = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objSuppliers.Country) Then
                  country = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objSuppliers.Phone) Then
                  phone = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objSuppliers.Fax) Then
                  fax = System.DBNull.Value
              End If

              If [String].IsNullOrEmpty(objSuppliers.HomePage) Then
                  homePage = System.DBNull.Value
              End If

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      If isUpdate Then
                          ' for update only
                          command.Parameters.AddWithValue("@supplierID", objSuppliers.SupplierID)
                      End If

                      command.Parameters.AddWithValue("@companyName", objSuppliers.CompanyName)
                      command.Parameters.AddWithValue("@contactName", contactName)
                      command.Parameters.AddWithValue("@contactTitle", contactTitle)
                      command.Parameters.AddWithValue("@address", address)
                      command.Parameters.AddWithValue("@city", city)
                      command.Parameters.AddWithValue("@region1", region1)
                      command.Parameters.AddWithValue("@postalCode", postalCode)
                      command.Parameters.AddWithValue("@country", country)
                      command.Parameters.AddWithValue("@phone", phone)
                      command.Parameters.AddWithValue("@fax", fax)
                      command.Parameters.AddWithValue("@homePage", homePage)

                      If isUpdate Then
                          command.ExecuteNonQuery()
                      Else
                          newlyCreatedSupplierID = DirectCast(command.ExecuteScalar(), Integer)
                      End If
                  End Using
              End Using

              Return newlyCreatedSupplierID
         End Function

         ''' <summary>
         ''' Deletes a record based on primary key(s)
         ''' </summary>
         Public Shared Sub Delete(ByVal supplierID As Integer)
              Dim dynamicSqlScript As String = SuppliersSql.Delete()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@supplierID", supplierID)

                      ' execute
                      command.ExecuteNonQuery()
                  End Using
              End Using
         End Sub

         ''' <summary>
         ''' Adds search parameters to the Command object
         ''' </summary>
         Private Shared Sub AddSearchCommandParamsShared(command As SqlCommand, supplierID As Integer?, companyName As String, contactName As String, contactTitle As String, address As String, city As String, region1 As String, postalCode As String, country As String, phone As String, fax As String)
              If Not supplierID Is Nothing Then
                  command.Parameters.AddWithValue("@supplierID", supplierID)
              Else
                  command.Parameters.AddWithValue("@supplierID", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(companyName) Then
                  command.Parameters.AddWithValue("@companyName", companyName)
              Else
                  command.Parameters.AddWithValue("@companyName", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(contactName) Then
                  command.Parameters.AddWithValue("@contactName", contactName)
              Else
                  command.Parameters.AddWithValue("@contactName", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(contactTitle) Then
                  command.Parameters.AddWithValue("@contactTitle", contactTitle)
              Else
                  command.Parameters.AddWithValue("@contactTitle", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(address) Then
                  command.Parameters.AddWithValue("@address", address)
              Else
                  command.Parameters.AddWithValue("@address", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(city) Then
                  command.Parameters.AddWithValue("@city", city)
              Else
                  command.Parameters.AddWithValue("@city", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(region1) Then
                  command.Parameters.AddWithValue("@region1", region1)
              Else
                  command.Parameters.AddWithValue("@region1", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(postalCode) Then
                  command.Parameters.AddWithValue("@postalCode", postalCode)
              Else
                  command.Parameters.AddWithValue("@postalCode", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(country) Then
                  command.Parameters.AddWithValue("@country", country)
              Else
                  command.Parameters.AddWithValue("@country", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(phone) Then
                  command.Parameters.AddWithValue("@phone", phone)
              Else
                  command.Parameters.AddWithValue("@phone", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(fax) Then
                  command.Parameters.AddWithValue("@fax", fax)
              Else
                  command.Parameters.AddWithValue("@fax", System.DBNull.Value)
              End If

         End Sub

         ''' <summary>
         ''' Creates a Suppliers object from the passed data row
         ''' </summary>
         Private Shared Function CreateSuppliersFromDataRowShared(dr As DataRow) As Suppliers
             Dim objSuppliers As Suppliers = New Suppliers()


             If Not DbNull.Value.Equals(dr("SupplierID")) Then
                 objSuppliers.SupplierID = DirectCast(dr("SupplierID"), Integer)
             Else
                 objSuppliers.SupplierID = Nothing
             End If

             If Not DbNull.Value.Equals(dr("CompanyName")) Then
                 objSuppliers.CompanyName = dr("CompanyName").ToString()
             Else
                 objSuppliers.CompanyName = Nothing
             End If

             If Not DbNull.Value.Equals(dr("ContactName")) Then
                 objSuppliers.ContactName = dr("ContactName").ToString()
             Else
                 objSuppliers.ContactName = Nothing
             End If

             If Not DbNull.Value.Equals(dr("ContactTitle")) Then
                 objSuppliers.ContactTitle = dr("ContactTitle").ToString()
             Else
                 objSuppliers.ContactTitle = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Address")) Then
                 objSuppliers.Address = dr("Address").ToString()
             Else
                 objSuppliers.Address = Nothing
             End If

             If Not DbNull.Value.Equals(dr("City")) Then
                 objSuppliers.City = dr("City").ToString()
             Else
                 objSuppliers.City = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Region1")) Then
                 objSuppliers.Region1 = dr("Region1").ToString()
             Else
                 objSuppliers.Region1 = Nothing
             End If

             If Not DbNull.Value.Equals(dr("PostalCode")) Then
                 objSuppliers.PostalCode = dr("PostalCode").ToString()
             Else
                 objSuppliers.PostalCode = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Country")) Then
                 objSuppliers.Country = dr("Country").ToString()
             Else
                 objSuppliers.Country = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Phone")) Then
                 objSuppliers.Phone = dr("Phone").ToString()
             Else
                 objSuppliers.Phone = Nothing
             End If

             If Not DbNull.Value.Equals(dr("Fax")) Then
                 objSuppliers.Fax = dr("Fax").ToString()
             Else
                 objSuppliers.Fax = Nothing
             End If

             If Not DbNull.Value.Equals(dr("HomePage")) Then
                 objSuppliers.HomePage = dr("HomePage").ToString()
             Else
                 objSuppliers.HomePage = Nothing
             End If

             Return objSuppliers
         End Function
     End Class
End Namespace
