Imports System
Imports System.Data
Imports System.Data.SqlClient
Imports System.Configuration
Imports NorthwindWinFormDynamic_VB.BusinessObject

Namespace DataLayer.Base
     ''' <summary>
     ''' Base class for TerritoriesDataLayer.  Do not make changes to this class,
     ''' instead, put additional code in the TerritoriesDataLayer class
     ''' </summary>
     Public Class TerritoriesDataLayerBase

         ' constructor
         Public Sub New()
         End Sub

         ''' <summary>
         ''' Selects a record by primary key(s)
         ''' </summary>
         Public Shared Function SelectByPrimaryKey(ByVal territoryID As String) As Territories
              Dim objTerritories As Territories = Nothing
              Dim dynamicSqlScript As String = TerritoriesSql.SelectByPrimaryKey()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@territoryID", territoryID)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objTerritories = CreateTerritoriesFromDataRowShared(dt.Rows(0))
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objTerritories
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Territories table
         ''' </summary>
         Public Shared Function GetRecordCount() As Integer
             Dim sql As String = TerritoriesSql.GetRecordCount()
             Return GetRecordCountShared(Nothing, Nothing, Nothing, False, sql)
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Territories table by RegionID
         ''' </summary>
         Public Shared Function GetRecordCountByRegionID(regionID As Integer) As Integer
             Dim sql As String = TerritoriesSql.GetRecordCountByRegionID(regionID)
             Return GetRecordCountShared(Nothing, "regionID", regionID, False, sql)
         End Function

         Public Shared Function GetRecordCountShared(Optional ByVal storedProcName As String = Nothing, Optional ByVal param As String = Nothing, Optional ByVal paramValue As Object = Nothing, Optional ByVal isUseStoredProc As Boolean = True, Optional ByVal dynamicSqlScript As String = Nothing) As Integer
              Dim recordCount As Integer = 0

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      Select Case param
                          Case "regionID"
                              command.Parameters.AddWithValue("@regionID", paramValue)
                          Case Else
                              Exit Select
                      End Select

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  recordCount = DirectCast(dt.Rows(0)("RecordCount"), Integer)
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return recordCount
         End Function

         ''' <summary>
         ''' Gets the total number of records in the Territories table based on search parameters
         ''' </summary>
         Public Shared Function GetRecordCountDynamicWhere(territoryID As String, territoryDescription As String, regionID As Integer?) As Integer
              Dim recordCount As Integer = 0
              Dim dynamicSqlScript As String = TerritoriesSql.GetRecordCountDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' search parameters
                      AddSearchCommandParamsShared(command, territoryID, territoryDescription, regionID)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  recordCount = DirectCast(dt.Rows(0)("RecordCount"), Integer)
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return recordCount
         End Function

         ''' <summary>
         ''' Selects Territories records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of records)
         ''' </summary>
         Public Shared Function SelectSkipAndTake(sortByExpression As String, startRowIndex As Integer, rows As Integer) As TerritoriesCollection
             Dim dynamicSqlScript As String = TerritoriesSql.SelectSkipAndTake()
             Return SelectShared(dynamicSqlScript, Nothing, Nothing, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects records by RegionID as a collection (List) of Territories sorted by the sortByExpression and returns the rows (# of records) starting from the startRowIndex
         ''' </summary>
         Public Shared Function SelectSkipAndTakeByRegionID(sortByExpression As String, startRowIndex As Integer, rows As Integer, regionID As Integer) As TerritoriesCollection
             Dim dynamicSqlScript As String = TerritoriesSql.SelectSkipAndTakeByRegionID(regionID)
             Return SelectShared(dynamicSqlScript, "regionID", regionID, sortByExpression, startRowIndex, rows)
         End Function

         ''' <summary>
         ''' Selects Territories records sorted by the sortByExpression and returns records between the start and end based on search parameters
         ''' </summary>
         Public Shared Function SelectSkipAndTakeDynamicWhere(territoryID As String, territoryDescription As String, regionID As Integer?, sortByExpression As String, startRowIndex As Integer, rows  As Integer) As TerritoriesCollection
              Dim  objTerritoriesCol As TerritoriesCollection = Nothing
              Dim dynamicSqlScript As String = TerritoriesSql.SelectSkipAndTakeDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' select, skip, take, sort parameters
                      command.Parameters.AddWithValue("@start", startRowIndex)
                      command.Parameters.AddWithValue("@numberOfRows", rows)
                      command.Parameters.AddWithValue("@sortByExpression", sortByExpression)

                      ' search parameters
                      AddSearchCommandParamsShared(command, territoryID, territoryDescription, regionID)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objTerritoriesCol = New TerritoriesCollection()

                                  For Each dr As DataRow in dt.Rows
                                      Dim objTerritories As Territories = CreateTerritoriesFromDataRowShared(dr)
                                      objTerritoriesCol.Add(objTerritories)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objTerritoriesCol
         End Function

         ''' <summary>
         ''' Selects all Territories
         ''' </summary>
         Public Shared Function SelectAll() As TerritoriesCollection
             Dim dynamicSqlScript As String = TerritoriesSql.SelectAll()
             Return SelectShared(dynamicSqlScript, String.Empty, Nothing)
         End Function

         ''' <summary>
         ''' Selects records based on the passed filters as a collection (List) of Territories.
         ''' </summary>
         Public Shared Function SelectAllDynamicWhere(territoryID As String, territoryDescription As String, regionID As Integer?) As TerritoriesCollection
              Dim objTerritoriesCol As TerritoriesCollection = Nothing
              Dim dynamicSqlScript As String = TerritoriesSql.SelectAllDynamicWhere()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' search parameters
                      AddSearchCommandParamsShared(command, territoryID, territoryDescription, regionID)

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objTerritoriesCol = New TerritoriesCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objTerritories As Territories = CreateTerritoriesFromDataRowShared(dr)
                                      objTerritoriesCol.Add(objTerritories)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objTerritoriesCol
         End Function

         ''' <summary>
         ''' Selects all Territories by Region, related to column RegionID
         ''' </summary>
         Public Shared Function SelectTerritoriesCollectionByRegionID(regionID As Integer) As TerritoriesCollection
             Dim dynamicSqlScript As String = TerritoriesSql.SelectAllByRegionID(regionID)
             Return SelectShared(dynamicSqlScript, "regionID", regionID)
         End Function

         ''' <summary>
         ''' Selects TerritoryID and TerritoryDescription columns for use with a DropDownList web control
         ''' </summary>
         Public Shared Function SelectTerritoriesDropDownListData() As TerritoriesCollection
              Dim objTerritoriesCol As TerritoriesCollection = Nothing
              Dim dynamicSqlScript As String = TerritoriesSql.SelectTerritoriesDropDownListData()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objTerritoriesCol = New TerritoriesCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objTerritories As Territories  = New Territories()
                                      objTerritories.TerritoryID = DirectCast(dr("TerritoryID"), String)
                                      objTerritories.TerritoryDescription = DirectCast(dr("TerritoryDescription"), String)

                                      objTerritoriesCol.Add(objTerritories)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objTerritoriesCol
         End Function

         Public Shared Function SelectShared(dynamicSqlScript As String, param As String, paramValue As Object, Optional ByVal sortByExpression As String = Nothing, Optional ByVal startRowIndex As Integer? = Nothing, Optional ByVal rows As Integer? = Nothing) As TerritoriesCollection
              Dim objTerritoriesCol As TerritoriesCollection = Nothing

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' select, skip, take, sort parameters
                      If Not [String].IsNullOrEmpty(sortByExpression) AndAlso startRowIndex IsNot Nothing AndAlso rows IsNot Nothing Then
                          command.Parameters.AddWithValue("@start", startRowIndex.Value)
                          command.Parameters.AddWithValue("@numberOfRows", rows.Value)
                          command.Parameters.AddWithValue("@sortByExpression", sortByExpression)
                      End If

                      ' parameters
                      Select Case param
                          Case "regionID"
                              command.Parameters.AddWithValue("@regionID", paramValue)
                              Exit Select
                          Case Else
                              Exit Select
                      End Select

                      Using da As New SqlDataAdapter(command)
                          Dim dt As New DataTable()
                          da.Fill(dt)

                          If dt IsNot Nothing Then
                              If dt.Rows.Count > 0 Then
                                  objTerritoriesCol = New TerritoriesCollection()

                                  For Each dr As DataRow In dt.Rows
                                      Dim objTerritories As Territories = CreateTerritoriesFromDataRowShared(dr)
                                      objTerritoriesCol.Add(objTerritories)
                                  Next
                              End If
                          End If
                      End Using
                  End Using
              End Using

              Return objTerritoriesCol
         End Function

         ''' <summary>
         ''' Inserts a record
         ''' </summary>
         Public Shared Function Insert(objTerritories As Territories) As String
             Return InsertUpdate(objTerritories, False)
         End Function

         ''' <summary>
         ''' Updates a record
         ''' </summary>
         Public Shared Sub Update(objTerritories As Territories)
             InsertUpdate(objTerritories, True)
         End Sub

         Private Shared Function InsertUpdate(objTerritories As Territories, isUpdate As Boolean) As String
              Dim newlyCreatedTerritoryID As String = objTerritories.TerritoryID

              Dim dynamicSqlScript As String = String.Empty

              If isUpdate Then
                  dynamicSqlScript = TerritoriesSql.Update()
              Else
                  dynamicSqlScript = TerritoriesSql.Insert()
              End If
              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@territoryID", objTerritories.TerritoryID)
                      command.Parameters.AddWithValue("@territoryDescription", objTerritories.TerritoryDescription)
                      command.Parameters.AddWithValue("@regionID", objTerritories.RegionID)

                      If isUpdate Then
                          command.ExecuteNonQuery()
                      Else
                          newlyCreatedTerritoryID = DirectCast(command.ExecuteScalar(), String)
                      End If
                  End Using
              End Using

              Return newlyCreatedTerritoryID
         End Function

         ''' <summary>
         ''' Deletes a record based on primary key(s)
         ''' </summary>
         Public Shared Sub Delete(ByVal territoryID As String)
              Dim dynamicSqlScript As String = TerritoriesSql.Delete()

              Using connection As New SqlConnection(ConfigurationManager.ConnectionStrings("ConnectionString").ConnectionString)
                  connection.Open()

                  Using command As New SqlCommand(dynamicSqlScript, connection)
                      command.CommandType = CommandType.Text

                      ' parameters
                      command.Parameters.AddWithValue("@territoryID", territoryID)

                      ' execute
                      command.ExecuteNonQuery()
                  End Using
              End Using
         End Sub

         ''' <summary>
         ''' Adds search parameters to the Command object
         ''' </summary>
         Private Shared Sub AddSearchCommandParamsShared(command As SqlCommand, territoryID As String, territoryDescription As String, regionID As Integer?)
              If Not [String].IsNullOrEmpty(territoryID) Then
                  command.Parameters.AddWithValue("@territoryID", territoryID)
              Else
                  command.Parameters.AddWithValue("@territoryID", System.DBNull.Value)
              End If

              If Not [String].IsNullOrEmpty(territoryDescription) Then
                  command.Parameters.AddWithValue("@territoryDescription", territoryDescription)
              Else
                  command.Parameters.AddWithValue("@territoryDescription", System.DBNull.Value)
              End If

              If Not regionID Is Nothing Then
                  command.Parameters.AddWithValue("@regionID", regionID)
              Else
                  command.Parameters.AddWithValue("@regionID", System.DBNull.Value)
              End If

         End Sub

         ''' <summary>
         ''' Creates a Territories object from the passed data row
         ''' </summary>
         Private Shared Function CreateTerritoriesFromDataRowShared(dr As DataRow) As Territories
             Dim objTerritories As Territories = New Territories()


             If Not DbNull.Value.Equals(dr("TerritoryID")) Then
                 objTerritories.TerritoryID = dr("TerritoryID").ToString()
             Else
                 objTerritories.TerritoryID = Nothing
             End If

             If Not DbNull.Value.Equals(dr("TerritoryDescription")) Then
                 objTerritories.TerritoryDescription = dr("TerritoryDescription").ToString()
             Else
                 objTerritories.TerritoryDescription = Nothing
             End If

             If Not DbNull.Value.Equals(dr("RegionID")) Then
                 objTerritories.RegionID = DirectCast(dr("RegionID"), Integer)
             Else
                 objTerritories.RegionID = Nothing
             End If


             Return objTerritories
         End Function
     End Class
End Namespace
