Imports System
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports System.Windows.Forms

''' <summary>
''' These are data-centric code examples for the Categories _table.
''' You can cut and paste the respective codes into your application
''' by changing the sample values assigned from these examples.
''' NOTE: This class contains private methods because they're
''' not meant to be called by an outside client.  Each method contains
''' code for the respective example being shown
''' </summary>
Public NotInheritable Class CategoriesExample

    Private Sub New()
    End Sub

    Private Sub SelectAll()
        ' select all records
        Dim objCategoriesCol As CategoriesCollection = Categories.SelectAll()

        ' Example 1:  you can optionally sort the collection in ascending order by your chosen field  
        objCategoriesCol.Sort(Categories.ByCategoryName)

        ' Example 2:  to sort in descending order, add this line to the Sort code in Example 1
        objCategoriesCol.Reverse()

        ' Example 3:  assign to a DataGridView
        Dim grid As DataGridView = New DataGridView()
        grid.DataSource = objCategoriesCol

        ' Example 4:  loop through all the Categories
        For Each objCategories As Categories In objCategoriesCol
            Dim categoryID As Integer = objCategories.CategoryID
            Dim categoryName As String = objCategories.CategoryName
            Dim description As String = objCategories.Description
        Next
    End Sub

    Private Sub SelectByPrimaryKey()
        ' select a record by primary key(s)
        Dim objCategories As Categories = Categories.SelectByPrimaryKey(1)

        If objCategories IsNot Nothing Then
            ' if record is found, a record is returned
            Dim categoryID As Integer = objCategories.CategoryID
            Dim categoryName As String = objCategories.CategoryName
            Dim description As String = objCategories.Description
        End If
    End Sub

    ''' <summary>
    ''' Selects CategoryID and CategoryName columns for use with a with a Drop Down List, Combo Box, Checked Box List, List View, List Box, etc
    ''' </summary>
    Private Sub SelectCategoriesDropDownListData()
        Dim objCategoriesCol As CategoriesCollection = Categories.SelectCategoriesDropDownListData()

        ' Example 1:  directly bind to a combo box
        Dim cbx1 As ComboBox = New ComboBox()
        cbx1.ValueMember = "CategoryID"
        cbx1.DisplayMember = "CategoryName"
        cbx1.DataSource = objCategoriesCol

        ' Example 2:  add each item through a loop.  Web app example
        ' Dim cbx2 As DropDownList = New DropDownList()

        ' For Each objCategories As Categories In objCategoriesCol
        '     cbx2.Items.Add(New ListItem(objCategories.CategoryName, objCategories.CategoryID.ToString()))
        ' Next
    End Sub

    Private Sub Insert()
        ' first instantiate a new Categories
        Dim objCategories As New Categories()

        ' assign values you want inserted
        objCategories.CategoryName = "Beverages"
        objCategories.Description = "Soft drinks, coffees, teas, beers, and ales"

        ' finally, insert a new record
        ' the insert method returns the newly created primary key
        Dim newlyCreatedPrimaryKey As Integer = objCategories.Insert()
    End Sub

    Private Sub Update()
        ' first instantiate a new Categories
        Dim objCategories As New Categories()

        ' assign the existing primary key(s)
        ' of the record you want updated
        objCategories.CategoryID = 1

        ' assign values you want updated
        objCategories.CategoryName = "Beverages"
        objCategories.Description = "Soft drinks, coffees, teas, beers, and ales"

        ' finally, update an existing record
        objCategories.Update()
    End Sub

    Private Sub Delete()
        ' delete a record by primary key
        Categories.Delete(35)
    End Sub

    ''' <summary>
    ''' Shows how to get the total number of records
    ''' </summary>
    Private Sub GetRecordCount()
        ' get the total number of records in the Categories table
        Dim totalRecordCount As Integer = Categories.GetRecordCount()
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index.  The total number of records are also retrieved when using the SelectSkipAndTake() method.
    ''' For example, if ther are 200 records (totalRecordCount), take only 10 records (numberOfRecordsToRetrieve), starting from the first index (startRetrievalFromRecordIndex = 0)
    ''' The example below uses some variables, here are their definitions:
    ''' totalRecordCount - total number of records if you were to retrieve everything
    ''' startRetrievalFromRecordIndex - the index to start taking records from. Zero (0) E.g. If you want to skip the first 20 records, then assign 19 here.
    ''' numberOfRecordsToRetrieve - take n records starting from the startRetrievalFromRecordIndex
    ''' sortBy - to sort in Ascending order by Field Name, just assign just the Field Name, do not pass 'asc'
    ''' sortBy - to sort in Descending order by Field Name, use the Field Name, a space and the word 'desc'
    ''' </summary>
    Private Sub SelectSkipAndTake()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "CategoryID"
        ' Dim sortBy As String = "CategoryID desc"

        ' 1. select a specific number of sorted records starting from the index you specify with totalRecordCount
        Dim objCategoriesCol As CategoriesCollection = Categories.SelectSkipAndTake(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy)

        ' 2. or you can also select a specific number of sorted records starting from the index you specify Without the totalRecordCount
        Dim objCategoriesCol2 As CategoriesCollection = Categories.SelectSkipAndTake(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, sortBy)

        ' to use objCategoriesCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the Categories(s).  The example above will only loop for 10 items.
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index, based on Search Parameters.  The number of records are also retrieved.
    ''' </summary>
    Private Sub SelectSkipAndTakeDynamicWhere()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "CategoryID"
        ' Dim sortBy As String = "CategoryID desc"

        ' search parameters, everything is nullable, only items being searched for should be filled
        ' note: fields with String type uses a LIKE search, everything else uses an exact match
        ' also, every field you're searching for uses the AND operator
        ' e.g. Dim productID As Integer? = 1, Dim productName As String = "ch"
        ' will translate to: SELECT....WHERE productID = 1 AND productName LIKE '%ch%'
        Dim categoryID As Integer? = Nothing
        Dim categoryName As String = Nothing

        ' 1. select a specific number of sorted records starting from the index you specify with totalRecordCount based on Search Parameters
        Dim objCategoriesCol As CategoriesCollection = Categories.SelectSkipAndTakeDynamicWhere(categoryID, categoryName, numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy)

        ' to use objCategoriesCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the Categories(s).  The example above will only loop for 10 items.
    End Sub
End Class
