Imports System
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports System.Windows.Forms

''' <summary>
''' These are data-centric code examples for the Customers _table.
''' You can cut and paste the respective codes into your application
''' by changing the sample values assigned from these examples.
''' NOTE: This class contains private methods because they're
''' not meant to be called by an outside client.  Each method contains
''' code for the respective example being shown
''' </summary>
Public NotInheritable Class CustomersExample

    Private Sub New()
    End Sub

    Private Sub SelectAll()
        ' select all records
        Dim objCustomersCol As CustomersCollection = Customers.SelectAll()

        ' Example 1:  you can optionally sort the collection in ascending order by your chosen field  
        objCustomersCol.Sort(Customers.ByCompanyName)

        ' Example 2:  to sort in descending order, add this line to the Sort code in Example 1
        objCustomersCol.Reverse()

        ' Example 3:  assign to a DataGridView
        Dim grid As DataGridView = New DataGridView()
        grid.DataSource = objCustomersCol

        ' Example 4:  loop through all the Customers
        For Each objCustomers As Customers In objCustomersCol
            Dim customerID As String = objCustomers.CustomerID
            Dim companyName As String = objCustomers.CompanyName
            Dim contactName As String = objCustomers.ContactName
            Dim contactTitle As String = objCustomers.ContactTitle
            Dim address As String = objCustomers.Address
            Dim city As String = objCustomers.City
            Dim region1 As String = objCustomers.Region1
            Dim postalCode As String = objCustomers.PostalCode
            Dim country As String = objCustomers.Country
            Dim phone As String = objCustomers.Phone
            Dim fax As String = objCustomers.Fax
        Next
    End Sub

    Private Sub SelectByPrimaryKey()
        ' select a record by primary key(s)
        Dim objCustomers As Customers = Customers.SelectByPrimaryKey("AABBD")

        If objCustomers IsNot Nothing Then
            ' if record is found, a record is returned
            Dim customerID As String = objCustomers.CustomerID
            Dim companyName As String = objCustomers.CompanyName
            Dim contactName As String = objCustomers.ContactName
            Dim contactTitle As String = objCustomers.ContactTitle
            Dim address As String = objCustomers.Address
            Dim city As String = objCustomers.City
            Dim region1 As String = objCustomers.Region1
            Dim postalCode As String = objCustomers.PostalCode
            Dim country As String = objCustomers.Country
            Dim phone As String = objCustomers.Phone
            Dim fax As String = objCustomers.Fax
        End If
    End Sub

    ''' <summary>
    ''' Selects CustomerID and CompanyName columns for use with a with a Drop Down List, Combo Box, Checked Box List, List View, List Box, etc
    ''' </summary>
    Private Sub SelectCustomersDropDownListData()
        Dim objCustomersCol As CustomersCollection = Customers.SelectCustomersDropDownListData()

        ' Example 1:  directly bind to a combo box
        Dim cbx1 As ComboBox = New ComboBox()
        cbx1.ValueMember = "CustomerID"
        cbx1.DisplayMember = "CompanyName"
        cbx1.DataSource = objCustomersCol

        ' Example 2:  add each item through a loop.  Web app example
        ' Dim cbx2 As DropDownList = New DropDownList()

        ' For Each objCustomers As Customers In objCustomersCol
        '     cbx2.Items.Add(New ListItem(objCustomers.CompanyName, objCustomers.CustomerID))
        ' Next
    End Sub

    Private Sub Insert()
        ' first instantiate a new Customers
        Dim objCustomers As New Customers()

        ' assign values you want inserted
        objCustomers.CustomerID = "AABBD"
        objCustomers.CompanyName = "Testing"
        objCustomers.ContactName = "abc"
        objCustomers.ContactTitle = "sdfg"
        objCustomers.Address = "dsfg"
        objCustomers.City = "fg"
        objCustomers.Region1 = "abc"
        objCustomers.PostalCode = "abc"
        objCustomers.Country = "abc"
        objCustomers.Phone = "dgfdf"
        objCustomers.Fax = "abc"

        ' finally, insert a new record
        ' the insert method returns the newly created primary key
        Dim newlyCreatedPrimaryKey As String = objCustomers.Insert()
    End Sub

    Private Sub Update()
        ' first instantiate a new Customers
        Dim objCustomers As New Customers()

        ' assign the existing primary key(s)
        ' of the record you want updated
        objCustomers.CustomerID = "AABBD"

        ' assign values you want updated
        objCustomers.CompanyName = "Testing"
        objCustomers.ContactName = "abc"
        objCustomers.ContactTitle = "sdfg"
        objCustomers.Address = "dsfg"
        objCustomers.City = "fg"
        objCustomers.Region1 = "abc"
        objCustomers.PostalCode = "abc"
        objCustomers.Country = "abc"
        objCustomers.Phone = "dgfdf"
        objCustomers.Fax = "abc"

        ' finally, update an existing record
        objCustomers.Update()
    End Sub

    Private Sub Delete()
        ' delete a record by primary key
        Customers.Delete("AABBD")
    End Sub

    ''' <summary>
    ''' Shows how to get the total number of records
    ''' </summary>
    Private Sub GetRecordCount()
        ' get the total number of records in the Customers table
        Dim totalRecordCount As Integer = Customers.GetRecordCount()
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index.  The total number of records are also retrieved when using the SelectSkipAndTake() method.
    ''' For example, if ther are 200 records (totalRecordCount), take only 10 records (numberOfRecordsToRetrieve), starting from the first index (startRetrievalFromRecordIndex = 0)
    ''' The example below uses some variables, here are their definitions:
    ''' totalRecordCount - total number of records if you were to retrieve everything
    ''' startRetrievalFromRecordIndex - the index to start taking records from. Zero (0) E.g. If you want to skip the first 20 records, then assign 19 here.
    ''' numberOfRecordsToRetrieve - take n records starting from the startRetrievalFromRecordIndex
    ''' sortBy - to sort in Ascending order by Field Name, just assign just the Field Name, do not pass 'asc'
    ''' sortBy - to sort in Descending order by Field Name, use the Field Name, a space and the word 'desc'
    ''' </summary>
    Private Sub SelectSkipAndTake()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "CustomerID"
        ' Dim sortBy As String = "CustomerID desc"

        ' 1. select a specific number of sorted records starting from the index you specify with totalRecordCount
        Dim objCustomersCol As CustomersCollection = Customers.SelectSkipAndTake(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy)

        ' 2. or you can also select a specific number of sorted records starting from the index you specify Without the totalRecordCount
        Dim objCustomersCol2 As CustomersCollection = Customers.SelectSkipAndTake(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, sortBy)

        ' to use objCustomersCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the Customers(s).  The example above will only loop for 10 items.
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index, based on Search Parameters.  The number of records are also retrieved.
    ''' </summary>
    Private Sub SelectSkipAndTakeDynamicWhere()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "CustomerID"
        ' Dim sortBy As String = "CustomerID desc"

        ' search parameters, everything is nullable, only items being searched for should be filled
        ' note: fields with String type uses a LIKE search, everything else uses an exact match
        ' also, every field you're searching for uses the AND operator
        ' e.g. Dim productID As Integer? = 1, Dim productName As String = "ch"
        ' will translate to: SELECT....WHERE productID = 1 AND productName LIKE '%ch%'
        Dim customerID As String = Nothing
        Dim companyName As String = Nothing
        Dim contactName As String = Nothing
        Dim contactTitle As String = Nothing
        Dim address As String = Nothing
        Dim city As String = Nothing
        Dim region1 As String = Nothing
        Dim postalCode As String = Nothing
        Dim country As String = Nothing
        Dim phone As String = Nothing
        Dim fax As String = Nothing

        ' 1. select a specific number of sorted records starting from the index you specify with totalRecordCount based on Search Parameters
        Dim objCustomersCol As CustomersCollection = Customers.SelectSkipAndTakeDynamicWhere(customerID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax, numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy)

        ' to use objCustomersCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the Customers(s).  The example above will only loop for 10 items.
    End Sub
End Class
