Imports System
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports System.Windows.Forms

''' <summary>
''' These are data-centric code examples for the Employees _table.
''' You can cut and paste the respective codes into your application
''' by changing the sample values assigned from these examples.
''' NOTE: This class contains private methods because they're
''' not meant to be called by an outside client.  Each method contains
''' code for the respective example being shown
''' </summary>
Public NotInheritable Class EmployeesExample

    Private Sub New()
    End Sub

    Private Sub SelectAll()
        ' select all records
        Dim objEmployeesCol As EmployeesCollection = Employees.SelectAll()

        ' Example 1:  you can optionally sort the collection in ascending order by your chosen field  
        objEmployeesCol.Sort(Employees.ByLastName)

        ' Example 2:  to sort in descending order, add this line to the Sort code in Example 1
        objEmployeesCol.Reverse()

        ' Example 3:  assign to a DataGridView
        Dim grid As DataGridView = New DataGridView()
        grid.DataSource = objEmployeesCol

        ' Example 4:  loop through all the Employees
        For Each objEmployees As Employees In objEmployeesCol
            Dim employeeID As Integer = objEmployees.EmployeeID
            Dim lastName As String = objEmployees.LastName
            Dim firstName As String = objEmployees.FirstName
            Dim title As String = objEmployees.Title
            Dim titleOfCourtesy As String = objEmployees.TitleOfCourtesy
            Dim birthDate As Date = objEmployees.BirthDate
            Dim hireDate As Date = objEmployees.HireDate
            Dim address As String = objEmployees.Address
            Dim city As String = objEmployees.City
            Dim region1 As String = objEmployees.Region1
            Dim postalCode As String = objEmployees.PostalCode
            Dim country As String = objEmployees.Country
            Dim homePhone As String = objEmployees.HomePhone
            Dim extension As String = objEmployees.Extension
            Dim notes As String = objEmployees.Notes
            Dim reportsTo As Integer = objEmployees.ReportsTo
            Dim photoPath As String = objEmployees.PhotoPath

            ' optionally get the Employees related to ReportsTo.
            ' Note this is lazily loaded which means there's no value until you ask for it
            If objEmployees.ReportsTo IsNot Nothing Then
                Dim objEmployeesRelatedToReportsTo As Employees

                If objEmployees.EmployeesReportsTo.IsValueCreated Then
                    objEmployeesRelatedToReportsTo = objEmployees.EmployeesReportsTo.Value
                End If
            End If
        Next
    End Sub

    Private Sub SelectByPrimaryKey()
        ' select a record by primary key(s)
        Dim objEmployees As Employees = Employees.SelectByPrimaryKey(1)

        If objEmployees IsNot Nothing Then
            ' if record is found, a record is returned
            Dim employeeID As Integer = objEmployees.EmployeeID
            Dim lastName As String = objEmployees.LastName
            Dim firstName As String = objEmployees.FirstName
            Dim title As String = objEmployees.Title
            Dim titleOfCourtesy As String = objEmployees.TitleOfCourtesy
            Dim birthDate As Date = objEmployees.BirthDate
            Dim hireDate As Date = objEmployees.HireDate
            Dim address As String = objEmployees.Address
            Dim city As String = objEmployees.City
            Dim region1 As String = objEmployees.Region1
            Dim postalCode As String = objEmployees.PostalCode
            Dim country As String = objEmployees.Country
            Dim homePhone As String = objEmployees.HomePhone
            Dim extension As String = objEmployees.Extension
            Dim notes As String = objEmployees.Notes
            Dim reportsTo As Integer = objEmployees.ReportsTo
            Dim photoPath As String = objEmployees.PhotoPath

            ' optionally get the Employees related to ReportsTo.
            ' Note this is lazily loaded which means there's no value until you ask for it
            If objEmployees.ReportsTo IsNot Nothing Then
                Dim objEmployeesRelatedToReportsTo As Employees

                If objEmployees.EmployeesReportsTo.IsValueCreated Then
                    objEmployeesRelatedToReportsTo = objEmployees.EmployeesReportsTo.Value
                End If
            End If
        End If
    End Sub

    ''' <summary>
    ''' Select all records by Employees, related to column ReportsTo
    ''' </summary> 
    Private Sub SelectEmployeesCollectionByReportsTo()
        Dim objEmployeesCol As EmployeesCollection = Employees.SelectEmployeesCollectionByReportsTo(2)

        ' Example 1:  you can optionally sort the collection in ascending order by your chosen field  
        objEmployeesCol.Sort(Employees.ByLastName)

        ' Example 2:  to sort in descending order, add this line to the Sort code in Example 1
        objEmployeesCol.Reverse()

        ' Example 3:  assign to a DataGridView
        Dim grid As DataGridView = New DataGridView()
        grid.DataSource = objEmployeesCol

        ' Example 4:  loop through all the Employees
        For Each objEmployees As Employees In objEmployeesCol
            Dim employeeID As Integer = objEmployees.EmployeeID
            Dim lastName As String = objEmployees.LastName
            Dim firstName As String = objEmployees.FirstName
            Dim title As String = objEmployees.Title
            Dim titleOfCourtesy As String = objEmployees.TitleOfCourtesy
            Dim birthDate As Date = objEmployees.BirthDate
            Dim hireDate As Date = objEmployees.HireDate
            Dim address As String = objEmployees.Address
            Dim city As String = objEmployees.City
            Dim region1 As String = objEmployees.Region1
            Dim postalCode As String = objEmployees.PostalCode
            Dim country As String = objEmployees.Country
            Dim homePhone As String = objEmployees.HomePhone
            Dim extension As String = objEmployees.Extension
            Dim notes As String = objEmployees.Notes
            Dim reportsTo As Integer = objEmployees.ReportsTo
            Dim photoPath As String = objEmployees.PhotoPath

            ' optionally get the Employees related to ReportsTo.
            ' Note this is lazily loaded which means there's no value until you ask for it
            If objEmployees.ReportsTo IsNot Nothing Then
                Dim objEmployeesRelatedToReportsTo As Employees

                If objEmployees.EmployeesReportsTo.IsValueCreated Then
                    objEmployeesRelatedToReportsTo = objEmployees.EmployeesReportsTo.Value
                End If
            End If
        Next
    End Sub

    ''' <summary>
    ''' Selects EmployeeID and LastName columns for use with a with a Drop Down List, Combo Box, Checked Box List, List View, List Box, etc
    ''' </summary>
    Private Sub SelectEmployeesDropDownListData()
        Dim objEmployeesCol As EmployeesCollection = Employees.SelectEmployeesDropDownListData()

        ' Example 1:  directly bind to a combo box
        Dim cbx1 As ComboBox = New ComboBox()
        cbx1.ValueMember = "EmployeeID"
        cbx1.DisplayMember = "LastName"
        cbx1.DataSource = objEmployeesCol

        ' Example 2:  add each item through a loop.  Web app example
        ' Dim cbx2 As DropDownList = New DropDownList()

        ' For Each objEmployees As Employees In objEmployeesCol
        '     cbx2.Items.Add(New ListItem(objEmployees.LastName, objEmployees.EmployeeID.ToString()))
        ' Next
    End Sub

    Private Sub Insert()
        ' first instantiate a new Employees
        Dim objEmployees As New Employees()

        ' assign values you want inserted
        objEmployees.LastName = "Davolio"
        objEmployees.FirstName = "Nancy"
        objEmployees.Title = "Sales Representative"
        objEmployees.TitleOfCourtesy = "Ms."
        objEmployees.BirthDate = DateTime.Now
        objEmployees.HireDate = DateTime.Now
        objEmployees.Address = "507 - 20th Ave. E. Apt. 2A"
        objEmployees.City = "Seattle"
        objEmployees.Region1 = "WA"
        objEmployees.PostalCode = "98122"
        objEmployees.Country = "USA"
        objEmployees.HomePhone = "(206) 555-9857"
        objEmployees.Extension = "5467"
        objEmployees.Notes = "Education includes a BA in psychology from Colorado State University in 1970.  She also completed ""The Art of the Cold Call.""  Nancy is a member of Toastmasters International."
        objEmployees.ReportsTo = 2
        objEmployees.PhotoPath = "http://accweb/emmployees/davolio.bmp"

        ' finally, insert a new record
        ' the insert method returns the newly created primary key
        Dim newlyCreatedPrimaryKey As Integer = objEmployees.Insert()
    End Sub

    Private Sub Update()
        ' first instantiate a new Employees
        Dim objEmployees As New Employees()

        ' assign the existing primary key(s)
        ' of the record you want updated
        objEmployees.EmployeeID = 1

        ' assign values you want updated
        objEmployees.LastName = "Davolio"
        objEmployees.FirstName = "Nancy"
        objEmployees.Title = "Sales Representative"
        objEmployees.TitleOfCourtesy = "Ms."
        objEmployees.BirthDate = DateTime.Now
        objEmployees.HireDate = DateTime.Now
        objEmployees.Address = "507 - 20th Ave. E. Apt. 2A"
        objEmployees.City = "Seattle"
        objEmployees.Region1 = "WA"
        objEmployees.PostalCode = "98122"
        objEmployees.Country = "USA"
        objEmployees.HomePhone = "(206) 555-9857"
        objEmployees.Extension = "5467"
        objEmployees.Notes = "Education includes a BA in psychology from Colorado State University in 1970.  She also completed ""The Art of the Cold Call.""  Nancy is a member of Toastmasters International."
        objEmployees.ReportsTo = 2
        objEmployees.PhotoPath = "http://accweb/emmployees/davolio.bmp"

        ' finally, update an existing record
        objEmployees.Update()
    End Sub

    Private Sub Delete()
        ' delete a record by primary key
        Employees.Delete(21)
    End Sub

    ''' <summary>
    ''' Shows how to get the total number of records
    ''' </summary>
    Private Sub GetRecordCount()
        ' get the total number of records in the Employees table
        Dim totalRecordCount As Integer = Employees.GetRecordCount()
    End Sub

    ''' <summary>
    ''' Shows how to get the total number of records by ReportsTo
    ''' </summary>
    Private Sub GetRecordCountByReportsTo()
        ' get the total number of records in the Employees table by ReportsTo
        ' 2 here is just a sample ReportsTo change the value as you see fit
        Dim totalRecordCount As Integer = Employees.GetRecordCountByReportsTo(2)
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index.  The total number of records are also retrieved when using the SelectSkipAndTake() method.
    ''' For example, if ther are 200 records (totalRecordCount), take only 10 records (numberOfRecordsToRetrieve), starting from the first index (startRetrievalFromRecordIndex = 0)
    ''' The example below uses some variables, here are their definitions:
    ''' totalRecordCount - total number of records if you were to retrieve everything
    ''' startRetrievalFromRecordIndex - the index to start taking records from. Zero (0) E.g. If you want to skip the first 20 records, then assign 19 here.
    ''' numberOfRecordsToRetrieve - take n records starting from the startRetrievalFromRecordIndex
    ''' sortBy - to sort in Ascending order by Field Name, just assign just the Field Name, do not pass 'asc'
    ''' sortBy - to sort in Descending order by Field Name, use the Field Name, a space and the word 'desc'
    ''' </summary>
    Private Sub SelectSkipAndTake()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "EmployeeID"
        ' Dim sortBy As String = "EmployeeID desc"

        ' 1. select a specific number of sorted records starting from the index you specify with totalRecordCount
        Dim objEmployeesCol As EmployeesCollection = Employees.SelectSkipAndTake(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy)

        ' 2. or you can also select a specific number of sorted records starting from the index you specify Without the totalRecordCount
        Dim objEmployeesCol2 As EmployeesCollection = Employees.SelectSkipAndTake(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, sortBy)

        ' to use objEmployeesCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the Employees(s).  The example above will only loop for 10 items.
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index by the related Field Name.  The total number of records are also retrieved when using the SelectSkipAndTake() method.
    ''' For example, if ther are 200 records (totalRecordCount), take only 10 records (numberOfRecordsToRetrieve), starting from the first index (startRetrievalFromRecordIndex = 0)
    ''' The example below uses some variables, here are their definitions:
    ''' totalRecordCount - total number of records if you were to retrieve everything
    ''' startRetrievalFromRecordIndex - the index to start taking records from. Zero (0) E.g. If you want to skip the first 20 records, then assign 19 here.
    ''' numberOfRecordsToRetrieve - take n records starting from the startRetrievalFromRecordIndex
    ''' sortBy - to sort in Ascending order by Field Name, just assign just the Field Name, do not pass 'asc'
    ''' sortBy - to sort in Descending order by Field Name, use the Field Name, a space and the word 'desc'
    ''' </summary>
    Private Sub SelectSkipAndTakeByReportsTo()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "EmployeeID"
        ' Dim sortBy As String = "EmployeeID desc"

        ' 1. select a specific number of sorted records with a ReportsTo = 2
        ' starting from the index you specify with totalRecordCount
        Dim objEmployeesCol As EmployeesCollection = Employees.SelectSkipAndTakeByReportsTo(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy, 2)

        ' to use objEmployeesCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the Employees(s).  The example above will only loop for 10 items.
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index, based on Search Parameters.  The number of records are also retrieved.
    ''' </summary>
    Private Sub SelectSkipAndTakeDynamicWhere()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "EmployeeID"
        ' Dim sortBy As String = "EmployeeID desc"

        ' search parameters, everything is nullable, only items being searched for should be filled
        ' note: fields with String type uses a LIKE search, everything else uses an exact match
        ' also, every field you're searching for uses the AND operator
        ' e.g. Dim productID As Integer? = 1, Dim productName As String = "ch"
        ' will translate to: SELECT....WHERE productID = 1 AND productName LIKE '%ch%'
        Dim employeeID As Integer? = Nothing
        Dim lastName As String = Nothing
        Dim firstName As String = Nothing
        Dim title As String = Nothing
        Dim titleOfCourtesy As String = Nothing
        Dim birthDate As Date? = Nothing
        Dim hireDate As Date? = Nothing
        Dim address As String = Nothing
        Dim city As String = Nothing
        Dim region1 As String = Nothing
        Dim postalCode As String = Nothing
        Dim country As String = Nothing
        Dim homePhone As String = Nothing
        Dim extension As String = Nothing
        Dim reportsTo As Integer? = Nothing
        Dim photoPath As String = Nothing

        ' 1. select a specific number of sorted records starting from the index you specify with totalRecordCount based on Search Parameters
        Dim objEmployeesCol As EmployeesCollection = Employees.SelectSkipAndTakeDynamicWhere(employeeID, lastName, firstName, title, titleOfCourtesy, birthDate, hireDate, address, city, region1, postalCode, country, homePhone, extension, reportsTo, photoPath, numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy)

        ' to use objEmployeesCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the Employees(s).  The example above will only loop for 10 items.
    End Sub
End Class
