Imports System
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports System.Windows.Forms

''' <summary>
''' These are data-centric code examples for the OrderDetails _table.
''' You can cut and paste the respective codes into your application
''' by changing the sample values assigned from these examples.
''' NOTE: This class contains private methods because they're
''' not meant to be called by an outside client.  Each method contains
''' code for the respective example being shown
''' </summary>
Public NotInheritable Class OrderDetailsExample

    Private Sub New()
    End Sub

    Private Sub SelectAll()
        ' select all records
        Dim objOrderDetailsCol As OrderDetailsCollection = OrderDetails.SelectAll()

        ' Example 1:  you can optionally sort the collection in ascending order by your chosen field  
        objOrderDetailsCol.Sort(OrderDetails.ByUnitPrice)

        ' Example 2:  to sort in descending order, add this line to the Sort code in Example 1
        objOrderDetailsCol.Reverse()

        ' Example 3:  assign to a DataGridView
        Dim grid As DataGridView = New DataGridView()
        grid.DataSource = objOrderDetailsCol

        ' Example 4:  loop through all the OrderDetails
        For Each objOrderDetails As OrderDetails In objOrderDetailsCol
            Dim orderID As Integer = objOrderDetails.OrderID
            Dim productID As Integer = objOrderDetails.ProductID
            Dim unitPrice As Decimal = objOrderDetails.UnitPrice
            Dim quantity As Short = objOrderDetails.Quantity
            Dim discount As Single = objOrderDetails.Discount

            ' optionally get the Orders related to OrderID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            Dim objOrdersRelatedToOrderID As Orders

            If objOrderDetails.Orders.IsValueCreated Then
                objOrdersRelatedToOrderID = objOrderDetails.Orders.Value
            End If

            ' optionally get the Products related to ProductID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            Dim objProductsRelatedToProductID As Products

            If objOrderDetails.Products.IsValueCreated Then
                objProductsRelatedToProductID = objOrderDetails.Products.Value
            End If
        Next
    End Sub

    Private Sub SelectByPrimaryKey()
        ' select a record by primary key(s)
        Dim objOrderDetails As OrderDetails = OrderDetails.SelectByPrimaryKey(10248, 
42)

        If objOrderDetails IsNot Nothing Then
            ' if record is found, a record is returned
            Dim orderID As Integer = objOrderDetails.OrderID
            Dim productID As Integer = objOrderDetails.ProductID
            Dim unitPrice As Decimal = objOrderDetails.UnitPrice
            Dim quantity As Short = objOrderDetails.Quantity
            Dim discount As Single = objOrderDetails.Discount

            ' optionally get the Orders related to OrderID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            Dim objOrdersRelatedToOrderID As Orders

            If objOrderDetails.Orders.IsValueCreated Then
                objOrdersRelatedToOrderID = objOrderDetails.Orders.Value
            End If

            ' optionally get the Products related to ProductID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            Dim objProductsRelatedToProductID As Products

            If objOrderDetails.Products.IsValueCreated Then
                objProductsRelatedToProductID = objOrderDetails.Products.Value
            End If
        End If
    End Sub

    ''' <summary>
    ''' Select all records by Orders, related to column OrderID
    ''' </summary> 
    Private Sub SelectOrderDetailsCollectionByOrderID()
        Dim objOrderDetailsCol As OrderDetailsCollection = OrderDetails.SelectOrderDetailsCollectionByOrderID(10248)

        ' Example 1:  you can optionally sort the collection in ascending order by your chosen field  
        objOrderDetailsCol.Sort(OrderDetails.ByUnitPrice)

        ' Example 2:  to sort in descending order, add this line to the Sort code in Example 1
        objOrderDetailsCol.Reverse()

        ' Example 3:  assign to a DataGridView
        Dim grid As DataGridView = New DataGridView()
        grid.DataSource = objOrderDetailsCol

        ' Example 4:  loop through all the OrderDetails
        For Each objOrderDetails As OrderDetails In objOrderDetailsCol
            Dim orderID As Integer = objOrderDetails.OrderID
            Dim productID As Integer = objOrderDetails.ProductID
            Dim unitPrice As Decimal = objOrderDetails.UnitPrice
            Dim quantity As Short = objOrderDetails.Quantity
            Dim discount As Single = objOrderDetails.Discount

            ' optionally get the Orders related to OrderID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            Dim objOrdersRelatedToOrderID As Orders

            If objOrderDetails.Orders.IsValueCreated Then
                objOrdersRelatedToOrderID = objOrderDetails.Orders.Value
            End If

            ' optionally get the Products related to ProductID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            Dim objProductsRelatedToProductID As Products

            If objOrderDetails.Products.IsValueCreated Then
                objProductsRelatedToProductID = objOrderDetails.Products.Value
            End If
        Next
    End Sub

    ''' <summary>
    ''' Select all records by Products, related to column ProductID
    ''' </summary> 
    Private Sub SelectOrderDetailsCollectionByProductID()
        Dim objOrderDetailsCol As OrderDetailsCollection = OrderDetails.SelectOrderDetailsCollectionByProductID(42)

        ' Example 1:  you can optionally sort the collection in ascending order by your chosen field  
        objOrderDetailsCol.Sort(OrderDetails.ByUnitPrice)

        ' Example 2:  to sort in descending order, add this line to the Sort code in Example 1
        objOrderDetailsCol.Reverse()

        ' Example 3:  assign to a DataGridView
        Dim grid As DataGridView = New DataGridView()
        grid.DataSource = objOrderDetailsCol

        ' Example 4:  loop through all the OrderDetails
        For Each objOrderDetails As OrderDetails In objOrderDetailsCol
            Dim orderID As Integer = objOrderDetails.OrderID
            Dim productID As Integer = objOrderDetails.ProductID
            Dim unitPrice As Decimal = objOrderDetails.UnitPrice
            Dim quantity As Short = objOrderDetails.Quantity
            Dim discount As Single = objOrderDetails.Discount

            ' optionally get the Orders related to OrderID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            Dim objOrdersRelatedToOrderID As Orders

            If objOrderDetails.Orders.IsValueCreated Then
                objOrdersRelatedToOrderID = objOrderDetails.Orders.Value
            End If

            ' optionally get the Products related to ProductID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            Dim objProductsRelatedToProductID As Products

            If objOrderDetails.Products.IsValueCreated Then
                objProductsRelatedToProductID = objOrderDetails.Products.Value
            End If
        Next
    End Sub

    Private Sub Insert()
        ' first instantiate a new OrderDetails
        Dim objOrderDetails As New OrderDetails()

        ' assign values you want inserted
        objOrderDetails.OrderID = 11078
        objOrderDetails.ProductID = 201
        objOrderDetails.UnitPrice = Convert.ToDecimal(9.8000)
        objOrderDetails.Quantity = 10
        objOrderDetails.Discount = 1

        ' finally, insert a new record
        objOrderDetails.Insert()
    End Sub

    Private Sub Update()
        ' first instantiate a new OrderDetails
        Dim objOrderDetails As New OrderDetails()

        ' assign the existing primary key(s)
        ' of the record you want updated
        objOrderDetails.OrderID = 10248
        objOrderDetails.ProductID = 42

        ' assign values you want updated
        objOrderDetails.UnitPrice = Convert.ToDecimal(9.8000)
        objOrderDetails.Quantity = 10
        objOrderDetails.Discount = 1

        ' finally, update an existing record
        objOrderDetails.Update()
    End Sub

    Private Sub Delete()
        ' delete a record by primary key
        OrderDetails.Delete(11078, 
201)
    End Sub

    ''' <summary>
    ''' Shows how to get the total number of records
    ''' </summary>
    Private Sub GetRecordCount()
        ' get the total number of records in the OrderDetails table
        Dim totalRecordCount As Integer = OrderDetails.GetRecordCount()
    End Sub

    ''' <summary>
    ''' Shows how to get the total number of records by OrderID
    ''' </summary>
    Private Sub GetRecordCountByOrderID()
        ' get the total number of records in the OrderDetails table by OrderID
        ' 10248 here is just a sample OrderID change the value as you see fit
        Dim totalRecordCount As Integer = OrderDetails.GetRecordCountByOrderID(10248)
    End Sub

    ''' <summary>
    ''' Shows how to get the total number of records by ProductID
    ''' </summary>
    Private Sub GetRecordCountByProductID()
        ' get the total number of records in the OrderDetails table by ProductID
        ' 42 here is just a sample ProductID change the value as you see fit
        Dim totalRecordCount As Integer = OrderDetails.GetRecordCountByProductID(42)
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index.  The total number of records are also retrieved when using the SelectSkipAndTake() method.
    ''' For example, if ther are 200 records (totalRecordCount), take only 10 records (numberOfRecordsToRetrieve), starting from the first index (startRetrievalFromRecordIndex = 0)
    ''' The example below uses some variables, here are their definitions:
    ''' totalRecordCount - total number of records if you were to retrieve everything
    ''' startRetrievalFromRecordIndex - the index to start taking records from. Zero (0) E.g. If you want to skip the first 20 records, then assign 19 here.
    ''' numberOfRecordsToRetrieve - take n records starting from the startRetrievalFromRecordIndex
    ''' sortBy - to sort in Ascending order by Field Name, just assign just the Field Name, do not pass 'asc'
    ''' sortBy - to sort in Descending order by Field Name, use the Field Name, a space and the word 'desc'
    ''' </summary>
    Private Sub SelectSkipAndTake()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "OrderID"
        ' Dim sortBy As String = "OrderID desc"

        ' 1. select a specific number of sorted records starting from the index you specify with totalRecordCount
        Dim objOrderDetailsCol As OrderDetailsCollection = OrderDetails.SelectSkipAndTake(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy)

        ' 2. or you can also select a specific number of sorted records starting from the index you specify Without the totalRecordCount
        Dim objOrderDetailsCol2 As OrderDetailsCollection = OrderDetails.SelectSkipAndTake(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, sortBy)

        ' to use objOrderDetailsCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the OrderDetails(s).  The example above will only loop for 10 items.
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index by the related Field Name.  The total number of records are also retrieved when using the SelectSkipAndTake() method.
    ''' For example, if ther are 200 records (totalRecordCount), take only 10 records (numberOfRecordsToRetrieve), starting from the first index (startRetrievalFromRecordIndex = 0)
    ''' The example below uses some variables, here are their definitions:
    ''' totalRecordCount - total number of records if you were to retrieve everything
    ''' startRetrievalFromRecordIndex - the index to start taking records from. Zero (0) E.g. If you want to skip the first 20 records, then assign 19 here.
    ''' numberOfRecordsToRetrieve - take n records starting from the startRetrievalFromRecordIndex
    ''' sortBy - to sort in Ascending order by Field Name, just assign just the Field Name, do not pass 'asc'
    ''' sortBy - to sort in Descending order by Field Name, use the Field Name, a space and the word 'desc'
    ''' </summary>
    Private Sub SelectSkipAndTakeByOrderID()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "OrderID"
        ' Dim sortBy As String = "OrderID desc"

        ' 1. select a specific number of sorted records with a OrderID = 10248
        ' starting from the index you specify with totalRecordCount
        Dim objOrderDetailsCol As OrderDetailsCollection = OrderDetails.SelectSkipAndTakeByOrderID(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy, 10248)

        ' to use objOrderDetailsCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the OrderDetails(s).  The example above will only loop for 10 items.
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index by the related Field Name.  The total number of records are also retrieved when using the SelectSkipAndTake() method.
    ''' For example, if ther are 200 records (totalRecordCount), take only 10 records (numberOfRecordsToRetrieve), starting from the first index (startRetrievalFromRecordIndex = 0)
    ''' The example below uses some variables, here are their definitions:
    ''' totalRecordCount - total number of records if you were to retrieve everything
    ''' startRetrievalFromRecordIndex - the index to start taking records from. Zero (0) E.g. If you want to skip the first 20 records, then assign 19 here.
    ''' numberOfRecordsToRetrieve - take n records starting from the startRetrievalFromRecordIndex
    ''' sortBy - to sort in Ascending order by Field Name, just assign just the Field Name, do not pass 'asc'
    ''' sortBy - to sort in Descending order by Field Name, use the Field Name, a space and the word 'desc'
    ''' </summary>
    Private Sub SelectSkipAndTakeByProductID()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "OrderID"
        ' Dim sortBy As String = "OrderID desc"

        ' 1. select a specific number of sorted records with a ProductID = 42
        ' starting from the index you specify with totalRecordCount
        Dim objOrderDetailsCol As OrderDetailsCollection = OrderDetails.SelectSkipAndTakeByProductID(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy, 42)

        ' to use objOrderDetailsCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the OrderDetails(s).  The example above will only loop for 10 items.
    End Sub

    ''' <summary>
    ''' Shows how to get fields with Totals
    ''' </summary>
    Private Sub SelectTotals()
        ' get all fields with totals for the OrderDetails table
        Dim objOrderDetails As OrderDetails = OrderDetails.SelectTotals()

        ' assign each field with a total
        Dim unitPriceTotal As Decimal = objOrderDetails.UnitPriceTotal
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index, based on Search Parameters.  The number of records are also retrieved.
    ''' </summary>
    Private Sub SelectSkipAndTakeDynamicWhere()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "OrderID"
        ' Dim sortBy As String = "OrderID desc"

        ' search parameters, everything is nullable, only items being searched for should be filled
        ' note: fields with String type uses a LIKE search, everything else uses an exact match
        ' also, every field you're searching for uses the AND operator
        ' e.g. Dim productID As Integer? = 1, Dim productName As String = "ch"
        ' will translate to: SELECT....WHERE productID = 1 AND productName LIKE '%ch%'
        Dim orderID As Integer? = Nothing
        Dim productID As Integer? = Nothing
        Dim unitPrice As Decimal? = Nothing
        Dim quantity As Short? = Nothing
        Dim discount As Single? = Nothing

        ' 1. select a specific number of sorted records starting from the index you specify with totalRecordCount based on Search Parameters
        Dim objOrderDetailsCol As OrderDetailsCollection = OrderDetails.SelectSkipAndTakeDynamicWhere(orderID, productID, unitPrice, quantity, discount, numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy)

        ' to use objOrderDetailsCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the OrderDetails(s).  The example above will only loop for 10 items.
    End Sub
End Class
