Imports System
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports System.Windows.Forms

''' <summary>
''' These are data-centric code examples for the Orders _table.
''' You can cut and paste the respective codes into your application
''' by changing the sample values assigned from these examples.
''' NOTE: This class contains private methods because they're
''' not meant to be called by an outside client.  Each method contains
''' code for the respective example being shown
''' </summary>
Public NotInheritable Class OrdersExample

    Private Sub New()
    End Sub

    Private Sub SelectAll()
        ' select all records
        Dim objOrdersCol As OrdersCollection = Orders.SelectAll()

        ' Example 1:  you can optionally sort the collection in ascending order by your chosen field  
        objOrdersCol.Sort(Orders.ByCustomerID)

        ' Example 2:  to sort in descending order, add this line to the Sort code in Example 1
        objOrdersCol.Reverse()

        ' Example 3:  assign to a DataGridView
        Dim grid As DataGridView = New DataGridView()
        grid.DataSource = objOrdersCol

        ' Example 4:  loop through all the Orders
        For Each objOrders As Orders In objOrdersCol
            Dim orderID As Integer = objOrders.OrderID
            Dim customerID As String = objOrders.CustomerID
            Dim employeeID As Integer = objOrders.EmployeeID
            Dim orderDate As Date = objOrders.OrderDate
            Dim requiredDate As Date = objOrders.RequiredDate
            Dim shippedDate As Date = objOrders.ShippedDate
            Dim shipVia As Integer = objOrders.ShipVia
            Dim freight As Decimal = objOrders.Freight
            Dim shipName As String = objOrders.ShipName
            Dim shipAddress As String = objOrders.ShipAddress
            Dim shipCity As String = objOrders.ShipCity
            Dim shipRegion As String = objOrders.ShipRegion
            Dim shipPostalCode As String = objOrders.ShipPostalCode
            Dim shipCountry As String = objOrders.ShipCountry

            ' optionally get the Customers related to CustomerID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            If Not String.IsNullOrEmpty(objOrders.CustomerID) Then
                Dim objCustomersRelatedToCustomerID As Customers

                If objOrders.Customers.IsValueCreated Then
                    objCustomersRelatedToCustomerID = objOrders.Customers.Value
                End If
            End If

            ' optionally get the Employees related to EmployeeID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            If objOrders.EmployeeID IsNot Nothing Then
                Dim objEmployeesRelatedToEmployeeID As Employees

                If objOrders.Employees.IsValueCreated Then
                    objEmployeesRelatedToEmployeeID = objOrders.Employees.Value
                End If
            End If

            ' optionally get the Shippers related to ShipVia.
            ' Note this is lazily loaded which means there's no value until you ask for it
            If objOrders.ShipVia IsNot Nothing Then
                Dim objShippersRelatedToShipVia As Shippers

                If objOrders.Shippers.IsValueCreated Then
                    objShippersRelatedToShipVia = objOrders.Shippers.Value
                End If
            End If
        Next
    End Sub

    Private Sub SelectByPrimaryKey()
        ' select a record by primary key(s)
        Dim objOrders As Orders = Orders.SelectByPrimaryKey(10248)

        If objOrders IsNot Nothing Then
            ' if record is found, a record is returned
            Dim orderID As Integer = objOrders.OrderID
            Dim customerID As String = objOrders.CustomerID
            Dim employeeID As Integer = objOrders.EmployeeID
            Dim orderDate As Date = objOrders.OrderDate
            Dim requiredDate As Date = objOrders.RequiredDate
            Dim shippedDate As Date = objOrders.ShippedDate
            Dim shipVia As Integer = objOrders.ShipVia
            Dim freight As Decimal = objOrders.Freight
            Dim shipName As String = objOrders.ShipName
            Dim shipAddress As String = objOrders.ShipAddress
            Dim shipCity As String = objOrders.ShipCity
            Dim shipRegion As String = objOrders.ShipRegion
            Dim shipPostalCode As String = objOrders.ShipPostalCode
            Dim shipCountry As String = objOrders.ShipCountry

            ' optionally get the Customers related to CustomerID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            If Not String.IsNullOrEmpty(objOrders.CustomerID) Then
                Dim objCustomersRelatedToCustomerID As Customers

                If objOrders.Customers.IsValueCreated Then
                    objCustomersRelatedToCustomerID = objOrders.Customers.Value
                End If
            End If

            ' optionally get the Employees related to EmployeeID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            If objOrders.EmployeeID IsNot Nothing Then
                Dim objEmployeesRelatedToEmployeeID As Employees

                If objOrders.Employees.IsValueCreated Then
                    objEmployeesRelatedToEmployeeID = objOrders.Employees.Value
                End If
            End If

            ' optionally get the Shippers related to ShipVia.
            ' Note this is lazily loaded which means there's no value until you ask for it
            If objOrders.ShipVia IsNot Nothing Then
                Dim objShippersRelatedToShipVia As Shippers

                If objOrders.Shippers.IsValueCreated Then
                    objShippersRelatedToShipVia = objOrders.Shippers.Value
                End If
            End If
        End If
    End Sub

    ''' <summary>
    ''' Select all records by Customers, related to column CustomerID
    ''' </summary> 
    Private Sub SelectOrdersCollectionByCustomerID()
        Dim objOrdersCol As OrdersCollection = Orders.SelectOrdersCollectionByCustomerID("VINET")

        ' Example 1:  you can optionally sort the collection in ascending order by your chosen field  
        objOrdersCol.Sort(Orders.ByCustomerID)

        ' Example 2:  to sort in descending order, add this line to the Sort code in Example 1
        objOrdersCol.Reverse()

        ' Example 3:  assign to a DataGridView
        Dim grid As DataGridView = New DataGridView()
        grid.DataSource = objOrdersCol

        ' Example 4:  loop through all the Orders
        For Each objOrders As Orders In objOrdersCol
            Dim orderID As Integer = objOrders.OrderID
            Dim customerID As String = objOrders.CustomerID
            Dim employeeID As Integer = objOrders.EmployeeID
            Dim orderDate As Date = objOrders.OrderDate
            Dim requiredDate As Date = objOrders.RequiredDate
            Dim shippedDate As Date = objOrders.ShippedDate
            Dim shipVia As Integer = objOrders.ShipVia
            Dim freight As Decimal = objOrders.Freight
            Dim shipName As String = objOrders.ShipName
            Dim shipAddress As String = objOrders.ShipAddress
            Dim shipCity As String = objOrders.ShipCity
            Dim shipRegion As String = objOrders.ShipRegion
            Dim shipPostalCode As String = objOrders.ShipPostalCode
            Dim shipCountry As String = objOrders.ShipCountry

            ' optionally get the Customers related to CustomerID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            If Not String.IsNullOrEmpty(objOrders.CustomerID) Then
                Dim objCustomersRelatedToCustomerID As Customers

                If objOrders.Customers.IsValueCreated Then
                    objCustomersRelatedToCustomerID = objOrders.Customers.Value
                End If
            End If

            ' optionally get the Employees related to EmployeeID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            If objOrders.EmployeeID IsNot Nothing Then
                Dim objEmployeesRelatedToEmployeeID As Employees

                If objOrders.Employees.IsValueCreated Then
                    objEmployeesRelatedToEmployeeID = objOrders.Employees.Value
                End If
            End If

            ' optionally get the Shippers related to ShipVia.
            ' Note this is lazily loaded which means there's no value until you ask for it
            If objOrders.ShipVia IsNot Nothing Then
                Dim objShippersRelatedToShipVia As Shippers

                If objOrders.Shippers.IsValueCreated Then
                    objShippersRelatedToShipVia = objOrders.Shippers.Value
                End If
            End If
        Next
    End Sub

    ''' <summary>
    ''' Select all records by Employees, related to column EmployeeID
    ''' </summary> 
    Private Sub SelectOrdersCollectionByEmployeeID()
        Dim objOrdersCol As OrdersCollection = Orders.SelectOrdersCollectionByEmployeeID(5)

        ' Example 1:  you can optionally sort the collection in ascending order by your chosen field  
        objOrdersCol.Sort(Orders.ByCustomerID)

        ' Example 2:  to sort in descending order, add this line to the Sort code in Example 1
        objOrdersCol.Reverse()

        ' Example 3:  assign to a DataGridView
        Dim grid As DataGridView = New DataGridView()
        grid.DataSource = objOrdersCol

        ' Example 4:  loop through all the Orders
        For Each objOrders As Orders In objOrdersCol
            Dim orderID As Integer = objOrders.OrderID
            Dim customerID As String = objOrders.CustomerID
            Dim employeeID As Integer = objOrders.EmployeeID
            Dim orderDate As Date = objOrders.OrderDate
            Dim requiredDate As Date = objOrders.RequiredDate
            Dim shippedDate As Date = objOrders.ShippedDate
            Dim shipVia As Integer = objOrders.ShipVia
            Dim freight As Decimal = objOrders.Freight
            Dim shipName As String = objOrders.ShipName
            Dim shipAddress As String = objOrders.ShipAddress
            Dim shipCity As String = objOrders.ShipCity
            Dim shipRegion As String = objOrders.ShipRegion
            Dim shipPostalCode As String = objOrders.ShipPostalCode
            Dim shipCountry As String = objOrders.ShipCountry

            ' optionally get the Customers related to CustomerID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            If Not String.IsNullOrEmpty(objOrders.CustomerID) Then
                Dim objCustomersRelatedToCustomerID As Customers

                If objOrders.Customers.IsValueCreated Then
                    objCustomersRelatedToCustomerID = objOrders.Customers.Value
                End If
            End If

            ' optionally get the Employees related to EmployeeID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            If objOrders.EmployeeID IsNot Nothing Then
                Dim objEmployeesRelatedToEmployeeID As Employees

                If objOrders.Employees.IsValueCreated Then
                    objEmployeesRelatedToEmployeeID = objOrders.Employees.Value
                End If
            End If

            ' optionally get the Shippers related to ShipVia.
            ' Note this is lazily loaded which means there's no value until you ask for it
            If objOrders.ShipVia IsNot Nothing Then
                Dim objShippersRelatedToShipVia As Shippers

                If objOrders.Shippers.IsValueCreated Then
                    objShippersRelatedToShipVia = objOrders.Shippers.Value
                End If
            End If
        Next
    End Sub

    ''' <summary>
    ''' Select all records by Shippers, related to column ShipVia
    ''' </summary> 
    Private Sub SelectOrdersCollectionByShipVia()
        Dim objOrdersCol As OrdersCollection = Orders.SelectOrdersCollectionByShipVia(3)

        ' Example 1:  you can optionally sort the collection in ascending order by your chosen field  
        objOrdersCol.Sort(Orders.ByCustomerID)

        ' Example 2:  to sort in descending order, add this line to the Sort code in Example 1
        objOrdersCol.Reverse()

        ' Example 3:  assign to a DataGridView
        Dim grid As DataGridView = New DataGridView()
        grid.DataSource = objOrdersCol

        ' Example 4:  loop through all the Orders
        For Each objOrders As Orders In objOrdersCol
            Dim orderID As Integer = objOrders.OrderID
            Dim customerID As String = objOrders.CustomerID
            Dim employeeID As Integer = objOrders.EmployeeID
            Dim orderDate As Date = objOrders.OrderDate
            Dim requiredDate As Date = objOrders.RequiredDate
            Dim shippedDate As Date = objOrders.ShippedDate
            Dim shipVia As Integer = objOrders.ShipVia
            Dim freight As Decimal = objOrders.Freight
            Dim shipName As String = objOrders.ShipName
            Dim shipAddress As String = objOrders.ShipAddress
            Dim shipCity As String = objOrders.ShipCity
            Dim shipRegion As String = objOrders.ShipRegion
            Dim shipPostalCode As String = objOrders.ShipPostalCode
            Dim shipCountry As String = objOrders.ShipCountry

            ' optionally get the Customers related to CustomerID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            If Not String.IsNullOrEmpty(objOrders.CustomerID) Then
                Dim objCustomersRelatedToCustomerID As Customers

                If objOrders.Customers.IsValueCreated Then
                    objCustomersRelatedToCustomerID = objOrders.Customers.Value
                End If
            End If

            ' optionally get the Employees related to EmployeeID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            If objOrders.EmployeeID IsNot Nothing Then
                Dim objEmployeesRelatedToEmployeeID As Employees

                If objOrders.Employees.IsValueCreated Then
                    objEmployeesRelatedToEmployeeID = objOrders.Employees.Value
                End If
            End If

            ' optionally get the Shippers related to ShipVia.
            ' Note this is lazily loaded which means there's no value until you ask for it
            If objOrders.ShipVia IsNot Nothing Then
                Dim objShippersRelatedToShipVia As Shippers

                If objOrders.Shippers.IsValueCreated Then
                    objShippersRelatedToShipVia = objOrders.Shippers.Value
                End If
            End If
        Next
    End Sub

    ''' <summary>
    ''' Selects OrderID and ShipName columns for use with a with a Drop Down List, Combo Box, Checked Box List, List View, List Box, etc
    ''' </summary>
    Private Sub SelectOrdersDropDownListData()
        Dim objOrdersCol As OrdersCollection = Orders.SelectOrdersDropDownListData()

        ' Example 1:  directly bind to a combo box
        Dim cbx1 As ComboBox = New ComboBox()
        cbx1.ValueMember = "OrderID"
        cbx1.DisplayMember = "ShipName"
        cbx1.DataSource = objOrdersCol

        ' Example 2:  add each item through a loop.  Web app example
        ' Dim cbx2 As DropDownList = New DropDownList()

        ' For Each objOrders As Orders In objOrdersCol
        '     cbx2.Items.Add(New ListItem(objOrders.ShipName, objOrders.OrderID.ToString()))
        ' Next
    End Sub

    Private Sub Insert()
        ' first instantiate a new Orders
        Dim objOrders As New Orders()

        ' assign values you want inserted
        objOrders.CustomerID = "VINET"
        objOrders.EmployeeID = 5
        objOrders.OrderDate = DateTime.Now
        objOrders.RequiredDate = DateTime.Now
        objOrders.ShippedDate = DateTime.Now
        objOrders.ShipVia = 3
        objOrders.Freight = Convert.ToDecimal(32.3800)
        objOrders.ShipName = "Vins et alcools Chevalier"
        objOrders.ShipAddress = "59 rue de l'Abbaye"
        objOrders.ShipCity = "Reims"
        objOrders.ShipRegion = "abc"
        objOrders.ShipPostalCode = "51100"
        objOrders.ShipCountry = "France"

        ' finally, insert a new record
        ' the insert method returns the newly created primary key
        Dim newlyCreatedPrimaryKey As Integer = objOrders.Insert()
    End Sub

    Private Sub Update()
        ' first instantiate a new Orders
        Dim objOrders As New Orders()

        ' assign the existing primary key(s)
        ' of the record you want updated
        objOrders.OrderID = 10248

        ' assign values you want updated
        objOrders.CustomerID = "VINET"
        objOrders.EmployeeID = 5
        objOrders.OrderDate = DateTime.Now
        objOrders.RequiredDate = DateTime.Now
        objOrders.ShippedDate = DateTime.Now
        objOrders.ShipVia = 3
        objOrders.Freight = Convert.ToDecimal(32.3800)
        objOrders.ShipName = "Vins et alcools Chevalier"
        objOrders.ShipAddress = "59 rue de l'Abbaye"
        objOrders.ShipCity = "Reims"
        objOrders.ShipRegion = "abc"
        objOrders.ShipPostalCode = "51100"
        objOrders.ShipCountry = "France"

        ' finally, update an existing record
        objOrders.Update()
    End Sub

    Private Sub Delete()
        ' delete a record by primary key
        Orders.Delete(11095)
    End Sub

    ''' <summary>
    ''' Shows how to get the total number of records
    ''' </summary>
    Private Sub GetRecordCount()
        ' get the total number of records in the Orders table
        Dim totalRecordCount As Integer = Orders.GetRecordCount()
    End Sub

    ''' <summary>
    ''' Shows how to get the total number of records by CustomerID
    ''' </summary>
    Private Sub GetRecordCountByCustomerID()
        ' get the total number of records in the Orders table by CustomerID
        ' "VINET" here is just a sample CustomerID change the value as you see fit
        Dim totalRecordCount As Integer = Orders.GetRecordCountByCustomerID("VINET")
    End Sub

    ''' <summary>
    ''' Shows how to get the total number of records by EmployeeID
    ''' </summary>
    Private Sub GetRecordCountByEmployeeID()
        ' get the total number of records in the Orders table by EmployeeID
        ' 5 here is just a sample EmployeeID change the value as you see fit
        Dim totalRecordCount As Integer = Orders.GetRecordCountByEmployeeID(5)
    End Sub

    ''' <summary>
    ''' Shows how to get the total number of records by ShipVia
    ''' </summary>
    Private Sub GetRecordCountByShipVia()
        ' get the total number of records in the Orders table by ShipVia
        ' 3 here is just a sample ShipVia change the value as you see fit
        Dim totalRecordCount As Integer = Orders.GetRecordCountByShipVia(3)
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index.  The total number of records are also retrieved when using the SelectSkipAndTake() method.
    ''' For example, if ther are 200 records (totalRecordCount), take only 10 records (numberOfRecordsToRetrieve), starting from the first index (startRetrievalFromRecordIndex = 0)
    ''' The example below uses some variables, here are their definitions:
    ''' totalRecordCount - total number of records if you were to retrieve everything
    ''' startRetrievalFromRecordIndex - the index to start taking records from. Zero (0) E.g. If you want to skip the first 20 records, then assign 19 here.
    ''' numberOfRecordsToRetrieve - take n records starting from the startRetrievalFromRecordIndex
    ''' sortBy - to sort in Ascending order by Field Name, just assign just the Field Name, do not pass 'asc'
    ''' sortBy - to sort in Descending order by Field Name, use the Field Name, a space and the word 'desc'
    ''' </summary>
    Private Sub SelectSkipAndTake()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "OrderID"
        ' Dim sortBy As String = "OrderID desc"

        ' 1. select a specific number of sorted records starting from the index you specify with totalRecordCount
        Dim objOrdersCol As OrdersCollection = Orders.SelectSkipAndTake(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy)

        ' 2. or you can also select a specific number of sorted records starting from the index you specify Without the totalRecordCount
        Dim objOrdersCol2 As OrdersCollection = Orders.SelectSkipAndTake(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, sortBy)

        ' to use objOrdersCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the Orders(s).  The example above will only loop for 10 items.
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index by the related Field Name.  The total number of records are also retrieved when using the SelectSkipAndTake() method.
    ''' For example, if ther are 200 records (totalRecordCount), take only 10 records (numberOfRecordsToRetrieve), starting from the first index (startRetrievalFromRecordIndex = 0)
    ''' The example below uses some variables, here are their definitions:
    ''' totalRecordCount - total number of records if you were to retrieve everything
    ''' startRetrievalFromRecordIndex - the index to start taking records from. Zero (0) E.g. If you want to skip the first 20 records, then assign 19 here.
    ''' numberOfRecordsToRetrieve - take n records starting from the startRetrievalFromRecordIndex
    ''' sortBy - to sort in Ascending order by Field Name, just assign just the Field Name, do not pass 'asc'
    ''' sortBy - to sort in Descending order by Field Name, use the Field Name, a space and the word 'desc'
    ''' </summary>
    Private Sub SelectSkipAndTakeByCustomerID()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "OrderID"
        ' Dim sortBy As String = "OrderID desc"

        ' 1. select a specific number of sorted records with a CustomerID = "VINET"
        ' starting from the index you specify with totalRecordCount
        Dim objOrdersCol As OrdersCollection = Orders.SelectSkipAndTakeByCustomerID(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy, "VINET")

        ' to use objOrdersCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the Orders(s).  The example above will only loop for 10 items.
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index by the related Field Name.  The total number of records are also retrieved when using the SelectSkipAndTake() method.
    ''' For example, if ther are 200 records (totalRecordCount), take only 10 records (numberOfRecordsToRetrieve), starting from the first index (startRetrievalFromRecordIndex = 0)
    ''' The example below uses some variables, here are their definitions:
    ''' totalRecordCount - total number of records if you were to retrieve everything
    ''' startRetrievalFromRecordIndex - the index to start taking records from. Zero (0) E.g. If you want to skip the first 20 records, then assign 19 here.
    ''' numberOfRecordsToRetrieve - take n records starting from the startRetrievalFromRecordIndex
    ''' sortBy - to sort in Ascending order by Field Name, just assign just the Field Name, do not pass 'asc'
    ''' sortBy - to sort in Descending order by Field Name, use the Field Name, a space and the word 'desc'
    ''' </summary>
    Private Sub SelectSkipAndTakeByEmployeeID()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "OrderID"
        ' Dim sortBy As String = "OrderID desc"

        ' 1. select a specific number of sorted records with a EmployeeID = 5
        ' starting from the index you specify with totalRecordCount
        Dim objOrdersCol As OrdersCollection = Orders.SelectSkipAndTakeByEmployeeID(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy, 5)

        ' to use objOrdersCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the Orders(s).  The example above will only loop for 10 items.
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index by the related Field Name.  The total number of records are also retrieved when using the SelectSkipAndTake() method.
    ''' For example, if ther are 200 records (totalRecordCount), take only 10 records (numberOfRecordsToRetrieve), starting from the first index (startRetrievalFromRecordIndex = 0)
    ''' The example below uses some variables, here are their definitions:
    ''' totalRecordCount - total number of records if you were to retrieve everything
    ''' startRetrievalFromRecordIndex - the index to start taking records from. Zero (0) E.g. If you want to skip the first 20 records, then assign 19 here.
    ''' numberOfRecordsToRetrieve - take n records starting from the startRetrievalFromRecordIndex
    ''' sortBy - to sort in Ascending order by Field Name, just assign just the Field Name, do not pass 'asc'
    ''' sortBy - to sort in Descending order by Field Name, use the Field Name, a space and the word 'desc'
    ''' </summary>
    Private Sub SelectSkipAndTakeByShipVia()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "OrderID"
        ' Dim sortBy As String = "OrderID desc"

        ' 1. select a specific number of sorted records with a ShipVia = 3
        ' starting from the index you specify with totalRecordCount
        Dim objOrdersCol As OrdersCollection = Orders.SelectSkipAndTakeByShipVia(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy, 3)

        ' to use objOrdersCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the Orders(s).  The example above will only loop for 10 items.
    End Sub

    ''' <summary>
    ''' Shows how to get fields with Totals
    ''' </summary>
    Private Sub SelectTotals()
        ' get all fields with totals for the Orders table
        Dim objOrders As Orders = Orders.SelectTotals()

        ' assign each field with a total
        Dim freightTotal As Decimal = objOrders.FreightTotal
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index, based on Search Parameters.  The number of records are also retrieved.
    ''' </summary>
    Private Sub SelectSkipAndTakeDynamicWhere()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "OrderID"
        ' Dim sortBy As String = "OrderID desc"

        ' search parameters, everything is nullable, only items being searched for should be filled
        ' note: fields with String type uses a LIKE search, everything else uses an exact match
        ' also, every field you're searching for uses the AND operator
        ' e.g. Dim productID As Integer? = 1, Dim productName As String = "ch"
        ' will translate to: SELECT....WHERE productID = 1 AND productName LIKE '%ch%'
        Dim orderID As Integer? = Nothing
        Dim customerID As String = Nothing
        Dim employeeID As Integer? = Nothing
        Dim orderDate As Date? = Nothing
        Dim requiredDate As Date? = Nothing
        Dim shippedDate As Date? = Nothing
        Dim shipVia As Integer? = Nothing
        Dim freight As Decimal? = Nothing
        Dim shipName As String = Nothing
        Dim shipAddress As String = Nothing
        Dim shipCity As String = Nothing
        Dim shipRegion As String = Nothing
        Dim shipPostalCode As String = Nothing
        Dim shipCountry As String = Nothing

        ' 1. select a specific number of sorted records starting from the index you specify with totalRecordCount based on Search Parameters
        Dim objOrdersCol As OrdersCollection = Orders.SelectSkipAndTakeDynamicWhere(orderID, customerID, employeeID, orderDate, requiredDate, shippedDate, shipVia, freight, shipName, shipAddress, shipCity, shipRegion, shipPostalCode, shipCountry, numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy)

        ' to use objOrdersCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the Orders(s).  The example above will only loop for 10 items.
    End Sub
End Class
