Imports System
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports System.Windows.Forms

''' <summary>
''' These are data-centric code examples for the Region _table.
''' You can cut and paste the respective codes into your application
''' by changing the sample values assigned from these examples.
''' NOTE: This class contains private methods because they're
''' not meant to be called by an outside client.  Each method contains
''' code for the respective example being shown
''' </summary>
Public NotInheritable Class RegionExample

    Private Sub New()
    End Sub

    Private Sub SelectAll()
        ' select all records
        Dim objRegionCol As RegionCollection = Region.SelectAll()

        ' Example 1:  you can optionally sort the collection in ascending order by your chosen field  
        objRegionCol.Sort(Region.ByRegionDescription)

        ' Example 2:  to sort in descending order, add this line to the Sort code in Example 1
        objRegionCol.Reverse()

        ' Example 3:  assign to a DataGridView
        Dim grid As DataGridView = New DataGridView()
        grid.DataSource = objRegionCol

        ' Example 4:  loop through all the Region
        For Each objRegion As Region In objRegionCol
            Dim regionID As Integer = objRegion.RegionID
            Dim regionDescription As String = objRegion.RegionDescription
        Next
    End Sub

    Private Sub SelectByPrimaryKey()
        ' select a record by primary key(s)
        Dim objRegion As Region = Region.SelectByPrimaryKey(1)

        If objRegion IsNot Nothing Then
            ' if record is found, a record is returned
            Dim regionID As Integer = objRegion.RegionID
            Dim regionDescription As String = objRegion.RegionDescription
        End If
    End Sub

    ''' <summary>
    ''' Selects RegionID and RegionDescription columns for use with a with a Drop Down List, Combo Box, Checked Box List, List View, List Box, etc
    ''' </summary>
    Private Sub SelectRegionDropDownListData()
        Dim objRegionCol As RegionCollection = Region.SelectRegionDropDownListData()

        ' Example 1:  directly bind to a combo box
        Dim cbx1 As ComboBox = New ComboBox()
        cbx1.ValueMember = "RegionID"
        cbx1.DisplayMember = "RegionDescription"
        cbx1.DataSource = objRegionCol

        ' Example 2:  add each item through a loop.  Web app example
        ' Dim cbx2 As DropDownList = New DropDownList()

        ' For Each objRegion As Region In objRegionCol
        '     cbx2.Items.Add(New ListItem(objRegion.RegionDescription, objRegion.RegionID.ToString()))
        ' Next
    End Sub

    Private Sub Insert()
        ' first instantiate a new Region
        Dim objRegion As New Region()

        ' assign values you want inserted
        objRegion.RegionID = 5
        objRegion.RegionDescription = "Eastern"

        ' finally, insert a new record
        ' the insert method returns the newly created primary key
        Dim newlyCreatedPrimaryKey As Integer = objRegion.Insert()
    End Sub

    Private Sub Update()
        ' first instantiate a new Region
        Dim objRegion As New Region()

        ' assign the existing primary key(s)
        ' of the record you want updated
        objRegion.RegionID = 1

        ' assign values you want updated
        objRegion.RegionDescription = "Eastern"

        ' finally, update an existing record
        objRegion.Update()
    End Sub

    Private Sub Delete()
        ' delete a record by primary key
        Region.Delete(5)
    End Sub

    ''' <summary>
    ''' Shows how to get the total number of records
    ''' </summary>
    Private Sub GetRecordCount()
        ' get the total number of records in the Region table
        Dim totalRecordCount As Integer = Region.GetRecordCount()
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index.  The total number of records are also retrieved when using the SelectSkipAndTake() method.
    ''' For example, if ther are 200 records (totalRecordCount), take only 10 records (numberOfRecordsToRetrieve), starting from the first index (startRetrievalFromRecordIndex = 0)
    ''' The example below uses some variables, here are their definitions:
    ''' totalRecordCount - total number of records if you were to retrieve everything
    ''' startRetrievalFromRecordIndex - the index to start taking records from. Zero (0) E.g. If you want to skip the first 20 records, then assign 19 here.
    ''' numberOfRecordsToRetrieve - take n records starting from the startRetrievalFromRecordIndex
    ''' sortBy - to sort in Ascending order by Field Name, just assign just the Field Name, do not pass 'asc'
    ''' sortBy - to sort in Descending order by Field Name, use the Field Name, a space and the word 'desc'
    ''' </summary>
    Private Sub SelectSkipAndTake()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "RegionID"
        ' Dim sortBy As String = "RegionID desc"

        ' 1. select a specific number of sorted records starting from the index you specify with totalRecordCount
        Dim objRegionCol As RegionCollection = Region.SelectSkipAndTake(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy)

        ' 2. or you can also select a specific number of sorted records starting from the index you specify Without the totalRecordCount
        Dim objRegionCol2 As RegionCollection = Region.SelectSkipAndTake(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, sortBy)

        ' to use objRegionCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the Region(s).  The example above will only loop for 10 items.
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index, based on Search Parameters.  The number of records are also retrieved.
    ''' </summary>
    Private Sub SelectSkipAndTakeDynamicWhere()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "RegionID"
        ' Dim sortBy As String = "RegionID desc"

        ' search parameters, everything is nullable, only items being searched for should be filled
        ' note: fields with String type uses a LIKE search, everything else uses an exact match
        ' also, every field you're searching for uses the AND operator
        ' e.g. Dim productID As Integer? = 1, Dim productName As String = "ch"
        ' will translate to: SELECT....WHERE productID = 1 AND productName LIKE '%ch%'
        Dim regionID As Integer? = Nothing
        Dim regionDescription As String = Nothing

        ' 1. select a specific number of sorted records starting from the index you specify with totalRecordCount based on Search Parameters
        Dim objRegionCol As RegionCollection = Region.SelectSkipAndTakeDynamicWhere(regionID, regionDescription, numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy)

        ' to use objRegionCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the Region(s).  The example above will only loop for 10 items.
    End Sub
End Class
