Imports System
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports System.Windows.Forms

''' <summary>
''' These are data-centric code examples for the Shippers _table.
''' You can cut and paste the respective codes into your application
''' by changing the sample values assigned from these examples.
''' NOTE: This class contains private methods because they're
''' not meant to be called by an outside client.  Each method contains
''' code for the respective example being shown
''' </summary>
Public NotInheritable Class ShippersExample

    Private Sub New()
    End Sub

    Private Sub SelectAll()
        ' select all records
        Dim objShippersCol As ShippersCollection = Shippers.SelectAll()

        ' Example 1:  you can optionally sort the collection in ascending order by your chosen field  
        objShippersCol.Sort(Shippers.ByCompanyName)

        ' Example 2:  to sort in descending order, add this line to the Sort code in Example 1
        objShippersCol.Reverse()

        ' Example 3:  assign to a DataGridView
        Dim grid As DataGridView = New DataGridView()
        grid.DataSource = objShippersCol

        ' Example 4:  loop through all the Shippers
        For Each objShippers As Shippers In objShippersCol
            Dim shipperID As Integer = objShippers.ShipperID
            Dim companyName As String = objShippers.CompanyName
            Dim phone As String = objShippers.Phone
        Next
    End Sub

    Private Sub SelectByPrimaryKey()
        ' select a record by primary key(s)
        Dim objShippers As Shippers = Shippers.SelectByPrimaryKey(1)

        If objShippers IsNot Nothing Then
            ' if record is found, a record is returned
            Dim shipperID As Integer = objShippers.ShipperID
            Dim companyName As String = objShippers.CompanyName
            Dim phone As String = objShippers.Phone
        End If
    End Sub

    ''' <summary>
    ''' Selects ShipperID and CompanyName columns for use with a with a Drop Down List, Combo Box, Checked Box List, List View, List Box, etc
    ''' </summary>
    Private Sub SelectShippersDropDownListData()
        Dim objShippersCol As ShippersCollection = Shippers.SelectShippersDropDownListData()

        ' Example 1:  directly bind to a combo box
        Dim cbx1 As ComboBox = New ComboBox()
        cbx1.ValueMember = "ShipperID"
        cbx1.DisplayMember = "CompanyName"
        cbx1.DataSource = objShippersCol

        ' Example 2:  add each item through a loop.  Web app example
        ' Dim cbx2 As DropDownList = New DropDownList()

        ' For Each objShippers As Shippers In objShippersCol
        '     cbx2.Items.Add(New ListItem(objShippers.CompanyName, objShippers.ShipperID.ToString()))
        ' Next
    End Sub

    Private Sub Insert()
        ' first instantiate a new Shippers
        Dim objShippers As New Shippers()

        ' assign values you want inserted
        objShippers.CompanyName = "Speedy Express"
        objShippers.Phone = "(503) 555-9831"

        ' finally, insert a new record
        ' the insert method returns the newly created primary key
        Dim newlyCreatedPrimaryKey As Integer = objShippers.Insert()
    End Sub

    Private Sub Update()
        ' first instantiate a new Shippers
        Dim objShippers As New Shippers()

        ' assign the existing primary key(s)
        ' of the record you want updated
        objShippers.ShipperID = 1

        ' assign values you want updated
        objShippers.CompanyName = "Speedy Express"
        objShippers.Phone = "(503) 555-9831"

        ' finally, update an existing record
        objShippers.Update()
    End Sub

    Private Sub Delete()
        ' delete a record by primary key
        Shippers.Delete(6)
    End Sub

    ''' <summary>
    ''' Shows how to get the total number of records
    ''' </summary>
    Private Sub GetRecordCount()
        ' get the total number of records in the Shippers table
        Dim totalRecordCount As Integer = Shippers.GetRecordCount()
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index.  The total number of records are also retrieved when using the SelectSkipAndTake() method.
    ''' For example, if ther are 200 records (totalRecordCount), take only 10 records (numberOfRecordsToRetrieve), starting from the first index (startRetrievalFromRecordIndex = 0)
    ''' The example below uses some variables, here are their definitions:
    ''' totalRecordCount - total number of records if you were to retrieve everything
    ''' startRetrievalFromRecordIndex - the index to start taking records from. Zero (0) E.g. If you want to skip the first 20 records, then assign 19 here.
    ''' numberOfRecordsToRetrieve - take n records starting from the startRetrievalFromRecordIndex
    ''' sortBy - to sort in Ascending order by Field Name, just assign just the Field Name, do not pass 'asc'
    ''' sortBy - to sort in Descending order by Field Name, use the Field Name, a space and the word 'desc'
    ''' </summary>
    Private Sub SelectSkipAndTake()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "ShipperID"
        ' Dim sortBy As String = "ShipperID desc"

        ' 1. select a specific number of sorted records starting from the index you specify with totalRecordCount
        Dim objShippersCol As ShippersCollection = Shippers.SelectSkipAndTake(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy)

        ' 2. or you can also select a specific number of sorted records starting from the index you specify Without the totalRecordCount
        Dim objShippersCol2 As ShippersCollection = Shippers.SelectSkipAndTake(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, sortBy)

        ' to use objShippersCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the Shippers(s).  The example above will only loop for 10 items.
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index, based on Search Parameters.  The number of records are also retrieved.
    ''' </summary>
    Private Sub SelectSkipAndTakeDynamicWhere()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "ShipperID"
        ' Dim sortBy As String = "ShipperID desc"

        ' search parameters, everything is nullable, only items being searched for should be filled
        ' note: fields with String type uses a LIKE search, everything else uses an exact match
        ' also, every field you're searching for uses the AND operator
        ' e.g. Dim productID As Integer? = 1, Dim productName As String = "ch"
        ' will translate to: SELECT....WHERE productID = 1 AND productName LIKE '%ch%'
        Dim shipperID As Integer? = Nothing
        Dim companyName As String = Nothing
        Dim phone As String = Nothing

        ' 1. select a specific number of sorted records starting from the index you specify with totalRecordCount based on Search Parameters
        Dim objShippersCol As ShippersCollection = Shippers.SelectSkipAndTakeDynamicWhere(shipperID, companyName, phone, numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy)

        ' to use objShippersCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the Shippers(s).  The example above will only loop for 10 items.
    End Sub
End Class
