Imports System
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports System.Windows.Forms

''' <summary>
''' These are data-centric code examples for the Territories _table.
''' You can cut and paste the respective codes into your application
''' by changing the sample values assigned from these examples.
''' NOTE: This class contains private methods because they're
''' not meant to be called by an outside client.  Each method contains
''' code for the respective example being shown
''' </summary>
Public NotInheritable Class TerritoriesExample

    Private Sub New()
    End Sub

    Private Sub SelectAll()
        ' select all records
        Dim objTerritoriesCol As TerritoriesCollection = Territories.SelectAll()

        ' Example 1:  you can optionally sort the collection in ascending order by your chosen field  
        objTerritoriesCol.Sort(Territories.ByTerritoryDescription)

        ' Example 2:  to sort in descending order, add this line to the Sort code in Example 1
        objTerritoriesCol.Reverse()

        ' Example 3:  assign to a DataGridView
        Dim grid As DataGridView = New DataGridView()
        grid.DataSource = objTerritoriesCol

        ' Example 4:  loop through all the Territories
        For Each objTerritories As Territories In objTerritoriesCol
            Dim territoryID As String = objTerritories.TerritoryID
            Dim territoryDescription As String = objTerritories.TerritoryDescription
            Dim regionID As Integer = objTerritories.RegionID

            ' optionally get the Region related to RegionID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            Dim objRegionRelatedToRegionID As Region

            If objTerritories.Region.IsValueCreated Then
                objRegionRelatedToRegionID = objTerritories.Region.Value
            End If
        Next
    End Sub

    Private Sub SelectByPrimaryKey()
        ' select a record by primary key(s)
        Dim objTerritories As Territories = Territories.SelectByPrimaryKey("01581")

        If objTerritories IsNot Nothing Then
            ' if record is found, a record is returned
            Dim territoryID As String = objTerritories.TerritoryID
            Dim territoryDescription As String = objTerritories.TerritoryDescription
            Dim regionID As Integer = objTerritories.RegionID

            ' optionally get the Region related to RegionID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            Dim objRegionRelatedToRegionID As Region

            If objTerritories.Region.IsValueCreated Then
                objRegionRelatedToRegionID = objTerritories.Region.Value
            End If
        End If
    End Sub

    ''' <summary>
    ''' Select all records by Region, related to column RegionID
    ''' </summary> 
    Private Sub SelectTerritoriesCollectionByRegionID()
        Dim objTerritoriesCol As TerritoriesCollection = Territories.SelectTerritoriesCollectionByRegionID(1)

        ' Example 1:  you can optionally sort the collection in ascending order by your chosen field  
        objTerritoriesCol.Sort(Territories.ByTerritoryDescription)

        ' Example 2:  to sort in descending order, add this line to the Sort code in Example 1
        objTerritoriesCol.Reverse()

        ' Example 3:  assign to a DataGridView
        Dim grid As DataGridView = New DataGridView()
        grid.DataSource = objTerritoriesCol

        ' Example 4:  loop through all the Territories
        For Each objTerritories As Territories In objTerritoriesCol
            Dim territoryID As String = objTerritories.TerritoryID
            Dim territoryDescription As String = objTerritories.TerritoryDescription
            Dim regionID As Integer = objTerritories.RegionID

            ' optionally get the Region related to RegionID.
            ' Note this is lazily loaded which means there's no value until you ask for it
            Dim objRegionRelatedToRegionID As Region

            If objTerritories.Region.IsValueCreated Then
                objRegionRelatedToRegionID = objTerritories.Region.Value
            End If
        Next
    End Sub

    ''' <summary>
    ''' Selects TerritoryID and TerritoryDescription columns for use with a with a Drop Down List, Combo Box, Checked Box List, List View, List Box, etc
    ''' </summary>
    Private Sub SelectTerritoriesDropDownListData()
        Dim objTerritoriesCol As TerritoriesCollection = Territories.SelectTerritoriesDropDownListData()

        ' Example 1:  directly bind to a combo box
        Dim cbx1 As ComboBox = New ComboBox()
        cbx1.ValueMember = "TerritoryID"
        cbx1.DisplayMember = "TerritoryDescription"
        cbx1.DataSource = objTerritoriesCol

        ' Example 2:  add each item through a loop.  Web app example
        ' Dim cbx2 As DropDownList = New DropDownList()

        ' For Each objTerritories As Territories In objTerritoriesCol
        '     cbx2.Items.Add(New ListItem(objTerritories.TerritoryDescription, objTerritories.TerritoryID))
        ' Next
    End Sub

    Private Sub Insert()
        ' first instantiate a new Territories
        Dim objTerritories As New Territories()

        ' assign values you want inserted
        objTerritories.TerritoryID = "01581"
        objTerritories.TerritoryDescription = "Westboro"
        objTerritories.RegionID = 1

        ' finally, insert a new record
        ' the insert method returns the newly created primary key
        Dim newlyCreatedPrimaryKey As String = objTerritories.Insert()
    End Sub

    Private Sub Update()
        ' first instantiate a new Territories
        Dim objTerritories As New Territories()

        ' assign the existing primary key(s)
        ' of the record you want updated
        objTerritories.TerritoryID = "01581"

        ' assign values you want updated
        objTerritories.TerritoryDescription = "Westboro"
        objTerritories.RegionID = 1

        ' finally, update an existing record
        objTerritories.Update()
    End Sub

    Private Sub Delete()
        ' delete a record by primary key
        Territories.Delete("01581")
    End Sub

    ''' <summary>
    ''' Shows how to get the total number of records
    ''' </summary>
    Private Sub GetRecordCount()
        ' get the total number of records in the Territories table
        Dim totalRecordCount As Integer = Territories.GetRecordCount()
    End Sub

    ''' <summary>
    ''' Shows how to get the total number of records by RegionID
    ''' </summary>
    Private Sub GetRecordCountByRegionID()
        ' get the total number of records in the Territories table by RegionID
        ' 1 here is just a sample RegionID change the value as you see fit
        Dim totalRecordCount As Integer = Territories.GetRecordCountByRegionID(1)
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index.  The total number of records are also retrieved when using the SelectSkipAndTake() method.
    ''' For example, if ther are 200 records (totalRecordCount), take only 10 records (numberOfRecordsToRetrieve), starting from the first index (startRetrievalFromRecordIndex = 0)
    ''' The example below uses some variables, here are their definitions:
    ''' totalRecordCount - total number of records if you were to retrieve everything
    ''' startRetrievalFromRecordIndex - the index to start taking records from. Zero (0) E.g. If you want to skip the first 20 records, then assign 19 here.
    ''' numberOfRecordsToRetrieve - take n records starting from the startRetrievalFromRecordIndex
    ''' sortBy - to sort in Ascending order by Field Name, just assign just the Field Name, do not pass 'asc'
    ''' sortBy - to sort in Descending order by Field Name, use the Field Name, a space and the word 'desc'
    ''' </summary>
    Private Sub SelectSkipAndTake()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "TerritoryID"
        ' Dim sortBy As String = "TerritoryID desc"

        ' 1. select a specific number of sorted records starting from the index you specify with totalRecordCount
        Dim objTerritoriesCol As TerritoriesCollection = Territories.SelectSkipAndTake(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy)

        ' 2. or you can also select a specific number of sorted records starting from the index you specify Without the totalRecordCount
        Dim objTerritoriesCol2 As TerritoriesCollection = Territories.SelectSkipAndTake(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, sortBy)

        ' to use objTerritoriesCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the Territories(s).  The example above will only loop for 10 items.
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index by the related Field Name.  The total number of records are also retrieved when using the SelectSkipAndTake() method.
    ''' For example, if ther are 200 records (totalRecordCount), take only 10 records (numberOfRecordsToRetrieve), starting from the first index (startRetrievalFromRecordIndex = 0)
    ''' The example below uses some variables, here are their definitions:
    ''' totalRecordCount - total number of records if you were to retrieve everything
    ''' startRetrievalFromRecordIndex - the index to start taking records from. Zero (0) E.g. If you want to skip the first 20 records, then assign 19 here.
    ''' numberOfRecordsToRetrieve - take n records starting from the startRetrievalFromRecordIndex
    ''' sortBy - to sort in Ascending order by Field Name, just assign just the Field Name, do not pass 'asc'
    ''' sortBy - to sort in Descending order by Field Name, use the Field Name, a space and the word 'desc'
    ''' </summary>
    Private Sub SelectSkipAndTakeByRegionID()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "TerritoryID"
        ' Dim sortBy As String = "TerritoryID desc"

        ' 1. select a specific number of sorted records with a RegionID = 1
        ' starting from the index you specify with totalRecordCount
        Dim objTerritoriesCol As TerritoriesCollection = Territories.SelectSkipAndTakeByRegionID(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy, 1)

        ' to use objTerritoriesCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the Territories(s).  The example above will only loop for 10 items.
    End Sub

    ''' <summary>
    ''' Shows how to get a specific number of sorted records, starting from an index, based on Search Parameters.  The number of records are also retrieved.
    ''' </summary>
    Private Sub SelectSkipAndTakeDynamicWhere()
        Dim totalRecordCount As Integer
        Dim startRetrievalFromRecordIndex As Integer = 0
        Dim numberOfRecordsToRetrieve As Integer = 10
        Dim sortBy As String = "TerritoryID"
        ' Dim sortBy As String = "TerritoryID desc"

        ' search parameters, everything is nullable, only items being searched for should be filled
        ' note: fields with String type uses a LIKE search, everything else uses an exact match
        ' also, every field you're searching for uses the AND operator
        ' e.g. Dim productID As Integer? = 1, Dim productName As String = "ch"
        ' will translate to: SELECT....WHERE productID = 1 AND productName LIKE '%ch%'
        Dim territoryID As String = Nothing
        Dim territoryDescription As String = Nothing
        Dim regionID As Integer? = Nothing

        ' 1. select a specific number of sorted records starting from the index you specify with totalRecordCount based on Search Parameters
        Dim objTerritoriesCol As TerritoriesCollection = Territories.SelectSkipAndTakeDynamicWhere(territoryID, territoryDescription, regionID, numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, totalRecordCount, sortBy)

        ' to use objTerritoriesCol please see the SelectAll() method examples
        ' No need for Examples 1 and 2 because the Collection here is already sorted
        ' Example 3:  directly bind to a GridView
        ' Example 4:  loop through all the Territories(s).  The example above will only loop for 10 items.
    End Sub
End Class
