Imports System 
Imports System.Text 

Namespace DataLayer.Base
    Public NotInheritable Class EmployeesSql
        Private Sub New() 
        End Sub 

        Public Shared Function SelectByPrimaryKey() As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append(selectStatement) 
            sb.Append(" WHERE ") 
            sb.Append("[EmployeeID] = @employeeID ") 

            Return sb.ToString() 
        End Function 

        Public Shared Function GetRecordCount() As String 
            Dim sb As New StringBuilder() 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Employees]")

            Return sb.ToString() 
        End Function 

        ''' <summary>
        ''' Related to column ReportsTo
        ''' </summary>
        Public Shared Function GetRecordCountByReportsTo(reportsTo As Integer) As String
            Dim sb As New StringBuilder() 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Employees] ")
            sb.Append("WHERE [ReportsTo] = @reportsTo ")

            Return sb.ToString() 
        End Function 

        Public Shared Function GetRecordCountDynamicWhere() As String 
            Dim sb As New StringBuilder() 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Employees]")

            sb.Append(" WHERE ")
            sb.Append("([EmployeeID] = @employeeID OR @employeeID IS NULL) AND ")
            sb.Append("([LastName] LIKE '%' + @lastName + '%' OR @lastName IS NULL) AND ")
            sb.Append("([FirstName] LIKE '%' + @firstName + '%' OR @firstName IS NULL) AND ")
            sb.Append("([Title] LIKE '%' + @title + '%' OR @title IS NULL) AND ")
            sb.Append("([TitleOfCourtesy] LIKE '%' + @titleOfCourtesy + '%' OR @titleOfCourtesy IS NULL) AND ")
            sb.Append("([BirthDate] = @birthDate OR @birthDate IS NULL) AND ")
            sb.Append("([HireDate] = @hireDate OR @hireDate IS NULL) AND ")
            sb.Append("([Address] LIKE '%' + @address + '%' OR @address IS NULL) AND ")
            sb.Append("([City] LIKE '%' + @city + '%' OR @city IS NULL) AND ")
            sb.Append("([Region] LIKE '%' + @region1 + '%' OR @region1 IS NULL) AND ")
            sb.Append("([PostalCode] LIKE '%' + @postalCode + '%' OR @postalCode IS NULL) AND ")
            sb.Append("([Country] LIKE '%' + @country + '%' OR @country IS NULL) AND ")
            sb.Append("([HomePhone] LIKE '%' + @homePhone + '%' OR @homePhone IS NULL) AND ")
            sb.Append("([Extension] LIKE '%' + @extension + '%' OR @extension IS NULL) AND ")
            sb.Append("([ReportsTo] = @reportsTo OR @reportsTo IS NULL) AND ")
            sb.Append("([PhotoPath] LIKE '%' + @photoPath + '%' OR @photoPath IS NULL) ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectSkipAndTake() As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start - 1; ")

            sb.Append("SELECT ")
            sb.Append("[EmployeeID], ")
            sb.Append("[LastName], ")
            sb.Append("[FirstName], ")
            sb.Append("[Title], ")
            sb.Append("[TitleOfCourtesy], ")
            sb.Append("[BirthDate], ")
            sb.Append("[HireDate], ")
            sb.Append("[Address], ")
            sb.Append("[City], ")
            sb.Append("[Region] AS [Region1], ")
            sb.Append("[PostalCode], ")
            sb.Append("[Country], ")
            sb.Append("[HomePhone], ")
            sb.Append("[Extension], ")
            sb.Append("[Notes], ")
            sb.Append("[ReportsTo], ")
            sb.Append("[PhotoPath] ")
            sb.Append("FROM [dbo].[Employees] ")
            sb.Append("ORDER BY ")
	        sb.Append("CASE WHEN @sortByExpression = 'EmployeeID' THEN [EmployeeID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'EmployeeID desc' THEN [EmployeeID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'LastName' THEN [LastName] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'LastName desc' THEN [LastName] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'FirstName' THEN [FirstName] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'FirstName desc' THEN [FirstName] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Title' THEN [Title] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Title desc' THEN [Title] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'TitleOfCourtesy' THEN [TitleOfCourtesy] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'TitleOfCourtesy desc' THEN [TitleOfCourtesy] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'BirthDate' THEN [BirthDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'BirthDate desc' THEN [BirthDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'HireDate' THEN [HireDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'HireDate desc' THEN [HireDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Address' THEN [Address] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Address desc' THEN [Address] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'City' THEN [City] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'City desc' THEN [City] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Region1' THEN [Region] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Region1 desc' THEN [Region] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'PostalCode' THEN [PostalCode] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'PostalCode desc' THEN [PostalCode] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Country' THEN [Country] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Country desc' THEN [Country] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'HomePhone' THEN [HomePhone] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'HomePhone desc' THEN [HomePhone] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Extension' THEN [Extension] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Extension desc' THEN [Extension] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ReportsTo' THEN [ReportsTo] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ReportsTo desc' THEN [ReportsTo] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'PhotoPath' THEN [PhotoPath] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'PhotoPath desc' THEN [PhotoPath] END DESC ")

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ")
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectSkipAndTakeByReportsTo(reportsTo As Integer) As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start - 1; ")

            sb.Append("SELECT ")
            sb.Append("[EmployeeID], ")
            sb.Append("[LastName], ")
            sb.Append("[FirstName], ")
            sb.Append("[Title], ")
            sb.Append("[TitleOfCourtesy], ")
            sb.Append("[BirthDate], ")
            sb.Append("[HireDate], ")
            sb.Append("[Address], ")
            sb.Append("[City], ")
            sb.Append("[Region] AS [Region1], ")
            sb.Append("[PostalCode], ")
            sb.Append("[Country], ")
            sb.Append("[HomePhone], ")
            sb.Append("[Extension], ")
            sb.Append("[Notes], ")
            sb.Append("[ReportsTo], ")
            sb.Append("[PhotoPath] ")
            sb.Append("FROM [dbo].[Employees] ")
            sb.Append("WHERE [ReportsTo] = @reportsTo ")
            sb.Append("ORDER BY ")
	        sb.Append("CASE WHEN @sortByExpression = 'EmployeeID' THEN [EmployeeID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'EmployeeID desc' THEN [EmployeeID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'LastName' THEN [LastName] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'LastName desc' THEN [LastName] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'FirstName' THEN [FirstName] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'FirstName desc' THEN [FirstName] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Title' THEN [Title] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Title desc' THEN [Title] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'TitleOfCourtesy' THEN [TitleOfCourtesy] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'TitleOfCourtesy desc' THEN [TitleOfCourtesy] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'BirthDate' THEN [BirthDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'BirthDate desc' THEN [BirthDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'HireDate' THEN [HireDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'HireDate desc' THEN [HireDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Address' THEN [Address] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Address desc' THEN [Address] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'City' THEN [City] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'City desc' THEN [City] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Region1' THEN [Region] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Region1 desc' THEN [Region] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'PostalCode' THEN [PostalCode] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'PostalCode desc' THEN [PostalCode] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Country' THEN [Country] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Country desc' THEN [Country] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'HomePhone' THEN [HomePhone] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'HomePhone desc' THEN [HomePhone] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Extension' THEN [Extension] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Extension desc' THEN [Extension] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ReportsTo' THEN [ReportsTo] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ReportsTo desc' THEN [ReportsTo] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'PhotoPath' THEN [PhotoPath] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'PhotoPath desc' THEN [PhotoPath] END DESC ")

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ")
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectSkipAndTakeDynamicWhere() As String 
            Dim sb As New StringBuilder() 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start - 1; ")

            sb.Append("SELECT ") 
            sb.Append("[EmployeeID], ") 
            sb.Append("[LastName], ") 
            sb.Append("[FirstName], ") 
            sb.Append("[Title], ") 
            sb.Append("[TitleOfCourtesy], ") 
            sb.Append("[BirthDate], ") 
            sb.Append("[HireDate], ") 
            sb.Append("[Address], ") 
            sb.Append("[City], ") 
            sb.Append("[Region] AS [Region1], ") 
            sb.Append("[PostalCode], ") 
            sb.Append("[Country], ") 
            sb.Append("[HomePhone], ") 
            sb.Append("[Extension], ") 
            sb.Append("[Notes], ") 
            sb.Append("[ReportsTo], ") 
            sb.Append("[PhotoPath] ") 
            sb.Append("FROM [dbo].[Employees] ") 
            sb.Append(" WHERE ")
            sb.Append("([EmployeeID] = @employeeID OR @employeeID IS NULL) AND ")
            sb.Append("([LastName] LIKE '%' + @lastName + '%' OR @lastName IS NULL) AND ")
            sb.Append("([FirstName] LIKE '%' + @firstName + '%' OR @firstName IS NULL) AND ")
            sb.Append("([Title] LIKE '%' + @title + '%' OR @title IS NULL) AND ")
            sb.Append("([TitleOfCourtesy] LIKE '%' + @titleOfCourtesy + '%' OR @titleOfCourtesy IS NULL) AND ")
            sb.Append("([BirthDate] = @birthDate OR @birthDate IS NULL) AND ")
            sb.Append("([HireDate] = @hireDate OR @hireDate IS NULL) AND ")
            sb.Append("([Address] LIKE '%' + @address + '%' OR @address IS NULL) AND ")
            sb.Append("([City] LIKE '%' + @city + '%' OR @city IS NULL) AND ")
            sb.Append("([Region] LIKE '%' + @region1 + '%' OR @region1 IS NULL) AND ")
            sb.Append("([PostalCode] LIKE '%' + @postalCode + '%' OR @postalCode IS NULL) AND ")
            sb.Append("([Country] LIKE '%' + @country + '%' OR @country IS NULL) AND ")
            sb.Append("([HomePhone] LIKE '%' + @homePhone + '%' OR @homePhone IS NULL) AND ")
            sb.Append("([Extension] LIKE '%' + @extension + '%' OR @extension IS NULL) AND ")
            sb.Append("([ReportsTo] = @reportsTo OR @reportsTo IS NULL) AND ")
            sb.Append("([PhotoPath] LIKE '%' + @photoPath + '%' OR @photoPath IS NULL) ")

            sb.Append("ORDER BY ")
	        sb.Append("CASE WHEN @sortByExpression = 'EmployeeID' THEN [EmployeeID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'EmployeeID desc' THEN [EmployeeID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'LastName' THEN [LastName] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'LastName desc' THEN [LastName] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'FirstName' THEN [FirstName] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'FirstName desc' THEN [FirstName] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Title' THEN [Title] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Title desc' THEN [Title] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'TitleOfCourtesy' THEN [TitleOfCourtesy] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'TitleOfCourtesy desc' THEN [TitleOfCourtesy] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'BirthDate' THEN [BirthDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'BirthDate desc' THEN [BirthDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'HireDate' THEN [HireDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'HireDate desc' THEN [HireDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Address' THEN [Address] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Address desc' THEN [Address] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'City' THEN [City] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'City desc' THEN [City] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Region1' THEN [Region] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Region1 desc' THEN [Region] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'PostalCode' THEN [PostalCode] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'PostalCode desc' THEN [PostalCode] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Country' THEN [Country] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Country desc' THEN [Country] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'HomePhone' THEN [HomePhone] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'HomePhone desc' THEN [HomePhone] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Extension' THEN [Extension] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Extension desc' THEN [Extension] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ReportsTo' THEN [ReportsTo] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ReportsTo desc' THEN [ReportsTo] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'PhotoPath' THEN [PhotoPath] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'PhotoPath desc' THEN [PhotoPath] END DESC ")

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ")
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectAll() As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append(selectStatement) 

            Return sb.ToString() 
        End Function 

        ''' <summary>
        ''' Related to column ReportsTo
        ''' </summary>
        Public Shared Function SelectAllByReportsTo(ByVal reportsTo As Integer) As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 
 
            sb.Append(selectStatement) 
            sb.Append("WHERE [ReportsTo] = " & reportsTo) 
 
            Return sb.ToString() 
        End Function 

        Public Shared Function SelectAllDynamicWhere() As String
            Dim selectStatement As String = GetSelectStatement()
            Dim sb As New StringBuilder()

            sb.Append(selectStatement)
            sb.Append(" WHERE ")
            sb.Append("([EmployeeID] = @employeeID OR @employeeID IS NULL) AND ")
            sb.Append("([LastName] LIKE '%' + @lastName + '%' OR @lastName IS NULL) AND ")
            sb.Append("([FirstName] LIKE '%' + @firstName + '%' OR @firstName IS NULL) AND ")
            sb.Append("([Title] LIKE '%' + @title + '%' OR @title IS NULL) AND ")
            sb.Append("([TitleOfCourtesy] LIKE '%' + @titleOfCourtesy + '%' OR @titleOfCourtesy IS NULL) AND ")
            sb.Append("([BirthDate] = @birthDate OR @birthDate IS NULL) AND ")
            sb.Append("([HireDate] = @hireDate OR @hireDate IS NULL) AND ")
            sb.Append("([Address] LIKE '%' + @address + '%' OR @address IS NULL) AND ")
            sb.Append("([City] LIKE '%' + @city + '%' OR @city IS NULL) AND ")
            sb.Append("([Region] LIKE '%' + @region1 + '%' OR @region1 IS NULL) AND ")
            sb.Append("([PostalCode] LIKE '%' + @postalCode + '%' OR @postalCode IS NULL) AND ")
            sb.Append("([Country] LIKE '%' + @country + '%' OR @country IS NULL) AND ")
            sb.Append("([HomePhone] LIKE '%' + @homePhone + '%' OR @homePhone IS NULL) AND ")
            sb.Append("([Extension] LIKE '%' + @extension + '%' OR @extension IS NULL) AND ")
            sb.Append("([ReportsTo] = @reportsTo OR @reportsTo IS NULL) AND ")
            sb.Append("([PhotoPath] LIKE '%' + @photoPath + '%' OR @photoPath IS NULL) ")

            Return sb.ToString() 
        End Function

        ''' <summary>
        ''' Selects EmployeeID and LastName columns for use with a DropDownList web control
        ''' </summary>
        Public Shared Function SelectEmployeesDropDownListData() As String
            Dim selectStatement As String = "SELECT [EmployeeID], [LastName] FROM [dbo].[Employees] ORDER BY [LastName] ASC " 
            Dim sb As New StringBuilder() 

            sb.Append(selectStatement)

            Return sb.ToString() 
        End Function 

        Public Shared Function Insert() As String
            Dim sb As New StringBuilder() 

            sb.Append("INSERT INTO [dbo].[Employees] ") 
            sb.Append("(") 
            sb.Append("[LastName], ")
            sb.Append("[FirstName], ")
            sb.Append("[Title], ")
            sb.Append("[TitleOfCourtesy], ")
            sb.Append("[BirthDate], ")
            sb.Append("[HireDate], ")
            sb.Append("[Address], ")
            sb.Append("[City], ")
            sb.Append("[Region], ")
            sb.Append("[PostalCode], ")
            sb.Append("[Country], ")
            sb.Append("[HomePhone], ")
            sb.Append("[Extension], ")
            sb.Append("[Notes], ")
            sb.Append("[ReportsTo], ")
            sb.Append("[PhotoPath] ")
            sb.Append(") ") 
            sb.Append("OUTPUT inserted.[EmployeeID] ") 
            sb.Append("VALUES ") 
            sb.Append("(") 
            sb.Append("@lastName,")
            sb.Append("@firstName,")
            sb.Append("@title,")
            sb.Append("@titleOfCourtesy,")
            sb.Append("@birthDate,")
            sb.Append("@hireDate,")
            sb.Append("@address,")
            sb.Append("@city,")
            sb.Append("@region1,")
            sb.Append("@postalCode,")
            sb.Append("@country,")
            sb.Append("@homePhone,")
            sb.Append("@extension,")
            sb.Append("@notes,")
            sb.Append("@reportsTo,")
            sb.Append("@photoPath")
            sb.Append(")") 

            Return sb.ToString() 
        End Function 

        Public Shared Function Update() As String
            Dim sb As New StringBuilder() 

            sb.Append("UPDATE [dbo].[Employees] ") 
            sb.Append("SET ") 
            sb.Append("[LastName] = @lastName,")
            sb.Append("[FirstName] = @firstName,")
            sb.Append("[Title] = @title,")
            sb.Append("[TitleOfCourtesy] = @titleOfCourtesy,")
            sb.Append("[BirthDate] = @birthDate,")
            sb.Append("[HireDate] = @hireDate,")
            sb.Append("[Address] = @address,")
            sb.Append("[City] = @city,")
            sb.Append("[Region] = @region1,")
            sb.Append("[PostalCode] = @postalCode,")
            sb.Append("[Country] = @country,")
            sb.Append("[HomePhone] = @homePhone,")
            sb.Append("[Extension] = @extension,")
            sb.Append("[Notes] = @notes,")
            sb.Append("[ReportsTo] = @reportsTo,")
            sb.Append("[PhotoPath] = @photoPath")
            sb.Append(" WHERE ") 
            sb.Append("[EmployeeID] = @employeeID ") 

            Return sb.ToString() 
        End Function 

        Public Shared Function Delete() As String 
            Dim sb As New StringBuilder() 

            sb.Append("DELETE FROM [dbo].[Employees] ") 
            sb.Append(" WHERE ") 
            sb.Append("[EmployeeID] = @employeeID ") 

            Return sb.ToString() 
        End Function 

        Public Shared Function GetSelectStatement() As String 
            Dim sb As New StringBuilder() 

            sb.Append("SELECT ") 
            sb.Append("[EmployeeID], ") 
            sb.Append("[LastName], ") 
            sb.Append("[FirstName], ") 
            sb.Append("[Title], ") 
            sb.Append("[TitleOfCourtesy], ") 
            sb.Append("[BirthDate], ") 
            sb.Append("[HireDate], ") 
            sb.Append("[Address], ") 
            sb.Append("[City], ") 
            sb.Append("[Region] AS [Region1], ") 
            sb.Append("[PostalCode], ") 
            sb.Append("[Country], ") 
            sb.Append("[HomePhone], ") 
            sb.Append("[Extension], ") 
            sb.Append("[Notes], ") 
            sb.Append("[ReportsTo], ") 
            sb.Append("[PhotoPath] ") 
            sb.Append("FROM [dbo].[Employees] ") 

            Return sb.ToString() 
        End Function 
    End Class 
End Namespace 
