Imports System 
Imports System.Text 

Namespace DataLayer.Base
    Public NotInheritable Class OrderDetailsSql
        Private Sub New() 
        End Sub 

        Public Shared Function SelectByPrimaryKey() As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append(selectStatement) 
            sb.Append(" WHERE ") 
            sb.Append("[OrderID] = @orderID ") 
            sb.Append(" AND ") 
            sb.Append("[ProductID] = @productID ") 

            Return sb.ToString() 
        End Function 

        Public Shared Function GetRecordCount() As String 
            Dim sb As New StringBuilder() 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Order Details]")

            Return sb.ToString() 
        End Function 

        ''' <summary>
        ''' Related to column OrderID
        ''' </summary>
        Public Shared Function GetRecordCountByOrderID(orderID As Integer) As String
            Dim sb As New StringBuilder() 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Order Details] ")
            sb.Append("WHERE [OrderID] = @orderID ")

            Return sb.ToString() 
        End Function 

        ''' <summary>
        ''' Related to column ProductID
        ''' </summary>
        Public Shared Function GetRecordCountByProductID(productID As Integer) As String
            Dim sb As New StringBuilder() 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Order Details] ")
            sb.Append("WHERE [ProductID] = @productID ")

            Return sb.ToString() 
        End Function 

        Public Shared Function GetRecordCountDynamicWhere() As String 
            Dim sb As New StringBuilder() 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Order Details]")

            sb.Append(" WHERE ")
            sb.Append("([OrderID] = @orderID OR @orderID IS NULL) AND ")
            sb.Append("([ProductID] = @productID OR @productID IS NULL) AND ")
            sb.Append("([UnitPrice] = @unitPrice OR @unitPrice IS NULL) AND ")
            sb.Append("([Quantity] = @quantity OR @quantity IS NULL) AND ")
            sb.Append("([Discount] = @discount OR @discount IS NULL) ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectSkipAndTake() As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start - 1; ")

            sb.Append("SELECT ")
            sb.Append("[OrderID], ")
            sb.Append("[ProductID], ")
            sb.Append("[UnitPrice], ")
            sb.Append("[Quantity], ")
            sb.Append("[Discount] ")
            sb.Append("FROM [dbo].[Order Details] ")
            sb.Append("ORDER BY ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderID' THEN [OrderID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderID desc' THEN [OrderID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ProductID' THEN [ProductID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ProductID desc' THEN [ProductID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'UnitPrice' THEN [UnitPrice] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'UnitPrice desc' THEN [UnitPrice] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Quantity' THEN [Quantity] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Quantity desc' THEN [Quantity] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Discount' THEN [Discount] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Discount desc' THEN [Discount] END DESC ")

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ")
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectSkipAndTakeByOrderID(orderID As Integer) As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start - 1; ")

            sb.Append("SELECT ")
            sb.Append("[OrderID], ")
            sb.Append("[ProductID], ")
            sb.Append("[UnitPrice], ")
            sb.Append("[Quantity], ")
            sb.Append("[Discount] ")
            sb.Append("FROM [dbo].[Order Details] ")
            sb.Append("WHERE [OrderID] = @orderID ")
            sb.Append("ORDER BY ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderID' THEN [OrderID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderID desc' THEN [OrderID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ProductID' THEN [ProductID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ProductID desc' THEN [ProductID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'UnitPrice' THEN [UnitPrice] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'UnitPrice desc' THEN [UnitPrice] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Quantity' THEN [Quantity] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Quantity desc' THEN [Quantity] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Discount' THEN [Discount] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Discount desc' THEN [Discount] END DESC ")

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ")
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectSkipAndTakeByProductID(productID As Integer) As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start - 1; ")

            sb.Append("SELECT ")
            sb.Append("[OrderID], ")
            sb.Append("[ProductID], ")
            sb.Append("[UnitPrice], ")
            sb.Append("[Quantity], ")
            sb.Append("[Discount] ")
            sb.Append("FROM [dbo].[Order Details] ")
            sb.Append("WHERE [ProductID] = @productID ")
            sb.Append("ORDER BY ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderID' THEN [OrderID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderID desc' THEN [OrderID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ProductID' THEN [ProductID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ProductID desc' THEN [ProductID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'UnitPrice' THEN [UnitPrice] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'UnitPrice desc' THEN [UnitPrice] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Quantity' THEN [Quantity] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Quantity desc' THEN [Quantity] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Discount' THEN [Discount] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Discount desc' THEN [Discount] END DESC ")

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ")
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectSkipAndTakeDynamicWhere() As String 
            Dim sb As New StringBuilder() 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start - 1; ")

            sb.Append("SELECT ") 
            sb.Append("[OrderID], ") 
            sb.Append("[ProductID], ") 
            sb.Append("[UnitPrice], ") 
            sb.Append("[Quantity], ") 
            sb.Append("[Discount] ") 
            sb.Append("FROM [dbo].[Order Details] ") 
            sb.Append(" WHERE ")
            sb.Append("([OrderID] = @orderID OR @orderID IS NULL) AND ")
            sb.Append("([ProductID] = @productID OR @productID IS NULL) AND ")
            sb.Append("([UnitPrice] = @unitPrice OR @unitPrice IS NULL) AND ")
            sb.Append("([Quantity] = @quantity OR @quantity IS NULL) AND ")
            sb.Append("([Discount] = @discount OR @discount IS NULL) ")

            sb.Append("ORDER BY ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderID' THEN [OrderID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderID desc' THEN [OrderID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ProductID' THEN [ProductID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ProductID desc' THEN [ProductID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'UnitPrice' THEN [UnitPrice] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'UnitPrice desc' THEN [UnitPrice] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Quantity' THEN [Quantity] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Quantity desc' THEN [Quantity] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Discount' THEN [Discount] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Discount desc' THEN [Discount] END DESC ")

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ")
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectTotals() As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append("SELECT ")
            sb.Append("SUM([UnitPrice]) AS [UnitPriceTotal] ")
            sb.Append("FROM [dbo].[Order Details]")
            Return sb.ToString() 
        End Function 

        Public Shared Function SelectAll() As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append(selectStatement) 

            Return sb.ToString() 
        End Function 

        ''' <summary>
        ''' Related to column OrderID
        ''' </summary>
        Public Shared Function SelectAllByOrderID(ByVal orderID As Integer) As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 
 
            sb.Append(selectStatement) 
            sb.Append("WHERE [OrderID] = " & orderID) 
 
            Return sb.ToString() 
        End Function 

        ''' <summary>
        ''' Related to column ProductID
        ''' </summary>
        Public Shared Function SelectAllByProductID(ByVal productID As Integer) As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 
 
            sb.Append(selectStatement) 
            sb.Append("WHERE [ProductID] = " & productID) 
 
            Return sb.ToString() 
        End Function 

        Public Shared Function SelectAllDynamicWhere() As String
            Dim selectStatement As String = GetSelectStatement()
            Dim sb As New StringBuilder()

            sb.Append(selectStatement)
            sb.Append(" WHERE ")
            sb.Append("([OrderID] = @orderID OR @orderID IS NULL) AND ")
            sb.Append("([ProductID] = @productID OR @productID IS NULL) AND ")
            sb.Append("([UnitPrice] = @unitPrice OR @unitPrice IS NULL) AND ")
            sb.Append("([Quantity] = @quantity OR @quantity IS NULL) AND ")
            sb.Append("([Discount] = @discount OR @discount IS NULL) ")

            Return sb.ToString() 
        End Function

        ''' <summary>
        ''' Selects OrderID and UnitPrice columns for use with a DropDownList web control
        ''' </summary>
        Public Shared Function SelectOrderDetailsDropDownListData() As String
            Dim selectStatement As String = "SELECT [OrderID], [UnitPrice] FROM [dbo].[Order Details] ORDER BY [UnitPrice] ASC " 
            Dim sb As New StringBuilder() 

            sb.Append(selectStatement)

            Return sb.ToString() 
        End Function 

        Public Shared Function Insert() As String
            Dim sb As New StringBuilder() 

            sb.Append("INSERT INTO [dbo].[Order Details] ") 
            sb.Append("(") 
            sb.Append("[OrderID], ")
            sb.Append("[ProductID], ")
            sb.Append("[UnitPrice], ")
            sb.Append("[Quantity], ")
            sb.Append("[Discount] ")
            sb.Append(") ") 
            sb.Append("VALUES ") 
            sb.Append("(") 
            sb.Append("@orderID,")
            sb.Append("@productID,")
            sb.Append("@unitPrice,")
            sb.Append("@quantity,")
            sb.Append("@discount")
            sb.Append(")") 

            Return sb.ToString() 
        End Function 

        Public Shared Function Update() As String
            Dim sb As New StringBuilder() 

            sb.Append("UPDATE [dbo].[Order Details] ") 
            sb.Append("SET ") 
            sb.Append("[OrderID] = @orderID,")
            sb.Append("[ProductID] = @productID,")
            sb.Append("[UnitPrice] = @unitPrice,")
            sb.Append("[Quantity] = @quantity,")
            sb.Append("[Discount] = @discount")
            sb.Append(" WHERE ") 
            sb.Append("[OrderID] = @orderID ") 
            sb.Append(" AND ") 
            sb.Append("[ProductID] = @productID ") 

            Return sb.ToString() 
        End Function 

        Public Shared Function Delete() As String 
            Dim sb As New StringBuilder() 

            sb.Append("DELETE FROM [dbo].[Order Details] ") 
            sb.Append(" WHERE ") 
            sb.Append("[OrderID] = @orderID ") 
            sb.Append(" AND ") 
            sb.Append("[ProductID] = @productID ") 

            Return sb.ToString() 
        End Function 

        Public Shared Function GetSelectStatement() As String 
            Dim sb As New StringBuilder() 

            sb.Append("SELECT ") 
            sb.Append("[OrderID], ") 
            sb.Append("[ProductID], ") 
            sb.Append("[UnitPrice], ") 
            sb.Append("[Quantity], ") 
            sb.Append("[Discount] ") 
            sb.Append("FROM [dbo].[Order Details] ") 

            Return sb.ToString() 
        End Function 
    End Class 
End Namespace 
