Imports System 
Imports System.Text 

Namespace DataLayer.Base
    Public NotInheritable Class OrdersSql
        Private Sub New() 
        End Sub 

        Public Shared Function SelectByPrimaryKey() As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append(selectStatement) 
            sb.Append(" WHERE ") 
            sb.Append("[OrderID] = @orderID ") 

            Return sb.ToString() 
        End Function 

        Public Shared Function GetRecordCount() As String 
            Dim sb As New StringBuilder() 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Orders]")

            Return sb.ToString() 
        End Function 

        ''' <summary>
        ''' Related to column CustomerID
        ''' </summary>
        Public Shared Function GetRecordCountByCustomerID(customerID As String) As String
            Dim sb As New StringBuilder() 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Orders] ")
            sb.Append("WHERE [CustomerID] = @customerID ")

            Return sb.ToString() 
        End Function 

        ''' <summary>
        ''' Related to column EmployeeID
        ''' </summary>
        Public Shared Function GetRecordCountByEmployeeID(employeeID As Integer) As String
            Dim sb As New StringBuilder() 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Orders] ")
            sb.Append("WHERE [EmployeeID] = @employeeID ")

            Return sb.ToString() 
        End Function 

        ''' <summary>
        ''' Related to column ShipVia
        ''' </summary>
        Public Shared Function GetRecordCountByShipVia(shipVia As Integer) As String
            Dim sb As New StringBuilder() 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Orders] ")
            sb.Append("WHERE [ShipVia] = @shipVia ")

            Return sb.ToString() 
        End Function 

        Public Shared Function GetRecordCountDynamicWhere() As String 
            Dim sb As New StringBuilder() 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Orders]")

            sb.Append(" WHERE ")
            sb.Append("([OrderID] = @orderID OR @orderID IS NULL) AND ")
            sb.Append("([CustomerID] LIKE '%' + @customerID + '%' OR @customerID IS NULL) AND ")
            sb.Append("([EmployeeID] = @employeeID OR @employeeID IS NULL) AND ")
            sb.Append("([OrderDate] = @orderDate OR @orderDate IS NULL) AND ")
            sb.Append("([RequiredDate] = @requiredDate OR @requiredDate IS NULL) AND ")
            sb.Append("([ShippedDate] = @shippedDate OR @shippedDate IS NULL) AND ")
            sb.Append("([ShipVia] = @shipVia OR @shipVia IS NULL) AND ")
            sb.Append("([Freight] = @freight OR @freight IS NULL) AND ")
            sb.Append("([ShipName] LIKE '%' + @shipName + '%' OR @shipName IS NULL) AND ")
            sb.Append("([ShipAddress] LIKE '%' + @shipAddress + '%' OR @shipAddress IS NULL) AND ")
            sb.Append("([ShipCity] LIKE '%' + @shipCity + '%' OR @shipCity IS NULL) AND ")
            sb.Append("([ShipRegion] LIKE '%' + @shipRegion + '%' OR @shipRegion IS NULL) AND ")
            sb.Append("([ShipPostalCode] LIKE '%' + @shipPostalCode + '%' OR @shipPostalCode IS NULL) AND ")
            sb.Append("([ShipCountry] LIKE '%' + @shipCountry + '%' OR @shipCountry IS NULL) ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectSkipAndTake() As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start - 1; ")

            sb.Append("SELECT ")
            sb.Append("[OrderID], ")
            sb.Append("[CustomerID], ")
            sb.Append("[EmployeeID], ")
            sb.Append("[OrderDate], ")
            sb.Append("[RequiredDate], ")
            sb.Append("[ShippedDate], ")
            sb.Append("[ShipVia], ")
            sb.Append("[Freight], ")
            sb.Append("[ShipName], ")
            sb.Append("[ShipAddress], ")
            sb.Append("[ShipCity], ")
            sb.Append("[ShipRegion], ")
            sb.Append("[ShipPostalCode], ")
            sb.Append("[ShipCountry] ")
            sb.Append("FROM [dbo].[Orders] ")
            sb.Append("ORDER BY ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderID' THEN [OrderID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderID desc' THEN [OrderID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'CustomerID' THEN [CustomerID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'CustomerID desc' THEN [CustomerID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'EmployeeID' THEN [EmployeeID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'EmployeeID desc' THEN [EmployeeID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'OrderDate' THEN [OrderDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderDate desc' THEN [OrderDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'RequiredDate' THEN [RequiredDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'RequiredDate desc' THEN [RequiredDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShippedDate' THEN [ShippedDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShippedDate desc' THEN [ShippedDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipVia' THEN [ShipVia] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipVia desc' THEN [ShipVia] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Freight' THEN [Freight] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Freight desc' THEN [Freight] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipName' THEN [ShipName] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipName desc' THEN [ShipName] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipAddress' THEN [ShipAddress] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipAddress desc' THEN [ShipAddress] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipCity' THEN [ShipCity] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipCity desc' THEN [ShipCity] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipRegion' THEN [ShipRegion] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipRegion desc' THEN [ShipRegion] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipPostalCode' THEN [ShipPostalCode] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipPostalCode desc' THEN [ShipPostalCode] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipCountry' THEN [ShipCountry] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipCountry desc' THEN [ShipCountry] END DESC ")

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ")
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectSkipAndTakeByCustomerID(customerID As String) As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start - 1; ")

            sb.Append("SELECT ")
            sb.Append("[OrderID], ")
            sb.Append("[CustomerID], ")
            sb.Append("[EmployeeID], ")
            sb.Append("[OrderDate], ")
            sb.Append("[RequiredDate], ")
            sb.Append("[ShippedDate], ")
            sb.Append("[ShipVia], ")
            sb.Append("[Freight], ")
            sb.Append("[ShipName], ")
            sb.Append("[ShipAddress], ")
            sb.Append("[ShipCity], ")
            sb.Append("[ShipRegion], ")
            sb.Append("[ShipPostalCode], ")
            sb.Append("[ShipCountry] ")
            sb.Append("FROM [dbo].[Orders] ")
            sb.Append("WHERE [CustomerID] = @customerID ")
            sb.Append("ORDER BY ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderID' THEN [OrderID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderID desc' THEN [OrderID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'CustomerID' THEN [CustomerID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'CustomerID desc' THEN [CustomerID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'EmployeeID' THEN [EmployeeID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'EmployeeID desc' THEN [EmployeeID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'OrderDate' THEN [OrderDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderDate desc' THEN [OrderDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'RequiredDate' THEN [RequiredDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'RequiredDate desc' THEN [RequiredDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShippedDate' THEN [ShippedDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShippedDate desc' THEN [ShippedDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipVia' THEN [ShipVia] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipVia desc' THEN [ShipVia] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Freight' THEN [Freight] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Freight desc' THEN [Freight] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipName' THEN [ShipName] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipName desc' THEN [ShipName] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipAddress' THEN [ShipAddress] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipAddress desc' THEN [ShipAddress] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipCity' THEN [ShipCity] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipCity desc' THEN [ShipCity] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipRegion' THEN [ShipRegion] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipRegion desc' THEN [ShipRegion] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipPostalCode' THEN [ShipPostalCode] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipPostalCode desc' THEN [ShipPostalCode] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipCountry' THEN [ShipCountry] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipCountry desc' THEN [ShipCountry] END DESC ")

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ")
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectSkipAndTakeByEmployeeID(employeeID As Integer) As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start - 1; ")

            sb.Append("SELECT ")
            sb.Append("[OrderID], ")
            sb.Append("[CustomerID], ")
            sb.Append("[EmployeeID], ")
            sb.Append("[OrderDate], ")
            sb.Append("[RequiredDate], ")
            sb.Append("[ShippedDate], ")
            sb.Append("[ShipVia], ")
            sb.Append("[Freight], ")
            sb.Append("[ShipName], ")
            sb.Append("[ShipAddress], ")
            sb.Append("[ShipCity], ")
            sb.Append("[ShipRegion], ")
            sb.Append("[ShipPostalCode], ")
            sb.Append("[ShipCountry] ")
            sb.Append("FROM [dbo].[Orders] ")
            sb.Append("WHERE [EmployeeID] = @employeeID ")
            sb.Append("ORDER BY ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderID' THEN [OrderID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderID desc' THEN [OrderID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'CustomerID' THEN [CustomerID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'CustomerID desc' THEN [CustomerID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'EmployeeID' THEN [EmployeeID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'EmployeeID desc' THEN [EmployeeID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'OrderDate' THEN [OrderDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderDate desc' THEN [OrderDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'RequiredDate' THEN [RequiredDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'RequiredDate desc' THEN [RequiredDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShippedDate' THEN [ShippedDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShippedDate desc' THEN [ShippedDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipVia' THEN [ShipVia] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipVia desc' THEN [ShipVia] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Freight' THEN [Freight] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Freight desc' THEN [Freight] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipName' THEN [ShipName] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipName desc' THEN [ShipName] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipAddress' THEN [ShipAddress] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipAddress desc' THEN [ShipAddress] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipCity' THEN [ShipCity] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipCity desc' THEN [ShipCity] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipRegion' THEN [ShipRegion] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipRegion desc' THEN [ShipRegion] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipPostalCode' THEN [ShipPostalCode] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipPostalCode desc' THEN [ShipPostalCode] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipCountry' THEN [ShipCountry] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipCountry desc' THEN [ShipCountry] END DESC ")

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ")
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectSkipAndTakeByShipVia(shipVia As Integer) As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start - 1; ")

            sb.Append("SELECT ")
            sb.Append("[OrderID], ")
            sb.Append("[CustomerID], ")
            sb.Append("[EmployeeID], ")
            sb.Append("[OrderDate], ")
            sb.Append("[RequiredDate], ")
            sb.Append("[ShippedDate], ")
            sb.Append("[ShipVia], ")
            sb.Append("[Freight], ")
            sb.Append("[ShipName], ")
            sb.Append("[ShipAddress], ")
            sb.Append("[ShipCity], ")
            sb.Append("[ShipRegion], ")
            sb.Append("[ShipPostalCode], ")
            sb.Append("[ShipCountry] ")
            sb.Append("FROM [dbo].[Orders] ")
            sb.Append("WHERE [ShipVia] = @shipVia ")
            sb.Append("ORDER BY ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderID' THEN [OrderID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderID desc' THEN [OrderID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'CustomerID' THEN [CustomerID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'CustomerID desc' THEN [CustomerID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'EmployeeID' THEN [EmployeeID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'EmployeeID desc' THEN [EmployeeID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'OrderDate' THEN [OrderDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderDate desc' THEN [OrderDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'RequiredDate' THEN [RequiredDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'RequiredDate desc' THEN [RequiredDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShippedDate' THEN [ShippedDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShippedDate desc' THEN [ShippedDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipVia' THEN [ShipVia] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipVia desc' THEN [ShipVia] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Freight' THEN [Freight] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Freight desc' THEN [Freight] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipName' THEN [ShipName] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipName desc' THEN [ShipName] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipAddress' THEN [ShipAddress] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipAddress desc' THEN [ShipAddress] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipCity' THEN [ShipCity] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipCity desc' THEN [ShipCity] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipRegion' THEN [ShipRegion] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipRegion desc' THEN [ShipRegion] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipPostalCode' THEN [ShipPostalCode] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipPostalCode desc' THEN [ShipPostalCode] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipCountry' THEN [ShipCountry] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipCountry desc' THEN [ShipCountry] END DESC ")

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ")
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectSkipAndTakeDynamicWhere() As String 
            Dim sb As New StringBuilder() 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start - 1; ")

            sb.Append("SELECT ") 
            sb.Append("[OrderID], ") 
            sb.Append("[CustomerID], ") 
            sb.Append("[EmployeeID], ") 
            sb.Append("[OrderDate], ") 
            sb.Append("[RequiredDate], ") 
            sb.Append("[ShippedDate], ") 
            sb.Append("[ShipVia], ") 
            sb.Append("[Freight], ") 
            sb.Append("[ShipName], ") 
            sb.Append("[ShipAddress], ") 
            sb.Append("[ShipCity], ") 
            sb.Append("[ShipRegion], ") 
            sb.Append("[ShipPostalCode], ") 
            sb.Append("[ShipCountry] ") 
            sb.Append("FROM [dbo].[Orders] ") 
            sb.Append(" WHERE ")
            sb.Append("([OrderID] = @orderID OR @orderID IS NULL) AND ")
            sb.Append("([CustomerID] LIKE '%' + @customerID + '%' OR @customerID IS NULL) AND ")
            sb.Append("([EmployeeID] = @employeeID OR @employeeID IS NULL) AND ")
            sb.Append("([OrderDate] = @orderDate OR @orderDate IS NULL) AND ")
            sb.Append("([RequiredDate] = @requiredDate OR @requiredDate IS NULL) AND ")
            sb.Append("([ShippedDate] = @shippedDate OR @shippedDate IS NULL) AND ")
            sb.Append("([ShipVia] = @shipVia OR @shipVia IS NULL) AND ")
            sb.Append("([Freight] = @freight OR @freight IS NULL) AND ")
            sb.Append("([ShipName] LIKE '%' + @shipName + '%' OR @shipName IS NULL) AND ")
            sb.Append("([ShipAddress] LIKE '%' + @shipAddress + '%' OR @shipAddress IS NULL) AND ")
            sb.Append("([ShipCity] LIKE '%' + @shipCity + '%' OR @shipCity IS NULL) AND ")
            sb.Append("([ShipRegion] LIKE '%' + @shipRegion + '%' OR @shipRegion IS NULL) AND ")
            sb.Append("([ShipPostalCode] LIKE '%' + @shipPostalCode + '%' OR @shipPostalCode IS NULL) AND ")
            sb.Append("([ShipCountry] LIKE '%' + @shipCountry + '%' OR @shipCountry IS NULL) ")

            sb.Append("ORDER BY ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderID' THEN [OrderID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderID desc' THEN [OrderID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'CustomerID' THEN [CustomerID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'CustomerID desc' THEN [CustomerID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'EmployeeID' THEN [EmployeeID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'EmployeeID desc' THEN [EmployeeID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'OrderDate' THEN [OrderDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'OrderDate desc' THEN [OrderDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'RequiredDate' THEN [RequiredDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'RequiredDate desc' THEN [RequiredDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShippedDate' THEN [ShippedDate] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShippedDate desc' THEN [ShippedDate] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipVia' THEN [ShipVia] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipVia desc' THEN [ShipVia] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Freight' THEN [Freight] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Freight desc' THEN [Freight] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipName' THEN [ShipName] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipName desc' THEN [ShipName] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipAddress' THEN [ShipAddress] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipAddress desc' THEN [ShipAddress] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipCity' THEN [ShipCity] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipCity desc' THEN [ShipCity] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipRegion' THEN [ShipRegion] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipRegion desc' THEN [ShipRegion] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipPostalCode' THEN [ShipPostalCode] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipPostalCode desc' THEN [ShipPostalCode] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'ShipCountry' THEN [ShipCountry] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipCountry desc' THEN [ShipCountry] END DESC ")

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ")
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectTotals() As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append("SELECT ")
            sb.Append("SUM([Freight]) AS [FreightTotal] ")
            sb.Append("FROM [dbo].[Orders]")
            Return sb.ToString() 
        End Function 

        Public Shared Function SelectAll() As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append(selectStatement) 

            Return sb.ToString() 
        End Function 

        ''' <summary>
        ''' Related to column CustomerID
        ''' </summary>
        Public Shared Function SelectAllByCustomerID(ByVal customerID As String) As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 
 
            sb.Append(selectStatement) 
            sb.Append("WHERE [CustomerID] = '" & customerID & "'") 
 
            Return sb.ToString() 
        End Function 

        ''' <summary>
        ''' Related to column EmployeeID
        ''' </summary>
        Public Shared Function SelectAllByEmployeeID(ByVal employeeID As Integer) As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 
 
            sb.Append(selectStatement) 
            sb.Append("WHERE [EmployeeID] = " & employeeID) 
 
            Return sb.ToString() 
        End Function 

        ''' <summary>
        ''' Related to column ShipVia
        ''' </summary>
        Public Shared Function SelectAllByShipVia(ByVal shipVia As Integer) As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 
 
            sb.Append(selectStatement) 
            sb.Append("WHERE [ShipVia] = " & shipVia) 
 
            Return sb.ToString() 
        End Function 

        Public Shared Function SelectAllDynamicWhere() As String
            Dim selectStatement As String = GetSelectStatement()
            Dim sb As New StringBuilder()

            sb.Append(selectStatement)
            sb.Append(" WHERE ")
            sb.Append("([OrderID] = @orderID OR @orderID IS NULL) AND ")
            sb.Append("([CustomerID] LIKE '%' + @customerID + '%' OR @customerID IS NULL) AND ")
            sb.Append("([EmployeeID] = @employeeID OR @employeeID IS NULL) AND ")
            sb.Append("([OrderDate] = @orderDate OR @orderDate IS NULL) AND ")
            sb.Append("([RequiredDate] = @requiredDate OR @requiredDate IS NULL) AND ")
            sb.Append("([ShippedDate] = @shippedDate OR @shippedDate IS NULL) AND ")
            sb.Append("([ShipVia] = @shipVia OR @shipVia IS NULL) AND ")
            sb.Append("([Freight] = @freight OR @freight IS NULL) AND ")
            sb.Append("([ShipName] LIKE '%' + @shipName + '%' OR @shipName IS NULL) AND ")
            sb.Append("([ShipAddress] LIKE '%' + @shipAddress + '%' OR @shipAddress IS NULL) AND ")
            sb.Append("([ShipCity] LIKE '%' + @shipCity + '%' OR @shipCity IS NULL) AND ")
            sb.Append("([ShipRegion] LIKE '%' + @shipRegion + '%' OR @shipRegion IS NULL) AND ")
            sb.Append("([ShipPostalCode] LIKE '%' + @shipPostalCode + '%' OR @shipPostalCode IS NULL) AND ")
            sb.Append("([ShipCountry] LIKE '%' + @shipCountry + '%' OR @shipCountry IS NULL) ")

            Return sb.ToString() 
        End Function

        ''' <summary>
        ''' Selects OrderID and ShipName columns for use with a DropDownList web control
        ''' </summary>
        Public Shared Function SelectOrdersDropDownListData() As String
            Dim selectStatement As String = "SELECT [OrderID], [ShipName] FROM [dbo].[Orders] ORDER BY [ShipName] ASC " 
            Dim sb As New StringBuilder() 

            sb.Append(selectStatement)

            Return sb.ToString() 
        End Function 

        Public Shared Function Insert() As String
            Dim sb As New StringBuilder() 

            sb.Append("INSERT INTO [dbo].[Orders] ") 
            sb.Append("(") 
            sb.Append("[CustomerID], ")
            sb.Append("[EmployeeID], ")
            sb.Append("[OrderDate], ")
            sb.Append("[RequiredDate], ")
            sb.Append("[ShippedDate], ")
            sb.Append("[ShipVia], ")
            sb.Append("[Freight], ")
            sb.Append("[ShipName], ")
            sb.Append("[ShipAddress], ")
            sb.Append("[ShipCity], ")
            sb.Append("[ShipRegion], ")
            sb.Append("[ShipPostalCode], ")
            sb.Append("[ShipCountry] ")
            sb.Append(") ") 
            sb.Append("OUTPUT inserted.[OrderID] ") 
            sb.Append("VALUES ") 
            sb.Append("(") 
            sb.Append("@customerID,")
            sb.Append("@employeeID,")
            sb.Append("@orderDate,")
            sb.Append("@requiredDate,")
            sb.Append("@shippedDate,")
            sb.Append("@shipVia,")
            sb.Append("@freight,")
            sb.Append("@shipName,")
            sb.Append("@shipAddress,")
            sb.Append("@shipCity,")
            sb.Append("@shipRegion,")
            sb.Append("@shipPostalCode,")
            sb.Append("@shipCountry")
            sb.Append(")") 

            Return sb.ToString() 
        End Function 

        Public Shared Function Update() As String
            Dim sb As New StringBuilder() 

            sb.Append("UPDATE [dbo].[Orders] ") 
            sb.Append("SET ") 
            sb.Append("[CustomerID] = @customerID,")
            sb.Append("[EmployeeID] = @employeeID,")
            sb.Append("[OrderDate] = @orderDate,")
            sb.Append("[RequiredDate] = @requiredDate,")
            sb.Append("[ShippedDate] = @shippedDate,")
            sb.Append("[ShipVia] = @shipVia,")
            sb.Append("[Freight] = @freight,")
            sb.Append("[ShipName] = @shipName,")
            sb.Append("[ShipAddress] = @shipAddress,")
            sb.Append("[ShipCity] = @shipCity,")
            sb.Append("[ShipRegion] = @shipRegion,")
            sb.Append("[ShipPostalCode] = @shipPostalCode,")
            sb.Append("[ShipCountry] = @shipCountry")
            sb.Append(" WHERE ") 
            sb.Append("[OrderID] = @orderID ") 

            Return sb.ToString() 
        End Function 

        Public Shared Function Delete() As String 
            Dim sb As New StringBuilder() 

            sb.Append("DELETE FROM [dbo].[Orders] ") 
            sb.Append(" WHERE ") 
            sb.Append("[OrderID] = @orderID ") 

            Return sb.ToString() 
        End Function 

        Public Shared Function GetSelectStatement() As String 
            Dim sb As New StringBuilder() 

            sb.Append("SELECT ") 
            sb.Append("[OrderID], ") 
            sb.Append("[CustomerID], ") 
            sb.Append("[EmployeeID], ") 
            sb.Append("[OrderDate], ") 
            sb.Append("[RequiredDate], ") 
            sb.Append("[ShippedDate], ") 
            sb.Append("[ShipVia], ") 
            sb.Append("[Freight], ") 
            sb.Append("[ShipName], ") 
            sb.Append("[ShipAddress], ") 
            sb.Append("[ShipCity], ") 
            sb.Append("[ShipRegion], ") 
            sb.Append("[ShipPostalCode], ") 
            sb.Append("[ShipCountry] ") 
            sb.Append("FROM [dbo].[Orders] ") 

            Return sb.ToString() 
        End Function 
    End Class 
End Namespace 
