Imports System 
Imports System.Text 

Namespace DataLayer.Base
    Public NotInheritable Class RegionSql
        Private Sub New() 
        End Sub 

        Public Shared Function SelectByPrimaryKey() As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append(selectStatement) 
            sb.Append(" WHERE ") 
            sb.Append("[RegionID] = @regionID ") 

            Return sb.ToString() 
        End Function 

        Public Shared Function GetRecordCount() As String 
            Dim sb As New StringBuilder() 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Region]")

            Return sb.ToString() 
        End Function 

        Public Shared Function GetRecordCountDynamicWhere() As String 
            Dim sb As New StringBuilder() 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Region]")

            sb.Append(" WHERE ")
            sb.Append("([RegionID] = @regionID OR @regionID IS NULL) AND ")
            sb.Append("([RegionDescription] LIKE '%' + @regionDescription + '%' OR @regionDescription IS NULL) ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectSkipAndTake() As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start - 1; ")

            sb.Append("SELECT ")
            sb.Append("[RegionID], ")
            sb.Append("[RegionDescription] ")
            sb.Append("FROM [dbo].[Region] ")
            sb.Append("ORDER BY ")
	        sb.Append("CASE WHEN @sortByExpression = 'RegionID' THEN [RegionID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'RegionID desc' THEN [RegionID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'RegionDescription' THEN [RegionDescription] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'RegionDescription desc' THEN [RegionDescription] END DESC ")

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ")
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectSkipAndTakeDynamicWhere() As String 
            Dim sb As New StringBuilder() 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start - 1; ")

            sb.Append("SELECT ") 
            sb.Append("[RegionID], ") 
            sb.Append("[RegionDescription] ") 
            sb.Append("FROM [dbo].[Region] ") 
            sb.Append(" WHERE ")
            sb.Append("([RegionID] = @regionID OR @regionID IS NULL) AND ")
            sb.Append("([RegionDescription] LIKE '%' + @regionDescription + '%' OR @regionDescription IS NULL) ")

            sb.Append("ORDER BY ")
	        sb.Append("CASE WHEN @sortByExpression = 'RegionID' THEN [RegionID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'RegionID desc' THEN [RegionID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'RegionDescription' THEN [RegionDescription] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'RegionDescription desc' THEN [RegionDescription] END DESC ")

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ")
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectAll() As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append(selectStatement) 

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectAllDynamicWhere() As String
            Dim selectStatement As String = GetSelectStatement()
            Dim sb As New StringBuilder()

            sb.Append(selectStatement)
            sb.Append(" WHERE ")
            sb.Append("([RegionID] = @regionID OR @regionID IS NULL) AND ")
            sb.Append("([RegionDescription] LIKE '%' + @regionDescription + '%' OR @regionDescription IS NULL) ")

            Return sb.ToString() 
        End Function

        ''' <summary>
        ''' Selects RegionID and RegionDescription columns for use with a DropDownList web control
        ''' </summary>
        Public Shared Function SelectRegionDropDownListData() As String
            Dim selectStatement As String = "SELECT [RegionID], [RegionDescription] FROM [dbo].[Region] ORDER BY [RegionDescription] ASC " 
            Dim sb As New StringBuilder() 

            sb.Append(selectStatement)

            Return sb.ToString() 
        End Function 

        Public Shared Function Insert() As String
            Dim sb As New StringBuilder() 

            sb.Append("INSERT INTO [dbo].[Region] ") 
            sb.Append("(") 
            sb.Append("[RegionID], ")
            sb.Append("[RegionDescription] ")
            sb.Append(") ") 
            sb.Append("OUTPUT inserted.[RegionID] ") 
            sb.Append("VALUES ") 
            sb.Append("(") 
            sb.Append("@regionID,")
            sb.Append("@regionDescription")
            sb.Append(")") 

            Return sb.ToString() 
        End Function 

        Public Shared Function Update() As String
            Dim sb As New StringBuilder() 

            sb.Append("UPDATE [dbo].[Region] ") 
            sb.Append("SET ") 
            sb.Append("[RegionID] = @regionID,")
            sb.Append("[RegionDescription] = @regionDescription")
            sb.Append(" WHERE ") 
            sb.Append("[RegionID] = @regionID ") 

            Return sb.ToString() 
        End Function 

        Public Shared Function Delete() As String 
            Dim sb As New StringBuilder() 

            sb.Append("DELETE FROM [dbo].[Region] ") 
            sb.Append(" WHERE ") 
            sb.Append("[RegionID] = @regionID ") 

            Return sb.ToString() 
        End Function 

        Public Shared Function GetSelectStatement() As String 
            Dim sb As New StringBuilder() 

            sb.Append("SELECT ") 
            sb.Append("[RegionID], ") 
            sb.Append("[RegionDescription] ") 
            sb.Append("FROM [dbo].[Region] ") 

            Return sb.ToString() 
        End Function 
    End Class 
End Namespace 
