Imports System 
Imports System.Text 

Namespace DataLayer.Base
    Public NotInheritable Class ShippersSql
        Private Sub New() 
        End Sub 

        Public Shared Function SelectByPrimaryKey() As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append(selectStatement) 
            sb.Append(" WHERE ") 
            sb.Append("[ShipperID] = @shipperID ") 

            Return sb.ToString() 
        End Function 

        Public Shared Function GetRecordCount() As String 
            Dim sb As New StringBuilder() 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Shippers]")

            Return sb.ToString() 
        End Function 

        Public Shared Function GetRecordCountDynamicWhere() As String 
            Dim sb As New StringBuilder() 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Shippers]")

            sb.Append(" WHERE ")
            sb.Append("([ShipperID] = @shipperID OR @shipperID IS NULL) AND ")
            sb.Append("([CompanyName] LIKE '%' + @companyName + '%' OR @companyName IS NULL) AND ")
            sb.Append("([Phone] LIKE '%' + @phone + '%' OR @phone IS NULL) ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectSkipAndTake() As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start - 1; ")

            sb.Append("SELECT ")
            sb.Append("[ShipperID], ")
            sb.Append("[CompanyName], ")
            sb.Append("[Phone] ")
            sb.Append("FROM [dbo].[Shippers] ")
            sb.Append("ORDER BY ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipperID' THEN [ShipperID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipperID desc' THEN [ShipperID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'CompanyName' THEN [CompanyName] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'CompanyName desc' THEN [CompanyName] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Phone' THEN [Phone] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Phone desc' THEN [Phone] END DESC ")

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ")
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectSkipAndTakeDynamicWhere() As String 
            Dim sb As New StringBuilder() 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start - 1; ")

            sb.Append("SELECT ") 
            sb.Append("[ShipperID], ") 
            sb.Append("[CompanyName], ") 
            sb.Append("[Phone] ") 
            sb.Append("FROM [dbo].[Shippers] ") 
            sb.Append(" WHERE ")
            sb.Append("([ShipperID] = @shipperID OR @shipperID IS NULL) AND ")
            sb.Append("([CompanyName] LIKE '%' + @companyName + '%' OR @companyName IS NULL) AND ")
            sb.Append("([Phone] LIKE '%' + @phone + '%' OR @phone IS NULL) ")

            sb.Append("ORDER BY ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipperID' THEN [ShipperID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'ShipperID desc' THEN [ShipperID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'CompanyName' THEN [CompanyName] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'CompanyName desc' THEN [CompanyName] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'Phone' THEN [Phone] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'Phone desc' THEN [Phone] END DESC ")

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ")
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectAll() As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append(selectStatement) 

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectAllDynamicWhere() As String
            Dim selectStatement As String = GetSelectStatement()
            Dim sb As New StringBuilder()

            sb.Append(selectStatement)
            sb.Append(" WHERE ")
            sb.Append("([ShipperID] = @shipperID OR @shipperID IS NULL) AND ")
            sb.Append("([CompanyName] LIKE '%' + @companyName + '%' OR @companyName IS NULL) AND ")
            sb.Append("([Phone] LIKE '%' + @phone + '%' OR @phone IS NULL) ")

            Return sb.ToString() 
        End Function

        ''' <summary>
        ''' Selects ShipperID and CompanyName columns for use with a DropDownList web control
        ''' </summary>
        Public Shared Function SelectShippersDropDownListData() As String
            Dim selectStatement As String = "SELECT [ShipperID], [CompanyName] FROM [dbo].[Shippers] ORDER BY [CompanyName] ASC " 
            Dim sb As New StringBuilder() 

            sb.Append(selectStatement)

            Return sb.ToString() 
        End Function 

        Public Shared Function Insert() As String
            Dim sb As New StringBuilder() 

            sb.Append("INSERT INTO [dbo].[Shippers] ") 
            sb.Append("(") 
            sb.Append("[CompanyName], ")
            sb.Append("[Phone] ")
            sb.Append(") ") 
            sb.Append("OUTPUT inserted.[ShipperID] ") 
            sb.Append("VALUES ") 
            sb.Append("(") 
            sb.Append("@companyName,")
            sb.Append("@phone")
            sb.Append(")") 

            Return sb.ToString() 
        End Function 

        Public Shared Function Update() As String
            Dim sb As New StringBuilder() 

            sb.Append("UPDATE [dbo].[Shippers] ") 
            sb.Append("SET ") 
            sb.Append("[CompanyName] = @companyName,")
            sb.Append("[Phone] = @phone")
            sb.Append(" WHERE ") 
            sb.Append("[ShipperID] = @shipperID ") 

            Return sb.ToString() 
        End Function 

        Public Shared Function Delete() As String 
            Dim sb As New StringBuilder() 

            sb.Append("DELETE FROM [dbo].[Shippers] ") 
            sb.Append(" WHERE ") 
            sb.Append("[ShipperID] = @shipperID ") 

            Return sb.ToString() 
        End Function 

        Public Shared Function GetSelectStatement() As String 
            Dim sb As New StringBuilder() 

            sb.Append("SELECT ") 
            sb.Append("[ShipperID], ") 
            sb.Append("[CompanyName], ") 
            sb.Append("[Phone] ") 
            sb.Append("FROM [dbo].[Shippers] ") 

            Return sb.ToString() 
        End Function 
    End Class 
End Namespace 
