Imports System 
Imports System.Text 

Namespace DataLayer.Base
    Public NotInheritable Class TerritoriesSql
        Private Sub New() 
        End Sub 

        Public Shared Function SelectByPrimaryKey() As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append(selectStatement) 
            sb.Append(" WHERE ") 
            sb.Append("[TerritoryID] = @territoryID ") 

            Return sb.ToString() 
        End Function 

        Public Shared Function GetRecordCount() As String 
            Dim sb As New StringBuilder() 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Territories]")

            Return sb.ToString() 
        End Function 

        ''' <summary>
        ''' Related to column RegionID
        ''' </summary>
        Public Shared Function GetRecordCountByRegionID(regionID As Integer) As String
            Dim sb As New StringBuilder() 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Territories] ")
            sb.Append("WHERE [RegionID] = @regionID ")

            Return sb.ToString() 
        End Function 

        Public Shared Function GetRecordCountDynamicWhere() As String 
            Dim sb As New StringBuilder() 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Territories]")

            sb.Append(" WHERE ")
            sb.Append("([TerritoryID] LIKE '%' + @territoryID + '%' OR @territoryID IS NULL) AND ")
            sb.Append("([TerritoryDescription] LIKE '%' + @territoryDescription + '%' OR @territoryDescription IS NULL) AND ")
            sb.Append("([RegionID] = @regionID OR @regionID IS NULL) ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectSkipAndTake() As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start - 1; ")

            sb.Append("SELECT ")
            sb.Append("[TerritoryID], ")
            sb.Append("[TerritoryDescription], ")
            sb.Append("[RegionID] ")
            sb.Append("FROM [dbo].[Territories] ")
            sb.Append("ORDER BY ")
	        sb.Append("CASE WHEN @sortByExpression = 'TerritoryID' THEN [TerritoryID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'TerritoryID desc' THEN [TerritoryID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'TerritoryDescription' THEN [TerritoryDescription] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'TerritoryDescription desc' THEN [TerritoryDescription] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'RegionID' THEN [RegionID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'RegionID desc' THEN [RegionID] END DESC ")

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ")
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectSkipAndTakeByRegionID(regionID As Integer) As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start - 1; ")

            sb.Append("SELECT ")
            sb.Append("[TerritoryID], ")
            sb.Append("[TerritoryDescription], ")
            sb.Append("[RegionID] ")
            sb.Append("FROM [dbo].[Territories] ")
            sb.Append("WHERE [RegionID] = @regionID ")
            sb.Append("ORDER BY ")
	        sb.Append("CASE WHEN @sortByExpression = 'TerritoryID' THEN [TerritoryID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'TerritoryID desc' THEN [TerritoryID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'TerritoryDescription' THEN [TerritoryDescription] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'TerritoryDescription desc' THEN [TerritoryDescription] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'RegionID' THEN [RegionID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'RegionID desc' THEN [RegionID] END DESC ")

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ")
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectSkipAndTakeDynamicWhere() As String 
            Dim sb As New StringBuilder() 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start - 1; ")

            sb.Append("SELECT ") 
            sb.Append("[TerritoryID], ") 
            sb.Append("[TerritoryDescription], ") 
            sb.Append("[RegionID] ") 
            sb.Append("FROM [dbo].[Territories] ") 
            sb.Append(" WHERE ")
            sb.Append("([TerritoryID] LIKE '%' + @territoryID + '%' OR @territoryID IS NULL) AND ")
            sb.Append("([TerritoryDescription] LIKE '%' + @territoryDescription + '%' OR @territoryDescription IS NULL) AND ")
            sb.Append("([RegionID] = @regionID OR @regionID IS NULL) ")

            sb.Append("ORDER BY ")
	        sb.Append("CASE WHEN @sortByExpression = 'TerritoryID' THEN [TerritoryID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'TerritoryID desc' THEN [TerritoryID] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'TerritoryDescription' THEN [TerritoryDescription] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'TerritoryDescription desc' THEN [TerritoryDescription] END DESC, ")

	        sb.Append("CASE WHEN @sortByExpression = 'RegionID' THEN [RegionID] END, ")
	        sb.Append("CASE WHEN @sortByExpression = 'RegionID desc' THEN [RegionID] END DESC ")

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ")
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ")

            Return sb.ToString() 
        End Function 

        Public Shared Function SelectAll() As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 

            sb.Append(selectStatement) 

            Return sb.ToString() 
        End Function 

        ''' <summary>
        ''' Related to column RegionID
        ''' </summary>
        Public Shared Function SelectAllByRegionID(ByVal regionID As Integer) As String 
            Dim selectStatement As String = GetSelectStatement() 
            Dim sb As New StringBuilder() 
 
            sb.Append(selectStatement) 
            sb.Append("WHERE [RegionID] = " & regionID) 
 
            Return sb.ToString() 
        End Function 

        Public Shared Function SelectAllDynamicWhere() As String
            Dim selectStatement As String = GetSelectStatement()
            Dim sb As New StringBuilder()

            sb.Append(selectStatement)
            sb.Append(" WHERE ")
            sb.Append("([TerritoryID] LIKE '%' + @territoryID + '%' OR @territoryID IS NULL) AND ")
            sb.Append("([TerritoryDescription] LIKE '%' + @territoryDescription + '%' OR @territoryDescription IS NULL) AND ")
            sb.Append("([RegionID] = @regionID OR @regionID IS NULL) ")

            Return sb.ToString() 
        End Function

        ''' <summary>
        ''' Selects TerritoryID and TerritoryDescription columns for use with a DropDownList web control
        ''' </summary>
        Public Shared Function SelectTerritoriesDropDownListData() As String
            Dim selectStatement As String = "SELECT [TerritoryID], [TerritoryDescription] FROM [dbo].[Territories] ORDER BY [TerritoryDescription] ASC " 
            Dim sb As New StringBuilder() 

            sb.Append(selectStatement)

            Return sb.ToString() 
        End Function 

        Public Shared Function Insert() As String
            Dim sb As New StringBuilder() 

            sb.Append("INSERT INTO [dbo].[Territories] ") 
            sb.Append("(") 
            sb.Append("[TerritoryID], ")
            sb.Append("[TerritoryDescription], ")
            sb.Append("[RegionID] ")
            sb.Append(") ") 
            sb.Append("OUTPUT inserted.[TerritoryID] ") 
            sb.Append("VALUES ") 
            sb.Append("(") 
            sb.Append("@territoryID,")
            sb.Append("@territoryDescription,")
            sb.Append("@regionID")
            sb.Append(")") 

            Return sb.ToString() 
        End Function 

        Public Shared Function Update() As String
            Dim sb As New StringBuilder() 

            sb.Append("UPDATE [dbo].[Territories] ") 
            sb.Append("SET ") 
            sb.Append("[TerritoryID] = @territoryID,")
            sb.Append("[TerritoryDescription] = @territoryDescription,")
            sb.Append("[RegionID] = @regionID")
            sb.Append(" WHERE ") 
            sb.Append("[TerritoryID] = @territoryID ") 

            Return sb.ToString() 
        End Function 

        Public Shared Function Delete() As String 
            Dim sb As New StringBuilder() 

            sb.Append("DELETE FROM [dbo].[Territories] ") 
            sb.Append(" WHERE ") 
            sb.Append("[TerritoryID] = @territoryID ") 

            Return sb.ToString() 
        End Function 

        Public Shared Function GetSelectStatement() As String 
            Dim sb As New StringBuilder() 

            sb.Append("SELECT ") 
            sb.Append("[TerritoryID], ") 
            sb.Append("[TerritoryDescription], ") 
            sb.Append("[RegionID] ") 
            sb.Append("FROM [dbo].[Territories] ") 

            Return sb.ToString() 
        End Function 
    End Class 
End Namespace 
