Imports System
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class ListCrudRedirectEmployees
        Dim _currentRecordNumber As Integer = 1
        Dim _currentPageNumber As Integer = 1
        Dim _totalRecords As Integer = 0
        Dim _totalNumberOfPages As Integer = 0
        Dim _sortColumnIndex As Integer = 0
        Dim _sortByExpression As String = "EmployeeID"
        Dim _sortOrder As SortOrder = SortOrder.Ascending
        Dim _currentRowIndex As Integer
        Dim _isFireToolStripComboBoxSelectedIndexChanged As Boolean

        #Region "winform events"

        Private Sub ListCrudRedirectEmployees_Load(sender As Object, e As EventArgs) Handles MyBase.Load
            SetGridProperties()
            SetGridColumnProperties()
            SetDataSource(False)
            ResetNavButtonsToDefault()

            BnvNavAddNewItem.Enabled = True
        End Sub

        #End Region

        #Region "binding navigator control events"

        Private Sub BnvNavMoveFirstItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveFirstItem.Click
            _currentRecordNumber = 1
            _currentPageNumber = 1
            MoveToFirstPage()
        End Sub

        Private Sub BnvNavMovePreviousItem_Click(sender As Object, e As EventArgs) Handles BnvNavMovePreviousItem.Click
            _currentPageNumber -= 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToPrevious()
        End Sub

        Private Sub BnvNavMoveNextItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveNextItem.Click
            _currentPageNumber += 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToNext()
        End Sub

        Private Sub BnvNavMoveLastItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveLastItem.Click
            _currentPageNumber = _totalNumberOfPages
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToLastPage()
        End Sub

        Private Sub BnvNavAddNewItem_Click(sender As Object, e As EventArgs) Handles BnvNavAddNewItem.Click
            ShowAddEditDialog(CrudOperation.Add)
        End Sub

        Private Sub BnvNavEditItem_Click(sender As Object, e As EventArgs) Handles BnvNavEditItem.Click
            ShowAddEditDialog(CrudOperation.Update)
        End Sub

        Private Sub BnvNavDeleteItem_Click(sender As Object, e As EventArgs) Handles BnvNavDeleteItem.Click
            DeleteItem()
        End Sub

        #End Region

        #Region "data gridview events"

        Private Sub DgvGrid_ColumnHeaderMouseClick(sender As Object, e As DataGridViewCellMouseEventArgs) Handles DgvGrid.ColumnHeaderMouseClick
            _sortColumnIndex = e.ColumnIndex
            SetDataSource(True)
        End Sub

        Private Sub DgvGrid_CellContextMenuStripNeeded(sender As Object, e As DataGridViewCellContextMenuStripNeededEventArgs) Handles DgvGrid.CellContextMenuStripNeeded
            HightlightRow(e.RowIndex)
            e.ContextMenuStrip = contextMenuStrip1
        End Sub

        Private Sub DgvGrid_CellClick(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.CellClick
            ' make sure the click did not come from the header
            If e.RowIndex <> -1 Then
                HightlightRow(e.RowIndex)
            End If
        End Sub

        Private Sub DgvGrid_CellContentClick(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.CellContentClick
            If e.ColumnIndex = 15  And e.RowIndex <> -1 Then
                Dim reportsTo1 As Integer = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("ReportsTo").Value.ToString())

                Dim form As New DetailsEmployees(reportsTo1)
                form.ShowDialog()
            End If
        End Sub

        #End Region

        #Region "toolstrip menu item events"

        Private Sub TspEditMenuItem_Click(sender As Object, e As EventArgs) Handles TspEditMenuItem.Click
            ShowAddEditDialog(CrudOperation.Update)
        End Sub

        Private Sub TspDeleteMenuItem_Click(sender As Object, e As EventArgs) Handles TspDeleteMenuItem.Click
            DeleteItem()
        End Sub

        Private Sub TscbPage_SelectedIndexChanged(sender As Object, e As EventArgs) Handles TscbPage.SelectedIndexChanged
            ' make sure this event is not called 
            ' when adding items to the tool strip combo box
            If Not _isFireToolStripComboBoxSelectedIndexChanged Then
                Return
            End If

            ' set current page number
            Dim lastVisitedPageNumber As Integer = _currentPageNumber
            _currentPageNumber = Cint(TscbPage.ComboBox.SelectedItem)

            If _currentPageNumber = 1 Then
                _currentRecordNumber = 1
            Else
                _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))
            End If

            ' go to the selected page
            If _currentPageNumber = 1 Then
                MoveToFirstPage()
            Else If _currentPageNumber = _totalNumberOfPages Then
                MoveToLastPage()
            Else If _currentPageNumber < lastVisitedPageNumber Then
                MoveToPrevious()
            Else
                MoveToNext()
            End If
        End Sub

        #End Region

        #Region "private methods"

        Private Sub SetGridProperties()
            DgvGrid.AutoGenerateColumns = False
            DgvGrid.AllowUserToResizeRows = False
            DgvGrid.ColumnCount = 17
            DgvGrid.ScrollBars = ScrollBars.Horizontal
            DgvGrid.AllowUserToOrderColumns = False
            DgvGrid.AllowUserToAddRows = False
            DgvGrid.AllowUserToDeleteRows = False
            DgvGrid.ReadOnly = True
        End Sub

        Private Sub SetGridColumnProperties()
            DgvGrid.Columns("EmployeeID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("LastName").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("FirstName").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("Title").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("TitleOfCourtesy").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("BirthDate").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("HireDate").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("Address").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("City").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("Region1").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("PostalCode").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("Country").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("HomePhone").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("Extension").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("Notes").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("ReportsTo").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("PhotoPath").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
        End Sub

        Private Sub MoveToFirstPage()
            SetDataSource(False)
            ToolStripHelper.MoveToFirst(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub MoveToPrevious()
            SetDataSource(False)
            ToolStripHelper.MoveToPrevious(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentRecordNumber)
        End Sub

        Private Sub MoveToNext()
            SetDataSource(False)
            ToolStripHelper.MoveToNext(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentPageNumber, _totalNumberOfPages)
        End Sub

        Private Sub MoveToLastPage()
            SetDataSource(False)
            ToolStripHelper.MoveToLast(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub SetDataSource(isSort As Boolean)
            If isSort Then
                _sortByExpression = GetSortByExpression()
            End If

            ' fill the datagridview
            DgvGrid.DataSource = Employees.SelectSkipAndTake(ConstantHelper.GridNumberOfRows, _currentRecordNumber, _totalRecords, _sortByExpression)
            DgvGrid.Columns(_sortColumnIndex).HeaderCell.SortGlyphDirection = _sortOrder

            If _totalRecords > 0 Then
                _totalNumberOfPages = CInt(Math.Ceiling(CSng(_totalRecords) / CSng(ConstantHelper.GridNumberOfRows)))
                TscbPage.ComboBox.Items.Clear()

                ' fill page numbers in toolstripcombobox
                For page As Integer = 1 To _totalNumberOfPages
                    _isFireToolStripComboBoxSelectedIndexChanged = False
                    TscbPage.ComboBox.Items.Add(page)
                Next

                TscbPage.ComboBox.SelectedItem = _currentPageNumber
                _isFireToolStripComboBoxSelectedIndexChanged = True
            End If

            ResetNavButtonsToDefault()

            ' disable delete and edit on the toolstrip
            BnvNavEditItem.Enabled = False
            BnvNavDeleteItem.Enabled = False
        End Sub

        Private Function GetSortByExpression() As String
            _sortOrder = SortOrder.Descending

            Select Case _sortColumnIndex
                Case 0
                    If _sortByExpression = "EmployeeID" Then
                        Return "EmployeeID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "EmployeeID"

                Case 1
                    If _sortByExpression = "LastName" Then
                        Return "LastName desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "LastName"

                Case 2
                    If _sortByExpression = "FirstName" Then
                        Return "FirstName desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "FirstName"

                Case 3
                    If _sortByExpression = "Title" Then
                        Return "Title desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Title"

                Case 4
                    If _sortByExpression = "TitleOfCourtesy" Then
                        Return "TitleOfCourtesy desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "TitleOfCourtesy"

                Case 5
                    If _sortByExpression = "BirthDate" Then
                        Return "BirthDate desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "BirthDate"

                Case 6
                    If _sortByExpression = "HireDate" Then
                        Return "HireDate desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "HireDate"

                Case 7
                    If _sortByExpression = "Address" Then
                        Return "Address desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Address"

                Case 8
                    If _sortByExpression = "City" Then
                        Return "City desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "City"

                Case 9
                    If _sortByExpression = "Region1" Then
                        Return "Region1 desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Region1"

                Case 10
                    If _sortByExpression = "PostalCode" Then
                        Return "PostalCode desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "PostalCode"

                Case 11
                    If _sortByExpression = "Country" Then
                        Return "Country desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Country"

                Case 12
                    If _sortByExpression = "HomePhone" Then
                        Return "HomePhone desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "HomePhone"

                Case 13
                    If _sortByExpression = "Extension" Then
                        Return "Extension desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Extension"

                Case 14
                    If _sortByExpression = "Notes" Then
                        Return "Notes desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Notes"

                Case 15
                    If _sortByExpression = "ReportsTo" Then
                        Return "ReportsTo desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ReportsTo"

                Case 16
                    If _sortByExpression = "PhotoPath" Then
                        Return "PhotoPath desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "PhotoPath"

                Case Else
                    If _sortByExpression = "EmployeeID" Then
                        Return "EmployeeID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "EmployeeID"
            End Select
        End Function

        Private Sub ShowAddEditDialog(operation As CrudOperation)
            Dim form As Form

            If operation = CrudOperation.Add Then
                form = New AddEmployees()
            Else
                Dim employeeID1 As Integer = Convert.ToInt32(DgvGrid.Rows(_currentRowIndex).Cells("EmployeeID").Value.ToString())
                form = New UpdateEmployees(employeeID1)
            End If

            Dim result As DialogResult = form.ShowDialog()

            If result = DialogResult.Cancel Then
                SetDataSource(False)
            End If
        End Sub

        Private Sub ResetNavButtonsToDefault()
            BnvNavCountItem.Text = "of " & _totalNumberOfPages & " pages"

            If _totalRecords <= ConstantHelper.GridNumberOfRows Then
                BnvNavMoveNextItem.Enabled = False
                BnvNavMoveLastItem.Enabled = False
            Else
                BnvNavMoveNextItem.Enabled = True
                BnvNavMoveLastItem.Enabled = True
            End If
        End Sub

        Private Sub HightlightRow(rowIndex As Integer)
            ' highlight row
            DgvGrid.ClearSelection()
            DgvGrid.Rows(rowIndex).Selected = True

            ' enable edit and delete buttons on toolbar
            BnvNavEditItem.Enabled = True
            BnvNavDeleteItem.Enabled = True

            ' remember the current row clicked by the user
            _currentRowIndex = rowIndex
        End Sub

        Private Sub DeleteItem()
            Dim employeeID1 As Integer = Convert.ToInt32(DgvGrid.Rows(_currentRowIndex).Cells("EmployeeID").Value.ToString())
            Dim result As DialogResult = MessageBox.Show("Are you sure you want to delete item: " & employeeID1 & "?", "Confirm Item Deletion", MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation)

            If result = DialogResult.OK Then
                ' delete item
                Employees.Delete(employeeID1)

                SetDataSource(False)
                BnvNavDeleteItem.Enabled = False
                BnvNavEditItem.Enabled = False
            End If
        End Sub

        #End Region
    End Class
