Imports System
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class ListInlineOrders
        Dim _currentRecordNumber As Integer = 1
        Dim _currentPageNumber As Integer = 1
        Dim _totalRecords As Integer = 0
        Dim _totalNumberOfPages As Integer = 0
        Dim _sortColumnIndex As Integer = 0
        Dim _sortByExpression As String = "OrderID"
        Dim _sortOrder As SortOrder = SortOrder.Ascending
        Dim _currentRowIndex As Integer
        Dim _isFireToolStripComboBoxSelectedIndexChanged As Boolean

        #Region "winform events"

        Private Sub ListInlineOrders_Load(sender As Object, e As EventArgs) Handles MyBase.Load
            SetGridProperties()
            SetGridColumnProperties()
            FillGridComboBoxColumns()
            SetDataSource(False)
            ResetNavButtonsToDefault()

            BnvNavAddNewItem.Enabled = True
        End Sub

        #End Region

        #Region "binding navigator control events"

        Private Sub BnvNavMoveFirstItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveFirstItem.Click
            _currentRecordNumber = 1
            _currentPageNumber = 1
            MoveToFirstPage()
        End Sub

        Private Sub BnvNavMovePreviousItem_Click(sender As Object, e As EventArgs) Handles BnvNavMovePreviousItem.Click
            _currentPageNumber -= 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToPrevious()
        End Sub

        Private Sub BnvNavMoveNextItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveNextItem.Click
            _currentPageNumber += 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToNext()
        End Sub

        Private Sub BnvNavMoveLastItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveLastItem.Click
            _currentPageNumber = _totalNumberOfPages
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToLastPage()
        End Sub

        Private Sub BnvNavAddNewItem_Click(sender As Object, e As EventArgs) Handles BnvNavAddNewItem.Click
            ' add just the required fields
            Dim objOrders As New Orders()

            objOrders.Insert()

            _currentRecordNumber = 1
            _currentPageNumber = 1
            _sortColumnIndex = 0
            _sortByExpression = "OrderID"

            SetDataSource(True)

            ' focus on the first text box
            DgvGrid.Focus()
            DgvGrid.CurrentCell = DgvGrid.Rows(0).Cells("ShipName")
        End Sub

        #End Region

        #Region "data gridview events"

        Private Sub DgvGrid_ColumnHeaderMouseClick(sender As Object, e As DataGridViewCellMouseEventArgs) Handles DgvGrid.ColumnHeaderMouseClick
            _sortColumnIndex = e.ColumnIndex
            SetDataSource(True)
        End Sub

        Private Sub DgvGrid_DataError(sender As Object, e As DataGridViewDataErrorEventArgs) Handles DgvGrid.DataError
            e.ThrowException = False
            e.Cancel = False
        End Sub

        Private Sub DgvGrid_RowLeave(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.RowLeave
            If DgvGrid.IsCurrentRowDirty Then
                Dim orderID As Integer
                Dim customerID As String = Nothing
                Dim employeeID As Integer? = Nothing
                Dim orderDate As Date? = DateTime.MinValue
                Dim requiredDate As Date? = DateTime.MinValue
                Dim shippedDate As Date? = DateTime.MinValue
                Dim shipVia As Integer? = Nothing
                Dim freight As Decimal? = Nothing
                Dim shipName As String = Nothing
                Dim shipAddress As String = Nothing
                Dim shipCity As String = Nothing
                Dim shipRegion As String = Nothing
                Dim shipPostalCode As String = Nothing
                Dim shipCountry As String = Nothing

                orderID = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("OrderID").Value.ToString())

                If DgvGrid.Rows(e.RowIndex).Cells("CustomerID").Value IsNot Nothing Then
                    customerID = DgvGrid.Rows(e.RowIndex).Cells("CustomerID").Value.ToString()
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("EmployeeID").Value IsNot Nothing Then
                    employeeID = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("EmployeeID").Value.ToString())
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("OrderDate").Value IsNot Nothing Then
                    orderDate = Convert.ToDateTime(DgvGrid.Rows(e.RowIndex).Cells("OrderDate").Value.ToString())
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("RequiredDate").Value IsNot Nothing Then
                    requiredDate = Convert.ToDateTime(DgvGrid.Rows(e.RowIndex).Cells("RequiredDate").Value.ToString())
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("ShippedDate").Value IsNot Nothing Then
                    shippedDate = Convert.ToDateTime(DgvGrid.Rows(e.RowIndex).Cells("ShippedDate").Value.ToString())
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("ShipVia").Value IsNot Nothing Then
                    shipVia = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("ShipVia").Value.ToString())
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("Freight").Value IsNot Nothing Then
                    freight = Convert.ToDecimal(DgvGrid.Rows(e.RowIndex).Cells("Freight").Value.ToString())
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("ShipName").Value IsNot Nothing Then
                    shipName = DgvGrid.Rows(e.RowIndex).Cells("ShipName").Value.ToString()
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("ShipAddress").Value IsNot Nothing Then
                    shipAddress = DgvGrid.Rows(e.RowIndex).Cells("ShipAddress").Value.ToString()
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("ShipCity").Value IsNot Nothing Then
                    shipCity = DgvGrid.Rows(e.RowIndex).Cells("ShipCity").Value.ToString()
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("ShipRegion").Value IsNot Nothing Then
                    shipRegion = DgvGrid.Rows(e.RowIndex).Cells("ShipRegion").Value.ToString()
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("ShipPostalCode").Value IsNot Nothing Then
                    shipPostalCode = DgvGrid.Rows(e.RowIndex).Cells("ShipPostalCode").Value.ToString()
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("ShipCountry").Value IsNot Nothing Then
                    shipCountry = DgvGrid.Rows(e.RowIndex).Cells("ShipCountry").Value.ToString()
                End If

                Dim objOrders As Orders = Orders.SelectByPrimaryKey(orderID)
                objOrders.OrderID = orderID
                objOrders.CustomerID = customerID
                objOrders.EmployeeID = employeeID.Value
                objOrders.OrderDate = orderDate.Value
                objOrders.RequiredDate = requiredDate.Value
                objOrders.ShippedDate = shippedDate.Value
                objOrders.ShipVia = shipVia.Value
                objOrders.Freight = freight.Value
                objOrders.ShipName = shipName
                objOrders.ShipAddress = shipAddress
                objOrders.ShipCity = shipCity
                objOrders.ShipRegion = shipRegion
                objOrders.ShipPostalCode = shipPostalCode
                objOrders.ShipCountry = shipCountry

                objOrders.Update()
            End If
        End Sub

        #End Region

        #Region "toolstrip menu item events"

        Private Sub TscbPage_SelectedIndexChanged(sender As Object, e As EventArgs) Handles TscbPage.SelectedIndexChanged
            ' make sure this event is not called 
            ' when adding items to the tool strip combo box
            If Not _isFireToolStripComboBoxSelectedIndexChanged Then
                Return
            End If

            ' set current page number
            Dim lastVisitedPageNumber As Integer = _currentPageNumber
            _currentPageNumber = Cint(TscbPage.ComboBox.SelectedItem)

            If _currentPageNumber = 1 Then
                _currentRecordNumber = 1
            Else
                _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))
            End If

            ' go to the selected page
            If _currentPageNumber = 1 Then
                MoveToFirstPage()
            Else If _currentPageNumber = _totalNumberOfPages Then
                MoveToLastPage()
            Else If _currentPageNumber < lastVisitedPageNumber Then
                MoveToPrevious()
            Else
                MoveToNext()
            End If
        End Sub

        #End Region

        #Region "private methods"

        Private Sub FillGridComboBoxColumns()
            ' fill CustomerIDGridColumn datagridview combo box column
            Dim objCustomersCol1 As New CustomersCollection()
            objCustomersCol1.Add(New Customers With { .CustomerID = "", .CompanyName = "" })
            objCustomersCol1.AddRange(Customers.SelectCustomersDropDownListData())

            CustomerIDGridColumn.DataSource = objCustomersCol1
            CustomerIDGridColumn.ValueMember = "CustomerID"
            CustomerIDGridColumn.DisplayMember = "CompanyName"

            ' fill EmployeeIDGridColumn datagridview combo box column
            Dim objEmployeesCol3 As New EmployeesCollection()
            objEmployeesCol3.Add(New Employees With { .EmployeeID = -1, .LastName = "" })
            objEmployeesCol3.AddRange(Employees.SelectEmployeesDropDownListData())

            EmployeeIDGridColumn.DataSource = objEmployeesCol3
            EmployeeIDGridColumn.ValueMember = "EmployeeID"
            EmployeeIDGridColumn.DisplayMember = "LastName"

            ' fill ShipViaGridColumn datagridview combo box column
            Dim objShippersCol5 As New ShippersCollection()
            objShippersCol5.Add(New Shippers With { .ShipperID = -1, .CompanyName = "" })
            objShippersCol5.AddRange(Shippers.SelectShippersDropDownListData())

            ShipViaGridColumn.DataSource = objShippersCol5
            ShipViaGridColumn.ValueMember = "ShipperID"
            ShipViaGridColumn.DisplayMember = "CompanyName"

        End Sub

        Private Sub SetGridProperties()
            DgvGrid.AutoGenerateColumns = False
            DgvGrid.AllowUserToResizeRows = False
            DgvGrid.ColumnCount = 14
            DgvGrid.ScrollBars = ScrollBars.Horizontal
            DgvGrid.AllowUserToOrderColumns = False
        End Sub

        Private Sub SetGridColumnProperties()
            DgvGrid.Columns("OrderID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("CustomerID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("EmployeeID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("OrderDate").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("RequiredDate").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("ShippedDate").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("ShipVia").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("Freight").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("Freight").DefaultCellStyle.Format = "0.00##"
            DgvGrid.Columns("ShipName").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("ShipAddress").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("ShipCity").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("ShipRegion").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("ShipPostalCode").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("ShipCountry").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
        End Sub

        Private Sub MoveToFirstPage()
            SetDataSource(False)
            ToolStripHelper.MoveToFirst(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub MoveToPrevious()
            SetDataSource(False)
            ToolStripHelper.MoveToPrevious(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentRecordNumber)
        End Sub

        Private Sub MoveToNext()
            SetDataSource(False)
            ToolStripHelper.MoveToNext(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentPageNumber, _totalNumberOfPages)
        End Sub

        Private Sub MoveToLastPage()
            SetDataSource(False)
            ToolStripHelper.MoveToLast(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub SetDataSource(isSort As Boolean)
            If isSort Then
                _sortByExpression = GetSortByExpression()
            End If

            ' fill the datagridview
            DgvGrid.DataSource = Orders.SelectSkipAndTake(ConstantHelper.GridNumberOfRows, _currentRecordNumber, _totalRecords, _sortByExpression)
            DgvGrid.Columns(_sortColumnIndex).HeaderCell.SortGlyphDirection = _sortOrder

            If _totalRecords > 0 Then
                _totalNumberOfPages = CInt(Math.Ceiling(CSng(_totalRecords) / CSng(ConstantHelper.GridNumberOfRows)))
                TscbPage.ComboBox.Items.Clear()

                ' fill page numbers in toolstripcombobox
                For page As Integer = 1 To _totalNumberOfPages
                    _isFireToolStripComboBoxSelectedIndexChanged = False
                    TscbPage.ComboBox.Items.Add(page)
                Next

                TscbPage.ComboBox.SelectedItem = _currentPageNumber
                _isFireToolStripComboBoxSelectedIndexChanged = True
            End If

            ResetNavButtonsToDefault()
        End Sub

        Private Function GetSortByExpression() As String
            _sortOrder = SortOrder.Descending

            Select Case _sortColumnIndex
                Case 0
                    If _sortByExpression = "OrderID" Then
                        Return "OrderID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "OrderID"

                Case 1
                    If _sortByExpression = "CustomerID" Then
                        Return "CustomerID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "CustomerID"

                Case 2
                    If _sortByExpression = "EmployeeID" Then
                        Return "EmployeeID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "EmployeeID"

                Case 3
                    If _sortByExpression = "OrderDate" Then
                        Return "OrderDate desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "OrderDate"

                Case 4
                    If _sortByExpression = "RequiredDate" Then
                        Return "RequiredDate desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "RequiredDate"

                Case 5
                    If _sortByExpression = "ShippedDate" Then
                        Return "ShippedDate desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShippedDate"

                Case 6
                    If _sortByExpression = "ShipVia" Then
                        Return "ShipVia desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipVia"

                Case 7
                    If _sortByExpression = "Freight" Then
                        Return "Freight desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Freight"

                Case 8
                    If _sortByExpression = "ShipName" Then
                        Return "ShipName desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipName"

                Case 9
                    If _sortByExpression = "ShipAddress" Then
                        Return "ShipAddress desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipAddress"

                Case 10
                    If _sortByExpression = "ShipCity" Then
                        Return "ShipCity desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipCity"

                Case 11
                    If _sortByExpression = "ShipRegion" Then
                        Return "ShipRegion desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipRegion"

                Case 12
                    If _sortByExpression = "ShipPostalCode" Then
                        Return "ShipPostalCode desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipPostalCode"

                Case 13
                    If _sortByExpression = "ShipCountry" Then
                        Return "ShipCountry desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipCountry"

                Case Else
                    If _sortByExpression = "OrderID" Then
                        Return "OrderID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "OrderID"
            End Select
        End Function

        Private Sub ResetNavButtonsToDefault()
            BnvNavCountItem.Text = "of " & _totalNumberOfPages & " pages"

            If _totalRecords <= ConstantHelper.GridNumberOfRows Then
                BnvNavMoveNextItem.Enabled = False
                BnvNavMoveLastItem.Enabled = False
            Else
                BnvNavMoveNextItem.Enabled = True
                BnvNavMoveLastItem.Enabled = True
            End If
        End Sub

        Private Sub HightlightRow(rowIndex As Integer)
            ' highlight row
            DgvGrid.ClearSelection()
            DgvGrid.Rows(rowIndex).Selected = True

            ' remember the current row clicked by the user
            _currentRowIndex = rowIndex
        End Sub

        Private Sub DeleteItem()
            Dim orderID1 As Integer = Convert.ToInt32(DgvGrid.Rows(_currentRowIndex).Cells("OrderID").Value.ToString())
            Dim result As DialogResult = MessageBox.Show("Are you sure you want to delete item: " & orderID1 & "?", "Confirm Item Deletion", MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation)

            If result = DialogResult.OK Then
                ' delete item
                Orders.Delete(orderID1)

                SetDataSource(False)
            End If
        End Sub

        #End Region
    End Class
