Imports System
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class ListInlineProducts
        Dim _currentRecordNumber As Integer = 1
        Dim _currentPageNumber As Integer = 1
        Dim _totalRecords As Integer = 0
        Dim _totalNumberOfPages As Integer = 0
        Dim _sortColumnIndex As Integer = 0
        Dim _sortByExpression As String = "ProductID"
        Dim _sortOrder As SortOrder = SortOrder.Ascending
        Dim _currentRowIndex As Integer
        Dim _isFireToolStripComboBoxSelectedIndexChanged As Boolean

        #Region "winform events"

        Private Sub ListInlineProducts_Load(sender As Object, e As EventArgs) Handles MyBase.Load
            SetGridProperties()
            SetGridColumnProperties()
            FillGridComboBoxColumns()
            SetDataSource(False)
            ResetNavButtonsToDefault()

            BnvNavAddNewItem.Enabled = True
        End Sub

        #End Region

        #Region "binding navigator control events"

        Private Sub BnvNavMoveFirstItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveFirstItem.Click
            _currentRecordNumber = 1
            _currentPageNumber = 1
            MoveToFirstPage()
        End Sub

        Private Sub BnvNavMovePreviousItem_Click(sender As Object, e As EventArgs) Handles BnvNavMovePreviousItem.Click
            _currentPageNumber -= 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToPrevious()
        End Sub

        Private Sub BnvNavMoveNextItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveNextItem.Click
            _currentPageNumber += 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToNext()
        End Sub

        Private Sub BnvNavMoveLastItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveLastItem.Click
            _currentPageNumber = _totalNumberOfPages
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToLastPage()
        End Sub

        Private Sub BnvNavAddNewItem_Click(sender As Object, e As EventArgs) Handles BnvNavAddNewItem.Click
            ' add just the required fields
            Dim objProducts As New Products()
            objProducts.ProductName = String.Empty
            objProducts.Discontinued = False

            objProducts.Insert()

            _currentRecordNumber = 1
            _currentPageNumber = 1
            _sortColumnIndex = 0
            _sortByExpression = "ProductID"

            SetDataSource(True)

            ' focus on the first text box
            DgvGrid.Focus()
            DgvGrid.CurrentCell = DgvGrid.Rows(0).Cells("ProductName")
        End Sub

        #End Region

        #Region "data gridview events"

        Private Sub DgvGrid_ColumnHeaderMouseClick(sender As Object, e As DataGridViewCellMouseEventArgs) Handles DgvGrid.ColumnHeaderMouseClick
            _sortColumnIndex = e.ColumnIndex
            SetDataSource(True)
        End Sub

        Private Sub DgvGrid_DataError(sender As Object, e As DataGridViewDataErrorEventArgs) Handles DgvGrid.DataError
            e.ThrowException = False
            e.Cancel = False
        End Sub

        Private Sub DgvGrid_RowLeave(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.RowLeave
            If DgvGrid.IsCurrentRowDirty Then
                If IsChangesValid(e.RowIndex)
                Dim productID As Integer
                Dim productName As String = Nothing
                Dim supplierID As Integer? = Nothing
                Dim categoryID As Integer? = Nothing
                Dim quantityPerUnit As String = Nothing
                Dim unitPrice As Decimal? = Nothing
                Dim unitsInStock As Short? = Nothing
                Dim unitsOnOrder As Short? = Nothing
                Dim reorderLevel As Short? = Nothing
                Dim discontinued As Boolean

                productID = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("ProductID").Value.ToString())

                If DgvGrid.Rows(e.RowIndex).Cells("ProductName").Value IsNot Nothing Then
                    productName = DgvGrid.Rows(e.RowIndex).Cells("ProductName").Value.ToString()
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("SupplierID").Value IsNot Nothing Then
                    supplierID = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("SupplierID").Value.ToString())
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("CategoryID").Value IsNot Nothing Then
                    categoryID = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("CategoryID").Value.ToString())
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("QuantityPerUnit").Value IsNot Nothing Then
                    quantityPerUnit = DgvGrid.Rows(e.RowIndex).Cells("QuantityPerUnit").Value.ToString()
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("UnitPrice").Value IsNot Nothing Then
                    unitPrice = Convert.ToDecimal(DgvGrid.Rows(e.RowIndex).Cells("UnitPrice").Value.ToString())
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("UnitsInStock").Value IsNot Nothing Then
                    unitsInStock = Convert.ToInt16(DgvGrid.Rows(e.RowIndex).Cells("UnitsInStock").Value.ToString())
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("UnitsOnOrder").Value IsNot Nothing Then
                    unitsOnOrder = Convert.ToInt16(DgvGrid.Rows(e.RowIndex).Cells("UnitsOnOrder").Value.ToString())
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("ReorderLevel").Value IsNot Nothing Then
                    reorderLevel = Convert.ToInt16(DgvGrid.Rows(e.RowIndex).Cells("ReorderLevel").Value.ToString())
                End If

                discontinued = Convert.ToBoolean(DgvGrid.Rows(e.RowIndex).Cells("Discontinued").Value.ToString())

                    Dim objProducts As Products = Products.SelectByPrimaryKey(productID)
                objProducts.ProductID = productID
                objProducts.ProductName = productName
                objProducts.SupplierID = supplierID.Value
                objProducts.CategoryID = categoryID.Value
                objProducts.QuantityPerUnit = quantityPerUnit
                objProducts.UnitPrice = unitPrice.Value
                objProducts.UnitsInStock = unitsInStock.Value
                objProducts.UnitsOnOrder = unitsOnOrder.Value
                objProducts.ReorderLevel = reorderLevel.Value
                objProducts.Discontinued = discontinued

                    objProducts.Update()
                End If
            End If
        End Sub

        #End Region

        #Region "toolstrip menu item events"

        Private Sub TscbPage_SelectedIndexChanged(sender As Object, e As EventArgs) Handles TscbPage.SelectedIndexChanged
            ' make sure this event is not called 
            ' when adding items to the tool strip combo box
            If Not _isFireToolStripComboBoxSelectedIndexChanged Then
                Return
            End If

            ' set current page number
            Dim lastVisitedPageNumber As Integer = _currentPageNumber
            _currentPageNumber = Cint(TscbPage.ComboBox.SelectedItem)

            If _currentPageNumber = 1 Then
                _currentRecordNumber = 1
            Else
                _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))
            End If

            ' go to the selected page
            If _currentPageNumber = 1 Then
                MoveToFirstPage()
            Else If _currentPageNumber = _totalNumberOfPages Then
                MoveToLastPage()
            Else If _currentPageNumber < lastVisitedPageNumber Then
                MoveToPrevious()
            Else
                MoveToNext()
            End If
        End Sub

        #End Region

        #Region "private methods"

        Private Sub FillGridComboBoxColumns()
            ' fill SupplierIDGridColumn datagridview combo box column
            Dim objSuppliersCol1 As New SuppliersCollection()
            objSuppliersCol1.Add(New Suppliers With { .SupplierID = -1, .CompanyName = "" })
            objSuppliersCol1.AddRange(Suppliers.SelectSuppliersDropDownListData())

            SupplierIDGridColumn.DataSource = objSuppliersCol1
            SupplierIDGridColumn.ValueMember = "SupplierID"
            SupplierIDGridColumn.DisplayMember = "CompanyName"

            ' fill CategoryIDGridColumn datagridview combo box column
            Dim objCategoriesCol3 As New CategoriesCollection()
            objCategoriesCol3.Add(New Categories With { .CategoryID = -1, .CategoryName = "" })
            objCategoriesCol3.AddRange(Categories.SelectCategoriesDropDownListData())

            CategoryIDGridColumn.DataSource = objCategoriesCol3
            CategoryIDGridColumn.ValueMember = "CategoryID"
            CategoryIDGridColumn.DisplayMember = "CategoryName"

        End Sub

        Private Function IsChangesValid(rowIndex As Integer) As Boolean
            Dim errors As String = String.Empty
            DgvGrid.CommitEdit(DataGridViewDataErrorContexts.Commit)

            If DgvGrid.Rows(rowIndex).Cells("ProductName").Value Is Nothing Then
                errors += "Product Name is required!\n\r"
            End If

            If Not String.IsNullOrEmpty(errors) Then
                MessageBox.Show(errors, "An Error Occured")
                Return False
            End If

            Return True
        End Function

        Private Sub SetGridProperties()
            DgvGrid.AutoGenerateColumns = False
            DgvGrid.AllowUserToResizeRows = False
            DgvGrid.ColumnCount = 10
            DgvGrid.ScrollBars = ScrollBars.None
            DgvGrid.AllowUserToOrderColumns = False
        End Sub

        Private Sub SetGridColumnProperties()
            DgvGrid.Columns("ProductID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("ProductName").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("SupplierID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("CategoryID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("QuantityPerUnit").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("UnitPrice").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("UnitPrice").DefaultCellStyle.Format = "0.00##"
            DgvGrid.Columns("UnitsInStock").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("UnitsOnOrder").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("ReorderLevel").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("Discontinued").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter
        End Sub

        Private Sub MoveToFirstPage()
            SetDataSource(False)
            ToolStripHelper.MoveToFirst(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub MoveToPrevious()
            SetDataSource(False)
            ToolStripHelper.MoveToPrevious(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentRecordNumber)
        End Sub

        Private Sub MoveToNext()
            SetDataSource(False)
            ToolStripHelper.MoveToNext(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentPageNumber, _totalNumberOfPages)
        End Sub

        Private Sub MoveToLastPage()
            SetDataSource(False)
            ToolStripHelper.MoveToLast(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub SetDataSource(isSort As Boolean)
            If isSort Then
                _sortByExpression = GetSortByExpression()
            End If

            ' fill the datagridview
            DgvGrid.DataSource = Products.SelectSkipAndTake(ConstantHelper.GridNumberOfRows, _currentRecordNumber, _totalRecords, _sortByExpression)
            DgvGrid.Columns(_sortColumnIndex).HeaderCell.SortGlyphDirection = _sortOrder

            If _totalRecords > 0 Then
                _totalNumberOfPages = CInt(Math.Ceiling(CSng(_totalRecords) / CSng(ConstantHelper.GridNumberOfRows)))
                TscbPage.ComboBox.Items.Clear()

                ' fill page numbers in toolstripcombobox
                For page As Integer = 1 To _totalNumberOfPages
                    _isFireToolStripComboBoxSelectedIndexChanged = False
                    TscbPage.ComboBox.Items.Add(page)
                Next

                TscbPage.ComboBox.SelectedItem = _currentPageNumber
                _isFireToolStripComboBoxSelectedIndexChanged = True
            End If

            ResetNavButtonsToDefault()
        End Sub

        Private Function GetSortByExpression() As String
            _sortOrder = SortOrder.Descending

            Select Case _sortColumnIndex
                Case 0
                    If _sortByExpression = "ProductID" Then
                        Return "ProductID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ProductID"

                Case 1
                    If _sortByExpression = "ProductName" Then
                        Return "ProductName desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ProductName"

                Case 2
                    If _sortByExpression = "SupplierID" Then
                        Return "SupplierID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "SupplierID"

                Case 3
                    If _sortByExpression = "CategoryID" Then
                        Return "CategoryID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "CategoryID"

                Case 4
                    If _sortByExpression = "QuantityPerUnit" Then
                        Return "QuantityPerUnit desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "QuantityPerUnit"

                Case 5
                    If _sortByExpression = "UnitPrice" Then
                        Return "UnitPrice desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "UnitPrice"

                Case 6
                    If _sortByExpression = "UnitsInStock" Then
                        Return "UnitsInStock desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "UnitsInStock"

                Case 7
                    If _sortByExpression = "UnitsOnOrder" Then
                        Return "UnitsOnOrder desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "UnitsOnOrder"

                Case 8
                    If _sortByExpression = "ReorderLevel" Then
                        Return "ReorderLevel desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ReorderLevel"

                Case 9
                    If _sortByExpression = "Discontinued" Then
                        Return "Discontinued desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Discontinued"

                Case Else
                    If _sortByExpression = "ProductID" Then
                        Return "ProductID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ProductID"
            End Select
        End Function

        Private Sub ResetNavButtonsToDefault()
            BnvNavCountItem.Text = "of " & _totalNumberOfPages & " pages"

            If _totalRecords <= ConstantHelper.GridNumberOfRows Then
                BnvNavMoveNextItem.Enabled = False
                BnvNavMoveLastItem.Enabled = False
            Else
                BnvNavMoveNextItem.Enabled = True
                BnvNavMoveLastItem.Enabled = True
            End If
        End Sub

        Private Sub HightlightRow(rowIndex As Integer)
            ' highlight row
            DgvGrid.ClearSelection()
            DgvGrid.Rows(rowIndex).Selected = True

            ' remember the current row clicked by the user
            _currentRowIndex = rowIndex
        End Sub

        Private Sub DeleteItem()
            Dim productID1 As Integer = Convert.ToInt32(DgvGrid.Rows(_currentRowIndex).Cells("ProductID").Value.ToString())
            Dim result As DialogResult = MessageBox.Show("Are you sure you want to delete item: " & productID1 & "?", "Confirm Item Deletion", MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation)

            If result = DialogResult.OK Then
                ' delete item
                Products.Delete(productID1)

                SetDataSource(False)
            End If
        End Sub

        #End Region
    End Class
