Imports System
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class ListInlineShippers
        Dim _currentRecordNumber As Integer = 1
        Dim _currentPageNumber As Integer = 1
        Dim _totalRecords As Integer = 0
        Dim _totalNumberOfPages As Integer = 0
        Dim _sortColumnIndex As Integer = 0
        Dim _sortByExpression As String = "ShipperID"
        Dim _sortOrder As SortOrder = SortOrder.Ascending
        Dim _currentRowIndex As Integer
        Dim _isFireToolStripComboBoxSelectedIndexChanged As Boolean

        #Region "winform events"

        Private Sub ListInlineShippers_Load(sender As Object, e As EventArgs) Handles MyBase.Load
            SetGridProperties()
            SetGridColumnProperties()
            SetDataSource(False)
            ResetNavButtonsToDefault()

            BnvNavAddNewItem.Enabled = True
        End Sub

        #End Region

        #Region "binding navigator control events"

        Private Sub BnvNavMoveFirstItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveFirstItem.Click
            _currentRecordNumber = 1
            _currentPageNumber = 1
            MoveToFirstPage()
        End Sub

        Private Sub BnvNavMovePreviousItem_Click(sender As Object, e As EventArgs) Handles BnvNavMovePreviousItem.Click
            _currentPageNumber -= 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToPrevious()
        End Sub

        Private Sub BnvNavMoveNextItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveNextItem.Click
            _currentPageNumber += 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToNext()
        End Sub

        Private Sub BnvNavMoveLastItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveLastItem.Click
            _currentPageNumber = _totalNumberOfPages
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToLastPage()
        End Sub

        Private Sub BnvNavAddNewItem_Click(sender As Object, e As EventArgs) Handles BnvNavAddNewItem.Click
            ' add just the required fields
            Dim objShippers As New Shippers()
            objShippers.CompanyName = String.Empty

            objShippers.Insert()

            _currentRecordNumber = 1
            _currentPageNumber = 1
            _sortColumnIndex = 0
            _sortByExpression = "ShipperID"

            SetDataSource(True)

            ' focus on the first text box
            DgvGrid.Focus()
            DgvGrid.CurrentCell = DgvGrid.Rows(0).Cells("CompanyName")
        End Sub

        #End Region

        #Region "data gridview events"

        Private Sub DgvGrid_ColumnHeaderMouseClick(sender As Object, e As DataGridViewCellMouseEventArgs) Handles DgvGrid.ColumnHeaderMouseClick
            _sortColumnIndex = e.ColumnIndex
            SetDataSource(True)
        End Sub

        Private Sub DgvGrid_DataError(sender As Object, e As DataGridViewDataErrorEventArgs) Handles DgvGrid.DataError
            e.ThrowException = False
            e.Cancel = False
        End Sub

        Private Sub DgvGrid_RowLeave(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.RowLeave
            If DgvGrid.IsCurrentRowDirty Then
                If IsChangesValid(e.RowIndex)
                Dim shipperID As Integer
                Dim companyName As String = Nothing
                Dim phone As String = Nothing

                shipperID = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("ShipperID").Value.ToString())

                If DgvGrid.Rows(e.RowIndex).Cells("CompanyName").Value IsNot Nothing Then
                    companyName = DgvGrid.Rows(e.RowIndex).Cells("CompanyName").Value.ToString()
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("Phone").Value IsNot Nothing Then
                    phone = DgvGrid.Rows(e.RowIndex).Cells("Phone").Value.ToString()
                End If

                    Dim objShippers As Shippers = Shippers.SelectByPrimaryKey(shipperID)
                objShippers.ShipperID = shipperID
                objShippers.CompanyName = companyName
                objShippers.Phone = phone

                    objShippers.Update()
                End If
            End If
        End Sub

        #End Region

        #Region "toolstrip menu item events"

        Private Sub TscbPage_SelectedIndexChanged(sender As Object, e As EventArgs) Handles TscbPage.SelectedIndexChanged
            ' make sure this event is not called 
            ' when adding items to the tool strip combo box
            If Not _isFireToolStripComboBoxSelectedIndexChanged Then
                Return
            End If

            ' set current page number
            Dim lastVisitedPageNumber As Integer = _currentPageNumber
            _currentPageNumber = Cint(TscbPage.ComboBox.SelectedItem)

            If _currentPageNumber = 1 Then
                _currentRecordNumber = 1
            Else
                _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))
            End If

            ' go to the selected page
            If _currentPageNumber = 1 Then
                MoveToFirstPage()
            Else If _currentPageNumber = _totalNumberOfPages Then
                MoveToLastPage()
            Else If _currentPageNumber < lastVisitedPageNumber Then
                MoveToPrevious()
            Else
                MoveToNext()
            End If
        End Sub

        #End Region

        #Region "private methods"

        Private Function IsChangesValid(rowIndex As Integer) As Boolean
            Dim errors As String = String.Empty
            DgvGrid.CommitEdit(DataGridViewDataErrorContexts.Commit)

            If DgvGrid.Rows(rowIndex).Cells("CompanyName").Value Is Nothing Then
                errors += "Company Name is required!\n\r"
            End If

            If Not String.IsNullOrEmpty(errors) Then
                MessageBox.Show(errors, "An Error Occured")
                Return False
            End If

            Return True
        End Function

        Private Sub SetGridProperties()
            DgvGrid.AutoGenerateColumns = False
            DgvGrid.AllowUserToResizeRows = False
            DgvGrid.ColumnCount = 3
            DgvGrid.ScrollBars = ScrollBars.None
            DgvGrid.AllowUserToOrderColumns = False
        End Sub

        Private Sub SetGridColumnProperties()
            DgvGrid.Columns("ShipperID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("CompanyName").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("Phone").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
        End Sub

        Private Sub MoveToFirstPage()
            SetDataSource(False)
            ToolStripHelper.MoveToFirst(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub MoveToPrevious()
            SetDataSource(False)
            ToolStripHelper.MoveToPrevious(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentRecordNumber)
        End Sub

        Private Sub MoveToNext()
            SetDataSource(False)
            ToolStripHelper.MoveToNext(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentPageNumber, _totalNumberOfPages)
        End Sub

        Private Sub MoveToLastPage()
            SetDataSource(False)
            ToolStripHelper.MoveToLast(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub SetDataSource(isSort As Boolean)
            If isSort Then
                _sortByExpression = GetSortByExpression()
            End If

            ' fill the datagridview
            DgvGrid.DataSource = Shippers.SelectSkipAndTake(ConstantHelper.GridNumberOfRows, _currentRecordNumber, _totalRecords, _sortByExpression)
            DgvGrid.Columns(_sortColumnIndex).HeaderCell.SortGlyphDirection = _sortOrder

            If _totalRecords > 0 Then
                _totalNumberOfPages = CInt(Math.Ceiling(CSng(_totalRecords) / CSng(ConstantHelper.GridNumberOfRows)))
                TscbPage.ComboBox.Items.Clear()

                ' fill page numbers in toolstripcombobox
                For page As Integer = 1 To _totalNumberOfPages
                    _isFireToolStripComboBoxSelectedIndexChanged = False
                    TscbPage.ComboBox.Items.Add(page)
                Next

                TscbPage.ComboBox.SelectedItem = _currentPageNumber
                _isFireToolStripComboBoxSelectedIndexChanged = True
            End If

            ResetNavButtonsToDefault()
        End Sub

        Private Function GetSortByExpression() As String
            _sortOrder = SortOrder.Descending

            Select Case _sortColumnIndex
                Case 0
                    If _sortByExpression = "ShipperID" Then
                        Return "ShipperID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipperID"

                Case 1
                    If _sortByExpression = "CompanyName" Then
                        Return "CompanyName desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "CompanyName"

                Case 2
                    If _sortByExpression = "Phone" Then
                        Return "Phone desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Phone"

                Case Else
                    If _sortByExpression = "ShipperID" Then
                        Return "ShipperID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipperID"
            End Select
        End Function

        Private Sub ResetNavButtonsToDefault()
            BnvNavCountItem.Text = "of " & _totalNumberOfPages & " pages"

            If _totalRecords <= ConstantHelper.GridNumberOfRows Then
                BnvNavMoveNextItem.Enabled = False
                BnvNavMoveLastItem.Enabled = False
            Else
                BnvNavMoveNextItem.Enabled = True
                BnvNavMoveLastItem.Enabled = True
            End If
        End Sub

        Private Sub HightlightRow(rowIndex As Integer)
            ' highlight row
            DgvGrid.ClearSelection()
            DgvGrid.Rows(rowIndex).Selected = True

            ' remember the current row clicked by the user
            _currentRowIndex = rowIndex
        End Sub

        Private Sub DeleteItem()
            Dim shipperID1 As Integer = Convert.ToInt32(DgvGrid.Rows(_currentRowIndex).Cells("ShipperID").Value.ToString())
            Dim result As DialogResult = MessageBox.Show("Are you sure you want to delete item: " & shipperID1 & "?", "Confirm Item Deletion", MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation)

            If result = DialogResult.OK Then
                ' delete item
                Shippers.Delete(shipperID1)

                SetDataSource(False)
            End If
        End Sub

        #End Region
    End Class
