Imports System
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class ListInlineTerritories
        Dim _currentRecordNumber As Integer = 1
        Dim _currentPageNumber As Integer = 1
        Dim _totalRecords As Integer = 0
        Dim _totalNumberOfPages As Integer = 0
        Dim _sortColumnIndex As Integer = 0
        Dim _sortByExpression As String = "TerritoryID"
        Dim _sortOrder As SortOrder = SortOrder.Ascending
        Dim _currentRowIndex As Integer
        Dim _isFireToolStripComboBoxSelectedIndexChanged As Boolean

        #Region "winform events"

        Private Sub ListInlineTerritories_Load(sender As Object, e As EventArgs) Handles MyBase.Load
            SetGridProperties()
            SetGridColumnProperties()
            FillGridComboBoxColumns()
            SetDataSource(False)
            ResetNavButtonsToDefault()

            BnvNavAddNewItem.Enabled = True
        End Sub

        #End Region

        #Region "binding navigator control events"

        Private Sub BnvNavMoveFirstItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveFirstItem.Click
            _currentRecordNumber = 1
            _currentPageNumber = 1
            MoveToFirstPage()
        End Sub

        Private Sub BnvNavMovePreviousItem_Click(sender As Object, e As EventArgs) Handles BnvNavMovePreviousItem.Click
            _currentPageNumber -= 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToPrevious()
        End Sub

        Private Sub BnvNavMoveNextItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveNextItem.Click
            _currentPageNumber += 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToNext()
        End Sub

        Private Sub BnvNavMoveLastItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveLastItem.Click
            _currentPageNumber = _totalNumberOfPages
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToLastPage()
        End Sub

        Private Sub BnvNavAddNewItem_Click(sender As Object, e As EventArgs) Handles BnvNavAddNewItem.Click
            ' add just the required fields
            Dim objTerritories As New Territories()
            objTerritories.TerritoryDescription = String.Empty
            objTerritories.RegionID = 0

            objTerritories.Insert()

            _currentRecordNumber = 1
            _currentPageNumber = 1
            _sortColumnIndex = 0
            _sortByExpression = "TerritoryID"

            SetDataSource(True)

            ' focus on the first text box
            DgvGrid.Focus()
            DgvGrid.CurrentCell = DgvGrid.Rows(0).Cells("TerritoryDescription")
        End Sub

        #End Region

        #Region "data gridview events"

        Private Sub DgvGrid_ColumnHeaderMouseClick(sender As Object, e As DataGridViewCellMouseEventArgs) Handles DgvGrid.ColumnHeaderMouseClick
            _sortColumnIndex = e.ColumnIndex
            SetDataSource(True)
        End Sub

        Private Sub DgvGrid_DataError(sender As Object, e As DataGridViewDataErrorEventArgs) Handles DgvGrid.DataError
            e.ThrowException = False
            e.Cancel = False
        End Sub

        Private Sub DgvGrid_RowLeave(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.RowLeave
            If DgvGrid.IsCurrentRowDirty Then
                If IsChangesValid(e.RowIndex)
                Dim territoryID As String = Nothing
                Dim territoryDescription As String = Nothing
                Dim regionID As Integer = 0

                territoryID = DgvGrid.Rows(e.RowIndex).Cells("TerritoryID").Value.ToString()

                If DgvGrid.Rows(e.RowIndex).Cells("TerritoryDescription").Value IsNot Nothing Then
                    territoryDescription = DgvGrid.Rows(e.RowIndex).Cells("TerritoryDescription").Value.ToString()
                End If

                If DgvGrid.Rows(e.RowIndex).Cells("RegionID").Value IsNot Nothing Then
                    regionID = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("RegionID").Value.ToString())
                End If

                    Dim objTerritories As Territories = Territories.SelectByPrimaryKey(territoryID)
                objTerritories.TerritoryID = territoryID
                objTerritories.TerritoryDescription = territoryDescription
                objTerritories.RegionID = regionID

                    objTerritories.Update()
                End If
            End If
        End Sub

        #End Region

        #Region "toolstrip menu item events"

        Private Sub TscbPage_SelectedIndexChanged(sender As Object, e As EventArgs) Handles TscbPage.SelectedIndexChanged
            ' make sure this event is not called 
            ' when adding items to the tool strip combo box
            If Not _isFireToolStripComboBoxSelectedIndexChanged Then
                Return
            End If

            ' set current page number
            Dim lastVisitedPageNumber As Integer = _currentPageNumber
            _currentPageNumber = Cint(TscbPage.ComboBox.SelectedItem)

            If _currentPageNumber = 1 Then
                _currentRecordNumber = 1
            Else
                _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))
            End If

            ' go to the selected page
            If _currentPageNumber = 1 Then
                MoveToFirstPage()
            Else If _currentPageNumber = _totalNumberOfPages Then
                MoveToLastPage()
            Else If _currentPageNumber < lastVisitedPageNumber Then
                MoveToPrevious()
            Else
                MoveToNext()
            End If
        End Sub

        #End Region

        #Region "private methods"

        Private Sub FillGridComboBoxColumns()
            ' fill RegionIDGridColumn datagridview combo box column
            Dim objRegionCol1 As New RegionCollection()
            objRegionCol1.Add(New BusinessObject.Region With { .RegionID = -1, .RegionDescription = "" })
            objRegionCol1.AddRange(BusinessObject.Region.SelectRegionDropDownListData())

            RegionIDGridColumn.DataSource = objRegionCol1
            RegionIDGridColumn.ValueMember = "RegionID"
            RegionIDGridColumn.DisplayMember = "RegionDescription"

        End Sub

        Private Function IsChangesValid(rowIndex As Integer) As Boolean
            Dim errors As String = String.Empty
            DgvGrid.CommitEdit(DataGridViewDataErrorContexts.Commit)

            If DgvGrid.Rows(rowIndex).Cells("TerritoryID").Value Is Nothing Then
                errors += "Territory ID is required!\n\r"
            End If

            If DgvGrid.Rows(rowIndex).Cells("TerritoryDescription").Value Is Nothing Then
                errors += "Territory Description is required!\n\r"
            End If

            If DgvGrid.Rows(rowIndex).Cells("RegionID").Value Is Nothing Then
                errors += "Region ID is required!\n\r"
            End If

            If Not String.IsNullOrEmpty(errors) Then
                MessageBox.Show(errors, "An Error Occured")
                Return False
            End If

            Return True
        End Function

        Private Sub SetGridProperties()
            DgvGrid.AutoGenerateColumns = False
            DgvGrid.AllowUserToResizeRows = False
            DgvGrid.ColumnCount = 3
            DgvGrid.ScrollBars = ScrollBars.None
            DgvGrid.AllowUserToOrderColumns = False
        End Sub

        Private Sub SetGridColumnProperties()
            DgvGrid.Columns("TerritoryID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("TerritoryDescription").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("RegionID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
        End Sub

        Private Sub MoveToFirstPage()
            SetDataSource(False)
            ToolStripHelper.MoveToFirst(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub MoveToPrevious()
            SetDataSource(False)
            ToolStripHelper.MoveToPrevious(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentRecordNumber)
        End Sub

        Private Sub MoveToNext()
            SetDataSource(False)
            ToolStripHelper.MoveToNext(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentPageNumber, _totalNumberOfPages)
        End Sub

        Private Sub MoveToLastPage()
            SetDataSource(False)
            ToolStripHelper.MoveToLast(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub SetDataSource(isSort As Boolean)
            If isSort Then
                _sortByExpression = GetSortByExpression()
            End If

            ' fill the datagridview
            DgvGrid.DataSource = Territories.SelectSkipAndTake(ConstantHelper.GridNumberOfRows, _currentRecordNumber, _totalRecords, _sortByExpression)
            DgvGrid.Columns(_sortColumnIndex).HeaderCell.SortGlyphDirection = _sortOrder

            If _totalRecords > 0 Then
                _totalNumberOfPages = CInt(Math.Ceiling(CSng(_totalRecords) / CSng(ConstantHelper.GridNumberOfRows)))
                TscbPage.ComboBox.Items.Clear()

                ' fill page numbers in toolstripcombobox
                For page As Integer = 1 To _totalNumberOfPages
                    _isFireToolStripComboBoxSelectedIndexChanged = False
                    TscbPage.ComboBox.Items.Add(page)
                Next

                TscbPage.ComboBox.SelectedItem = _currentPageNumber
                _isFireToolStripComboBoxSelectedIndexChanged = True
            End If

            ResetNavButtonsToDefault()
        End Sub

        Private Function GetSortByExpression() As String
            _sortOrder = SortOrder.Descending

            Select Case _sortColumnIndex
                Case 0
                    If _sortByExpression = "TerritoryID" Then
                        Return "TerritoryID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "TerritoryID"

                Case 1
                    If _sortByExpression = "TerritoryDescription" Then
                        Return "TerritoryDescription desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "TerritoryDescription"

                Case 2
                    If _sortByExpression = "RegionID" Then
                        Return "RegionID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "RegionID"

                Case Else
                    If _sortByExpression = "TerritoryID" Then
                        Return "TerritoryID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "TerritoryID"
            End Select
        End Function

        Private Sub ResetNavButtonsToDefault()
            BnvNavCountItem.Text = "of " & _totalNumberOfPages & " pages"

            If _totalRecords <= ConstantHelper.GridNumberOfRows Then
                BnvNavMoveNextItem.Enabled = False
                BnvNavMoveLastItem.Enabled = False
            Else
                BnvNavMoveNextItem.Enabled = True
                BnvNavMoveLastItem.Enabled = True
            End If
        End Sub

        Private Sub HightlightRow(rowIndex As Integer)
            ' highlight row
            DgvGrid.ClearSelection()
            DgvGrid.Rows(rowIndex).Selected = True

            ' remember the current row clicked by the user
            _currentRowIndex = rowIndex
        End Sub

        Private Sub DeleteItem()
            Dim territoryID1 As String = DgvGrid.Rows(_currentRowIndex).Cells("TerritoryID").Value.ToString()
            Dim result As DialogResult = MessageBox.Show("Are you sure you want to delete item: " & territoryID1 & "?", "Confirm Item Deletion", MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation)

            If result = DialogResult.OK Then
                ' delete item
                Territories.Delete(territoryID1)

                SetDataSource(False)
            End If
        End Sub

        #End Region
    End Class
