Imports System
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class ListReadOnlyOrderDetails
        Dim _currentRecordNumber As Integer = 1
        Dim _currentPageNumber As Integer = 1
        Dim _totalRecords As Integer = 0
        Dim _totalNumberOfPages As Integer = 0
        Dim _sortColumnIndex As Integer = 0
        Dim _sortByExpression As String = "OrderID"
        Dim _sortOrder As SortOrder = SortOrder.Ascending
        Dim _currentRowIndex As Integer
        Dim _isFireToolStripComboBoxSelectedIndexChanged As Boolean

        #Region "winform events"

        Private Sub ListReadOnlyOrderDetails_Load(sender As Object, e As EventArgs) Handles MyBase.Load
            SetGridProperties()
            SetGridColumnProperties()
            SetDataSource(False)
            ResetNavButtonsToDefault()

        End Sub

        #End Region

        #Region "binding navigator control events"

        Private Sub BnvNavMoveFirstItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveFirstItem.Click
            _currentRecordNumber = 1
            _currentPageNumber = 1
            MoveToFirstPage()
        End Sub

        Private Sub BnvNavMovePreviousItem_Click(sender As Object, e As EventArgs) Handles BnvNavMovePreviousItem.Click
            _currentPageNumber -= 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToPrevious()
        End Sub

        Private Sub BnvNavMoveNextItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveNextItem.Click
            _currentPageNumber += 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToNext()
        End Sub

        Private Sub BnvNavMoveLastItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveLastItem.Click
            _currentPageNumber = _totalNumberOfPages
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToLastPage()
        End Sub


        #End Region

        #Region "data gridview events"

        Private Sub DgvGrid_ColumnHeaderMouseClick(sender As Object, e As DataGridViewCellMouseEventArgs) Handles DgvGrid.ColumnHeaderMouseClick
            _sortColumnIndex = e.ColumnIndex
            SetDataSource(True)
        End Sub

        Private Sub DgvGrid_CellContextMenuStripNeeded(sender As Object, e As DataGridViewCellContextMenuStripNeededEventArgs) Handles DgvGrid.CellContextMenuStripNeeded
            HightlightRow(e.RowIndex)
        End Sub

        Private Sub DgvGrid_CellClick(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.CellClick
            ' make sure the click did not come from the header
            If e.RowIndex <> -1 Then
                HightlightRow(e.RowIndex)
            End If
        End Sub

        Private Sub DgvGrid_CellContentClick(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.CellContentClick
            If e.ColumnIndex = 0  And e.RowIndex <> -1 Then
                Dim orderID1 As Integer = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("OrderID").Value.ToString())

                Dim form As New DetailsOrders(orderID1)
                form.ShowDialog()
            End If
            If e.ColumnIndex = 1  And e.RowIndex <> -1 Then
                Dim productID1 As Integer = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("ProductID").Value.ToString())

                Dim form As New DetailsProducts(productID1)
                form.ShowDialog()
            End If
        End Sub

        #End Region

        #Region "toolstrip menu item events"

        Private Sub TscbPage_SelectedIndexChanged(sender As Object, e As EventArgs) Handles TscbPage.SelectedIndexChanged
            ' make sure this event is not called 
            ' when adding items to the tool strip combo box
            If Not _isFireToolStripComboBoxSelectedIndexChanged Then
                Return
            End If

            ' set current page number
            Dim lastVisitedPageNumber As Integer = _currentPageNumber
            _currentPageNumber = Cint(TscbPage.ComboBox.SelectedItem)

            If _currentPageNumber = 1 Then
                _currentRecordNumber = 1
            Else
                _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))
            End If

            ' go to the selected page
            If _currentPageNumber = 1 Then
                MoveToFirstPage()
            Else If _currentPageNumber = _totalNumberOfPages Then
                MoveToLastPage()
            Else If _currentPageNumber < lastVisitedPageNumber Then
                MoveToPrevious()
            Else
                MoveToNext()
            End If
        End Sub

        #End Region

        #Region "private methods"

        Private Sub SetGridProperties()
            DgvGrid.AutoGenerateColumns = False
            DgvGrid.AllowUserToResizeRows = False
            DgvGrid.ColumnCount = 5
            DgvGrid.ScrollBars = ScrollBars.None
            DgvGrid.AllowUserToOrderColumns = False
            DgvGrid.AllowUserToAddRows = False
            DgvGrid.AllowUserToDeleteRows = False
            DgvGrid.ReadOnly = True
        End Sub

        Private Sub SetGridColumnProperties()
            DgvGrid.Columns("OrderID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("ProductID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("UnitPrice").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("UnitPrice").DefaultCellStyle.Format = "0.00##"
            DgvGrid.Columns("Quantity").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("Discount").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
        End Sub

        Private Sub MoveToFirstPage()
            SetDataSource(False)
            ToolStripHelper.MoveToFirst(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub MoveToPrevious()
            SetDataSource(False)
            ToolStripHelper.MoveToPrevious(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentRecordNumber)
        End Sub

        Private Sub MoveToNext()
            SetDataSource(False)
            ToolStripHelper.MoveToNext(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentPageNumber, _totalNumberOfPages)
        End Sub

        Private Sub MoveToLastPage()
            SetDataSource(False)
            ToolStripHelper.MoveToLast(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub SetDataSource(isSort As Boolean)
            If isSort Then
                _sortByExpression = GetSortByExpression()
            End If

            ' fill the datagridview
            DgvGrid.DataSource = OrderDetails.SelectSkipAndTake(ConstantHelper.GridNumberOfRows, _currentRecordNumber, _totalRecords, _sortByExpression)
            DgvGrid.Columns(_sortColumnIndex).HeaderCell.SortGlyphDirection = _sortOrder

            If _totalRecords > 0 Then
                _totalNumberOfPages = CInt(Math.Ceiling(CSng(_totalRecords) / CSng(ConstantHelper.GridNumberOfRows)))
                TscbPage.ComboBox.Items.Clear()

                ' fill page numbers in toolstripcombobox
                For page As Integer = 1 To _totalNumberOfPages
                    _isFireToolStripComboBoxSelectedIndexChanged = False
                    TscbPage.ComboBox.Items.Add(page)
                Next

                TscbPage.ComboBox.SelectedItem = _currentPageNumber
                _isFireToolStripComboBoxSelectedIndexChanged = True
            End If

            ResetNavButtonsToDefault()
        End Sub

        Private Function GetSortByExpression() As String
            _sortOrder = SortOrder.Descending

            Select Case _sortColumnIndex
                Case 0
                    If _sortByExpression = "OrderID" Then
                        Return "OrderID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "OrderID"

                Case 1
                    If _sortByExpression = "ProductID" Then
                        Return "ProductID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ProductID"

                Case 2
                    If _sortByExpression = "UnitPrice" Then
                        Return "UnitPrice desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "UnitPrice"

                Case 3
                    If _sortByExpression = "Quantity" Then
                        Return "Quantity desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Quantity"

                Case 4
                    If _sortByExpression = "Discount" Then
                        Return "Discount desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Discount"

                Case Else
                    If _sortByExpression = "OrderID" Then
                        Return "OrderID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "OrderID"
            End Select
        End Function

        Private Sub ResetNavButtonsToDefault()
            BnvNavCountItem.Text = "of " & _totalNumberOfPages & " pages"

            If _totalRecords <= ConstantHelper.GridNumberOfRows Then
                BnvNavMoveNextItem.Enabled = False
                BnvNavMoveLastItem.Enabled = False
            Else
                BnvNavMoveNextItem.Enabled = True
                BnvNavMoveLastItem.Enabled = True
            End If
        End Sub

        Private Sub HightlightRow(rowIndex As Integer)
            ' highlight row
            DgvGrid.ClearSelection()
            DgvGrid.Rows(rowIndex).Selected = True

            ' remember the current row clicked by the user
            _currentRowIndex = rowIndex
        End Sub


        #End Region
    End Class
