Imports System
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class ListRelatedFieldOrdersByEmployeeID
        Dim _isOnLoad As Boolean = True
        Dim _employeeID1 As Integer
        Dim _currentRecordNumber As Integer = 1
        Dim _currentPageNumber As Integer = 1
        Dim _totalRecords As Integer = 0
        Dim _totalNumberOfPages As Integer = 0
        Dim _sortColumnIndex As Integer = 0
        Dim _sortByExpression As String = "OrderID"
        Dim _sortOrder As SortOrder = SortOrder.Ascending
        Dim _currentRowIndex As Integer
        Dim _isFireToolStripComboBoxSelectedIndexChanged As Boolean

        #Region "winform events"

        Private Sub ListRelatedFieldOrdersByEmployeeID_Load(sender As Object, e As EventArgs) Handles MyBase.Load
            FillComboBoxWithData()

            If DdlEmployeeID.Items.Count > 0 Then
                DdlEmployeeID.SelectedIndex = 0
            End If

            SetGridProperties()
            SetGridColumnProperties()
            SetDataSource(False)
            ResetNavButtonsToDefault()

            LblLine.Width = Me.Width
        End Sub

        #End Region

        #Region "binding navigator control events"

        Private Sub BnvNavMoveFirstItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveFirstItem.Click
            _currentRecordNumber = 1
            _currentPageNumber = 1
            MoveToFirstPage()
        End Sub

        Private Sub BnvNavMovePreviousItem_Click(sender As Object, e As EventArgs) Handles BnvNavMovePreviousItem.Click
            _currentPageNumber -= 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToPrevious()
        End Sub

        Private Sub BnvNavMoveNextItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveNextItem.Click
            _currentPageNumber += 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToNext()
        End Sub

        Private Sub BnvNavMoveLastItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveLastItem.Click
            _currentPageNumber = _totalNumberOfPages
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToLastPage()
        End Sub


        #End Region

        #Region "data gridview events"

        Private Sub DgvGrid_ColumnHeaderMouseClick(sender As Object, e As DataGridViewCellMouseEventArgs) Handles DgvGrid.ColumnHeaderMouseClick
            _sortColumnIndex = e.ColumnIndex
            SetDataSource(True)
        End Sub

        Private Sub DgvGrid_CellContextMenuStripNeeded(sender As Object, e As DataGridViewCellContextMenuStripNeededEventArgs) Handles DgvGrid.CellContextMenuStripNeeded
            HightlightRow(e.RowIndex)
        End Sub

        Private Sub DgvGrid_CellClick(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.CellClick
            ' make sure the click did not come from the header
            If e.RowIndex <> -1 Then
                HightlightRow(e.RowIndex)
            End If
        End Sub

        Private Sub DgvGrid_CellContentClick(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.CellContentClick
            If e.ColumnIndex = 1  And e.RowIndex <> -1 Then
                Dim customerID1 As String = DgvGrid.Rows(e.RowIndex).Cells("CustomerID").Value.ToString()

                Dim form As New DetailsCustomers(customerID1)
                form.ShowDialog()
            End If
            If e.ColumnIndex = 2  And e.RowIndex <> -1 Then
                Dim employeeID1 As Integer = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("EmployeeID").Value.ToString())

                Dim form As New DetailsEmployees(employeeID1)
                form.ShowDialog()
            End If
            If e.ColumnIndex = 6  And e.RowIndex <> -1 Then
                Dim shipVia1 As Integer = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("ShipVia").Value.ToString())

                Dim form As New DetailsShippers(shipVia1)
                form.ShowDialog()
            End If
        End Sub

        #End Region

        #Region "toolstrip menu item events"

        Private Sub TscbPage_SelectedIndexChanged(sender As Object, e As EventArgs) Handles TscbPage.SelectedIndexChanged
            ' make sure this event is not called 
            ' when adding items to the tool strip combo box
            If Not _isFireToolStripComboBoxSelectedIndexChanged Then
                Return
            End If

            ' set current page number
            Dim lastVisitedPageNumber As Integer = _currentPageNumber
            _currentPageNumber = Cint(TscbPage.ComboBox.SelectedItem)

            If _currentPageNumber = 1 Then
                _currentRecordNumber = 1
            Else
                _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))
            End If

            ' go to the selected page
            If _currentPageNumber = 1 Then
                MoveToFirstPage()
            Else If _currentPageNumber = _totalNumberOfPages Then
                MoveToLastPage()
            Else If _currentPageNumber < lastVisitedPageNumber Then
                MoveToPrevious()
            Else
                MoveToNext()
            End If
        End Sub

        #End Region

        #Region "private methods"

        Private Sub FillComboBoxWithData()
            DdlEmployeeID.DataSource = Employees.SelectEmployeesDropDownListData()
            DdlEmployeeID.ValueMember = "EmployeeID"
            DdlEmployeeID.DisplayMember = "LastName"

            _isOnLoad = False
        End Sub

        Private Sub SetGridProperties()
            DgvGrid.AutoGenerateColumns = False
            DgvGrid.AllowUserToResizeRows = False
            DgvGrid.ColumnCount = 14
            DgvGrid.ScrollBars = ScrollBars.Horizontal
            DgvGrid.AllowUserToOrderColumns = False
            DgvGrid.AllowUserToAddRows = False
            DgvGrid.AllowUserToDeleteRows = False
            DgvGrid.ReadOnly = True
        End Sub

        Private Sub SetGridColumnProperties()
            DgvGrid.Columns("OrderID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("CustomerID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("EmployeeID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("OrderDate").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("RequiredDate").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("ShippedDate").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("ShipVia").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("Freight").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("Freight").DefaultCellStyle.Format = "0.00##"
            DgvGrid.Columns("ShipName").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("ShipAddress").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("ShipCity").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("ShipRegion").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("ShipPostalCode").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("ShipCountry").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
        End Sub

        Private Sub MoveToFirstPage()
            SetDataSource(False)
            ToolStripHelper.MoveToFirst(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub MoveToPrevious()
            SetDataSource(False)
            ToolStripHelper.MoveToPrevious(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentRecordNumber)
        End Sub

        Private Sub MoveToNext()
            SetDataSource(False)
            ToolStripHelper.MoveToNext(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentPageNumber, _totalNumberOfPages)
        End Sub

        Private Sub MoveToLastPage()
            SetDataSource(False)
            ToolStripHelper.MoveToLast(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub SetDataSource(isSort As Boolean)
            If isSort Then
                _sortByExpression = GetSortByExpression()
            End If

            If DdlEmployeeID.SelectedValue Is Nothing Then
                _totalRecords = 0
                ResetNavButtonsToDefault()
                Return
            End If

            ' fill the datagridview
            _employeeID1 = Convert.ToInt32(DdlEmployeeID.SelectedValue.ToString())

            DgvGrid.DataSource = Orders.SelectSkipAndTakeByEmployeeID(ConstantHelper.GridNumberOfRows, _currentRecordNumber, _totalRecords, _sortByExpression, _employeeID1)
            DgvGrid.Columns(_sortColumnIndex).HeaderCell.SortGlyphDirection = _sortOrder

            If _totalRecords > 0 Then
                _totalNumberOfPages = CInt(Math.Ceiling(CSng(_totalRecords) / CSng(ConstantHelper.GridNumberOfRows)))
                TscbPage.ComboBox.Items.Clear()

                ' fill page numbers in toolstripcombobox
                For page As Integer = 1 To _totalNumberOfPages
                    _isFireToolStripComboBoxSelectedIndexChanged = False
                    TscbPage.ComboBox.Items.Add(page)
                Next

                TscbPage.ComboBox.SelectedItem = _currentPageNumber
                _isFireToolStripComboBoxSelectedIndexChanged = True
            End If

            ResetNavButtonsToDefault()
        End Sub

        Private Function GetSortByExpression() As String
            _sortOrder = SortOrder.Descending

            Select Case _sortColumnIndex
                Case 0
                    If _sortByExpression = "OrderID" Then
                        Return "OrderID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "OrderID"

                Case 1
                    If _sortByExpression = "CustomerID" Then
                        Return "CustomerID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "CustomerID"

                Case 2
                    If _sortByExpression = "EmployeeID" Then
                        Return "EmployeeID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "EmployeeID"

                Case 3
                    If _sortByExpression = "OrderDate" Then
                        Return "OrderDate desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "OrderDate"

                Case 4
                    If _sortByExpression = "RequiredDate" Then
                        Return "RequiredDate desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "RequiredDate"

                Case 5
                    If _sortByExpression = "ShippedDate" Then
                        Return "ShippedDate desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShippedDate"

                Case 6
                    If _sortByExpression = "ShipVia" Then
                        Return "ShipVia desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipVia"

                Case 7
                    If _sortByExpression = "Freight" Then
                        Return "Freight desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Freight"

                Case 8
                    If _sortByExpression = "ShipName" Then
                        Return "ShipName desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipName"

                Case 9
                    If _sortByExpression = "ShipAddress" Then
                        Return "ShipAddress desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipAddress"

                Case 10
                    If _sortByExpression = "ShipCity" Then
                        Return "ShipCity desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipCity"

                Case 11
                    If _sortByExpression = "ShipRegion" Then
                        Return "ShipRegion desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipRegion"

                Case 12
                    If _sortByExpression = "ShipPostalCode" Then
                        Return "ShipPostalCode desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipPostalCode"

                Case 13
                    If _sortByExpression = "ShipCountry" Then
                        Return "ShipCountry desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipCountry"

                Case Else
                    If _sortByExpression = "OrderID" Then
                        Return "OrderID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "OrderID"
            End Select
        End Function

        Private Sub ResetNavButtonsToDefault()
            BnvNavCountItem.Text = "of " & _totalNumberOfPages & " pages"

            If _totalRecords <= ConstantHelper.GridNumberOfRows Then
                BnvNavMoveNextItem.Enabled = False
                BnvNavMoveLastItem.Enabled = False
            Else
                BnvNavMoveNextItem.Enabled = True
                BnvNavMoveLastItem.Enabled = True
            End If
        End Sub

        Private Sub HightlightRow(rowIndex As Integer)
            ' highlight row
            DgvGrid.ClearSelection()
            DgvGrid.Rows(rowIndex).Selected = True

            ' remember the current row clicked by the user
            _currentRowIndex = rowIndex
        End Sub


        #End Region

        #Region "control events"

        Private Sub DdlEmployeeID_SelectedIndexChanged(sender As Object, e As EventArgs) Handles DdlEmployeeID.SelectedIndexChanged
            If Not _isOnLoad
                SetDataSource(False)
            End If
        End Sub

        #End Region
    End Class
