Imports System
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class ListSearchCustomers
        Dim _currentRecordNumber As Integer = 1
        Dim _currentPageNumber As Integer = 1
        Dim _totalRecords As Integer = 0
        Dim _totalNumberOfPages As Integer = 0
        Dim _sortColumnIndex As Integer = 0
        Dim _sortByExpression As String = "CustomerID"
        Dim _sortOrder As SortOrder = SortOrder.Ascending
        Dim _currentRowIndex As Integer
        Dim _isFireToolStripComboBoxSelectedIndexChanged As Boolean

        #Region "winform events"

        Private Sub ListSearchCustomers_Load(sender As Object, e As EventArgs) Handles MyBase.Load
            SetGridProperties()
            SetGridColumnProperties()
            SetDataSource(False)
            ResetNavButtonsToDefault()

            LblLine.Width = Me.Width
        End Sub

        Private Sub ListSearchCustomers_Resize(sender As Object, e As EventArgs) Handles MyBase.Resize
            LblLine.Width = Me.Width
        End Sub

        #End Region

        #Region "binding navigator control events"

        Private Sub BnvNavMoveFirstItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveFirstItem.Click
            _currentRecordNumber = 1
            _currentPageNumber = 1
            MoveToFirstPage()
        End Sub

        Private Sub BnvNavMovePreviousItem_Click(sender As Object, e As EventArgs) Handles BnvNavMovePreviousItem.Click
            _currentPageNumber -= 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToPrevious()
        End Sub

        Private Sub BnvNavMoveNextItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveNextItem.Click
            _currentPageNumber += 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToNext()
        End Sub

        Private Sub BnvNavMoveLastItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveLastItem.Click
            _currentPageNumber = _totalNumberOfPages
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToLastPage()
        End Sub


        #End Region

        #Region "data gridview events"

        Private Sub DgvGrid_ColumnHeaderMouseClick(sender As Object, e As DataGridViewCellMouseEventArgs) Handles DgvGrid.ColumnHeaderMouseClick
            _sortColumnIndex = e.ColumnIndex
            SetDataSource(True)
        End Sub

        Private Sub DgvGrid_CellContextMenuStripNeeded(sender As Object, e As DataGridViewCellContextMenuStripNeededEventArgs) Handles DgvGrid.CellContextMenuStripNeeded
            HightlightRow(e.RowIndex)
        End Sub

        Private Sub DgvGrid_CellClick(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.CellClick
            ' make sure the click did not come from the header
            If e.RowIndex <> -1 Then
                HightlightRow(e.RowIndex)
            End If
        End Sub

        #End Region

        #Region "toolstrip menu item events"

        Private Sub TscbPage_SelectedIndexChanged(sender As Object, e As EventArgs) Handles TscbPage.SelectedIndexChanged
            ' make sure this event is not called 
            ' when adding items to the tool strip combo box
            If Not _isFireToolStripComboBoxSelectedIndexChanged Then
                Return
            End If

            ' set current page number
            Dim lastVisitedPageNumber As Integer = _currentPageNumber
            _currentPageNumber = Cint(TscbPage.ComboBox.SelectedItem)

            If _currentPageNumber = 1 Then
                _currentRecordNumber = 1
            Else
                _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))
            End If

            ' go to the selected page
            If _currentPageNumber = 1 Then
                MoveToFirstPage()
            Else If _currentPageNumber = _totalNumberOfPages Then
                MoveToLastPage()
            Else If _currentPageNumber < lastVisitedPageNumber Then
                MoveToPrevious()
            Else
                MoveToNext()
            End If
        End Sub

        #End Region

        #Region "private methods"

        Private Sub SetGridProperties()
            DgvGrid.AutoGenerateColumns = False
            DgvGrid.AllowUserToResizeRows = False
            DgvGrid.ColumnCount = 11
            DgvGrid.ScrollBars = ScrollBars.None
            DgvGrid.AllowUserToOrderColumns = False
            DgvGrid.AllowUserToAddRows = False
            DgvGrid.AllowUserToDeleteRows = False
            DgvGrid.ReadOnly = True
        End Sub

        Private Sub SetGridColumnProperties()
            DgvGrid.Columns("CustomerID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("CompanyName").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("ContactName").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("ContactTitle").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("Address").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("City").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("Region1").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("PostalCode").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("Country").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("Phone").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("Fax").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
        End Sub

        Private Sub MoveToFirstPage()
            SetDataSource(False)
            ToolStripHelper.MoveToFirst(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub MoveToPrevious()
            SetDataSource(False)
            ToolStripHelper.MoveToPrevious(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentRecordNumber)
        End Sub

        Private Sub MoveToNext()
            SetDataSource(False)
            ToolStripHelper.MoveToNext(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentPageNumber, _totalNumberOfPages)
        End Sub

        Private Sub MoveToLastPage()
            SetDataSource(False)
            ToolStripHelper.MoveToLast(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub SetDataSource(isSort As Boolean)
            If isSort Then
                _sortByExpression = GetSortByExpression()
            End If

            ' fill the datagridview
            If IsUseSearch() Then
                Dim customerID As String = Nothing
                Dim companyName As String = Nothing
                Dim contactName As String = Nothing
                Dim contactTitle As String = Nothing
                Dim address As String = Nothing
                Dim city As String = Nothing
                Dim region1 As String = Nothing
                Dim postalCode As String = Nothing
                Dim country As String = Nothing
                Dim phone As String = Nothing
                Dim fax As String = Nothing

                If Not String.IsNullOrEmpty(TxtCustomerID.Text.Trim()) Then
                    customerID = TxtCustomerID.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtCompanyName.Text.Trim()) Then
                    companyName = TxtCompanyName.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtContactName.Text.Trim()) Then
                    contactName = TxtContactName.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtContactTitle.Text.Trim()) Then
                    contactTitle = TxtContactTitle.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtAddress.Text.Trim()) Then
                    address = TxtAddress.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtCity.Text.Trim()) Then
                    city = TxtCity.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtRegion1.Text.Trim()) Then
                    region1 = TxtRegion1.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtPostalCode.Text.Trim()) Then
                    postalCode = TxtPostalCode.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtCountry.Text.Trim()) Then
                    country = TxtCountry.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtPhone.Text.Trim()) Then
                    phone = TxtPhone.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtFax.Text.Trim()) Then
                    fax = TxtFax.Text.ToString()
                End if

                _totalRecords = Customers.GetRecordCountDynamicWhere(customerID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax)
                DgvGrid.DataSource = Customers.SelectSkipAndTakeDynamicWhere(customerID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax, ConstantHelper.GridNumberOfRows, _currentRecordNumber, _sortByExpression)
            Else
                DgvGrid.DataSource = Customers.SelectSkipAndTake(ConstantHelper.GridNumberOfRows, _currentRecordNumber, _totalRecords, _sortByExpression)
            End If

            DgvGrid.Columns(_sortColumnIndex).HeaderCell.SortGlyphDirection = _sortOrder

            If _totalRecords > 0 Then
                _totalNumberOfPages = CInt(Math.Ceiling(CSng(_totalRecords) / CSng(ConstantHelper.GridNumberOfRows)))
                TscbPage.ComboBox.Items.Clear()

                ' fill page numbers in toolstripcombobox
                For page As Integer = 1 To _totalNumberOfPages
                    _isFireToolStripComboBoxSelectedIndexChanged = False
                    TscbPage.ComboBox.Items.Add(page)
                Next

                TscbPage.ComboBox.SelectedItem = _currentPageNumber
                _isFireToolStripComboBoxSelectedIndexChanged = True
            End If

            ResetNavButtonsToDefault()
        End Sub

        Private Function GetSortByExpression() As String
            _sortOrder = SortOrder.Descending

            Select Case _sortColumnIndex
                Case 0
                    If _sortByExpression = "CustomerID" Then
                        Return "CustomerID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "CustomerID"

                Case 1
                    If _sortByExpression = "CompanyName" Then
                        Return "CompanyName desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "CompanyName"

                Case 2
                    If _sortByExpression = "ContactName" Then
                        Return "ContactName desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ContactName"

                Case 3
                    If _sortByExpression = "ContactTitle" Then
                        Return "ContactTitle desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ContactTitle"

                Case 4
                    If _sortByExpression = "Address" Then
                        Return "Address desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Address"

                Case 5
                    If _sortByExpression = "City" Then
                        Return "City desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "City"

                Case 6
                    If _sortByExpression = "Region1" Then
                        Return "Region1 desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Region1"

                Case 7
                    If _sortByExpression = "PostalCode" Then
                        Return "PostalCode desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "PostalCode"

                Case 8
                    If _sortByExpression = "Country" Then
                        Return "Country desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Country"

                Case 9
                    If _sortByExpression = "Phone" Then
                        Return "Phone desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Phone"

                Case 10
                    If _sortByExpression = "Fax" Then
                        Return "Fax desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Fax"

                Case Else
                    If _sortByExpression = "CustomerID" Then
                        Return "CustomerID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "CustomerID"
            End Select
        End Function

        Private Sub ResetNavButtonsToDefault()
            BnvNavCountItem.Text = "of " & _totalNumberOfPages & " pages"

            If _totalRecords <= ConstantHelper.GridNumberOfRows Then
                BnvNavMoveNextItem.Enabled = False
                BnvNavMoveLastItem.Enabled = False
            Else
                BnvNavMoveNextItem.Enabled = True
                BnvNavMoveLastItem.Enabled = True
            End If
        End Sub

        Private Function IsUseSearch() As Boolean
            If TxtCustomerID.Text.Trim() <> "" Or TxtCompanyName.Text.Trim() <> "" Or TxtContactName.Text.Trim() <> "" Or TxtContactTitle.Text.Trim() <> "" Or TxtAddress.Text.Trim() <> "" Or TxtCity.Text.Trim() <> "" Or TxtRegion1.Text.Trim() <> "" Or TxtPostalCode.Text.Trim() <> "" Or TxtCountry.Text.Trim() <> "" Or TxtPhone.Text.Trim() <> "" Or TxtFax.Text.Trim() <> "" Then
                BtnCancel.Enabled = True
                Return True
            End If

            Return False
        End Function

        Private Sub HightlightRow(rowIndex As Integer)
            ' highlight row
            DgvGrid.ClearSelection()
            DgvGrid.Rows(rowIndex).Selected = True

            ' remember the current row clicked by the user
            _currentRowIndex = rowIndex
        End Sub


        #End Region

        #Region "search page control events"

        Private Sub BtnSearch_Click(sender As Object, e As EventArgs) Handles BtnSearch.Click
            If IsUseSearch() Then
                _currentRecordNumber = 1
                _currentPageNumber = 1
                _sortByExpression = "CustomerID"

                BnvNavMoveFirstItem.Enabled = False
                BnvNavMovePreviousItem.Enabled = False

                SetDataSource(False)
                ResetNavButtonsToDefault()
            Else
                MessageBox.Show("At least one (1) search field must be filled!", "Empty Search Fields")
            End If
        End Sub

        Private Sub BtnCancel_Click(sender As Object, e As EventArgs) Handles BtnCancel.Click
            ' reset all control's values
            TxtCustomerID.Text = ""
            TxtCompanyName.Text = ""
            TxtContactName.Text = ""
            TxtContactTitle.Text = ""
            TxtAddress.Text = ""
            TxtCity.Text = ""
            TxtRegion1.Text = ""
            TxtPostalCode.Text = ""
            TxtCountry.Text = ""
            TxtPhone.Text = ""
            TxtFax.Text = ""

            SetDataSource(False)
            ResetNavButtonsToDefault()

            BtnCancel.Enabled = False
        End Sub

        #End Region
    End Class
