Imports System
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class ListSearchEmployees
        Dim _currentRecordNumber As Integer = 1
        Dim _currentPageNumber As Integer = 1
        Dim _totalRecords As Integer = 0
        Dim _totalNumberOfPages As Integer = 0
        Dim _sortColumnIndex As Integer = 0
        Dim _sortByExpression As String = "EmployeeID"
        Dim _sortOrder As SortOrder = SortOrder.Ascending
        Dim _currentRowIndex As Integer
        Dim _isFireToolStripComboBoxSelectedIndexChanged As Boolean

        #Region "winform events"

        Private Sub ListSearchEmployees_Load(sender As Object, e As EventArgs) Handles MyBase.Load
            FillComboBoxWithData()
            SetDateFieldsToDefault()
            SetGridProperties()
            SetGridColumnProperties()
            SetDataSource(False)
            ResetNavButtonsToDefault()

            LblLine.Width = DgvGrid.Width + 10
        End Sub

        Private Sub ListSearchEmployees_Resize(sender As Object, e As EventArgs) Handles MyBase.Resize
            LblLine.Width = Me.Width
        End Sub

        #End Region

        #Region "binding navigator control events"

        Private Sub BnvNavMoveFirstItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveFirstItem.Click
            _currentRecordNumber = 1
            _currentPageNumber = 1
            MoveToFirstPage()
        End Sub

        Private Sub BnvNavMovePreviousItem_Click(sender As Object, e As EventArgs) Handles BnvNavMovePreviousItem.Click
            _currentPageNumber -= 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToPrevious()
        End Sub

        Private Sub BnvNavMoveNextItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveNextItem.Click
            _currentPageNumber += 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToNext()
        End Sub

        Private Sub BnvNavMoveLastItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveLastItem.Click
            _currentPageNumber = _totalNumberOfPages
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToLastPage()
        End Sub


        #End Region

        #Region "data gridview events"

        Private Sub DgvGrid_ColumnHeaderMouseClick(sender As Object, e As DataGridViewCellMouseEventArgs) Handles DgvGrid.ColumnHeaderMouseClick
            _sortColumnIndex = e.ColumnIndex
            SetDataSource(True)
        End Sub

        Private Sub DgvGrid_CellContextMenuStripNeeded(sender As Object, e As DataGridViewCellContextMenuStripNeededEventArgs) Handles DgvGrid.CellContextMenuStripNeeded
            HightlightRow(e.RowIndex)
        End Sub

        Private Sub DgvGrid_CellClick(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.CellClick
            ' make sure the click did not come from the header
            If e.RowIndex <> -1 Then
                HightlightRow(e.RowIndex)
            End If
        End Sub

        Private Sub DgvGrid_CellContentClick(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.CellContentClick
            If e.ColumnIndex = 15  And e.RowIndex <> -1 Then
                Dim reportsTo1 As Integer = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("ReportsTo").Value.ToString())

                Dim form As New DetailsEmployees(reportsTo1)
                form.ShowDialog()
            End If
        End Sub

        #End Region

        #Region "toolstrip menu item events"

        Private Sub TscbPage_SelectedIndexChanged(sender As Object, e As EventArgs) Handles TscbPage.SelectedIndexChanged
            ' make sure this event is not called 
            ' when adding items to the tool strip combo box
            If Not _isFireToolStripComboBoxSelectedIndexChanged Then
                Return
            End If

            ' set current page number
            Dim lastVisitedPageNumber As Integer = _currentPageNumber
            _currentPageNumber = Cint(TscbPage.ComboBox.SelectedItem)

            If _currentPageNumber = 1 Then
                _currentRecordNumber = 1
            Else
                _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))
            End If

            ' go to the selected page
            If _currentPageNumber = 1 Then
                MoveToFirstPage()
            Else If _currentPageNumber = _totalNumberOfPages Then
                MoveToLastPage()
            Else If _currentPageNumber < lastVisitedPageNumber Then
                MoveToPrevious()
            Else
                MoveToNext()
            End If
        End Sub

        #End Region

        #Region "private methods"

        Private Sub FillComboBoxWithData()
            ' fill ReportsTo combo box
            Dim objEmployeesCol1 As New EmployeesCollection()
            objEmployeesCol1.Add(New Employees With { .EmployeeID = -1, .LastName = "" })

            ' make sure collection is not null
            Dim objEmployeesCol2 As EmployeesCollection = Employees.SelectEmployeesDropDownListData()

            If objEmployeesCol2 IsNot Nothing Then
                objEmployeesCol1.AddRange(objEmployeesCol2)
            End If

            DdlReportsTo.DataSource = objEmployeesCol1
            DdlReportsTo.ValueMember = "EmployeeID"
            DdlReportsTo.DisplayMember = "LastName"

        End Sub

        Private Sub SetDateFieldsToDefault()
            Me.DtpBirthDate.CustomFormat = " "
            Me.DtpBirthDate.Format = DateTimePickerFormat.Custom

            Me.DtpHireDate.CustomFormat = " "
            Me.DtpHireDate.Format = DateTimePickerFormat.Custom

        End Sub

        Private Sub SetGridProperties()
            DgvGrid.AutoGenerateColumns = False
            DgvGrid.AllowUserToResizeRows = False
            DgvGrid.ColumnCount = 17
            DgvGrid.ScrollBars = ScrollBars.None
            DgvGrid.AllowUserToOrderColumns = False
            DgvGrid.AllowUserToAddRows = False
            DgvGrid.AllowUserToDeleteRows = False
            DgvGrid.ReadOnly = True
        End Sub

        Private Sub SetGridColumnProperties()
            DgvGrid.Columns("EmployeeID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("LastName").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("FirstName").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("Title").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("TitleOfCourtesy").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("BirthDate").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("HireDate").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("Address").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("City").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("Region1").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("PostalCode").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("Country").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("HomePhone").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("Extension").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("Notes").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("ReportsTo").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("PhotoPath").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
        End Sub

        Private Sub MoveToFirstPage()
            SetDataSource(False)
            ToolStripHelper.MoveToFirst(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub MoveToPrevious()
            SetDataSource(False)
            ToolStripHelper.MoveToPrevious(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentRecordNumber)
        End Sub

        Private Sub MoveToNext()
            SetDataSource(False)
            ToolStripHelper.MoveToNext(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentPageNumber, _totalNumberOfPages)
        End Sub

        Private Sub MoveToLastPage()
            SetDataSource(False)
            ToolStripHelper.MoveToLast(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub SetDataSource(isSort As Boolean)
            If isSort Then
                _sortByExpression = GetSortByExpression()
            End If

            ' fill the datagridview
            If IsUseSearch() Then
                Dim employeeID As Integer? = Nothing
                Dim lastName As String = Nothing
                Dim firstName As String = Nothing
                Dim title As String = Nothing
                Dim titleOfCourtesy As String = Nothing
                Dim birthDate As Date? = Nothing
                Dim hireDate As Date? = Nothing
                Dim address As String = Nothing
                Dim city As String = Nothing
                Dim region1 As String = Nothing
                Dim postalCode As String = Nothing
                Dim country As String = Nothing
                Dim homePhone As String = Nothing
                Dim extension As String = Nothing
                Dim notes As String = Nothing
                Dim reportsTo As Integer? = Nothing
                Dim photoPath As String = Nothing

                If Not String.IsNullOrEmpty(TxtEmployeeID.Text.Trim()) Then
                    employeeID = Convert.ToInt32(TxtEmployeeID.Text.ToString())
                End if

                If Not String.IsNullOrEmpty(TxtLastName.Text.Trim()) Then
                    lastName = TxtLastName.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtFirstName.Text.Trim()) Then
                    firstName = TxtFirstName.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtTitle.Text.Trim()) Then
                    title = TxtTitle.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtTitleOfCourtesy.Text.Trim()) Then
                    titleOfCourtesy = TxtTitleOfCourtesy.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(DtpBirthDate.Text.Trim()) Then
                    birthDate = DtpBirthDate.Value
                End if

                If Not String.IsNullOrEmpty(DtpHireDate.Text.Trim()) Then
                    hireDate = DtpHireDate.Value
                End if

                If Not String.IsNullOrEmpty(TxtAddress.Text.Trim()) Then
                    address = TxtAddress.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtCity.Text.Trim()) Then
                    city = TxtCity.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtRegion1.Text.Trim()) Then
                    region1 = TxtRegion1.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtPostalCode.Text.Trim()) Then
                    postalCode = TxtPostalCode.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtCountry.Text.Trim()) Then
                    country = TxtCountry.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtHomePhone.Text.Trim()) Then
                    homePhone = TxtHomePhone.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtExtension.Text.Trim()) Then
                    extension = TxtExtension.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtNotes.Text.Trim()) Then
                    notes = TxtNotes.Text.ToString()
                End if

                If DdlReportsTo.SelectedIndex <> 0 Then
                    reportsTo = Convert.ToInt32(DdlReportsTo.SelectedValue.ToString())
                End if

                If Not String.IsNullOrEmpty(TxtPhotoPath.Text.Trim()) Then
                    photoPath = TxtPhotoPath.Text.ToString()
                End if

                _totalRecords = Employees.GetRecordCountDynamicWhere(employeeID, lastName, firstName, title, titleOfCourtesy, birthDate, hireDate, address, city, region1, postalCode, country, homePhone, extension, reportsTo, photoPath)
                DgvGrid.DataSource = Employees.SelectSkipAndTakeDynamicWhere(employeeID, lastName, firstName, title, titleOfCourtesy, birthDate, hireDate, address, city, region1, postalCode, country, homePhone, extension, reportsTo, photoPath, ConstantHelper.GridNumberOfRows, _currentRecordNumber, _sortByExpression)
            Else
                DgvGrid.DataSource = Employees.SelectSkipAndTake(ConstantHelper.GridNumberOfRows, _currentRecordNumber, _totalRecords, _sortByExpression)
            End If

            DgvGrid.Columns(_sortColumnIndex).HeaderCell.SortGlyphDirection = _sortOrder

            If _totalRecords > 0 Then
                _totalNumberOfPages = CInt(Math.Ceiling(CSng(_totalRecords) / CSng(ConstantHelper.GridNumberOfRows)))
                TscbPage.ComboBox.Items.Clear()

                ' fill page numbers in toolstripcombobox
                For page As Integer = 1 To _totalNumberOfPages
                    _isFireToolStripComboBoxSelectedIndexChanged = False
                    TscbPage.ComboBox.Items.Add(page)
                Next

                TscbPage.ComboBox.SelectedItem = _currentPageNumber
                _isFireToolStripComboBoxSelectedIndexChanged = True
            End If

            ResetNavButtonsToDefault()
        End Sub

        Private Function GetSortByExpression() As String
            _sortOrder = SortOrder.Descending

            Select Case _sortColumnIndex
                Case 0
                    If _sortByExpression = "EmployeeID" Then
                        Return "EmployeeID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "EmployeeID"

                Case 1
                    If _sortByExpression = "LastName" Then
                        Return "LastName desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "LastName"

                Case 2
                    If _sortByExpression = "FirstName" Then
                        Return "FirstName desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "FirstName"

                Case 3
                    If _sortByExpression = "Title" Then
                        Return "Title desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Title"

                Case 4
                    If _sortByExpression = "TitleOfCourtesy" Then
                        Return "TitleOfCourtesy desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "TitleOfCourtesy"

                Case 5
                    If _sortByExpression = "BirthDate" Then
                        Return "BirthDate desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "BirthDate"

                Case 6
                    If _sortByExpression = "HireDate" Then
                        Return "HireDate desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "HireDate"

                Case 7
                    If _sortByExpression = "Address" Then
                        Return "Address desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Address"

                Case 8
                    If _sortByExpression = "City" Then
                        Return "City desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "City"

                Case 9
                    If _sortByExpression = "Region1" Then
                        Return "Region1 desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Region1"

                Case 10
                    If _sortByExpression = "PostalCode" Then
                        Return "PostalCode desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "PostalCode"

                Case 11
                    If _sortByExpression = "Country" Then
                        Return "Country desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Country"

                Case 12
                    If _sortByExpression = "HomePhone" Then
                        Return "HomePhone desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "HomePhone"

                Case 13
                    If _sortByExpression = "Extension" Then
                        Return "Extension desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Extension"

                Case 14
                    If _sortByExpression = "Notes" Then
                        Return "Notes desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Notes"

                Case 15
                    If _sortByExpression = "ReportsTo" Then
                        Return "ReportsTo desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ReportsTo"

                Case 16
                    If _sortByExpression = "PhotoPath" Then
                        Return "PhotoPath desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "PhotoPath"

                Case Else
                    If _sortByExpression = "EmployeeID" Then
                        Return "EmployeeID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "EmployeeID"
            End Select
        End Function

        Private Sub ResetNavButtonsToDefault()
            BnvNavCountItem.Text = "of " & _totalNumberOfPages & " pages"

            If _totalRecords <= ConstantHelper.GridNumberOfRows Then
                BnvNavMoveNextItem.Enabled = False
                BnvNavMoveLastItem.Enabled = False
            Else
                BnvNavMoveNextItem.Enabled = True
                BnvNavMoveLastItem.Enabled = True
            End If
        End Sub

        Private Function IsUseSearch() As Boolean
            If TxtEmployeeID.Text.Trim() <> "" Or TxtLastName.Text.Trim() <> "" Or TxtFirstName.Text.Trim() <> "" Or TxtTitle.Text.Trim() <> "" Or TxtTitleOfCourtesy.Text.Trim() <> "" Or DtpBirthDate.Text.Trim() <> "" Or DtpHireDate.Text.Trim() <> "" Or TxtAddress.Text.Trim() <> "" Or TxtCity.Text.Trim() <> "" Or TxtRegion1.Text.Trim() <> "" Or TxtPostalCode.Text.Trim() <> "" Or TxtCountry.Text.Trim() <> "" Or TxtHomePhone.Text.Trim() <> "" Or TxtExtension.Text.Trim() <> "" Or TxtNotes.Text.Trim() <> "" Or DdlReportsTo.SelectedIndex <> 0 Or TxtPhotoPath.Text.Trim() <> "" Then
                BtnCancel.Enabled = True
                Return True
            End If

            Return False
        End Function

        Private Sub HightlightRow(rowIndex As Integer)
            ' highlight row
            DgvGrid.ClearSelection()
            DgvGrid.Rows(rowIndex).Selected = True

            ' remember the current row clicked by the user
            _currentRowIndex = rowIndex
        End Sub


        #End Region

        #Region "search page control events"

        Private Sub DdlReportsTo_KeyPress(sender As Object, e As KeyPressEventArgs) Handles DdlReportsTo.KeyPress
            e.Handled = True
        End Sub

        Private Sub BtnSearch_Click(sender As Object, e As EventArgs) Handles BtnSearch.Click
            If IsUseSearch() Then
                _currentRecordNumber = 1
                _currentPageNumber = 1
                _sortByExpression = "EmployeeID"

                BnvNavMoveFirstItem.Enabled = False
                BnvNavMovePreviousItem.Enabled = False

                SetDataSource(False)
                ResetNavButtonsToDefault()
            Else
                MessageBox.Show("At least one (1) search field must be filled!", "Empty Search Fields")
            End If
        End Sub

        Private Sub BtnCancel_Click(sender As Object, e As EventArgs) Handles BtnCancel.Click
            ' reset all control's values
            TxtEmployeeID.Text = ""
            TxtLastName.Text = ""
            TxtFirstName.Text = ""
            TxtTitle.Text = ""
            TxtTitleOfCourtesy.Text = ""
            TxtAddress.Text = ""
            TxtCity.Text = ""
            TxtRegion1.Text = ""
            TxtPostalCode.Text = ""
            TxtCountry.Text = ""
            TxtHomePhone.Text = ""
            TxtExtension.Text = ""
            TxtNotes.Text = ""
            DdlReportsTo.SelectedIndex = 0
            TxtPhotoPath.Text = ""

            SetDateFieldsToDefault()
            SetDataSource(False)
            ResetNavButtonsToDefault()

            BtnCancel.Enabled = False
        End Sub

        Private Sub DtpBirthDate_ValueChanged(sender As Object, e As EventArgs) Handles DtpBirthDate.ValueChanged
            Me.DtpBirthDate.CustomFormat = "MM/dd/yyyy"
        End Sub

        Private Sub DtpHireDate_ValueChanged(sender As Object, e As EventArgs) Handles DtpHireDate.ValueChanged
            Me.DtpHireDate.CustomFormat = "MM/dd/yyyy"
        End Sub


        #End Region
    End Class
