Imports System
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class ListSearchOrderDetails
        Dim _currentRecordNumber As Integer = 1
        Dim _currentPageNumber As Integer = 1
        Dim _totalRecords As Integer = 0
        Dim _totalNumberOfPages As Integer = 0
        Dim _sortColumnIndex As Integer = 0
        Dim _sortByExpression As String = "OrderID"
        Dim _sortOrder As SortOrder = SortOrder.Ascending
        Dim _currentRowIndex As Integer
        Dim _isFireToolStripComboBoxSelectedIndexChanged As Boolean

        #Region "winform events"

        Private Sub ListSearchOrderDetails_Load(sender As Object, e As EventArgs) Handles MyBase.Load
            FillComboBoxWithData()
            SetGridProperties()
            SetGridColumnProperties()
            SetDataSource(False)
            ResetNavButtonsToDefault()

            LblLine.Width = Me.Width
        End Sub

        Private Sub ListSearchOrderDetails_Resize(sender As Object, e As EventArgs) Handles MyBase.Resize
            LblLine.Width = Me.Width
        End Sub

        #End Region

        #Region "binding navigator control events"

        Private Sub BnvNavMoveFirstItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveFirstItem.Click
            _currentRecordNumber = 1
            _currentPageNumber = 1
            MoveToFirstPage()
        End Sub

        Private Sub BnvNavMovePreviousItem_Click(sender As Object, e As EventArgs) Handles BnvNavMovePreviousItem.Click
            _currentPageNumber -= 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToPrevious()
        End Sub

        Private Sub BnvNavMoveNextItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveNextItem.Click
            _currentPageNumber += 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToNext()
        End Sub

        Private Sub BnvNavMoveLastItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveLastItem.Click
            _currentPageNumber = _totalNumberOfPages
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToLastPage()
        End Sub


        #End Region

        #Region "data gridview events"

        Private Sub DgvGrid_ColumnHeaderMouseClick(sender As Object, e As DataGridViewCellMouseEventArgs) Handles DgvGrid.ColumnHeaderMouseClick
            _sortColumnIndex = e.ColumnIndex
            SetDataSource(True)
        End Sub

        Private Sub DgvGrid_CellContextMenuStripNeeded(sender As Object, e As DataGridViewCellContextMenuStripNeededEventArgs) Handles DgvGrid.CellContextMenuStripNeeded
            HightlightRow(e.RowIndex)
        End Sub

        Private Sub DgvGrid_CellClick(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.CellClick
            ' make sure the click did not come from the header
            If e.RowIndex <> -1 Then
                HightlightRow(e.RowIndex)
            End If
        End Sub

        Private Sub DgvGrid_CellContentClick(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.CellContentClick
            If e.ColumnIndex = 0  And e.RowIndex <> -1 Then
                Dim orderID1 As Integer = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("OrderID").Value.ToString())

                Dim form As New DetailsOrders(orderID1)
                form.ShowDialog()
            End If
            If e.ColumnIndex = 1  And e.RowIndex <> -1 Then
                Dim productID1 As Integer = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("ProductID").Value.ToString())

                Dim form As New DetailsProducts(productID1)
                form.ShowDialog()
            End If
        End Sub

        #End Region

        #Region "toolstrip menu item events"

        Private Sub TscbPage_SelectedIndexChanged(sender As Object, e As EventArgs) Handles TscbPage.SelectedIndexChanged
            ' make sure this event is not called 
            ' when adding items to the tool strip combo box
            If Not _isFireToolStripComboBoxSelectedIndexChanged Then
                Return
            End If

            ' set current page number
            Dim lastVisitedPageNumber As Integer = _currentPageNumber
            _currentPageNumber = Cint(TscbPage.ComboBox.SelectedItem)

            If _currentPageNumber = 1 Then
                _currentRecordNumber = 1
            Else
                _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))
            End If

            ' go to the selected page
            If _currentPageNumber = 1 Then
                MoveToFirstPage()
            Else If _currentPageNumber = _totalNumberOfPages Then
                MoveToLastPage()
            Else If _currentPageNumber < lastVisitedPageNumber Then
                MoveToPrevious()
            Else
                MoveToNext()
            End If
        End Sub

        #End Region

        #Region "private methods"

        Private Sub FillComboBoxWithData()
            ' fill OrderID combo box
            Dim objOrdersCol1 As New OrdersCollection()
            objOrdersCol1.Add(New Orders With { .OrderID = -1, .ShipName = "" })

            ' make sure collection is not null
            Dim objOrdersCol2 As OrdersCollection = Orders.SelectOrdersDropDownListData()

            If objOrdersCol2 IsNot Nothing Then
                objOrdersCol1.AddRange(objOrdersCol2)
            End If

            DdlOrderID.DataSource = objOrdersCol1
            DdlOrderID.ValueMember = "OrderID"
            DdlOrderID.DisplayMember = "ShipName"

            ' fill ProductID combo box
            Dim objProductsCol3 As New ProductsCollection()
            objProductsCol3.Add(New Products With { .ProductID = -1, .ProductName = "" })

            ' make sure collection is not null
            Dim objProductsCol4 As ProductsCollection = Products.SelectProductsDropDownListData()

            If objProductsCol4 IsNot Nothing Then
                objProductsCol3.AddRange(objProductsCol4)
            End If

            DdlProductID.DataSource = objProductsCol3
            DdlProductID.ValueMember = "ProductID"
            DdlProductID.DisplayMember = "ProductName"

        End Sub

        Private Sub SetGridProperties()
            DgvGrid.AutoGenerateColumns = False
            DgvGrid.AllowUserToResizeRows = False
            DgvGrid.ColumnCount = 5
            DgvGrid.ScrollBars = ScrollBars.None
            DgvGrid.AllowUserToOrderColumns = False
            DgvGrid.AllowUserToAddRows = False
            DgvGrid.AllowUserToDeleteRows = False
            DgvGrid.ReadOnly = True
        End Sub

        Private Sub SetGridColumnProperties()
            DgvGrid.Columns("OrderID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("ProductID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("UnitPrice").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("UnitPrice").DefaultCellStyle.Format = "0.00##"
            DgvGrid.Columns("Quantity").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("Discount").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
        End Sub

        Private Sub MoveToFirstPage()
            SetDataSource(False)
            ToolStripHelper.MoveToFirst(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub MoveToPrevious()
            SetDataSource(False)
            ToolStripHelper.MoveToPrevious(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentRecordNumber)
        End Sub

        Private Sub MoveToNext()
            SetDataSource(False)
            ToolStripHelper.MoveToNext(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentPageNumber, _totalNumberOfPages)
        End Sub

        Private Sub MoveToLastPage()
            SetDataSource(False)
            ToolStripHelper.MoveToLast(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub SetDataSource(isSort As Boolean)
            If isSort Then
                _sortByExpression = GetSortByExpression()
            End If

            ' fill the datagridview
            If IsUseSearch() Then
                Dim orderID As Integer? = Nothing
                Dim productID As Integer? = Nothing
                Dim unitPrice As Decimal? = Nothing
                Dim quantity As Short? = Nothing
                Dim discount As Single? = Nothing

                If DdlOrderID.SelectedIndex <> 0 Then
                    orderID = Convert.ToInt32(DdlOrderID.SelectedValue.ToString())
                End if

                If DdlProductID.SelectedIndex <> 0 Then
                    productID = Convert.ToInt32(DdlProductID.SelectedValue.ToString())
                End if

                If Not String.IsNullOrEmpty(MtbUnitPrice.Text.Trim()) Then
                    unitPrice = Convert.ToDecimal(MtbUnitPrice.Text.ToString())
                End if

                If Not String.IsNullOrEmpty(MtbQuantity.Text.Trim()) Then
                    quantity = Convert.ToInt16(MtbQuantity.Text.ToString())
                End if

                If Not String.IsNullOrEmpty(MtbDiscount.Text.Trim()) Then
                    discount = Convert.ToSingle(MtbDiscount.Text.ToString())
                End if

                _totalRecords = OrderDetails.GetRecordCountDynamicWhere(orderID, productID, unitPrice, quantity, discount)
                DgvGrid.DataSource = OrderDetails.SelectSkipAndTakeDynamicWhere(orderID, productID, unitPrice, quantity, discount, ConstantHelper.GridNumberOfRows, _currentRecordNumber, _sortByExpression)
            Else
                DgvGrid.DataSource = OrderDetails.SelectSkipAndTake(ConstantHelper.GridNumberOfRows, _currentRecordNumber, _totalRecords, _sortByExpression)
            End If

            DgvGrid.Columns(_sortColumnIndex).HeaderCell.SortGlyphDirection = _sortOrder

            If _totalRecords > 0 Then
                _totalNumberOfPages = CInt(Math.Ceiling(CSng(_totalRecords) / CSng(ConstantHelper.GridNumberOfRows)))
                TscbPage.ComboBox.Items.Clear()

                ' fill page numbers in toolstripcombobox
                For page As Integer = 1 To _totalNumberOfPages
                    _isFireToolStripComboBoxSelectedIndexChanged = False
                    TscbPage.ComboBox.Items.Add(page)
                Next

                TscbPage.ComboBox.SelectedItem = _currentPageNumber
                _isFireToolStripComboBoxSelectedIndexChanged = True
            End If

            ResetNavButtonsToDefault()
        End Sub

        Private Function GetSortByExpression() As String
            _sortOrder = SortOrder.Descending

            Select Case _sortColumnIndex
                Case 0
                    If _sortByExpression = "OrderID" Then
                        Return "OrderID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "OrderID"

                Case 1
                    If _sortByExpression = "ProductID" Then
                        Return "ProductID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ProductID"

                Case 2
                    If _sortByExpression = "UnitPrice" Then
                        Return "UnitPrice desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "UnitPrice"

                Case 3
                    If _sortByExpression = "Quantity" Then
                        Return "Quantity desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Quantity"

                Case 4
                    If _sortByExpression = "Discount" Then
                        Return "Discount desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Discount"

                Case Else
                    If _sortByExpression = "OrderID" Then
                        Return "OrderID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "OrderID"
            End Select
        End Function

        Private Sub ResetNavButtonsToDefault()
            BnvNavCountItem.Text = "of " & _totalNumberOfPages & " pages"

            If _totalRecords <= ConstantHelper.GridNumberOfRows Then
                BnvNavMoveNextItem.Enabled = False
                BnvNavMoveLastItem.Enabled = False
            Else
                BnvNavMoveNextItem.Enabled = True
                BnvNavMoveLastItem.Enabled = True
            End If
        End Sub

        Private Function IsUseSearch() As Boolean
            If DdlOrderID.SelectedIndex <> 0 Or DdlProductID.SelectedIndex <> 0 Or MtbUnitPrice.Text.Trim() <> "" Or MtbQuantity.Text.Trim() <> "" Or MtbDiscount.Text.Trim() <> "" Then
                BtnCancel.Enabled = True
                Return True
            End If

            Return False
        End Function

        Private Sub HightlightRow(rowIndex As Integer)
            ' highlight row
            DgvGrid.ClearSelection()
            DgvGrid.Rows(rowIndex).Selected = True

            ' remember the current row clicked by the user
            _currentRowIndex = rowIndex
        End Sub


        #End Region

        #Region "search page control events"

        Private Sub DdlOrderID_KeyPress(sender As Object, e As KeyPressEventArgs) Handles DdlOrderID.KeyPress
            e.Handled = True
        End Sub

        Private Sub DdlProductID_KeyPress(sender As Object, e As KeyPressEventArgs) Handles DdlProductID.KeyPress
            e.Handled = True
        End Sub

        Private Sub BtnSearch_Click(sender As Object, e As EventArgs) Handles BtnSearch.Click
            If IsUseSearch() Then
                _currentRecordNumber = 1
                _currentPageNumber = 1
                _sortByExpression = "OrderID"

                BnvNavMoveFirstItem.Enabled = False
                BnvNavMovePreviousItem.Enabled = False

                SetDataSource(False)
                ResetNavButtonsToDefault()
            Else
                MessageBox.Show("At least one (1) search field must be filled!", "Empty Search Fields")
            End If
        End Sub

        Private Sub BtnCancel_Click(sender As Object, e As EventArgs) Handles BtnCancel.Click
            ' reset all control's values
            DdlOrderID.SelectedIndex = 0
            DdlProductID.SelectedIndex = 0
            MtbUnitPrice.Text = ""
            MtbQuantity.Text = ""
            MtbDiscount.Text = ""

            SetDataSource(False)
            ResetNavButtonsToDefault()

            BtnCancel.Enabled = False
        End Sub

        #End Region
    End Class
