Imports System
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class ListSearchOrders
        Dim _currentRecordNumber As Integer = 1
        Dim _currentPageNumber As Integer = 1
        Dim _totalRecords As Integer = 0
        Dim _totalNumberOfPages As Integer = 0
        Dim _sortColumnIndex As Integer = 0
        Dim _sortByExpression As String = "OrderID"
        Dim _sortOrder As SortOrder = SortOrder.Ascending
        Dim _currentRowIndex As Integer
        Dim _isFireToolStripComboBoxSelectedIndexChanged As Boolean

        #Region "winform events"

        Private Sub ListSearchOrders_Load(sender As Object, e As EventArgs) Handles MyBase.Load
            FillComboBoxWithData()
            SetDateFieldsToDefault()
            SetGridProperties()
            SetGridColumnProperties()
            SetDataSource(False)
            ResetNavButtonsToDefault()

            LblLine.Width = DgvGrid.Width + 10
        End Sub

        Private Sub ListSearchOrders_Resize(sender As Object, e As EventArgs) Handles MyBase.Resize
            LblLine.Width = Me.Width
        End Sub

        #End Region

        #Region "binding navigator control events"

        Private Sub BnvNavMoveFirstItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveFirstItem.Click
            _currentRecordNumber = 1
            _currentPageNumber = 1
            MoveToFirstPage()
        End Sub

        Private Sub BnvNavMovePreviousItem_Click(sender As Object, e As EventArgs) Handles BnvNavMovePreviousItem.Click
            _currentPageNumber -= 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToPrevious()
        End Sub

        Private Sub BnvNavMoveNextItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveNextItem.Click
            _currentPageNumber += 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToNext()
        End Sub

        Private Sub BnvNavMoveLastItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveLastItem.Click
            _currentPageNumber = _totalNumberOfPages
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToLastPage()
        End Sub


        #End Region

        #Region "data gridview events"

        Private Sub DgvGrid_ColumnHeaderMouseClick(sender As Object, e As DataGridViewCellMouseEventArgs) Handles DgvGrid.ColumnHeaderMouseClick
            _sortColumnIndex = e.ColumnIndex
            SetDataSource(True)
        End Sub

        Private Sub DgvGrid_CellContextMenuStripNeeded(sender As Object, e As DataGridViewCellContextMenuStripNeededEventArgs) Handles DgvGrid.CellContextMenuStripNeeded
            HightlightRow(e.RowIndex)
        End Sub

        Private Sub DgvGrid_CellClick(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.CellClick
            ' make sure the click did not come from the header
            If e.RowIndex <> -1 Then
                HightlightRow(e.RowIndex)
            End If
        End Sub

        Private Sub DgvGrid_CellContentClick(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.CellContentClick
            If e.ColumnIndex = 1  And e.RowIndex <> -1 Then
                Dim customerID1 As String = DgvGrid.Rows(e.RowIndex).Cells("CustomerID").Value.ToString()

                Dim form As New DetailsCustomers(customerID1)
                form.ShowDialog()
            End If
            If e.ColumnIndex = 2  And e.RowIndex <> -1 Then
                Dim employeeID1 As Integer = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("EmployeeID").Value.ToString())

                Dim form As New DetailsEmployees(employeeID1)
                form.ShowDialog()
            End If
            If e.ColumnIndex = 6  And e.RowIndex <> -1 Then
                Dim shipVia1 As Integer = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("ShipVia").Value.ToString())

                Dim form As New DetailsShippers(shipVia1)
                form.ShowDialog()
            End If
        End Sub

        #End Region

        #Region "toolstrip menu item events"

        Private Sub TscbPage_SelectedIndexChanged(sender As Object, e As EventArgs) Handles TscbPage.SelectedIndexChanged
            ' make sure this event is not called 
            ' when adding items to the tool strip combo box
            If Not _isFireToolStripComboBoxSelectedIndexChanged Then
                Return
            End If

            ' set current page number
            Dim lastVisitedPageNumber As Integer = _currentPageNumber
            _currentPageNumber = Cint(TscbPage.ComboBox.SelectedItem)

            If _currentPageNumber = 1 Then
                _currentRecordNumber = 1
            Else
                _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))
            End If

            ' go to the selected page
            If _currentPageNumber = 1 Then
                MoveToFirstPage()
            Else If _currentPageNumber = _totalNumberOfPages Then
                MoveToLastPage()
            Else If _currentPageNumber < lastVisitedPageNumber Then
                MoveToPrevious()
            Else
                MoveToNext()
            End If
        End Sub

        #End Region

        #Region "private methods"

        Private Sub FillComboBoxWithData()
            ' fill CustomerID combo box
            Dim objCustomersCol1 As New CustomersCollection()
            objCustomersCol1.Add(New Customers With { .CustomerID = "", .CompanyName = "" })

            ' make sure collection is not null
            Dim objCustomersCol2 As CustomersCollection = Customers.SelectCustomersDropDownListData()

            If objCustomersCol2 IsNot Nothing Then
                objCustomersCol1.AddRange(objCustomersCol2)
            End If

            DdlCustomerID.DataSource = objCustomersCol1
            DdlCustomerID.ValueMember = "CustomerID"
            DdlCustomerID.DisplayMember = "CompanyName"

            ' fill EmployeeID combo box
            Dim objEmployeesCol3 As New EmployeesCollection()
            objEmployeesCol3.Add(New Employees With { .EmployeeID = -1, .LastName = "" })

            ' make sure collection is not null
            Dim objEmployeesCol4 As EmployeesCollection = Employees.SelectEmployeesDropDownListData()

            If objEmployeesCol4 IsNot Nothing Then
                objEmployeesCol3.AddRange(objEmployeesCol4)
            End If

            DdlEmployeeID.DataSource = objEmployeesCol3
            DdlEmployeeID.ValueMember = "EmployeeID"
            DdlEmployeeID.DisplayMember = "LastName"

            ' fill ShipVia combo box
            Dim objShippersCol5 As New ShippersCollection()
            objShippersCol5.Add(New Shippers With { .ShipperID = -1, .CompanyName = "" })

            ' make sure collection is not null
            Dim objShippersCol6 As ShippersCollection = Shippers.SelectShippersDropDownListData()

            If objShippersCol6 IsNot Nothing Then
                objShippersCol5.AddRange(objShippersCol6)
            End If

            DdlShipVia.DataSource = objShippersCol5
            DdlShipVia.ValueMember = "ShipperID"
            DdlShipVia.DisplayMember = "CompanyName"

        End Sub

        Private Sub SetDateFieldsToDefault()
            Me.DtpOrderDate.CustomFormat = " "
            Me.DtpOrderDate.Format = DateTimePickerFormat.Custom

            Me.DtpRequiredDate.CustomFormat = " "
            Me.DtpRequiredDate.Format = DateTimePickerFormat.Custom

            Me.DtpShippedDate.CustomFormat = " "
            Me.DtpShippedDate.Format = DateTimePickerFormat.Custom

        End Sub

        Private Sub SetGridProperties()
            DgvGrid.AutoGenerateColumns = False
            DgvGrid.AllowUserToResizeRows = False
            DgvGrid.ColumnCount = 14
            DgvGrid.ScrollBars = ScrollBars.None
            DgvGrid.AllowUserToOrderColumns = False
            DgvGrid.AllowUserToAddRows = False
            DgvGrid.AllowUserToDeleteRows = False
            DgvGrid.ReadOnly = True
        End Sub

        Private Sub SetGridColumnProperties()
            DgvGrid.Columns("OrderID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("CustomerID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("EmployeeID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("OrderDate").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("RequiredDate").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("ShippedDate").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("ShipVia").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("Freight").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("Freight").DefaultCellStyle.Format = "0.00##"
            DgvGrid.Columns("ShipName").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("ShipAddress").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("ShipCity").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("ShipRegion").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("ShipPostalCode").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("ShipCountry").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
        End Sub

        Private Sub MoveToFirstPage()
            SetDataSource(False)
            ToolStripHelper.MoveToFirst(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub MoveToPrevious()
            SetDataSource(False)
            ToolStripHelper.MoveToPrevious(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentRecordNumber)
        End Sub

        Private Sub MoveToNext()
            SetDataSource(False)
            ToolStripHelper.MoveToNext(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentPageNumber, _totalNumberOfPages)
        End Sub

        Private Sub MoveToLastPage()
            SetDataSource(False)
            ToolStripHelper.MoveToLast(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub SetDataSource(isSort As Boolean)
            If isSort Then
                _sortByExpression = GetSortByExpression()
            End If

            ' fill the datagridview
            If IsUseSearch() Then
                Dim orderID As Integer? = Nothing
                Dim customerID As String = Nothing
                Dim employeeID As Integer? = Nothing
                Dim orderDate As Date? = Nothing
                Dim requiredDate As Date? = Nothing
                Dim shippedDate As Date? = Nothing
                Dim shipVia As Integer? = Nothing
                Dim freight As Decimal? = Nothing
                Dim shipName As String = Nothing
                Dim shipAddress As String = Nothing
                Dim shipCity As String = Nothing
                Dim shipRegion As String = Nothing
                Dim shipPostalCode As String = Nothing
                Dim shipCountry As String = Nothing

                If Not String.IsNullOrEmpty(TxtOrderID.Text.Trim()) Then
                    orderID = Convert.ToInt32(TxtOrderID.Text.ToString())
                End if

                If DdlCustomerID.SelectedIndex <> 0 Then
                    customerID = DdlCustomerID.SelectedValue.ToString()
                End if

                If DdlEmployeeID.SelectedIndex <> 0 Then
                    employeeID = Convert.ToInt32(DdlEmployeeID.SelectedValue.ToString())
                End if

                If Not String.IsNullOrEmpty(DtpOrderDate.Text.Trim()) Then
                    orderDate = DtpOrderDate.Value
                End if

                If Not String.IsNullOrEmpty(DtpRequiredDate.Text.Trim()) Then
                    requiredDate = DtpRequiredDate.Value
                End if

                If Not String.IsNullOrEmpty(DtpShippedDate.Text.Trim()) Then
                    shippedDate = DtpShippedDate.Value
                End if

                If DdlShipVia.SelectedIndex <> 0 Then
                    shipVia = Convert.ToInt32(DdlShipVia.SelectedValue.ToString())
                End if

                If Not String.IsNullOrEmpty(MtbFreight.Text.Trim()) Then
                    freight = Convert.ToDecimal(MtbFreight.Text.ToString())
                End if

                If Not String.IsNullOrEmpty(TxtShipName.Text.Trim()) Then
                    shipName = TxtShipName.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtShipAddress.Text.Trim()) Then
                    shipAddress = TxtShipAddress.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtShipCity.Text.Trim()) Then
                    shipCity = TxtShipCity.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtShipRegion.Text.Trim()) Then
                    shipRegion = TxtShipRegion.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtShipPostalCode.Text.Trim()) Then
                    shipPostalCode = TxtShipPostalCode.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtShipCountry.Text.Trim()) Then
                    shipCountry = TxtShipCountry.Text.ToString()
                End if

                _totalRecords = Orders.GetRecordCountDynamicWhere(orderID, customerID, employeeID, orderDate, requiredDate, shippedDate, shipVia, freight, shipName, shipAddress, shipCity, shipRegion, shipPostalCode, shipCountry)
                DgvGrid.DataSource = Orders.SelectSkipAndTakeDynamicWhere(orderID, customerID, employeeID, orderDate, requiredDate, shippedDate, shipVia, freight, shipName, shipAddress, shipCity, shipRegion, shipPostalCode, shipCountry, ConstantHelper.GridNumberOfRows, _currentRecordNumber, _sortByExpression)
            Else
                DgvGrid.DataSource = Orders.SelectSkipAndTake(ConstantHelper.GridNumberOfRows, _currentRecordNumber, _totalRecords, _sortByExpression)
            End If

            DgvGrid.Columns(_sortColumnIndex).HeaderCell.SortGlyphDirection = _sortOrder

            If _totalRecords > 0 Then
                _totalNumberOfPages = CInt(Math.Ceiling(CSng(_totalRecords) / CSng(ConstantHelper.GridNumberOfRows)))
                TscbPage.ComboBox.Items.Clear()

                ' fill page numbers in toolstripcombobox
                For page As Integer = 1 To _totalNumberOfPages
                    _isFireToolStripComboBoxSelectedIndexChanged = False
                    TscbPage.ComboBox.Items.Add(page)
                Next

                TscbPage.ComboBox.SelectedItem = _currentPageNumber
                _isFireToolStripComboBoxSelectedIndexChanged = True
            End If

            ResetNavButtonsToDefault()
        End Sub

        Private Function GetSortByExpression() As String
            _sortOrder = SortOrder.Descending

            Select Case _sortColumnIndex
                Case 0
                    If _sortByExpression = "OrderID" Then
                        Return "OrderID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "OrderID"

                Case 1
                    If _sortByExpression = "CustomerID" Then
                        Return "CustomerID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "CustomerID"

                Case 2
                    If _sortByExpression = "EmployeeID" Then
                        Return "EmployeeID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "EmployeeID"

                Case 3
                    If _sortByExpression = "OrderDate" Then
                        Return "OrderDate desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "OrderDate"

                Case 4
                    If _sortByExpression = "RequiredDate" Then
                        Return "RequiredDate desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "RequiredDate"

                Case 5
                    If _sortByExpression = "ShippedDate" Then
                        Return "ShippedDate desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShippedDate"

                Case 6
                    If _sortByExpression = "ShipVia" Then
                        Return "ShipVia desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipVia"

                Case 7
                    If _sortByExpression = "Freight" Then
                        Return "Freight desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Freight"

                Case 8
                    If _sortByExpression = "ShipName" Then
                        Return "ShipName desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipName"

                Case 9
                    If _sortByExpression = "ShipAddress" Then
                        Return "ShipAddress desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipAddress"

                Case 10
                    If _sortByExpression = "ShipCity" Then
                        Return "ShipCity desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipCity"

                Case 11
                    If _sortByExpression = "ShipRegion" Then
                        Return "ShipRegion desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipRegion"

                Case 12
                    If _sortByExpression = "ShipPostalCode" Then
                        Return "ShipPostalCode desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipPostalCode"

                Case 13
                    If _sortByExpression = "ShipCountry" Then
                        Return "ShipCountry desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ShipCountry"

                Case Else
                    If _sortByExpression = "OrderID" Then
                        Return "OrderID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "OrderID"
            End Select
        End Function

        Private Sub ResetNavButtonsToDefault()
            BnvNavCountItem.Text = "of " & _totalNumberOfPages & " pages"

            If _totalRecords <= ConstantHelper.GridNumberOfRows Then
                BnvNavMoveNextItem.Enabled = False
                BnvNavMoveLastItem.Enabled = False
            Else
                BnvNavMoveNextItem.Enabled = True
                BnvNavMoveLastItem.Enabled = True
            End If
        End Sub

        Private Function IsUseSearch() As Boolean
            If TxtOrderID.Text.Trim() <> "" Or DdlCustomerID.SelectedIndex <> 0 Or DdlEmployeeID.SelectedIndex <> 0 Or DtpOrderDate.Text.Trim() <> "" Or DtpRequiredDate.Text.Trim() <> "" Or DtpShippedDate.Text.Trim() <> "" Or DdlShipVia.SelectedIndex <> 0 Or MtbFreight.Text.Trim() <> "" Or TxtShipName.Text.Trim() <> "" Or TxtShipAddress.Text.Trim() <> "" Or TxtShipCity.Text.Trim() <> "" Or TxtShipRegion.Text.Trim() <> "" Or TxtShipPostalCode.Text.Trim() <> "" Or TxtShipCountry.Text.Trim() <> "" Then
                BtnCancel.Enabled = True
                Return True
            End If

            Return False
        End Function

        Private Sub HightlightRow(rowIndex As Integer)
            ' highlight row
            DgvGrid.ClearSelection()
            DgvGrid.Rows(rowIndex).Selected = True

            ' remember the current row clicked by the user
            _currentRowIndex = rowIndex
        End Sub


        #End Region

        #Region "search page control events"

        Private Sub DdlCustomerID_KeyPress(sender As Object, e As KeyPressEventArgs) Handles DdlCustomerID.KeyPress
            e.Handled = True
        End Sub

        Private Sub DdlEmployeeID_KeyPress(sender As Object, e As KeyPressEventArgs) Handles DdlEmployeeID.KeyPress
            e.Handled = True
        End Sub

        Private Sub DdlShipVia_KeyPress(sender As Object, e As KeyPressEventArgs) Handles DdlShipVia.KeyPress
            e.Handled = True
        End Sub

        Private Sub BtnSearch_Click(sender As Object, e As EventArgs) Handles BtnSearch.Click
            If IsUseSearch() Then
                _currentRecordNumber = 1
                _currentPageNumber = 1
                _sortByExpression = "OrderID"

                BnvNavMoveFirstItem.Enabled = False
                BnvNavMovePreviousItem.Enabled = False

                SetDataSource(False)
                ResetNavButtonsToDefault()
            Else
                MessageBox.Show("At least one (1) search field must be filled!", "Empty Search Fields")
            End If
        End Sub

        Private Sub BtnCancel_Click(sender As Object, e As EventArgs) Handles BtnCancel.Click
            ' reset all control's values
            TxtOrderID.Text = ""
            DdlCustomerID.SelectedIndex = 0
            DdlEmployeeID.SelectedIndex = 0
            DdlShipVia.SelectedIndex = 0
            MtbFreight.Text = ""
            TxtShipName.Text = ""
            TxtShipAddress.Text = ""
            TxtShipCity.Text = ""
            TxtShipRegion.Text = ""
            TxtShipPostalCode.Text = ""
            TxtShipCountry.Text = ""

            SetDateFieldsToDefault()
            SetDataSource(False)
            ResetNavButtonsToDefault()

            BtnCancel.Enabled = False
        End Sub

        Private Sub DtpOrderDate_ValueChanged(sender As Object, e As EventArgs) Handles DtpOrderDate.ValueChanged
            Me.DtpOrderDate.CustomFormat = "MM/dd/yyyy"
        End Sub

        Private Sub DtpRequiredDate_ValueChanged(sender As Object, e As EventArgs) Handles DtpRequiredDate.ValueChanged
            Me.DtpRequiredDate.CustomFormat = "MM/dd/yyyy"
        End Sub

        Private Sub DtpShippedDate_ValueChanged(sender As Object, e As EventArgs) Handles DtpShippedDate.ValueChanged
            Me.DtpShippedDate.CustomFormat = "MM/dd/yyyy"
        End Sub


        #End Region
    End Class
