Imports System
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class ListSearchProducts
        Dim _currentRecordNumber As Integer = 1
        Dim _currentPageNumber As Integer = 1
        Dim _totalRecords As Integer = 0
        Dim _totalNumberOfPages As Integer = 0
        Dim _sortColumnIndex As Integer = 0
        Dim _sortByExpression As String = "ProductID"
        Dim _sortOrder As SortOrder = SortOrder.Ascending
        Dim _currentRowIndex As Integer
        Dim _isFireToolStripComboBoxSelectedIndexChanged As Boolean

        #Region "winform events"

        Private Sub ListSearchProducts_Load(sender As Object, e As EventArgs) Handles MyBase.Load
            FillComboBoxWithData()
            SetGridProperties()
            SetGridColumnProperties()
            SetDataSource(False)
            ResetNavButtonsToDefault()

            LblLine.Width = Me.Width
        End Sub

        Private Sub ListSearchProducts_Resize(sender As Object, e As EventArgs) Handles MyBase.Resize
            LblLine.Width = Me.Width
        End Sub

        #End Region

        #Region "binding navigator control events"

        Private Sub BnvNavMoveFirstItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveFirstItem.Click
            _currentRecordNumber = 1
            _currentPageNumber = 1
            MoveToFirstPage()
        End Sub

        Private Sub BnvNavMovePreviousItem_Click(sender As Object, e As EventArgs) Handles BnvNavMovePreviousItem.Click
            _currentPageNumber -= 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToPrevious()
        End Sub

        Private Sub BnvNavMoveNextItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveNextItem.Click
            _currentPageNumber += 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToNext()
        End Sub

        Private Sub BnvNavMoveLastItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveLastItem.Click
            _currentPageNumber = _totalNumberOfPages
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToLastPage()
        End Sub


        #End Region

        #Region "data gridview events"

        Private Sub DgvGrid_ColumnHeaderMouseClick(sender As Object, e As DataGridViewCellMouseEventArgs) Handles DgvGrid.ColumnHeaderMouseClick
            _sortColumnIndex = e.ColumnIndex
            SetDataSource(True)
        End Sub

        Private Sub DgvGrid_CellContextMenuStripNeeded(sender As Object, e As DataGridViewCellContextMenuStripNeededEventArgs) Handles DgvGrid.CellContextMenuStripNeeded
            HightlightRow(e.RowIndex)
        End Sub

        Private Sub DgvGrid_CellClick(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.CellClick
            ' make sure the click did not come from the header
            If e.RowIndex <> -1 Then
                HightlightRow(e.RowIndex)
            End If
        End Sub

        Private Sub DgvGrid_CellContentClick(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.CellContentClick
            If e.ColumnIndex = 2  And e.RowIndex <> -1 Then
                Dim supplierID1 As Integer = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("SupplierID").Value.ToString())

                Dim form As New DetailsSuppliers(supplierID1)
                form.ShowDialog()
            End If
            If e.ColumnIndex = 3  And e.RowIndex <> -1 Then
                Dim categoryID1 As Integer = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("CategoryID").Value.ToString())

                Dim form As New DetailsCategories(categoryID1)
                form.ShowDialog()
            End If
        End Sub

        #End Region

        #Region "toolstrip menu item events"

        Private Sub TscbPage_SelectedIndexChanged(sender As Object, e As EventArgs) Handles TscbPage.SelectedIndexChanged
            ' make sure this event is not called 
            ' when adding items to the tool strip combo box
            If Not _isFireToolStripComboBoxSelectedIndexChanged Then
                Return
            End If

            ' set current page number
            Dim lastVisitedPageNumber As Integer = _currentPageNumber
            _currentPageNumber = Cint(TscbPage.ComboBox.SelectedItem)

            If _currentPageNumber = 1 Then
                _currentRecordNumber = 1
            Else
                _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))
            End If

            ' go to the selected page
            If _currentPageNumber = 1 Then
                MoveToFirstPage()
            Else If _currentPageNumber = _totalNumberOfPages Then
                MoveToLastPage()
            Else If _currentPageNumber < lastVisitedPageNumber Then
                MoveToPrevious()
            Else
                MoveToNext()
            End If
        End Sub

        #End Region

        #Region "private methods"

        Private Sub FillComboBoxWithData()
            ' fill SupplierID combo box
            Dim objSuppliersCol1 As New SuppliersCollection()
            objSuppliersCol1.Add(New Suppliers With { .SupplierID = -1, .CompanyName = "" })

            ' make sure collection is not null
            Dim objSuppliersCol2 As SuppliersCollection = Suppliers.SelectSuppliersDropDownListData()

            If objSuppliersCol2 IsNot Nothing Then
                objSuppliersCol1.AddRange(objSuppliersCol2)
            End If

            DdlSupplierID.DataSource = objSuppliersCol1
            DdlSupplierID.ValueMember = "SupplierID"
            DdlSupplierID.DisplayMember = "CompanyName"

            ' fill CategoryID combo box
            Dim objCategoriesCol3 As New CategoriesCollection()
            objCategoriesCol3.Add(New Categories With { .CategoryID = -1, .CategoryName = "" })

            ' make sure collection is not null
            Dim objCategoriesCol4 As CategoriesCollection = Categories.SelectCategoriesDropDownListData()

            If objCategoriesCol4 IsNot Nothing Then
                objCategoriesCol3.AddRange(objCategoriesCol4)
            End If

            DdlCategoryID.DataSource = objCategoriesCol3
            DdlCategoryID.ValueMember = "CategoryID"
            DdlCategoryID.DisplayMember = "CategoryName"

            DdlDiscontinued.Items.Insert(0, "")
            DdlDiscontinued.Items.Insert(1, "True")
            DdlDiscontinued.Items.Insert(2, "False")
            DdlDiscontinued.SelectedIndex = 0

        End Sub

        Private Sub SetGridProperties()
            DgvGrid.AutoGenerateColumns = False
            DgvGrid.AllowUserToResizeRows = False
            DgvGrid.ColumnCount = 10
            DgvGrid.ScrollBars = ScrollBars.None
            DgvGrid.AllowUserToOrderColumns = False
            DgvGrid.AllowUserToAddRows = False
            DgvGrid.AllowUserToDeleteRows = False
            DgvGrid.ReadOnly = True
        End Sub

        Private Sub SetGridColumnProperties()
            DgvGrid.Columns("ProductID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("ProductName").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("SupplierID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("CategoryID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("QuantityPerUnit").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("UnitPrice").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("UnitPrice").DefaultCellStyle.Format = "0.00##"
            DgvGrid.Columns("UnitsInStock").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("UnitsOnOrder").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("ReorderLevel").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            DgvGrid.Columns("Discontinued").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter
        End Sub

        Private Sub MoveToFirstPage()
            SetDataSource(False)
            ToolStripHelper.MoveToFirst(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub MoveToPrevious()
            SetDataSource(False)
            ToolStripHelper.MoveToPrevious(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentRecordNumber)
        End Sub

        Private Sub MoveToNext()
            SetDataSource(False)
            ToolStripHelper.MoveToNext(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentPageNumber, _totalNumberOfPages)
        End Sub

        Private Sub MoveToLastPage()
            SetDataSource(False)
            ToolStripHelper.MoveToLast(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub SetDataSource(isSort As Boolean)
            If isSort Then
                _sortByExpression = GetSortByExpression()
            End If

            ' fill the datagridview
            If IsUseSearch() Then
                Dim productID As Integer? = Nothing
                Dim productName As String = Nothing
                Dim supplierID As Integer? = Nothing
                Dim categoryID As Integer? = Nothing
                Dim quantityPerUnit As String = Nothing
                Dim unitPrice As Decimal? = Nothing
                Dim unitsInStock As Short? = Nothing
                Dim unitsOnOrder As Short? = Nothing
                Dim reorderLevel As Short? = Nothing
                Dim discontinued As Boolean? = Nothing

                If Not String.IsNullOrEmpty(TxtProductID.Text.Trim()) Then
                    productID = Convert.ToInt32(TxtProductID.Text.ToString())
                End if

                If Not String.IsNullOrEmpty(TxtProductName.Text.Trim()) Then
                    productName = TxtProductName.Text.ToString()
                End if

                If DdlSupplierID.SelectedIndex <> 0 Then
                    supplierID = Convert.ToInt32(DdlSupplierID.SelectedValue.ToString())
                End if

                If DdlCategoryID.SelectedIndex <> 0 Then
                    categoryID = Convert.ToInt32(DdlCategoryID.SelectedValue.ToString())
                End if

                If Not String.IsNullOrEmpty(TxtQuantityPerUnit.Text.Trim()) Then
                    quantityPerUnit = TxtQuantityPerUnit.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(MtbUnitPrice.Text.Trim()) Then
                    unitPrice = Convert.ToDecimal(MtbUnitPrice.Text.ToString())
                End if

                If Not String.IsNullOrEmpty(MtbUnitsInStock.Text.Trim()) Then
                    unitsInStock = Convert.ToInt16(MtbUnitsInStock.Text.ToString())
                End if

                If Not String.IsNullOrEmpty(MtbUnitsOnOrder.Text.Trim()) Then
                    unitsOnOrder = Convert.ToInt16(MtbUnitsOnOrder.Text.ToString())
                End if

                If Not String.IsNullOrEmpty(MtbReorderLevel.Text.Trim()) Then
                    reorderLevel = Convert.ToInt16(MtbReorderLevel.Text.ToString())
                End if

                If DdlDiscontinued.SelectedIndex <> 0 Then
                    discontinued = Convert.ToBoolean(DdlDiscontinued.SelectedItem.ToString())
                End if

                _totalRecords = Products.GetRecordCountDynamicWhere(productID, productName, supplierID, categoryID, quantityPerUnit, unitPrice, unitsInStock, unitsOnOrder, reorderLevel, discontinued)
                DgvGrid.DataSource = Products.SelectSkipAndTakeDynamicWhere(productID, productName, supplierID, categoryID, quantityPerUnit, unitPrice, unitsInStock, unitsOnOrder, reorderLevel, discontinued, ConstantHelper.GridNumberOfRows, _currentRecordNumber, _sortByExpression)
            Else
                DgvGrid.DataSource = Products.SelectSkipAndTake(ConstantHelper.GridNumberOfRows, _currentRecordNumber, _totalRecords, _sortByExpression)
            End If

            DgvGrid.Columns(_sortColumnIndex).HeaderCell.SortGlyphDirection = _sortOrder

            If _totalRecords > 0 Then
                _totalNumberOfPages = CInt(Math.Ceiling(CSng(_totalRecords) / CSng(ConstantHelper.GridNumberOfRows)))
                TscbPage.ComboBox.Items.Clear()

                ' fill page numbers in toolstripcombobox
                For page As Integer = 1 To _totalNumberOfPages
                    _isFireToolStripComboBoxSelectedIndexChanged = False
                    TscbPage.ComboBox.Items.Add(page)
                Next

                TscbPage.ComboBox.SelectedItem = _currentPageNumber
                _isFireToolStripComboBoxSelectedIndexChanged = True
            End If

            ResetNavButtonsToDefault()
        End Sub

        Private Function GetSortByExpression() As String
            _sortOrder = SortOrder.Descending

            Select Case _sortColumnIndex
                Case 0
                    If _sortByExpression = "ProductID" Then
                        Return "ProductID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ProductID"

                Case 1
                    If _sortByExpression = "ProductName" Then
                        Return "ProductName desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ProductName"

                Case 2
                    If _sortByExpression = "SupplierID" Then
                        Return "SupplierID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "SupplierID"

                Case 3
                    If _sortByExpression = "CategoryID" Then
                        Return "CategoryID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "CategoryID"

                Case 4
                    If _sortByExpression = "QuantityPerUnit" Then
                        Return "QuantityPerUnit desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "QuantityPerUnit"

                Case 5
                    If _sortByExpression = "UnitPrice" Then
                        Return "UnitPrice desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "UnitPrice"

                Case 6
                    If _sortByExpression = "UnitsInStock" Then
                        Return "UnitsInStock desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "UnitsInStock"

                Case 7
                    If _sortByExpression = "UnitsOnOrder" Then
                        Return "UnitsOnOrder desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "UnitsOnOrder"

                Case 8
                    If _sortByExpression = "ReorderLevel" Then
                        Return "ReorderLevel desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ReorderLevel"

                Case 9
                    If _sortByExpression = "Discontinued" Then
                        Return "Discontinued desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "Discontinued"

                Case Else
                    If _sortByExpression = "ProductID" Then
                        Return "ProductID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "ProductID"
            End Select
        End Function

        Private Sub ResetNavButtonsToDefault()
            BnvNavCountItem.Text = "of " & _totalNumberOfPages & " pages"

            If _totalRecords <= ConstantHelper.GridNumberOfRows Then
                BnvNavMoveNextItem.Enabled = False
                BnvNavMoveLastItem.Enabled = False
            Else
                BnvNavMoveNextItem.Enabled = True
                BnvNavMoveLastItem.Enabled = True
            End If
        End Sub

        Private Function IsUseSearch() As Boolean
            If TxtProductID.Text.Trim() <> "" Or TxtProductName.Text.Trim() <> "" Or DdlSupplierID.SelectedIndex <> 0 Or DdlCategoryID.SelectedIndex <> 0 Or TxtQuantityPerUnit.Text.Trim() <> "" Or MtbUnitPrice.Text.Trim() <> "" Or MtbUnitsInStock.Text.Trim() <> "" Or MtbUnitsOnOrder.Text.Trim() <> "" Or MtbReorderLevel.Text.Trim() <> "" Or DdlDiscontinued.SelectedIndex <> 0 Then
                BtnCancel.Enabled = True
                Return True
            End If

            Return False
        End Function

        Private Sub HightlightRow(rowIndex As Integer)
            ' highlight row
            DgvGrid.ClearSelection()
            DgvGrid.Rows(rowIndex).Selected = True

            ' remember the current row clicked by the user
            _currentRowIndex = rowIndex
        End Sub


        #End Region

        #Region "search page control events"

        Private Sub DdlSupplierID_KeyPress(sender As Object, e As KeyPressEventArgs) Handles DdlSupplierID.KeyPress
            e.Handled = True
        End Sub

        Private Sub DdlCategoryID_KeyPress(sender As Object, e As KeyPressEventArgs) Handles DdlCategoryID.KeyPress
            e.Handled = True
        End Sub

        Private Sub DdlDiscontinued_KeyPress(sender As Object, e As KeyPressEventArgs) Handles DdlDiscontinued.KeyPress
            e.Handled = True
        End Sub

        Private Sub BtnSearch_Click(sender As Object, e As EventArgs) Handles BtnSearch.Click
            If IsUseSearch() Then
                _currentRecordNumber = 1
                _currentPageNumber = 1
                _sortByExpression = "ProductID"

                BnvNavMoveFirstItem.Enabled = False
                BnvNavMovePreviousItem.Enabled = False

                SetDataSource(False)
                ResetNavButtonsToDefault()
            Else
                MessageBox.Show("At least one (1) search field must be filled!", "Empty Search Fields")
            End If
        End Sub

        Private Sub BtnCancel_Click(sender As Object, e As EventArgs) Handles BtnCancel.Click
            ' reset all control's values
            TxtProductID.Text = ""
            TxtProductName.Text = ""
            DdlSupplierID.SelectedIndex = 0
            DdlCategoryID.SelectedIndex = 0
            TxtQuantityPerUnit.Text = ""
            MtbUnitPrice.Text = ""
            MtbUnitsInStock.Text = ""
            MtbUnitsOnOrder.Text = ""
            MtbReorderLevel.Text = ""
            DdlDiscontinued.SelectedIndex = 0

            SetDataSource(False)
            ResetNavButtonsToDefault()

            BtnCancel.Enabled = False
        End Sub

        #End Region
    End Class
