Imports System
Imports System.Windows.Forms
Imports NorthwindWinFormDynamic_VB.BusinessObject
Imports NorthwindWinFormDynamic_VB.Domain

    Public Class ListSearchTerritories
        Dim _currentRecordNumber As Integer = 1
        Dim _currentPageNumber As Integer = 1
        Dim _totalRecords As Integer = 0
        Dim _totalNumberOfPages As Integer = 0
        Dim _sortColumnIndex As Integer = 0
        Dim _sortByExpression As String = "TerritoryID"
        Dim _sortOrder As SortOrder = SortOrder.Ascending
        Dim _currentRowIndex As Integer
        Dim _isFireToolStripComboBoxSelectedIndexChanged As Boolean

        #Region "winform events"

        Private Sub ListSearchTerritories_Load(sender As Object, e As EventArgs) Handles MyBase.Load
            FillComboBoxWithData()
            SetGridProperties()
            SetGridColumnProperties()
            SetDataSource(False)
            ResetNavButtonsToDefault()

            LblLine.Width = Me.Width
        End Sub

        Private Sub ListSearchTerritories_Resize(sender As Object, e As EventArgs) Handles MyBase.Resize
            LblLine.Width = Me.Width
        End Sub

        #End Region

        #Region "binding navigator control events"

        Private Sub BnvNavMoveFirstItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveFirstItem.Click
            _currentRecordNumber = 1
            _currentPageNumber = 1
            MoveToFirstPage()
        End Sub

        Private Sub BnvNavMovePreviousItem_Click(sender As Object, e As EventArgs) Handles BnvNavMovePreviousItem.Click
            _currentPageNumber -= 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToPrevious()
        End Sub

        Private Sub BnvNavMoveNextItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveNextItem.Click
            _currentPageNumber += 1
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToNext()
        End Sub

        Private Sub BnvNavMoveLastItem_Click(sender As Object, e As EventArgs) Handles BnvNavMoveLastItem.Click
            _currentPageNumber = _totalNumberOfPages
            _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))

            MoveToLastPage()
        End Sub


        #End Region

        #Region "data gridview events"

        Private Sub DgvGrid_ColumnHeaderMouseClick(sender As Object, e As DataGridViewCellMouseEventArgs) Handles DgvGrid.ColumnHeaderMouseClick
            _sortColumnIndex = e.ColumnIndex
            SetDataSource(True)
        End Sub

        Private Sub DgvGrid_CellContextMenuStripNeeded(sender As Object, e As DataGridViewCellContextMenuStripNeededEventArgs) Handles DgvGrid.CellContextMenuStripNeeded
            HightlightRow(e.RowIndex)
        End Sub

        Private Sub DgvGrid_CellClick(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.CellClick
            ' make sure the click did not come from the header
            If e.RowIndex <> -1 Then
                HightlightRow(e.RowIndex)
            End If
        End Sub

        Private Sub DgvGrid_CellContentClick(sender As Object, e As DataGridViewCellEventArgs) Handles DgvGrid.CellContentClick
            If e.ColumnIndex = 2  And e.RowIndex <> -1 Then
                Dim regionID1 As Integer = Convert.ToInt32(DgvGrid.Rows(e.RowIndex).Cells("RegionID").Value.ToString())

                Dim form As New DetailsRegion(regionID1)
                form.ShowDialog()
            End If
        End Sub

        #End Region

        #Region "toolstrip menu item events"

        Private Sub TscbPage_SelectedIndexChanged(sender As Object, e As EventArgs) Handles TscbPage.SelectedIndexChanged
            ' make sure this event is not called 
            ' when adding items to the tool strip combo box
            If Not _isFireToolStripComboBoxSelectedIndexChanged Then
                Return
            End If

            ' set current page number
            Dim lastVisitedPageNumber As Integer = _currentPageNumber
            _currentPageNumber = Cint(TscbPage.ComboBox.SelectedItem)

            If _currentPageNumber = 1 Then
                _currentRecordNumber = 1
            Else
                _currentRecordNumber = 1 + (ConstantHelper.GridNumberOfRows * (_currentPageNumber - 1))
            End If

            ' go to the selected page
            If _currentPageNumber = 1 Then
                MoveToFirstPage()
            Else If _currentPageNumber = _totalNumberOfPages Then
                MoveToLastPage()
            Else If _currentPageNumber < lastVisitedPageNumber Then
                MoveToPrevious()
            Else
                MoveToNext()
            End If
        End Sub

        #End Region

        #Region "private methods"

        Private Sub FillComboBoxWithData()
            ' fill RegionID combo box
            Dim objRegionCol1 As New RegionCollection()
            objRegionCol1.Add(New BusinessObject.Region With { .RegionID = -1, .RegionDescription = "" })

            ' make sure collection is not null
            Dim objRegionCol2 As RegionCollection = BusinessObject.Region.SelectRegionDropDownListData()

            If objRegionCol2 IsNot Nothing Then
                objRegionCol1.AddRange(objRegionCol2)
            End If

            DdlRegionID.DataSource = objRegionCol1
            DdlRegionID.ValueMember = "RegionID"
            DdlRegionID.DisplayMember = "RegionDescription"

        End Sub

        Private Sub SetGridProperties()
            DgvGrid.AutoGenerateColumns = False
            DgvGrid.AllowUserToResizeRows = False
            DgvGrid.ColumnCount = 3
            DgvGrid.ScrollBars = ScrollBars.None
            DgvGrid.AllowUserToOrderColumns = False
            DgvGrid.AllowUserToAddRows = False
            DgvGrid.AllowUserToDeleteRows = False
            DgvGrid.ReadOnly = True
        End Sub

        Private Sub SetGridColumnProperties()
            DgvGrid.Columns("TerritoryID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("TerritoryDescription").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
            DgvGrid.Columns("RegionID").DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
        End Sub

        Private Sub MoveToFirstPage()
            SetDataSource(False)
            ToolStripHelper.MoveToFirst(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub MoveToPrevious()
            SetDataSource(False)
            ToolStripHelper.MoveToPrevious(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentRecordNumber)
        End Sub

        Private Sub MoveToNext()
            SetDataSource(False)
            ToolStripHelper.MoveToNext(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem, _currentPageNumber, _totalNumberOfPages)
        End Sub

        Private Sub MoveToLastPage()
            SetDataSource(False)
            ToolStripHelper.MoveToLast(BnvNavMoveFirstItem, BnvNavMovePreviousItem, BnvNavMoveNextItem, BnvNavMoveLastItem)
        End Sub

        Private Sub SetDataSource(isSort As Boolean)
            If isSort Then
                _sortByExpression = GetSortByExpression()
            End If

            ' fill the datagridview
            If IsUseSearch() Then
                Dim territoryID As String = Nothing
                Dim territoryDescription As String = Nothing
                Dim regionID As Integer? = Nothing

                If Not String.IsNullOrEmpty(TxtTerritoryID.Text.Trim()) Then
                    territoryID = TxtTerritoryID.Text.ToString()
                End if

                If Not String.IsNullOrEmpty(TxtTerritoryDescription.Text.Trim()) Then
                    territoryDescription = TxtTerritoryDescription.Text.ToString()
                End if

                If DdlRegionID.SelectedIndex <> 0 Then
                    regionID = Convert.ToInt32(DdlRegionID.SelectedValue.ToString())
                End if

                _totalRecords = Territories.GetRecordCountDynamicWhere(territoryID, territoryDescription, regionID)
                DgvGrid.DataSource = Territories.SelectSkipAndTakeDynamicWhere(territoryID, territoryDescription, regionID, ConstantHelper.GridNumberOfRows, _currentRecordNumber, _sortByExpression)
            Else
                DgvGrid.DataSource = Territories.SelectSkipAndTake(ConstantHelper.GridNumberOfRows, _currentRecordNumber, _totalRecords, _sortByExpression)
            End If

            DgvGrid.Columns(_sortColumnIndex).HeaderCell.SortGlyphDirection = _sortOrder

            If _totalRecords > 0 Then
                _totalNumberOfPages = CInt(Math.Ceiling(CSng(_totalRecords) / CSng(ConstantHelper.GridNumberOfRows)))
                TscbPage.ComboBox.Items.Clear()

                ' fill page numbers in toolstripcombobox
                For page As Integer = 1 To _totalNumberOfPages
                    _isFireToolStripComboBoxSelectedIndexChanged = False
                    TscbPage.ComboBox.Items.Add(page)
                Next

                TscbPage.ComboBox.SelectedItem = _currentPageNumber
                _isFireToolStripComboBoxSelectedIndexChanged = True
            End If

            ResetNavButtonsToDefault()
        End Sub

        Private Function GetSortByExpression() As String
            _sortOrder = SortOrder.Descending

            Select Case _sortColumnIndex
                Case 0
                    If _sortByExpression = "TerritoryID" Then
                        Return "TerritoryID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "TerritoryID"

                Case 1
                    If _sortByExpression = "TerritoryDescription" Then
                        Return "TerritoryDescription desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "TerritoryDescription"

                Case 2
                    If _sortByExpression = "RegionID" Then
                        Return "RegionID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "RegionID"

                Case Else
                    If _sortByExpression = "TerritoryID" Then
                        Return "TerritoryID desc"
                    End If

                    _sortOrder = SortOrder.Ascending
                    Return "TerritoryID"
            End Select
        End Function

        Private Sub ResetNavButtonsToDefault()
            BnvNavCountItem.Text = "of " & _totalNumberOfPages & " pages"

            If _totalRecords <= ConstantHelper.GridNumberOfRows Then
                BnvNavMoveNextItem.Enabled = False
                BnvNavMoveLastItem.Enabled = False
            Else
                BnvNavMoveNextItem.Enabled = True
                BnvNavMoveLastItem.Enabled = True
            End If
        End Sub

        Private Function IsUseSearch() As Boolean
            If TxtTerritoryID.Text.Trim() <> "" Or TxtTerritoryDescription.Text.Trim() <> "" Or DdlRegionID.SelectedIndex <> 0 Then
                BtnCancel.Enabled = True
                Return True
            End If

            Return False
        End Function

        Private Sub HightlightRow(rowIndex As Integer)
            ' highlight row
            DgvGrid.ClearSelection()
            DgvGrid.Rows(rowIndex).Selected = True

            ' remember the current row clicked by the user
            _currentRowIndex = rowIndex
        End Sub


        #End Region

        #Region "search page control events"

        Private Sub DdlRegionID_KeyPress(sender As Object, e As KeyPressEventArgs) Handles DdlRegionID.KeyPress
            e.Handled = True
        End Sub

        Private Sub BtnSearch_Click(sender As Object, e As EventArgs) Handles BtnSearch.Click
            If IsUseSearch() Then
                _currentRecordNumber = 1
                _currentPageNumber = 1
                _sortByExpression = "TerritoryID"

                BnvNavMoveFirstItem.Enabled = False
                BnvNavMovePreviousItem.Enabled = False

                SetDataSource(False)
                ResetNavButtonsToDefault()
            Else
                MessageBox.Show("At least one (1) search field must be filled!", "Empty Search Fields")
            End If
        End Sub

        Private Sub BtnCancel_Click(sender As Object, e As EventArgs) Handles BtnCancel.Click
            ' reset all control's values
            TxtTerritoryID.Text = ""
            TxtTerritoryDescription.Text = ""
            DdlRegionID.SelectedIndex = 0

            SetDataSource(False)
            ResetNavButtonsToDefault()

            BtnCancel.Enabled = False
        End Sub

        #End Region
    End Class
